SystemOrganization addCategory: #'Monticello-Base'!
SystemOrganization addCategory: #'Monticello-Chunk Format'!
SystemOrganization addCategory: #'Monticello-Loading'!
SystemOrganization addCategory: #'Monticello-Merging'!
SystemOrganization addCategory: #'Monticello-Modeling'!
SystemOrganization addCategory: #'Monticello-Orphanage'!
SystemOrganization addCategory: #'Monticello-Patching'!
SystemOrganization addCategory: #'Monticello-Repositories'!
SystemOrganization addCategory: #'Monticello-Storing'!
SystemOrganization addCategory: #'Monticello-UI'!
SystemOrganization addCategory: #'Monticello-Versioning'!

!String methodsFor: '*monticello' stamp: 'kph 5/26/2007 10:52'!
displayProgress: existingBarOrNil at: aPoint from: minVal to: maxVal during: workBlock  
 
	existingBarOrNil ifNil: [
		self displayProgressAt: Sensor cursorPoint
			from: minVal to: maxVal
			during: workBlock.	
	]
	ifNotNil: [ 
		existingBarOrNil value: (existingBarOrNil value: #setMax) + (maxVal - minVal). 
		existingBarOrNil value: self.
		workBlock value: existingBarOrNil.
	].

! !

!String methodsFor: '*monticello' stamp: 'avi 2/4/2004 14:14'!
extractNumber
	^ ('0', self select: [:ea | ea isDigit]) asNumber! !

!HTTPSocket class methodsFor: '*monticello-override' stamp: 'bf 11/10/2005 15:29'!
digestFor: serverText method: method url: url user: user password: password
	"RFC2069"
	| sock |
	sock := HTTPSocket new. "header decoder is on instance side"
	sock header: (serverText readStream upToAll: CrLf, CrLf).
	^self digestFrom: sock method: method url: url user: user password: password! !

!HTTPSocket class methodsFor: '*monticello-override' stamp: 'bf 11/10/2005 15:40'!
digestFrom: sock method: method url: url user: user password: password
	"RFC2069"
	| auth fields realm nonce uri a1 a2 response |
	sock responseCode = '401' ifFalse: [^nil].
	auth := sock getHeader: 'www-authenticate'.
	(auth asLowercase beginsWith: 'digest') ifFalse: [^nil].

	fields := (((auth allButFirst: 6) findTokens: ', 	') collect: [:ea |
		(ea copyUpTo: $=) asLowercase -> (ea copyAfter: $=) withoutQuoting]) as: Dictionary.

	realm := fields at: 'realm'.
	nonce := fields at: 'nonce'.
	uri := url readStream upToAll: '://'; skipTo: $/; skip: -1; upTo: $#.
	a1 := self md5Hash: user, ':', realm, ':', password.
	a2 := self md5Hash: method, ':', uri.
	a1 ifNil: [^nil "no MD5 support"].
	response := self md5Hash: a1, ':', nonce, ':', a2.

	^String streamContents: [:digest |
		digest
			nextPutAll: 'username="', user, '"';
			nextPutAll: ', realm="', realm, '"';
			nextPutAll: ', nonce="', nonce, '"';
			nextPutAll: ', uri="', uri, '"';
			nextPutAll: ', response="', response, '"'.
		fields at: 'opaque' ifPresent: [:opaque |
			digest nextPutAll: ', opaque="', opaque, '"'].
	]
! !

!HTTPSocket class methodsFor: '*monticello-override' stamp: 'bf 11/10/2005 15:06'!
httpGet: url args: args user: user passwd: passwd
	| authorization result |
	authorization := (Base64MimeConverter mimeEncode: (user , ':' , passwd) readStream) contents.
	result := self 
		httpGet: url args: args accept: '*/*' 
		request: 'Authorization: Basic ' , authorization , CrLf.
	result isString ifFalse: [^result].

	authorization := self digestFor: result method: 'GET' url: url user: user password: passwd.
	authorization ifNil: [^result].
	^self 
		httpGet: url args: args accept: '*/*' 
		request: 'Authorization: Digest ' , authorization , CrLf.
! !

!HTTPSocket class methodsFor: '*monticello-override' stamp: 'mtf 6/9/2008 20:02'!
httpGetDocument: url args: args accept: mimeType request: requestString
	"Return the exact contents of a web object. Asks for the given MIME 
type. If mimeType is nil, use 'text/html'. An extra requestString may be 
submitted and must end with crlf.  The parsed header is saved. Use a 
proxy server if one has been registered.  tk 7/23/97 17:12"
	"Note: To fetch raw data, you can use the MIME type 
'application/octet-stream'."

	| serverName serverAddr port sock header length bare page list firstData 
aStream index connectToHost connectToPort type newUrl |
	Socket initializeNetwork.
	bare := (url asLowercase beginsWith: 'http://') 
		ifTrue: [url copyFrom: 8 to: url size]
		ifFalse: [url].
	bare := bare copyUpTo: $#.  "remove fragment, if specified"
	serverName := bare copyUpTo: $/.
	page := bare copyFrom: serverName size + 1 to: bare size.
	(serverName includes: $:) 
		ifTrue: [ index := serverName indexOf: $:.
			port := (serverName copyFrom: index+1 to: serverName size) asNumber.
			serverName := serverName copyFrom: 1 to: index-1. ]
		ifFalse: [ port := self defaultPort ].
	page size = 0 ifTrue: [page := '/'].
	"add arguments"
	args ifNotNil: [page := page, (self argString: args) ].


	(self shouldUseProxy: serverName)
		ifFalse: [ 
			connectToHost := serverName.
			connectToPort := port ]
		ifTrue:  [
			page := 'http://', serverName, ':', port printString, page.		"put back 
together"
			connectToHost := HTTPProxyServer.
			connectToPort := HTTPProxyPort].
	

	serverAddr := NetNameResolver addressForName: connectToHost timeout: 20.
	serverAddr ifNil: [
		^ 'Could not resolve the server named: ', connectToHost].

3 timesRepeat: [
	sock := HTTPSocket new.
	sock connectTo: serverAddr port: connectToPort.
	(sock waitForConnectionUntil: (self deadlineSecs: 30)) ifFalse: [
		Socket deadServer: connectToHost.  sock destroy.
		^ 'Server ',connectToHost,' is not responding'].
	"Transcript cr;show: url; cr.
	Transcript show: page; cr."
	sock sendCommand: 'GET ', page, ' HTTP/1.0', CrLf, 
		(mimeType ifNotNil: ['ACCEPT: ', mimeType, CrLf] ifNil: ['']),
		'ACCEPT: text/html', CrLf,	"Always accept plain text"
		HTTPBlabEmail,	"may be empty"
		requestString,	"extra user request. Authorization"
		self userAgentString, CrLf,
		'Host: ', serverName, ':', port printString, CrLf.	"blank line 
automatically added"

	list := sock getResponseUpTo: CrLf, CrLf ignoring: (String with: CR).	"list = header, CrLf, CrLf, 
beginningOfData"
	header := list at: 1.
	"Transcript show: page; cr; show: header; cr."
	firstData := list at: 3.
	header isEmpty 
		ifTrue: [aStream := 'server aborted early']
		ifFalse: [
			"dig out some headers"
			sock header: header.
			length := sock getHeader: 'content-length'.
			length ifNotNil: [ length := length asNumber ].
			type := sock getHeader: 'content-type'.
			sock responseCode first = $3 ifTrue: [
				newUrl := sock getHeader: 'location'.
				newUrl ifNotNil: [ 
					Transcript show: 'redirecting to ', newUrl; cr.
					sock destroy.
					newUrl := self expandUrl: newUrl ip: serverAddr port: connectToPort.
					^self httpGetDocument: newUrl args: args  accept: mimeType request: requestString] ].
			aStream := sock getRestOfBuffer: firstData totalLength: length.
			"a 400-series error"
			sock responseCode first = $4 ifTrue: [^ header, aStream contents].
			].
	sock destroy.	"Always OK to destroy!!"
	aStream class ~~ String ifTrue: [
 		^ MIMEDocument contentType: type content: aStream contents url: url].
	aStream = 'server aborted early' ifTrue: [ ^aStream ].
	].

{'HTTPSocket class>>httpGetDocument:args:accept:request:'. aStream. url} inspect.

	^'some other bad thing happened!!'! !

!HTTPSocket class methodsFor: '*monticello-override' stamp: 'mtf 6/9/2008 20:03'!
httpPost: url args: args user: user passwd: passwd
	| authorization result |
	authorization := (Base64MimeConverter mimeEncode: (user , ':' , passwd) readStream) contents.
	result := self 
		httpPostDocument: url args: args accept: '*/*' 
		request: 'Authorization: Basic ' , authorization , CrLf.
	result isString ifFalse: [^result].

	authorization := self digestFor: result method: 'POST' url: url user: user password: passwd.
	authorization ifNil: [^result].
	^self 
		httpPostDocument: url args: args accept: '*/*' 
		request: 'Authorization: Digest ' , authorization , CrLf.
! !

!HTTPSocket class methodsFor: '*monticello-override' stamp: 'mtf 6/9/2008 20:03'!
httpPut: contents to: url user: user passwd: passwd
	"Upload the contents of the stream to a file on the server"

	| bare serverName specifiedServer port page serverAddr authorization s list header firstData length aStream command digest |
	Socket initializeNetwork.
 
	"parse url"
	bare := (url asLowercase beginsWith: 'http://') 
		ifTrue: [url copyFrom: 8 to: url size]
		ifFalse: [url].
	serverName := bare copyUpTo: $/.
	specifiedServer := serverName.
	(serverName includes: $:) ifFalse: [ port := self defaultPort ] ifTrue: [
		port := (serverName copyFrom: (serverName indexOf: $:) + 1 
				to: serverName size) asNumber.
		serverName := serverName copyUpTo: $:.
	].

	page := bare copyFrom: (bare indexOf: $/) to: bare size.
	page size = 0 ifTrue: [page := '/'].
	(self shouldUseProxy: serverName) ifTrue: [ 
		page := 'http://', serverName, ':', port printString, page.		"put back together"
		serverName := HTTPProxyServer.
		port := HTTPProxyPort].

  	"make the request"	
	serverAddr := NetNameResolver addressForName: serverName timeout: 20.
	serverAddr ifNil: [
		^ 'Could not resolve the server named: ', serverName].

	authorization := ' Basic ', (Base64MimeConverter mimeEncode: (user , ':' , passwd) readStream) contents.
[
	s := HTTPSocket new.
	s connectTo: serverAddr port: port.
	s waitForConnectionUntil: self standardDeadline.
	Transcript cr; show: url; cr.
	command := 
		'PUT ', page, ' HTTP/1.0', CrLf, 
		self userAgentString, CrLf,
		'Host: ', specifiedServer, CrLf, 
		'ACCEPT: */*', CrLf,
		HTTPProxyCredentials,
		'Authorization: ' , authorization , CrLf , 
		'Content-length: ', contents size printString, CrLf , CrLf , 
		contents.
	s sendCommand: command.
	"get the header of the reply"
	list := s getResponseUpTo: CrLf, CrLf ignoring: (String with: CR).	"list = header, CrLf, CrLf, beginningOfData"
	header := list at: 1.
	"Transcript show: page; cr; show: argsStream contents; cr; show: header; cr."
	firstData := list at: 3.

	"dig out some headers"
	s header: header.

(authorization beginsWith: 'Digest ') not
and: [(digest := self digestFrom: s method: 'PUT' url: url user: user password: passwd) notNil]]
	whileTrue: [authorization :=  'Digest ', digest].

	length := s getHeader: 'content-length'.
	length ifNotNil: [ length := length asNumber ].

	aStream := s getRestOfBuffer: firstData totalLength: length.
	s destroy.	"Always OK to destroy!!"
	^ header, aStream contents! !

!HTTPSocket class methodsFor: '*monticello-override' stamp: 'bf 11/10/2005 14:08'!
md5Hash: aString
	"Answer hash of aString as lowercase 32 digit hex String.
	There are several providers of MD5 hash ..."
	"(self md5Hash: 'user:realm:passwd') =  '007e68e539ed680c24f6d9a370f3bcb1'"
	| hash |
	hash := Smalltalk at: #CMD5Hasher ifPresent: [:cls |
		cls hashMessage: aString].
	hash ifNil: [
		hash := Smalltalk at: #TCryptoRandom ifPresent: [:cls |
			(cls basicNew md5HashMessage: aString) asInteger]].
	hash ifNotNil: [
		hash := hash hex asLowercase.
		(hash beginsWith: '16r') ifTrue: [hash := hash allButFirst: 3].
		hash := hash padded: #left to: 32 with: $0].
	^hash! !

!Stream methodsFor: '*monticello' stamp: 'cwp 8/9/2003 12:02'!
isMessageStream
	^ false! !

!Trait methodsFor: '*monticello' stamp: 'al 3/26/2006 21:44'!
asClassDefinition
	^ MCTraitDefinition
		name: self name
		traitComposition: self traitCompositionString
		category: self category 
		comment: self organization classComment asString
		commentStamp: self organization commentStamp.! !

!Trait methodsFor: '*monticello' stamp: 'al 3/26/2006 21:45'!
classDefinitions
	| definitions |
	definitions := OrderedCollection with: self asClassDefinition.
	(self hasClassTrait
		and: [self classTrait hasTraitComposition]
		and: [self classTrait traitComposition isEmpty not])
			ifTrue: [definitions add: self classTrait asMCDefinition].
	^definitions asArray! !

!Class methodsFor: '*monticello' stamp: 'al 3/26/2006 21:31'!
asClassDefinition
	^ MCClassDefinition
		name: self name
		superclassName: self superclass name
		traitComposition: self traitCompositionString
		classTraitComposition: self class traitCompositionString
		category: self category 
		instVarNames: self instVarNames
		classVarNames: self classVarNames
		poolDictionaryNames: self poolDictionaryNames
		classInstVarNames: self class instVarNames
		type: self typeOfClass
		comment: self organization classComment	 asString
		commentStamp: self organization commentStamp	! !

!Class methodsFor: '*monticello' stamp: 'avi 3/10/2004 13:32'!
classDefinitions
	^ Array with: self asClassDefinition! !

!Class methodsFor: '*monticello' stamp: 'ab 4/14/2003 22:30'!
poolDictionaryNames
	^ self sharedPools collect: [:ea | self environment keyAtIdentityValue: ea]! !

!TimeStamp class methodsFor: '*monticello-instance creation' stamp: 'stephaneducasse 2/4/2006 20:47'!
fromMethodTimeStamp: aString
	| stream |
	stream := ReadStream on: aString.
	stream skipSeparators.
	stream skipTo: Character space.
	^self readFrom: stream.! !

!TimeStamp class methodsFor: '*monticello-instance creation' stamp: 'stephaneducasse 2/4/2006 20:47'!
readFrom: stream
	| date time |
	stream skipSeparators.
	date := Date readFrom: stream.
	stream skipSeparators.
	time := Time readFrom: stream.
	^self 
		date: date
		time: time! !

!MethodReference methodsFor: '*monticello' stamp: 'ab 8/22/2003 17:55'!
asMethodDefinition
	^ MCMethodDefinition forMethodReference: self! !

!MethodReference methodsFor: '*monticello' stamp: 'ab 8/22/2003 17:58'!
compiledMethod
	^ self actualClass compiledMethodAt: methodSymbol! !

!MethodReference methodsFor: '*monticello' stamp: 'ab 8/22/2003 17:59'!
source
	^ (self actualClass sourceCodeAt: methodSymbol) asString withSqueakLineEndings! !

!MethodReference methodsFor: '*monticello' stamp: 'ab 8/22/2003 17:58'!
timeStamp
	^ self compiledMethod timeStamp! !

!PseudoClass methodsFor: '*monticello' stamp: 'bf 7/25/2005 15:50'!
asClassDefinition
	^ MCClassDefinition
		name: self name
		superclassName: self superclass name
		category: self category 
		instVarNames: self instVarNames
		classVarNames: self classVarNames asSortedCollection
		poolDictionaryNames: self poolDictionaryNames
		classInstVarNames: self class instVarNames
		type: self typeOfClass
		comment: self organization classComment	 asString
		commentStamp: self organization commentStamp	! !

!PseudoClass methodsFor: '*monticello-override' stamp: 'nk 2/18/2004 18:30'!
isMeta
	^false! !

Object subclass: #MCAncestry
	instanceVariableNames: 'ancestors stepChildren'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Versioning'!

!MCAncestry commentStamp: '<historical>' prior: 0!
Abstract superclass of records of ancestry.!

!MCAncestry methodsFor: 'ancestry' stamp: 'avi 2/12/2004 20:57'!
allAncestorsDo: aBlock
	self ancestors do:
		[:ea |
		aBlock value: ea.
		ea allAncestorsDo: aBlock]! !

!MCAncestry methodsFor: 'ancestry' stamp: 'avi 2/12/2004 20:57'!
allAncestorsOnPathTo: aVersionInfo
	^ MCFilteredVersionSorter new
		target: aVersionInfo;
		addAllVersionInfos: self ancestors;
		sortedVersionInfos! !

!MCAncestry methodsFor: 'ancestry' stamp: 'avi 2/12/2004 20:57'!
ancestorString
	^ String streamContents:
		[:s | self ancestors do: [:ea | s nextPutAll: ea name] separatedBy: [s nextPutAll: ', ']]! !

!MCAncestry methodsFor: 'ancestry' stamp: 'avi 2/12/2004 20:57'!
ancestors
	^ ancestors ifNil: [#()]! !

!MCAncestry methodsFor: 'ancestry' stamp: 'bf 12/22/2004 21:55'!
ancestorsDoWhileTrue: aBlock
	self ancestors do:
		[:ea |
		(aBlock value: ea) ifTrue: 
			[ea ancestorsDoWhileTrue: aBlock]]! !

!MCAncestry methodsFor: 'ancestry' stamp: 'avi 9/17/2005 16:03'!
breadthFirstAncestors
	^ Array streamContents: [:s | self breadthFirstAncestorsDo: [:ea | s nextPut: ea]]! !

!MCAncestry methodsFor: 'ancestry' stamp: 'stephaneducasse 2/4/2006 20:47'!
breadthFirstAncestorsDo: aBlock
	| seen todo next |
	seen := Set with: self.
	todo := OrderedCollection with: self.
	[todo isEmpty] whileFalse:
		[next := todo removeFirst.
		next ancestors do:
			[:ea |
			(seen includes: ea) ifFalse:
				[aBlock value: ea.
				seen add: ea.
				todo add: ea]]]! !

!MCAncestry methodsFor: 'ancestry' stamp: 'stephaneducasse 2/4/2006 20:47'!
commonAncestorWith: aNode
	| commonAncestors |
	commonAncestors := self commonAncestorsWith: aNode.
	^ commonAncestors at: 1 ifAbsent: [nil]! !

!MCAncestry methodsFor: 'ancestry' stamp: 'stephaneducasse 2/4/2006 20:47'!
commonAncestorsWith: aVersionInfo

	| sharedAncestors mergedOrder sorter |
	sorter := MCVersionSorter new
						addVersionInfo: self;
						addVersionInfo: aVersionInfo.
	mergedOrder := sorter sortedVersionInfos.
	sharedAncestors := (sorter allAncestorsOf: self) intersection: (sorter allAncestorsOf: aVersionInfo).
	^ mergedOrder select: [:ea | sharedAncestors includes: ea]! !

!MCAncestry methodsFor: 'ancestry' stamp: 'jrp 7/12/2004 08:16'!
hasAncestor: aVersionInfo
	^ self
		hasAncestor: aVersionInfo
		alreadySeen: OrderedCollection new! !

!MCAncestry methodsFor: 'ancestry' stamp: 'jrp 7/12/2004 08:16'!
hasAncestor: aVersionInfo alreadySeen: aList
	(aList includes: self) ifTrue: [^ false].
	aList add: self.

	^ self = aVersionInfo or: [self ancestors anySatisfy: [:ea | ea hasAncestor: aVersionInfo alreadySeen: aList]]
! !

!MCAncestry methodsFor: 'initializing' stamp: 'kph 5/17/2007 05:50'!
initialize
 ! !

!MCAncestry methodsFor: 'ancestry' stamp: 'avi 2/12/2004 20:58'!
isRelatedTo: aVersionInfo
	^ aVersionInfo timeStamp < self timeStamp
		ifTrue: [self hasAncestor: aVersionInfo]
		ifFalse: [aVersionInfo hasAncestor: self]! !

!MCAncestry methodsFor: 'ancestry' stamp: 'kph 5/30/2008 21:01'!
size

^ self ancestors size! !

!MCAncestry methodsFor: 'ancestry' stamp: 'avi 9/11/2004 15:08'!
stepChildren
	^ stepChildren ifNil: [#()]! !

!MCAncestry methodsFor: 'ancestry' stamp: 'avi 9/14/2004 15:21'!
stepChildrenString
	^ String streamContents:
		[:s | self stepChildren do: [:ea | s nextPutAll: ea name] separatedBy: [s nextPutAll: ', ']]! !

!MCAncestry methodsFor: 'ancestry' stamp: 'stephaneducasse 2/4/2006 20:47'!
topologicalAncestors
	| frontier f |
	^ Array streamContents:
		[:s |
		frontier := MCFrontier frontierOn: self.
		[f := frontier frontier.
		s nextPutAll: f.
		frontier removeAll: f.
		f isEmpty] whileFalse] ! !

!MCAncestry methodsFor: 'ancestry' stamp: 'mtf 6/9/2008 20:03'!
trimAfterVersionInfo: aVersionInfo
	aVersionInfo = self
		ifTrue: [ancestors := #()]
		ifFalse:
			[aVersionInfo date <= self date ifTrue:
				[self ancestors do: [:ea | ea trimAfterVersionInfo: aVersionInfo]]
		]! !

!MCAncestry methodsFor: 'ancestry' stamp: 'avi 9/17/2005 16:03'!
withBreadthFirstAncestors
	^ (Array with: self), self breadthFirstAncestors! !

MCAncestry subclass: #MCVersionInfo
	instanceVariableNames: 'id name message date time author version dotPos'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Versioning'!

!MCVersionInfo commentStamp: 'kph 6/2/2008 14:11' prior: 0!
I am used as a handle for the UI to reference persisted versions and to obtain the relationships between versions.

Some repositories only supply a filename as an initial handle, and so to realise the
full data from the repository, use #realizeFrom: aRepository

Adds to the record of ancestry, other identifying details.

Names:

The defining point in the name is the 'firstDot' after the first $- (i.e. in some cases it may not actually be the first dot overall), all parts in the name are parsed relative to this. My-Package-kph.234.mcz is split into 'My-Package-kph' and '234.mcz' first. 
The #namePreFirstDot is split on the last $- to obtain the packgeName/author.
 
The version is obtained upTo the last $( or $. in the second #postFirstDot portion,  this may be a uuid or, it may even contain $. as in traditional version numbering schemes. 
  
VersionInfo does not have any notion of a file extn, whereas VersionInfoFilename does, and this is taken to be everything after the last $. 

PackageName - any characters including $- .
Author - any characters except $- or $.
Version - any characters including $- or $.

Any older packages which have $- in the author/version suffix must be renamed

UUID's are valid versionnumbers if $- is replcaed by $.!

MCVersionInfo subclass: #MCSystemChangeSetVersionInfo
	instanceVariableNames: 'csName'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Versioning'!

!MCSystemChangeSetVersionInfo class methodsFor: 'as yet unclassified' stamp: 'kph 5/17/2007 13:51'!
on: cs

	^ (self new) initializeFromCS: cs! !

!MCSystemChangeSetVersionInfo methodsFor: 'accessing' stamp: 'kph 5/17/2007 14:04'!
= other
	^ other species = self species
		and: [other csName = self csName]! !

!MCSystemChangeSetVersionInfo methodsFor: 'accessing' stamp: 'kph 5/17/2007 13:36'!
author

	^ Utilities authorInitials! !

!MCSystemChangeSetVersionInfo methodsFor: 'accessing' stamp: 'kph 5/17/2007 13:33'!
csName
	"Answer the value of csName"

	^ csName! !

!MCSystemChangeSetVersionInfo methodsFor: 'accessing' stamp: 'kph 5/17/2007 13:33'!
csName: anObject
	"Set the value of csName"

	csName := anObject! !

!MCSystemChangeSetVersionInfo methodsFor: 'accessing' stamp: 'kph 5/17/2007 14:03'!
hash

^ csName hash! !

!MCSystemChangeSetVersionInfo methodsFor: 'accessing' stamp: 'kph 5/17/2007 13:45'!
initializeFromCS: cs

	csName := cs name.
	
	message := 'System Change Set'.! !

!MCSystemChangeSetVersionInfo methodsFor: 'accessing' stamp: 'kph 5/17/2007 13:43'!
name

	^ csName, '-', self author! !

!MCSystemChangeSetVersionInfo methodsFor: 'accessing' stamp: 'kph 5/17/2007 13:37'!
packageName

	^ csName! !

!MCSystemChangeSetVersionInfo methodsFor: 'accessing' stamp: 'kph 5/17/2007 13:36'!
version

	^ '1'! !

!MCVersionInfo class methodsFor: 'as yet unclassified' stamp: 'kph 6/29/2007 03:47'!
firstVersionForPackage: packageName

	^ (self new) name: (packageName, '-', Utilities authorInitials, '.1')! !

!MCVersionInfo class methodsFor: 'as yet unclassified' stamp: 'kph 8/1/2007 01:40'!
name: vName id: id author: author 
	^ self new
		name: vName
		id: id 
		author: author
	 ! !

!MCVersionInfo class methodsFor: 'as yet unclassified' stamp: 'avi 9/11/2004 10:44'!
name: vName id: id message: message date: date time: time author: author ancestors: ancestors
	^ self 
		name: vName
		id: id
		message: message
		date: date
		time: time
		author: author
		ancestors: ancestors
		stepChildren: #()! !

!MCVersionInfo class methodsFor: 'as yet unclassified' stamp: 'avi 9/11/2004 10:43'!
name: vName id: id message: message date: date time: time author: author ancestors: ancestors stepChildren: stepChildren
	^ self new
		initializeWithName: vName
		id: id
		message: message
		date: date
		time: time
		author: author
		ancestors: ancestors
		stepChildren: stepChildren! !

!MCVersionInfo methodsFor: 'pillaging' stamp: 'mtf 6/27/2008 22:53'!
<= otherInfo
	^ (self versionNumbers copyWithFirst: self packageName)
		mcLessThan: (otherInfo versionNumbers copyWithFirst: otherInfo packageName)
		ifEqual: [self name <= otherInfo name]! !

!MCVersionInfo methodsFor: 'comparing' stamp: 'ab 7/5/2003 14:23'!
= other
	^ other species = self species
		and: [other hasID: id]! !

!MCVersionInfo methodsFor: 'pillaging' stamp: 'mtf 6/27/2008 23:08'!
> otherInfo
	^ (self <= otherInfo) not! !

!MCVersionInfo methodsFor: 'converting' stamp: 'kph 5/17/2007 13:40'!
asDictionary
	^ Dictionary new
		at: #name put: self name;
		at: #id put: self id;
		at: #message put: self message;
		at: #date put: self date;
		at: #time put: self time;
		at: #author put: self author;
		at: #ancestors put: (self ancestors collect: [:a | a asDictionary]);
		yourself! !

!MCVersionInfo methodsFor: 'pillaging' stamp: 'kph 5/17/2007 03:01'!
author
	^ author ifNil: [ self authorFromName ]
		 ! !

!MCVersionInfo methodsFor: 'pillaging' stamp: 'kph 6/28/2008 08:04'!
authorFromName
 	
	| authorTokens |
	
	authorTokens := OrderedCollection new.
	
	self nameAfterLastDashAsTokens do: [ : token |

		token first isDigit ifFalse: [ authorTokens add: token ].
	
	].

	^ authorTokens	reduce: [ :a :b | a , '.' , b ]
 
! !

!MCVersionInfo methodsFor: '' stamp: 'kph 8/1/2007 12:24'!
browseVersionFrom: repository

	(repository versionWithInfo: self) browseFrom: repository
	
	! !

!MCVersionInfo methodsFor: 'pillaging' stamp: 'cwp 8/1/2003 00:26'!
date
	^ date! !

!MCVersionInfo methodsFor: 'printing' stamp: 'kph 11/16/2007 23:23'!
description

^ self name! !

!MCVersionInfo methodsFor: 'pillaging' stamp: 'kph 6/29/2007 06:50'!
extractLeastSignificantVersionNumberPartFrom: v

	| foundDigits first num |

	foundDigits := nil.
	first := nil.

	v size to: 1 by: -1 do: [ :i | 
		first ifNil: [ (v at: i) isDigit ifTrue: [ foundDigits := i	]
						ifFalse: [ foundDigits ifNotNil: [ first := foundDigits ] ] ]
	].
	first ifNil: [ first := 1 ].
	^ Array 
		with: (first > 1 ifTrue: [ v copyFrom: 1 to: first -1  ] ifFalse: [ '' ])
		with: (num := (v copyFrom: first to: v size) asNumber)
		with: (((num := first + num asString size) > v size) ifTrue: ['']
				ifFalse: [ v copyFrom: num to: v size ])  

! !

!MCVersionInfo methodsFor: 'private' stamp: 'ab 7/5/2003 14:10'!
hasID: aUUID
	^ id = aUUID! !

!MCVersionInfo methodsFor: '' stamp: 'kph 8/1/2007 12:30'!
hasSnapshot

	^ true! !

!MCVersionInfo methodsFor: 'comparing' stamp: 'ab 7/5/2003 14:09'!
hash
	^ id hash! !

!MCVersionInfo methodsFor: 'pillaging' stamp: 'cwp 8/1/2003 00:24'!
id
	^ id ! !

!MCVersionInfo methodsFor: 'initialize-release' stamp: 'stephaneducasse 2/4/2006 20:47'!
initializeWithName: vName id: aUUID message: aString date: aDate time: aTime author: initials ancestors: aCollection stepChildren: stepCollection
	name := vName.
	id := aUUID.
	message := aString.
	date := aDate.
	time := aTime.
	author := initials.
	ancestors :=  aCollection.
	stepChildren := stepCollection! !

!MCVersionInfo methodsFor: 'pillaging' stamp: 'kph 5/17/2007 05:52'!
isFilename

	^ false! !

!MCVersionInfo methodsFor: 'pillaging' stamp: 'kph 6/28/2008 09:39'!
isUIDVersion
 
 	self versionTokens reverse 
	
	in: [ :vn |
	
		((vn size > 2 and: [ vn third size = 8]) and: [ vn second size = 9 ]) ifFalse: [ ^ false ]

	].

	^ true
! !

!MCVersionInfo methodsFor: '' stamp: 'kph 8/1/2007 12:07'!
loadVersionFrom: repository

	(repository versionWithInfo: self) loadFrom: repository
	
	! !

!MCVersionInfo methodsFor: '' stamp: 'kph 8/1/2007 12:52'!
mergeVersionFrom: repository

	(repository versionWithInfo: self) mergeFrom: repository
	
	! !

!MCVersionInfo methodsFor: 'accessing' stamp: 'ab 7/12/2003 00:04'!
message
	^ message ifNil: ['']! !

!MCVersionInfo methodsFor: 'accessing' stamp: 'ab 7/11/2003 23:33'!
name
	^ name ifNil: ['<working copy>']! !

!MCVersionInfo methodsFor: 'accessing' stamp: 'kph 6/29/2007 02:44'!
name: aName
	
	name := aName! !

!MCVersionInfo methodsFor: '' stamp: 'kph 8/1/2007 01:41'!
name: vName id: anId author: anAuthor 

	self name: vName.
	id := anId.
	author := anAuthor.! !

!MCVersionInfo methodsFor: 'accessing' stamp: 'kph 6/2/2008 13:27'!
nameAfterLastDash

	^ name copyAfterLast: $-
 ! !

!MCVersionInfo methodsFor: 'pillaging' stamp: 'kph 6/28/2008 08:02'!
nameAfterLastDashAsTokens
	| stream tokens |
	stream := self nameAfterLastDashUpToBracket readStream.
	tokens := OrderedCollection new.
	[stream atEnd] whileFalse: [
		
		tokens add: (stream upTo: $.)
	].
	^ tokens! !

!MCVersionInfo methodsFor: 'pillaging' stamp: 'kph 6/28/2008 07:38'!
nameAfterLastDashUpToBracket
	
	"version string includes the author initials and the version numbers" 
	
	^ self nameAfterLastDash copyUpTo: $(! !

!MCVersionInfo methodsFor: 'accessing' stamp: 'kph 6/2/2008 13:29'!
nameBeforeLastDash

	^ name copyUpToLast: $-
 ! !

!MCVersionInfo methodsFor: 'pillaging' stamp: 'xxx 6/28/2008 09:21'!
nextUIDBasedVersion
	
	^ self class new name: (self packageName, '-', Utilities authorInitials, '.' , (self versionStringFrom: self nextUIDBasedVersionTokens)); yourself

 ! !

!MCVersionInfo methodsFor: 'pillaging' stamp: 'xxx 6/28/2008 09:45'!
nextUIDBasedVersionTokens
	
	"The Universal ID is not a true GUID but a simpler sortable timestamp based equivalent."
	
 	| t count tokens |
	
	"copyReplaceAll is more backwards compatible than replacing:with: "
	
	tokens := self versionTokens.
	
	count := tokens last asNumber.
	
	t := DateAndTime now.
	tokens at: (tokens size - 2) put: (t asDate printFormat: #(3 2 1 0 1 1 2)).
	t := t asTime.
	tokens at: (tokens size - 1) put: (t print24 reject: [ :ea | ea = $: ]), 
	(t milliSecond asString padded: #left to: 3	with: $0).
	tokens at: tokens size put: (count + 1) asString.
	
	^ tokens
	! !

!MCVersionInfo methodsFor: 'pillaging' stamp: 'kph 6/28/2008 07:45'!
nextVersion

	|v num newVersion result  |
	self isUIDVersion
		ifTrue: [ ^self nextUIDBasedVersion ].

	"we hunt backwards through the version string, starting at the first number we constuce that
number until we find a non digit. e.g. from 4.1.13b we would obtain 13."
	v := self versionString.
	 
	result := 	self extractLeastSignificantVersionNumberPartFrom: v.
	
	num := result second.
	num = 0 ifTrue: [ " no number found"  self error: 'no version number found' ].

	newVersion := result first , (num + 1) asString , result last.
 
 	^ self copy name: (self packageName, '-', Utilities authorInitials, '.', newVersion)
	

 ! !

!MCVersionInfo methodsFor: 'pillaging' stamp: 'kph 7/11/2007 07:44'!
nextVersionConsidering: ancestry

	| v  myParts ancestorParts num |
 
	v := self versionString.
	myParts := self extractLeastSignificantVersionNumberPartFrom: v.
	num := myParts second.

	ancestry ancestors do: [ :each | 
	
		ancestorParts := each extractLeastSignificantVersionNumberPartFrom: v.
	
		((myParts first = ancestorParts first) and: [ myParts last = ancestorParts last ]) 
		ifTrue: [ num := num max: ancestorParts second ].

	].
 
 	^ self copy name: (self packageName, '-', Utilities authorInitials, '.', myParts first , (num + 1) asString , myParts last). 
! !

!MCVersionInfo methodsFor: 'accessing' stamp: 'kph 6/2/2008 13:29'!
packageName

	^ self nameBeforeLastDash! !

!MCVersionInfo methodsFor: 'printing' stamp: 'ab 7/5/2003 18:00'!
printOn: aStream
	super printOn: aStream.
	aStream nextPut: $(; nextPutAll: self name; nextPut: $)
	! !

!MCVersionInfo methodsFor: 'pillaging' stamp: 'kph 5/17/2007 05:54'!
realizeFrom: aRepository

	^ self ! !

!MCVersionInfo methodsFor: 'accessing' stamp: 'avi 1/20/2004 16:08'!
summary
	^ String streamContents:
		[:s |
		s
			nextPutAll: self summaryHeader; cr; cr;
			nextPutAll: self message.
		]! !

!MCVersionInfo methodsFor: 'accessing' stamp: 'kph 5/17/2007 13:42'!
summaryHeader
	^ String streamContents:
		[:s |
		s
			nextPutAll: 'Name: '; nextPutAll: self name; cr.
		date ifNotNil:
			[s
				nextPutAll: 'Author: '; nextPutAll: self author; cr;
				nextPutAll: 'Time: '; nextPutAll:  self date asString, ', ', self time asString; cr].
		id ifNotNil:
			[s nextPutAll: 'UUID: '; nextPutAll: self id asString; cr].
		s
			nextPutAll: 'Ancestors: '; nextPutAll: self ancestorString.
		self stepChildren isEmpty ifFalse:
			[s cr; nextPutAll: 'Backported From: '; nextPutAll: self stepChildrenString].
		]! !

!MCVersionInfo methodsFor: 'pillaging' stamp: 'cwp 8/1/2003 00:26'!
time
	^ time! !

!MCVersionInfo methodsFor: 'accessing' stamp: 'avi 1/22/2004 16:45'!
timeStamp
	^ TimeStamp date: date time: time! !

!MCVersionInfo methodsFor: 'accessing' stamp: 'avi 9/17/2003 11:24'!
timeString
	^ date asString, ', ', time asString! !

!MCVersionInfo methodsFor: 'accessing' stamp: 'kph 8/16/2007 18:14'!
versionName

	^ name! !

!MCVersionInfo methodsFor: 'pillaging' stamp: 'mtf 6/28/2008 11:02'!
versionNumbers
"To enable a smooth transition to more informative version numbers, we cause old, single-digit version numbers to sort prior to newer version numbers, by sorting as if old versions are 0.0.x"

	| array |
	array := self versionTokens collect: [ :ea | ea asInteger ].
	^ array size <= 1
		ifTrue: [#(0 0), array]
		ifFalse: [array]
	

 ! !

!MCVersionInfo methodsFor: 'pillaging' stamp: 'xxx 6/28/2008 09:10'!
versionString
	
	^ self versionStringFrom: self versionTokens
	 ! !

!MCVersionInfo methodsFor: 'pillaging' stamp: 'xxx 6/28/2008 09:10'!
versionStringFrom: tokens
	
	"version string includes the author initials and the version numbers" 
	
	^ String streamContents: [:stream | tokens
		do: [:ea | stream nextPutAll: ea]
		separatedBy: [stream nextPut: $.]]! !

!MCVersionInfo methodsFor: 'pillaging' stamp: 'mtf 7/21/2008 21:52'!
versionTokens
	^ self nameAfterLastDashAsTokens select: [ :ea | ea notEmpty and: [ ea first isDigit ] ] 

 ! !

MCVersionInfo subclass: #MCVersionInfoFilename
	instanceVariableNames: 'isFilename versionName'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Versioning'!

!MCVersionInfoFilename commentStamp: 'kph 6/28/2007 23:55' prior: 0!
For filenames

Project-Subcategory-author.123.mcz

Everything after the first $. is the version string, this allows for more variation, and is more flexible in the case of different file types and conventions, e.g. .mcm!

!MCVersionInfoFilename class methodsFor: 'as yet unclassified' stamp: 'kph 5/17/2007 05:51'!
filename: filename

	"using MCVersionInfo to represent a filename"
	
	^ (self new) 
		setFilename: filename; yourself
		! !

!MCVersionInfoFilename methodsFor: 'as yet unclassified' stamp: 'kph 5/17/2007 06:16'!
= other
	^ other species = self species
		and: [other name = name]! !

!MCVersionInfoFilename methodsFor: '' stamp: 'kph 9/28/2007 11:13'!
ext
 
	^ name  copyAfterLast: $.! !

!MCVersionInfoFilename methodsFor: '' stamp: 'kph 9/28/2007 11:13'!
hasSnapshot

	^ self ext = 'mcz'! !

!MCVersionInfoFilename methodsFor: 'as yet unclassified' stamp: 'kph 5/17/2007 06:43'!
hash
	^ name hash! !

!MCVersionInfoFilename methodsFor: 'as yet unclassified' stamp: 'kph 5/17/2007 05:53'!
isFilename

	^ true! !

!MCVersionInfoFilename methodsFor: 'accessing' stamp: 'mtf 6/27/2008 01:18'!
nameAfterLastDash

	"we strip the extension and everyone else should be happy"
	^ super nameAfterLastDash copyUpToLast: $.

 ! !

!MCVersionInfoFilename methodsFor: 'accessing' stamp: 'xxx 6/28/2008 09:22'!
nextUIDBasedVersion
	
	^ self class new name: (self packageName, '-', Utilities authorInitials, '.' , (self versionStringFrom: self nextUIDBasedVersionTokens), '.', self ext); yourself

 ! !

!MCVersionInfoFilename methodsFor: 'as yet unclassified' stamp: 'kph 5/17/2007 05:54'!
realizeFrom: aRepository

	^ aRepository realizeFullVersionInfo: self ! !

!MCVersionInfoFilename methodsFor: 'accessing' stamp: 'kph 5/17/2007 05:55'!
setFilename: n

	name := n.
 ! !

!MCVersionInfoFilename methodsFor: 'accessing' stamp: 'kph 1/28/2008 02:09'!
versionName
	"we probably dont need to but..."
	^ versionName ifNil: [versionName := name copyUpToLast: $.  ]

 ! !

MCAncestry subclass: #MCWorkingAncestry
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Versioning'!

!MCWorkingAncestry commentStamp: '<historical>' prior: 0!
The interim record of ancestry for a working copy, gets merged version added to the ancestry, and is used to create the VersionInfo when the working copy becomes a version. !

!MCWorkingAncestry methodsFor: 'as yet unclassified' stamp: 'bf 11/7/2005 11:37'!
addAncestor: aNode
	"Add aNode, remove ancestors of it, preserve original order"
	| writeNode |
	writeNode := true.
	"Write aNode only once, replacing the first of its ancestors, or add last"
	ancestors := Array streamContents: [:newAncestors |
		self ancestors do: [:each |
			(aNode hasAncestor: each)
				ifTrue: [writeNode ifTrue: [newAncestors nextPut: aNode. writeNode := false]]
				ifFalse: [newAncestors nextPut: each]].
		writeNode ifTrue: [newAncestors nextPut: aNode. writeNode := false]].! !

!MCWorkingAncestry methodsFor: 'as yet unclassified' stamp: 'abc 7/3/2007 22:07'!
addStepChild: aVersionInfo
	stepChildren := self stepChildren copyWith: aVersionInfo! !

!MCWorkingAncestry methodsFor: 'as yet unclassified' stamp: 'avi 10/22/2005 19:58'!
date
	^ Date today! !

!MCWorkingAncestry methodsFor: 'as yet unclassified' stamp: 'ul 6/8/2007 01:31'!
infoWithName: nameString message: messageString
	^ MCVersionInfo
		name: nameString
		id: UUID new
		message: messageString
		date: Date today
		time: Time now
		author: Utilities authorInitials
		ancestors: (ancestors ifNil: [ #() ]) asArray
		stepChildren: self stepChildren asArray! !

!MCWorkingAncestry methodsFor: 'as yet unclassified' stamp: 'avi 2/13/2004 01:14'!
name
	^ '<working copy>'! !

!MCWorkingAncestry methodsFor: 'as yet unclassified' stamp: 'avi 2/13/2004 01:14'!
summary
	^ 'Ancestors: ', self ancestorString! !

Object subclass: #MCConflict
	instanceVariableNames: 'operation chooseRemote'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Merging'!

!MCConflict class methodsFor: 'as yet unclassified' stamp: 'ab 6/1/2003 13:07'!
operation: anOperation
	^ self new operation: anOperation	! !

!MCConflict methodsFor: 'as yet unclassified' stamp: 'nk 11/10/2003 21:57'!
annotations
	^operation ifNotNilDo: [ :op | op annotations ]! !

!MCConflict methodsFor: 'as yet unclassified' stamp: 'ab 6/1/2003 13:04'!
applyTo: anObject
	self isResolved ifFalse: [self error: 'Cannot continue until this conflict has been resolved'].
	self remoteChosen ifTrue: [operation applyTo: anObject].! !

!MCConflict methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
chooseLocal
	chooseRemote := false! !

!MCConflict methodsFor: 'as yet unclassified' stamp: 'nk 10/21/2003 23:16'!
chooseNewer
	self isLocalNewer ifTrue: [ self chooseLocal ]
		ifFalse: [ self isRemoteNewer ifTrue: [ self chooseRemote ]]! !

!MCConflict methodsFor: 'as yet unclassified' stamp: 'nk 10/21/2003 23:22'!
chooseOlder
	self isRemoteNewer ifTrue: [ self chooseLocal ]
		ifFalse: [ self isLocalNewer ifTrue: [ self chooseRemote ]]! !

!MCConflict methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
chooseRemote
	chooseRemote := true! !

!MCConflict methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
clearChoice
	chooseRemote := nil! !

!MCConflict methodsFor: 'as yet unclassified' stamp: 'nk 11/10/2003 21:58'!
definition
	^operation ifNotNilDo: [ :op | op definition ]! !

!MCConflict methodsFor: 'as yet unclassified' stamp: 'dvf 8/10/2004 23:24'!
isConflict
	^true! !

!MCConflict methodsFor: 'as yet unclassified' stamp: 'nk 10/21/2003 23:11'!
isLocalNewer
	^ self localDefinition fullTimeStamp > self remoteDefinition fullTimeStamp! !

!MCConflict methodsFor: 'as yet unclassified' stamp: 'nk 10/21/2003 23:15'!
isRemoteNewer
	^ self localDefinition fullTimeStamp < self remoteDefinition fullTimeStamp! !

!MCConflict methodsFor: 'as yet unclassified' stamp: 'ab 6/1/2003 13:02'!
isResolved
	^ chooseRemote notNil! !

!MCConflict methodsFor: 'as yet unclassified' stamp: 'ab 6/2/2003 01:45'!
localChosen
	^ chooseRemote notNil and: [chooseRemote not]! !

!MCConflict methodsFor: 'as yet unclassified' stamp: 'ab 6/1/2003 13:10'!
localDefinition
	^ operation baseDefinition! !

!MCConflict methodsFor: 'as yet unclassified' stamp: 'avi 9/19/2005 02:19'!
operation
	^ operation! !

!MCConflict methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
operation: anOperation
	operation := anOperation! !

!MCConflict methodsFor: 'as yet unclassified' stamp: 'ab 6/2/2003 01:45'!
remoteChosen
	^ chooseRemote notNil and: [chooseRemote]! !

!MCConflict methodsFor: 'as yet unclassified' stamp: 'ab 6/1/2003 13:10'!
remoteDefinition
	^ operation targetDefinition! !

!MCConflict methodsFor: 'as yet unclassified' stamp: 'ab 7/18/2003 16:47'!
source
	^ self localChosen
		ifTrue: [operation fromSource]
		ifFalse: [operation source]! !

!MCConflict methodsFor: 'as yet unclassified' stamp: 'ab 6/1/2003 13:48'!
status
	^ self isResolved
		ifFalse: ['']
		ifTrue: [self remoteChosen
					ifFalse: ['L']
					ifTrue: ['R']]! !

!MCConflict methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
summary
	| attribute |
	attribute := 
		self isResolved
			ifTrue: [self remoteChosen ifTrue: [#underlined] ifFalse: [#struckOut]]
			ifFalse: [#bold].
	^ Text string: operation summary attribute: (TextEmphasis perform: attribute)! !

Object subclass: #MCDefinition
	instanceVariableNames: ''
	classVariableNames: 'Instances'
	poolDictionaries: ''
	category: 'Monticello-Base'!

!MCDefinition commentStamp: 'kph 8/1/2007 10:10' prior: 0!
Notes:

Given that monticello uses a binary fileOut, it is not possible to add instance vars to MCDefinition or its subclasses. With the benefit of hindsight this is a slight oversight by the original developers. It makes it almost impossible to improve this heirarchy without breaking compatibility.

First of all we implement a flag which enables us to enforce MC compatibility #storeAsMC1CompatibleBinary.
MCDefinition's #storeDataOn: defers to #storeMC1DataOn: which enforces a fixed number of instVars (defined in #storeMaxInstVarsForMC1 and a fixed output class even for specialized subclasses, which old MC's may not recognize.

This ensures that our packages can be viewed in repositoryies by old MC implementations.

For the future.
1) We remove ignore any restriction upon the number of inst vars that we save, unless a class wants to imlement a specific policy of its own (MCMethodDefinition has a number of instVars which are not worth persisting). 
2) We implement a properties interface in each class as best we can, we cannot implement it in MCDefinition because the format comatability has to be preserved with older MCs. So we place the instance var in subclasses of MCDefinition after their own inst vars.
3) we change the loading code to not-care if when loading if finds extra instance variables. It puts the extra instance varables in the properties dictionary under #extraInstVars. Then when storing we write those extraInstVars so as to preseve the format that we were given.
4) New subclasses save themselves as a known superclass with extra variables and their class name stored in property #class.

Accessors to properties can be placed in MCDefinition to acceive the same effect as a local instVar. For an example see #because, used in #orphanedBecause:

Other Monticellos need to implement the more tolerant #instVarAt: anInteger put: anObject in order to read our 'relaxed' format. At present I can see no other way of moving forward than to force all other monticellos to patch up!!

 !

!MCDefinition class methodsFor: '' stamp: 'mtf 12/15/2007 03:06'!
initialize
	"MCDefinition initialize"

"for the future"
	"
	Preferences addPreference: #storeMC1CompatibleBinary
		categories: #('monticello') default: true
		balloonHelp: 'Save using MC1 compatibility, readable by older Monticello versions, but not extensible'
		
	"
	self clearInstances! !

MCDefinition subclass: #MCClassDefinition
	instanceVariableNames: 'name superclassName variables category type comment commentStamp traitComposition classTraitComposition properties oldInstVars'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Modeling'!

!MCClassDefinition commentStamp: 'kph 5/15/2007 19:25' prior: 0!
The strategy for loading is to preload a class definiton is to merge the variables 
wanted by both the new and the old classes, and load this 'union' class definition.

This should allow old code to continue running, and new code to be compiled.

Since variable order is significant, the original order is preserved for the #preload, and changed on the #install.

In the atomic #install phase, the new traitComposition and comment is switched in.

In the atomic #postinstall phase the #initialize methods are run (note they have access to both the old and new class variables etc. 

In the postload phase the new class definition is loaded, eliminating the now redundant parts.

!

!MCClassDefinition class methodsFor: 'obsolete' stamp: 'ab 4/1/2003 01:22'!
name: nameString
superclassName: superclassString
category: categoryString 
instVarNames: ivarArray
classVarNames: cvarArray
poolDictionaryNames: poolArray
classInstVarNames: civarArray
comment: commentString
	^ self	name: nameString
			superclassName: superclassString
			category: categoryString 
			instVarNames: ivarArray
			classVarNames: cvarArray
			poolDictionaryNames: poolArray
			classInstVarNames: civarArray
			type: #normal
			comment: commentString
! !

!MCClassDefinition class methodsFor: 'obsolete' stamp: 'cwp 8/10/2003 16:33'!
name: nameString
superclassName: superclassString
category: categoryString 
instVarNames: ivarArray
classVarNames: cvarArray
poolDictionaryNames: poolArray
classInstVarNames: civarArray
type: typeSymbol
comment: commentString
	^ self 
		name: nameString
		superclassName: superclassString
		category: categoryString 
		instVarNames: ivarArray
		classVarNames: cvarArray
		poolDictionaryNames: poolArray
		classInstVarNames: civarArray
		type: typeSymbol
		comment: commentString
		commentStamp: nil! !

!MCClassDefinition class methodsFor: 'instance creation' stamp: 'al 10/9/2005 19:16'!
name: nameString
superclassName: superclassString
category: categoryString 
instVarNames: ivarArray
classVarNames: cvarArray
poolDictionaryNames: poolArray
classInstVarNames: civarArray
type: typeSymbol
comment: commentString
commentStamp: stampString
	^ self instanceLike:
		(self new initializeWithName: nameString
					superclassName: superclassString
					traitComposition: '{}'
					classTraitComposition: '{}'
					category: categoryString 
					instVarNames: ivarArray
					classVarNames: cvarArray
					poolDictionaryNames: poolArray
					classInstVarNames: civarArray
					type: typeSymbol
					comment: commentString
					commentStamp: stampString)! !

!MCClassDefinition class methodsFor: 'obsolete' stamp: 'ab 4/1/2003 01:22'!
name: nameString
superclassName: superclassString
category: categoryString 
instVarNames: ivarArray
comment: commentString
	^ self	name: nameString
			superclassName: superclassString
			category: categoryString 
			instVarNames: ivarArray
			classVarNames: #()
			poolDictionaryNames: #()
			classInstVarNames: #()
			comment: commentString
! !

!MCClassDefinition class methodsFor: 'instance creation' stamp: 'al 10/10/2005 13:58'!
name: nameString
superclassName: superclassString
traitComposition: traitCompositionString
classTraitComposition: classTraitCompositionString
category: categoryString 
instVarNames: ivarArray
classVarNames: cvarArray
poolDictionaryNames: poolArray
classInstVarNames: civarArray
type: typeSymbol
comment: commentString
commentStamp: stampString
	
	^ self instanceLike:
		(self new initializeWithName: nameString
					superclassName: superclassString
					traitComposition: traitCompositionString
					classTraitComposition: classTraitCompositionString
					category: categoryString 
					instVarNames: ivarArray
					classVarNames: cvarArray
					poolDictionaryNames: poolArray
					classInstVarNames: civarArray
					type: typeSymbol
					comment: commentString
					commentStamp: stampString)! !

!MCClassDefinition methodsFor: 'visiting' stamp: 'kph 10/25/2008 02:10'!
= aDefinition
	super = aDefinition ifFalse: [ ^ false ].
	superclassName = aDefinition superclassName ifFalse: [ ^ false ].
	self traitCompositionString = aDefinition traitCompositionString ifFalse: [ ^ false ].
	self classTraitCompositionString = aDefinition classTraitCompositionString ifFalse: [ ^ false ].
	category = aDefinition category ifFalse: [ ^ false ].
	type = aDefinition type ifFalse: [ ^false ].
	(variables = aDefinition variables
			or: [self sortedVariables = aDefinition sortedVariables]) ifFalse: [ ^false ].
	comment = aDefinition comment ifFalse: [ ^ false ].
	
	^ true
		! !

!MCClassDefinition methodsFor: 'visiting' stamp: 'al 10/9/2005 19:33'!
accept: aVisitor
	aVisitor visitClassDefinition: self.
	(self hasClassInstanceVariables or: [self hasClassTraitComposition])
		ifTrue: [aVisitor visitMetaclassDefinition: self].
! !

!MCClassDefinition methodsFor: 'accessing' stamp: 'nk 2/25/2005 09:49'!
actualClass
	^Smalltalk classNamed: self className! !

!MCClassDefinition methodsFor: 'visiting' stamp: 'kph 5/13/2007 05:13'!
actualSuperclass
	^Smalltalk at: self superclassName ifAbsent: [ ^self orphanedBecause: 'Missing superclass  ', self superclassName]! !

!MCClassDefinition methodsFor: 'accessing' stamp: 'kph 5/13/2007 02:43'!
actualThing
 	
	^ self actualClass! !

!MCClassDefinition methodsFor: 'initializing' stamp: 'cwp 7/7/2003 23:19'!
addVariables: aCollection ofType: aClass
	variables addAll: (aCollection collect: [:var | aClass name: var asString]).! !

!MCClassDefinition methodsFor: 'accessing' stamp: 'cwp 11/24/2002 06:23'!
category
	^ category! !

!MCClassDefinition methodsFor: 'accessing' stamp: 'cwp 7/7/2003 23:32'!
classInstVarNames
	^ self selectVariables: #isClassInstanceVariable! !

!MCClassDefinition methodsFor: 'printing' stamp: 'cwp 7/7/2003 23:35'!
classInstanceVariablesString
	^ self stringForVariablesOfType: #isClassInstanceVariable! !

!MCClassDefinition methodsFor: 'accessing' stamp: 'ab 11/13/2002 01:52'!
className
	^ name! !

!MCClassDefinition methodsFor: 'accessing' stamp: 'al 10/9/2005 20:07'!
classTraitComposition
	^classTraitComposition! !

!MCClassDefinition methodsFor: 'testing' stamp: 'kph 6/5/2008 02:04'!
classTraitComposition: aString
	aString = '{}' ifTrue: [ ^ self ].
	classTraitComposition := aString! !

!MCClassDefinition methodsFor: 'accessing' stamp: 'al 10/9/2005 21:55'!
classTraitCompositionString
	^self classTraitComposition ifNil: ['{}'].! !

!MCClassDefinition methodsFor: 'accessing' stamp: 'cwp 7/7/2003 23:33'!
classVarNames
	^ self selectVariables: #isClassVariable! !

!MCClassDefinition methodsFor: 'printing' stamp: 'cwp 7/7/2003 23:36'!
classVariablesString
	^ self stringForVariablesOfType: #isClassVariable! !

!MCClassDefinition methodsFor: 'accessing' stamp: 'ab 1/15/2003 13:42'!
comment
	^ comment! !

!MCClassDefinition methodsFor: 'accessing' stamp: 'cwp 8/10/2003 16:40'!
commentStamp
	^ commentStamp! !

!MCClassDefinition methodsFor: 'installing' stamp: 'kph 6/7/2008 01:35'!
createUnionClassWith: obsDefinition

	| newClass |
	
	newClass :=  (MCClassBuilder new)
			name: name 
			inEnvironment: self actualSuperclass environment 
			subclassOf: self actualSuperclass
			type: type 
			instanceVariableNames: (self stringForVariablesOfType: #isInstanceVariable plus: obsDefinition) 
			classVariableNames:   (self stringForVariablesOfType: #isClassVariable plus: obsDefinition) 
			poolDictionaries:  (self stringForVariablesOfType: #isPoolImport plus: obsDefinition) 
			category: category.
			
			
	newClass 
		ifNotNil: [ newClass class instanceVariableNames: 
						(self stringForVariablesOfType: #isClassInstanceVariable plus: obsDefinition) ].
		
	^ newClass! !

!MCClassDefinition methodsFor: 'initializing' stamp: 'cwp 8/10/2003 17:39'!
defaultCommentStamp
	^ String new

	"The version below avoids stomping on stamps already in the image

	^ (Smalltalk at: name ifPresent: [:c | c organization commentStamp])
		ifNil: ['']
	"
! !

!MCClassDefinition methodsFor: 'printing' stamp: 'cwp 8/2/2003 02:03'!
definitionString
	^ String streamContents: [:stream | self printDefinitionOn: stream]! !

!MCClassDefinition methodsFor: 'accessing' stamp: 'ab 12/5/2002 21:24'!
description
	^ Array with: name
! !

!MCClassDefinition methodsFor: 'system editor' stamp: 'mtf 6/10/2008 09:43'!
edLoad: editor

	| theClass |
	theClass := (editor at: superclassName) subclass: name
		instanceVariableNames: (self stringForVariablesOfType: #isInstanceVariable)
		classVariableNames: (self stringForVariablesOfType: #isClassVariable)
		poolDictionaries: (self stringForVariablesOfType: #isPoolImport)
		category: category.
		
	theClass typeOfClass: type.
	theClass class instanceVariableNames: (self stringForVariablesOfType: #isClassInstanceVariable). 
	theClass classComment: comment stamp: commentStamp.
	^ false "we dont need an edPostload"! !

!MCClassDefinition methodsFor: 'system editor' stamp: 'mtf 6/9/2008 20:14'!
edUnload: editor

	(editor at: name) removeFromSystem.
	
	^ false  "we dont need an edPostload"! !

!MCClassDefinition methodsFor: 'installing' stamp: 'kph 6/5/2008 23:05'!
extensions

	^ self propertyAt: #tmpExtensions ifAbsent: [ self propertyAt: #tmpExtensions put: OrderedCollection new  ]! !

!MCClassDefinition methodsFor: 'converting' stamp: 'mtf 10/7/2008 12:14'!
fromImage
	^ self actualClass ifNotNil: [self actualClass asClassDefinition]! !

!MCClassDefinition methodsFor: 'testing' stamp: 'cwp 8/2/2003 02:54'!
hasClassInstanceVariables
	^ (self selectVariables: #isClassInstanceVariable) isEmpty not! !

!MCClassDefinition methodsFor: 'testing' stamp: 'kph 6/5/2008 02:40'!
hasClassTraitComposition
	^ classTraitComposition notNil! !

!MCClassDefinition methodsFor: 'testing' stamp: 'al 10/9/2005 20:13'!
hasComment
	^ comment isEmptyOrNil not! !

!MCClassDefinition methodsFor: 'testing' stamp: 'kph 6/5/2008 02:40'!
hasTraitComposition
	^ traitComposition notNil! !

!MCClassDefinition methodsFor: 'comparing' stamp: 'stephaneducasse 2/4/2006 20:47'!
hash
	| hash |
	hash := String stringHash: name initialHash: 0.
	hash := String stringHash: superclassName initialHash: hash.
	hash := String stringHash: self traitCompositionString initialHash: hash.
	hash := String stringHash: self classTraitComposition asString initialHash: hash.
	hash := String stringHash: (category ifNil: ['']) initialHash: hash.
	hash := String stringHash: type initialHash: hash.
	variables do: [
		:v |
		hash := String stringHash: v name initialHash: hash.
	].
	^ hash! !

!MCClassDefinition methodsFor: 'initializing' stamp: 'kph 5/11/2007 01:34'!
initializeWithName: nameString
superclassName: superclassString
category: categoryString 
instVarNames: ivarArray
classVarNames: cvarArray
poolDictionaryNames: poolArray
classInstVarNames: civarArray
type: typeSymbol
comment: commentString
commentStamp: stampStringOrNil
	name := nameString asSymbol.
	self setSuperclassName: superclassString.
	category := categoryString.
	name = #CompiledMethod ifTrue: [type := #compiledMethod] ifFalse: [type := typeSymbol].
	comment := commentString withSqueakLineEndings.
	commentStamp := stampStringOrNil ifNil: [self defaultCommentStamp].
	variables := OrderedCollection  new.
	self addVariables: ivarArray ofType: MCInstanceVariableDefinition.
	self addVariables: cvarArray ofType: MCClassVariableDefinition.
	self addVariables: poolArray ofType: MCPoolImportDefinition.
	self addVariables: civarArray ofType: MCClassInstanceVariableDefinition.! !

!MCClassDefinition methodsFor: 'initializing' stamp: 'kph 6/5/2008 02:04'!
initializeWithName: nameString
superclassName: superclassString
traitComposition: traitCompositionString
classTraitComposition: classTraitCompositionString
category: categoryString 
instVarNames: ivarArray
classVarNames: cvarArray
poolDictionaryNames: poolArray
classInstVarNames: civarArray
type: typeSymbol
comment: commentString
commentStamp: stampStringOrNil
	name := nameString asSymbol.
	self setSuperclassName: superclassString.
	self traitComposition: traitCompositionString.
	self classTraitComposition:  classTraitCompositionString.
	category := categoryString.
	name = #CompiledMethod ifTrue: [type := #compiledMethod] ifFalse: [type := typeSymbol].
	comment := commentString withSqueakLineEndings.
	commentStamp := stampStringOrNil ifNil: [self defaultCommentStamp].
	variables := OrderedCollection  new.
	self addVariables: ivarArray ofType: MCInstanceVariableDefinition.
	self addVariables: cvarArray ofType: MCClassVariableDefinition.
	self addVariables: poolArray ofType: MCPoolImportDefinition.
	self addVariables: civarArray ofType: MCClassInstanceVariableDefinition.! !

!MCClassDefinition methodsFor: 'accessing' stamp: 'cwp 7/7/2003 23:33'!
instVarNames
	^ self selectVariables: #isInstanceVariable! !

!MCClassDefinition methodsFor: 'printing' stamp: 'cwp 7/7/2003 23:35'!
instanceVariablesString
	^ self stringForVariablesOfType: #isInstanceVariable! !

!MCClassDefinition methodsFor: 'testing' stamp: 'ab 12/4/2002 21:52'!
isClassDefinition
	^ true! !

!MCClassDefinition methodsFor: 'testing' stamp: 'ab 5/24/2003 13:49'!
isCodeDefinition
	^ true! !

!MCClassDefinition methodsFor: 'accessing' stamp: 'kph 5/15/2007 01:11'!
isPartOfPackage: aPackageInfo or: orphanage

	^ aPackageInfo includesSystemCategory: self category! !

!MCClassDefinition methodsFor: 'printing' stamp: 'cwp 11/24/2002 22:16'!
kindOfSubclass
	type = #normal ifTrue: [^ ' subclass: '].
	type = #words ifTrue: [^ ' variableWordSubclass: '].
	type = #variable ifTrue: [^ ' variableSubclass: '].
	type = #bytes ifTrue: [^ ' variableByteSubclass: '].
	type = #weak ifTrue: [^ ' weakSubclass: ' ].
	type = #compiledMethod ifTrue: [^ ' variableByteSubclass: ' ].
	self error: 'Unrecognized class type'! !

!MCClassDefinition methodsFor: 'installing' stamp: 'kph 6/5/2008 22:57'!
load

	"for test cases and original MCPackageLoader"
	^ self postloadOver: nil! !

!MCClassDefinition methodsFor: 'accessing' stamp: 'cwp 7/7/2003 23:53'!
poolDictionaries
	^ self selectVariables: #isPoolImport! !

!MCClassDefinition methodsFor: 'installing' stamp: 'mtf 6/10/2008 19:27'!
postinstall
 		
	(self createUnionClassWith: nil) 
	
		ifNotNilDo: [ :newClass |
	 		
			self hasTraitComposition ifTrue: [
					newClass setTraitComposition: (Compiler
					evaluate: self traitComposition) asTraitComposition ].
			self hasClassTraitComposition ifTrue: [
				newClass class setTraitComposition: (Compiler
				evaluate: self classTraitComposition) asTraitComposition ].
			
			self hasComment
				ifTrue: [newClass classComment: comment stamp: commentStamp]
				ifFalse: [newClass classComment: ''].
		
		].
	
	self extensions do: [ :ext | 
		ext ensuredCreateCompiledMethod.
		ext installMethod  ]! !

!MCClassDefinition methodsFor: 'installing' stamp: 'kph 6/7/2008 02:58'!
postloadOver: obs		

	self extensions do: [ :ext |  (ext postloadOver: nil).  ext compiledMethod ifNil: [ self error: 'class extension has been obsoleted' ]].
  
	self propertyRemoveKey: #tmpExtensions.
	
	MCClassBuilder new doneCompiling: self actualClass.! !

!MCClassDefinition methodsFor: 'installing' stamp: 'mtf 8/23/2008 15:19'!
preloadOver: obsDefinition

	| |
	
	super preloadOver: obsDefinition.
	
	obsDefinition ifNotNil: [ self removeExtensionsFrom: obsDefinition ].
		
	"we ignore syntax errors on the first time around, postinstall should sort them out"
	[ (self createUnionClassWith: obsDefinition).
	   ] on: SyntaxErrorNotification
	do: [ :ex |   ex isResumable ifTrue: [ ex resume: false] ]
		
	! !

!MCClassDefinition methodsFor: 'annotations' stamp: 'nk 7/24/2003 16:05'!
printAnnotations: requests on: aStream
	"Add a string for an annotation pane, trying to fulfill the annotation requests.
	These might include anything that
		Preferences defaultAnnotationRequests 
	might return. Which includes anything in
		Preferences annotationInfo
	To edit these, use:"
	"Preferences editAnnotations"

	requests do: [ :aRequest |
		aRequest == #requirements ifTrue: [
			self requirements do: [ :req | aStream nextPutAll: req ] separatedBy: [ aStream space ]]
	] separatedBy: [ aStream space ].! !

!MCClassDefinition methodsFor: 'printing' stamp: 'kph 5/21/2007 16:29'!
printDefinitionAndCommentOn: stream
		self printDefinitionOn: stream.

		stream cr; cr; nextPutAll: 'comment:'; cr.

		stream nextPutAll: self comment! !

!MCClassDefinition methodsFor: 'printing' stamp: 'al 3/28/2006 23:42'!
printDefinitionOn: stream
		stream 
			nextPutAll: self superclassName;
			nextPutAll: self kindOfSubclass;
			nextPut: $# ;
			nextPutAll: self className;
			cr; tab.
		self hasTraitComposition ifTrue: [
			stream 
				nextPutAll: 'uses: ';
		 		nextPutAll: self traitCompositionString;
				cr; tab ].
		stream
			nextPutAll: 'instanceVariableNames: ';
			store: self instanceVariablesString;
			cr; tab;
			nextPutAll: 'classVariableNames: ';
			store: self classVariablesString;
			cr; tab;
			nextPutAll: 'poolDictionaries: ';
			store: self sharedPoolsString;
			cr; tab;
			nextPutAll: 'category: ';
			store: self category asString! !

!MCClassDefinition methodsFor: 'comparing' stamp: 'ab 5/24/2003 14:12'!
provisions
	^ Array with: name! !

!MCClassDefinition methodsFor: 'installing' stamp: 'kph 10/1/2007 20:04'!
removeExtensionsFrom: obsDefinition 

	"For any Obsoleted Class Definitions, take all of the extension methods (in this and subclasses) and make them
	into MethodAdditions to be applied later"
		
	| cat |
	obsDefinition actualClass withAllSubclassesDo: [ :theClass |
	(Array with: theClass with: theClass classSide ) do: [ :behavior |
		behavior selectorsAndMethodsDo: [ :selector :method |
			cat := behavior organization categoryOfElement: selector.
			
			cat ifNotNil: [ 
				(cat beginsWith: '*')
					ifTrue: [ self extensions add: method methodReference asMethodDefinition ] 
				]
			]
		]
	].
	
	self extensions do:[ :each |  each unload ].! !

!MCClassDefinition methodsFor: 'orphanage' stamp: 'kph 5/10/2007 23:24'!
removeFromOrphanage: theOrphanage

	super removeFromOrphanage: theOrphanage.
	
	theOrphanage orphans removeAllSuchThat: [ :defn | 
		defn isMethodDefinition and: [ defn className = self className ] ]! !

!MCClassDefinition methodsFor: 'comparing' stamp: 'avi 2/17/2004 03:13'!
requirements
	^ (Array with: superclassName), self poolDictionaries! !

!MCClassDefinition methodsFor: 'accessing' stamp: 'cwp 7/7/2003 23:28'!
selectVariables: aSelector
	^ variables select: [:v | v perform: aSelector] thenCollect: [:v | v name]! !

!MCClassDefinition methodsFor: 'initializing' stamp: 'kph 5/11/2007 01:32'!
setSuperclassName: theName

	superclassName := theName.
	
	(theName beginsWith: 'AnObsolete') ifTrue: [ superclassName := superclassName allButFirst: 10 ].
	
	superclassName := superclassName ifNil: ['nil'] ifNotNil: [superclassName asSymbol].! !

!MCClassDefinition methodsFor: 'printing' stamp: 'cwp 7/7/2003 23:54'!
sharedPoolsString
	^ self stringForVariablesOfType: #isPoolImport! !

!MCClassDefinition methodsFor: 'accessing' stamp: 'ab 7/19/2003 18:00'!
sortKey
	^ self className! !

!MCClassDefinition methodsFor: 'accessing' stamp: 'bf 8/29/2006 11:45'!
sortedVariables
	"sort variables for comparison purposes"

	| sorted |
	sorted := variables select: [:var | var isOrderDependend].
	sorted addAll: ((variables reject: [:var | var isOrderDependend])
		asSortedCollection: [:a :b | a name <= b name]).
	^sorted! !

!MCClassDefinition methodsFor: 'printing' stamp: 'cwp 7/10/2003 01:29'!
source
	^ self definitionString! !

!MCClassDefinition methodsFor: 'installing' stamp: 'kph 5/11/2007 14:44'!
stringForVariablesOfType: typeSymbol  

	^ self stringForVariablesOfType: typeSymbol plus: nil
! !

!MCClassDefinition methodsFor: 'installing' stamp: 'kph 5/23/2007 02:58'!
stringForVariablesOfType: typeSymbol plus: addToMe
	
	| vars diff orig |
	
	vars := self selectVariables: typeSymbol.
	
	addToMe ifNotNil: [ 
		orig := addToMe selectVariables: typeSymbol.
		diff := vars difference: orig. 
		vars := orig , diff.
		vars := vars difference: (self superclassesExistingVariablesOfType: typeSymbol).
	].

	^ String streamContents:
		[:stream |
			vars
				do: [:ea | stream nextPutAll: ea]
				separatedBy: [stream space]]! !

!MCClassDefinition methodsFor: 'printing' stamp: 'ab 11/16/2002 17:33'!
summary
	^ name! !

!MCClassDefinition methodsFor: 'accessing' stamp: 'ab 11/13/2002 17:41'!
superclassName
	^ superclassName! !

!MCClassDefinition methodsFor: 'installing' stamp: 'kph 5/22/2007 17:27'!
superclassesExistingVariablesOfType: typeSymbol  
	
	
	typeSymbol = #isInstanceVariable ifTrue: [ 
		^ self actualSuperclass withAllSuperclasses gather: [:cl | cl instVarNames ] 
	].
	 
	typeSymbol = #isClassVariable ifTrue: [ 
		^ self actualSuperclass withAllSuperclasses gather: [:cl | cl classVarNames ] 
	].

	typeSymbol = #isClassInstanceVariable ifTrue: [ 
		^ self actualSuperclass withAllSuperclasses gather: [:cl | cl class instVarNames ] 
	].
	 
	typeSymbol = #isPoolImport ifTrue: [ 
		^ self actualSuperclass withAllSuperclasses gather: [:cl | cl poolDictionaryNames ] 
	].
 
 	^ Set new! !

!MCClassDefinition methodsFor: 'accessing' stamp: 'al 10/9/2005 20:16'!
traitComposition
	^traitComposition! !

!MCClassDefinition methodsFor: 'testing' stamp: 'kph 6/5/2008 02:03'!
traitComposition: aString
	aString = '{}' ifTrue: [ ^ self ].
	traitComposition := aString! !

!MCClassDefinition methodsFor: 'accessing' stamp: 'al 10/9/2005 21:55'!
traitCompositionString
	^self traitComposition ifNil: ['{}'].! !

!MCClassDefinition methodsFor: 'accessing' stamp: 'cwp 11/24/2002 22:35'!
type
	^ type! !

!MCClassDefinition methodsFor: 'installing' stamp: 'kph 5/11/2007 03:10'!
unload

	Smalltalk removeClassNamed: name.! !

!MCClassDefinition methodsFor: 'accessing' stamp: 'cwp 11/25/2002 06:51'!
variables
	^ variables! !

MCClassDefinition subclass: #MCTraitDefinition
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Modeling'!

!MCTraitDefinition class methodsFor: 'as yet unclassified' stamp: 'al 10/9/2005 20:28'!
name: classNameString traitComposition:  traitCompositionString category:  categoryString comment:  commentString commentStamp:   commentStamp
	^ self instanceLike:
		(self new initializeWithName: classNameString 
			traitComposition:  traitCompositionString
			category:  categoryString
			comment:  commentString  
			commentStamp:   commentStamp)
! !

!MCTraitDefinition methodsFor: 'visiting' stamp: 'kph 10/25/2008 02:11'!
= aDefinition
	self flag: #traits. "Ugly we harcoded the super superclass method.  We will have to refactor the definition hierarchy"
	
	(self isRevisionOf: aDefinition) ifFalse: [ ^ false ].
	self traitCompositionString = aDefinition traitCompositionString  ifFalse: [ ^ false ].
	category = aDefinition category ifFalse: [ ^ false ].
	comment = aDefinition comment ifFalse: [ ^ false ].
	
	^ true! !

!MCTraitDefinition methodsFor: 'visiting' stamp: 'al 10/9/2005 20:28'!
accept: aVisitor
	^ aVisitor visitTraitDefinition: self
! !

!MCTraitDefinition methodsFor: 'visiting' stamp: 'kph 5/1/2008 19:18'!
classTrait
	^ Smalltalk at: #Trait ifAbsent: [ self error: 'Trait''s are not available in this image' ].! !

!MCTraitDefinition methodsFor: 'accessing' stamp: 'al 6/5/2006 14:05'!
classTraitCompositionString
	^self traitComposition ifNil: ['{}'].! !

!MCTraitDefinition methodsFor: 'visiting' stamp: 'kph 5/1/2008 19:17'!
createTrait
	^self classTrait
		named: name
		uses: (Compiler evaluate: self traitCompositionString)
		category: category
		
! !

!MCTraitDefinition methodsFor: 'system editor' stamp: 'mtf 6/26/2008 11:07'!
edLoad: aSystemEditor
	^ (aSystemEditor at: #Trait) named: self name
		uses: (Compiler evaluate: self traitCompositionString for: aSystemEditor logged: false)
		category: self category! !

!MCTraitDefinition methodsFor: 'testing' stamp: 'al 10/9/2005 20:28'!
hasClassInstanceVariables
	^ false

! !

!MCTraitDefinition methodsFor: 'testing' stamp: 'al 6/5/2006 14:13'!
hasTraitComposition
	^self traitCompositionString ~= '{}'! !

!MCTraitDefinition methodsFor: 'comparing' stamp: 'al 6/5/2006 14:14'!
hash
	| hash |
	hash := String stringHash: name initialHash: 0.
	hash := String stringHash: self traitCompositionString initialHash: hash.
	hash := String stringHash: (category ifNil: ['']) initialHash: hash.
	^ hash
! !

!MCTraitDefinition methodsFor: 'initializing' stamp: 'kph 6/5/2008 02:02'!
initializeWithName: classNameString 
	traitComposition:  traitCompositionString
	category:  categoryString
	comment:  commentString  
	commentStamp:   commentStampString
					
		name := classNameString asSymbol.
		self traitComposition: traitCompositionString.
	     category := categoryString.
		comment := commentString withSqueakLineEndings.
		commentStamp :=  commentStampString ifNil: [self defaultCommentStamp]
! !

!MCTraitDefinition methodsFor: 'installing' stamp: 'kph 8/1/2007 09:57'!
install
	! !

!MCTraitDefinition methodsFor: 'installing' stamp: 'kph 5/13/2007 02:48'!
obsoletion

	^ nil! !

!MCTraitDefinition methodsFor: '' stamp: 'kph 8/1/2007 09:55'!
postinstall
	 ! !

!MCTraitDefinition methodsFor: '' stamp: 'kph 6/5/2008 22:58'!
postloadOver: obs! !

!MCTraitDefinition methodsFor: '' stamp: 'kph 8/1/2007 09:57'!
preloadOver: obsDefinition

 	self createTrait ifNotNilDo: [:trait |
		self hasComment ifTrue: [trait classComment: comment stamp: commentStamp]]! !

!MCTraitDefinition methodsFor: 'printing' stamp: 'al 6/5/2006 14:15'!
printDefinitionOn: stream
	stream nextPutAll: 'Trait named: #', self className;
		 cr;
		 tab;
		 nextPutAll: 'uses: ';
		 nextPutAll: self traitCompositionString;
		 cr;
		 tab;
		 nextPutAll: 'category: ';
		 store: self category asString
! !

!MCTraitDefinition methodsFor: 'comparing' stamp: 'al 6/5/2006 14:13'!
requirements
	"Assuming that traits in a composition can be identified by
	testing for the first character beeing an uppercase character
	(and thus not a special character such as {, # etc.)"

	| tokens traitNames |
	self hasTraitComposition ifFalse: [ ^Array new ].
	tokens := Scanner new scanTokens: self traitComposition.
	traitNames := tokens select: [:each | each first isUppercase].
	^traitNames asArray! !

!MCTraitDefinition methodsFor: 'accessing' stamp: 'al 10/9/2005 20:29'!
traitComposition
	^traitComposition! !

MCDefinition subclass: #MCClassTraitDefinition
	instanceVariableNames: 'baseTrait classTraitComposition properties'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Modeling'!

!MCClassTraitDefinition class methodsFor: 'as yet unclassified' stamp: 'al 10/9/2005 20:22'!
baseTraitName: aString classTraitComposition: classTraitCompositionString
	^self instanceLike: (
		self new
			initializeWithBaseTraitName: aString
			classTraitComposition: classTraitCompositionString).! !

!MCClassTraitDefinition methodsFor: 'accessing' stamp: 'al 10/9/2005 21:59'!
= aDefinition
	^ (super = aDefinition)
		and: [baseTrait = aDefinition baseTrait]
		and: [self classTraitCompositionString = aDefinition classTraitCompositionString]

! !

!MCClassTraitDefinition methodsFor: 'as yet unclassified' stamp: 'al 10/9/2005 20:23'!
accept: aVisitor
	^ aVisitor visitClassTraitDefinition: self.! !

!MCClassTraitDefinition methodsFor: 'accessing' stamp: 'al 10/9/2005 20:23'!
baseTrait
	^baseTrait
! !

!MCClassTraitDefinition methodsFor: 'accessing' stamp: 'al 12/15/2005 11:31'!
className
	^self baseTrait! !

!MCClassTraitDefinition methodsFor: 'accessing' stamp: 'al 10/9/2005 20:23'!
classTraitComposition
	^classTraitComposition

! !

!MCClassTraitDefinition methodsFor: 'accessing' stamp: 'al 10/10/2005 10:12'!
classTraitCompositionString
	^self classTraitComposition ifNil: ['{}'].

! !

!MCClassTraitDefinition methodsFor: 'accessing' stamp: 'al 10/9/2005 21:59'!
definitionString
	^self baseTrait , ' classTrait
	uses: ' , self classTraitCompositionString.
! !

!MCClassTraitDefinition methodsFor: 'accessing' stamp: 'al 10/9/2005 20:24'!
description
	^Array
		with: baseTrait
		with: classTraitComposition! !

!MCClassTraitDefinition methodsFor: 'accessing' stamp: 'al 6/5/2006 14:04'!
hash
	| hash |
	hash := String stringHash: baseTrait initialHash: 0.
	hash := String stringHash: self classTraitCompositionString initialHash: hash.
	^hash
! !

!MCClassTraitDefinition methodsFor: 'initialization' stamp: 'stephaneducasse 2/4/2006 20:47'!
initializeWithBaseTraitName: aTraitName classTraitComposition: aString
	baseTrait := aTraitName.
	classTraitComposition := aString.! !

!MCClassTraitDefinition methodsFor: 'as yet unclassified' stamp: 'kph 5/15/2007 19:47'!
preloadOver: obs
	super preloadOver: obs.	
	Compiler evaluate: self definitionString! !

!MCClassTraitDefinition methodsFor: 'accessing' stamp: 'al 10/9/2005 20:24'!
requirements
	^Array with: baseTrait! !

!MCClassTraitDefinition methodsFor: 'accessing' stamp: 'al 10/9/2005 20:25'!
sortKey
	^ self baseTrait name , '.classTrait'! !

!MCClassTraitDefinition methodsFor: 'accessing' stamp: 'al 10/9/2005 20:25'!
source
	^self definitionString! !

!MCClassTraitDefinition methodsFor: 'accessing' stamp: 'al 10/9/2005 20:25'!
summary
	^self baseTrait , ' classTrait'
! !

!MCDefinition class methodsFor: 'as yet unclassified' stamp: 'kph 5/30/2008 03:32'!
cleanUp

^ self clearInstances! !

!MCDefinition class methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
clearInstances
	WeakArray removeWeakDependent: Instances.
	Instances := nil! !

!MCDefinition class methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
instanceLike: aDefinition
	Instances ifNil: [Instances := WeakSet new].
	^ (Instances like: aDefinition) ifNil: [Instances add: aDefinition]! !

!MCDefinition class methodsFor: 'as yet unclassified' stamp: 'kph 5/19/2007 01:45'!
storeAsClassForMC1

^ self! !

!MCDefinition methodsFor: 'comparing' stamp: 'ab 7/19/2003 17:59'!
<= other
	^ self sortKey <= other sortKey! !

!MCDefinition methodsFor: 'comparing' stamp: 'ab 7/19/2003 18:24'!
= aDefinition
	^ self isRevisionOf: aDefinition! !

!MCDefinition methodsFor: 'accessing' stamp: 'gvc 10/26/2006 12:17'!
actualClass
	"Since the targetClass call on a patch operation will fail
	otherwise."
	
	^nil! !

!MCDefinition methodsFor: 'annotations' stamp: 'nk 7/24/2003 12:27'!
annotations
	^self annotations: Preferences defaultAnnotationRequests! !

!MCDefinition methodsFor: 'annotations' stamp: 'nk 7/24/2003 12:26'!
annotations: requests
	"Answer a string for an annotation pane, trying to fulfill the annotation requests.
	These might include anything that
		Preferences defaultAnnotationRequests 
	might return. Which includes anything in
		Preferences annotationInfo
	To edit these, use:"
	"Preferences editAnnotations"

	^String streamContents: [ :s | self printAnnotations: requests on: s ].! !

!MCDefinition methodsFor: 'accessing' stamp: 'kph 6/5/2008 02:30'!
because

	^ self propertyAt: #because ifAbsent: [ nil ]! !

!MCDefinition methodsFor: 'accessing' stamp: 'kph 6/5/2008 02:31'!
because: value
	
	value ifNil: [ ^ self propertyRemoveKey: #because  ].

	^ self propertyAt: #because put: value! !

!MCDefinition methodsFor: 'comparing' stamp: 'ab 7/19/2003 18:14'!
description
	self subclassResponsibility! !

!MCDefinition methodsFor: 'system editor' stamp: 'kph 5/25/2007 07:13'!
edLoad: editor

	^ false! !

!MCDefinition methodsFor: 'system editor' stamp: 'kph 5/25/2007 09:09'!
edUnload: editor

	^ false! !

!MCDefinition methodsFor: 'converting' stamp: 'mtf 10/7/2008 12:12'!
fromImage
"Answer a copy of myself, but at the version that exists in the image. Answers nil if no such version exists"

	^ nil! !

!MCDefinition methodsFor: 'comparing' stamp: 'nk 10/21/2003 23:18'!
fullTimeStamp
	^TimeStamp current! !

!MCDefinition methodsFor: 'comparing' stamp: 'ab 7/19/2003 18:14'!
hash
	^ self description hash! !

!MCDefinition methodsFor: 'annotations' stamp: 'kph 6/5/2008 01:39'!
instVarAt: anInteger put: anObject 
	"Primitive. Store a value into a fixed variable in the receiver. The 
	numbering of the variables corresponds to the named instance variables. 
	Fail if the index is not an Integer or is not the index of a fixed variable. 
	Answer the value stored as the result. Using this message violates the 
	principle that each object has sovereign control over the storing of 
	values into its instance variables. Essential. See Object documentation 
	whatIsAPrimitive."

	| |
	
	<primitive: 74>


	"Access beyond fixed fields"

	"When loading in any definition classes we get called if #instVarAt:put: fails which will occur if in a future version if extra instVars have been added to theses classes, ok, so we dont know what to do with this extra data, so we can slap it into our properties dictionary just in case!!"

	 'extra', anInteger asString in: [ :newName |

			self class addInstVarName: newName. 
	 		self instVarNamed: newName put: anObject. 
	 ].	
	
 ! !

!MCDefinition methodsFor: 'installing' stamp: 'kph 5/11/2007 11:17'!
install
	! !

!MCDefinition methodsFor: 'testing' stamp: 'ab 12/4/2002 21:51'!
isClassDefinition
	^false! !

!MCDefinition methodsFor: 'testing' stamp: 'bf 11/12/2004 14:46'!
isClassDefinitionExtension
	"Answer true if this definition extends the regular class definition"
	^false! !

!MCDefinition methodsFor: 'testing' stamp: 'ab 12/4/2002 21:51'!
isMethodDefinition
	^false! !

!MCDefinition methodsFor: 'testing' stamp: 'cwp 7/11/2003 01:32'!
isOrganizationDefinition
	^false! !

!MCDefinition methodsFor: 'comparing' stamp: 'kph 5/15/2007 01:12'!
isPartOfPackage: aPackageInfo or: orphanage
	self subclassResponsibility	! !

!MCDefinition methodsFor: 'comparing' stamp: 'ab 12/5/2002 21:24'!
isRevisionOf: aDefinition
	^ aDefinition description = self description! !

!MCDefinition methodsFor: 'comparing' stamp: 'ab 7/19/2003 18:25'!
isSameRevisionAs: aDefinition
	^ self = aDefinition! !

!MCDefinition methodsFor: 'testing' stamp: 'kph 5/11/2007 13:23'!
isUnlinkedClassDefinition
	 
	^false! !

!MCDefinition methodsFor: 'installing' stamp: 'ab 7/18/2003 21:31'!
load
	! !

!MCDefinition methodsFor: 'obsolete' stamp: 'kph 5/15/2007 20:17'!
loadOver: aDefinition
	"this method is now obsolete, it runs when old versions of Monticello are used to load this one.
	It is also the only place in the transition where you can put 'initialization' code if you have to"
	
	self load
	! !

!MCDefinition methodsFor: 'installing' stamp: 'kph 5/13/2007 02:53'!
manuallyObsoleted
 	 
	^ nil! !

!MCDefinition methodsFor: 'accessing' stamp: 'kph 5/10/2007 21:06'!
orphanage

	^ MCOrphanage current! !

!MCDefinition methodsFor: 'orphanage' stamp: 'kph 5/15/2007 13:05'!
orphanedBecause: explanation

	"During installation anticipated errors result in the definiton becoming orphaned"
	
	self because: explanation.
	
	^ MCOrphanedNotification orphan: self signal: explanation! !

!MCDefinition methodsFor: 'installing' stamp: 'kph 5/11/2007 12:43'!
postinstall
	! !

!MCDefinition methodsFor: 'installing' stamp: 'kph 6/5/2008 22:58'!
postloadOver: aDefinition
	"this method is now obsolete, it runs when old versions of Monticello are used to load this one.
	It is also the only place in the transition where you can put 'initialization' code if you have to"
 ! !

!MCDefinition methodsFor: 'installing' stamp: 'kph 5/15/2007 19:22'!
preloadOver: aDefinition
	self because: nil. "clear out any previous"
 	! !

!MCDefinition methodsFor: 'annotations' stamp: 'nk 11/10/2003 21:46'!
printAnnotations: requests on: aStream
	"Add a string for an annotation pane, trying to fulfill the annotation requests.
	These might include anything that
		Preferences defaultAnnotationRequests 
	might return. Which includes anything in
		Preferences annotationInfo
	To edit these, use:"
	"Preferences editAnnotations"

	aStream nextPutAll: 'not yet implemented'! !

!MCDefinition methodsFor: 'printing' stamp: 'kph 5/13/2007 05:43'!
printOn: aStream
	super printOn: aStream.
	aStream nextPutAll: '(', self summary, ') ' ! !

!MCDefinition methodsFor: 'accessing' stamp: 'kph 6/5/2008 02:26'!
propertiesInitialize

	^ self instVarNamed: 'properties' put: IdentityDictionary new! !

!MCDefinition methodsFor: 'accessing' stamp: 'kph 6/5/2008 02:28'!
propertiesPrivate

	^ self instVarNamed: 'properties'! !

!MCDefinition methodsFor: 'accessing' stamp: 'kph 6/5/2008 02:29'!
propertyAt: key ifAbsent: aBlock

	self propertiesPrivate ifNil: [ ^ aBlock value ].
	^ self propertiesPrivate at: key ifAbsent: aBlock! !

!MCDefinition methodsFor: 'accessing' stamp: 'kph 6/5/2008 02:29'!
propertyAt: key put: value

	self propertiesPrivate ifNil: [ self propertiesInitialize ].
	^ self propertiesPrivate at: key put: value.! !

!MCDefinition methodsFor: 'accessing' stamp: 'kph 6/5/2008 22:03'!
propertyRemoveKey: key

	self propertiesPrivate ifNotNilDo: [ :p | p removeKey: key ifAbsent: nil.
 		p isEmpty ifTrue: [ self instVarNamed: 'properties' put: nil ].
	].! !

!MCDefinition methodsFor: 'accessing' stamp: 'ab 5/24/2003 14:12'!
provisions
	^ #()! !

!MCDefinition methodsFor: 'testing' stamp: 'kph 6/5/2008 23:02'!
readDataFrom: aDataStream size: varsOnDisk

	| read theClass |
	
	read := super readDataFrom: aDataStream size: varsOnDisk.
		
	theClass := read propertyAt: #class ifAbsent: [ ^ read ].
	
	theClass := Smalltalk at: theClass ifAbsent: [ ^ read ]. 
	
	^ theClass basicNew copyFrom: read! !

!MCDefinition methodsFor: 'orphanage' stamp: 'kph 5/10/2007 23:05'!
removeFromOrphanage: theOrphanage
	theOrphanage orphans remove: self ifAbsent: []! !

!MCDefinition methodsFor: 'accessing' stamp: 'ab 5/24/2003 14:12'!
requirements
	^ #()! !

!MCDefinition methodsFor: 'accessing' stamp: 'kph 6/5/2008 02:26'!
setProperties

	^ self instVarNamed: 'properties'! !

!MCDefinition methodsFor: 'comparing' stamp: 'ab 7/19/2003 18:04'!
sortKey
	self subclassResponsibility ! !

!MCDefinition methodsFor: 'annotations' stamp: 'kph 6/19/2008 17:31'!
storeDataOn: aDataStream

	"we only ignore trailing instVars that are nil"
 
	| lastNotNil |

	self class ~= self class storeAsClassForMC1 ifTrue: [ self propertyAt: #class put: self class name ].

	self propertyRemoveKey: #because.

	lastNotNil := (self storeMaxInstVars to: 1 by: -1) detect: [ :n |  (self instVarAt: n) notNil ].
	 
	aDataStream
		beginInstance: self class storeAsClassForMC1
		size: lastNotNil.
	1 to: lastNotNil do:
		[:i | aDataStream nextPut: (self instVarAt: i)].


! !

!MCDefinition methodsFor: '' stamp: 'kph 6/5/2008 02:18'!
storeMaxInstVars 

^ self class instSize! !

!MCDefinition methodsFor: 'printing' stamp: 'ab 7/19/2003 18:23'!
summary
	self subclassResponsibility ! !

!MCDefinition methodsFor: 'installing' stamp: 'ab 11/14/2002 00:08'!
unload! !

MCDefinition subclass: #MCMethodDefinition
	instanceVariableNames: 'classIsMeta source category selector className timeStamp properties dummy1 dummy2 methodAndNode requestor compiledMethod priorMethodOrNil compiledSelector isInstalled priorMethodCategory theClass defnToInstall'
	classVariableNames: 'Definitions'
	poolDictionaries: ''
	category: 'Monticello-Modeling'!

!MCMethodDefinition class methodsFor: 'as yet unclassified' stamp: 'kph 12/15/2007 09:24'!
initialize
	self cleanUp.
	Smalltalk addToShutDownList: self! !

MCMethodDefinition subclass: #MCMethodAfterRemovalActionDefinition
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Modeling'!

!MCMethodAfterRemovalActionDefinition methodsFor: 'as yet unclassified' stamp: 'kph 11/18/2007 03:55'!
edPostunload 

	"
	this hackish method, takes our method source and evaluates it
	"

	| return code |
	
	return := super edPostunload.
	
	code :=  (source copyAfter: $  ) 
	 
	self class compilerClass evaluate:  code.
		
	^ return! !

!MCMethodAfterRemovalActionDefinition methodsFor: 'as yet unclassified' stamp: 'kph 5/26/2007 07:11'!
edUnload 

	super edUnload.
	
	^ true! !

!MCMethodAfterRemovalActionDefinition methodsFor: 'as yet unclassified' stamp: 'kph 5/26/2007 07:13'!
sortKey
	^ '!!', super sortKey "force to the beginning so it gets run last when unloading"! !

!MCMethodAfterRemovalActionDefinition methodsFor: 'as yet unclassified' stamp: 'kph 11/18/2007 03:45'!
unload 

	"
	this hackish method, takes our method source and evaluates it
	"

	| return code |
	
	return := super unload.
	
	code :=  (source copyAfter: $  ) 
	 
	self class compilerClass evaluate:  code.
		
	^ return! !

MCMethodDefinition subclass: #MCMethodBeforeCommitLoadDefinition
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Modeling'!

!MCMethodBeforeCommitLoadDefinition commentStamp: 'kph 5/30/2007 16:20' prior: 0!
MCMethodSpecialActionDefinition

 !

!MCMethodBeforeCommitLoadDefinition methodsFor: 'as yet unclassified' stamp: 'kph 5/30/2007 16:23'!
edLoad: theEditor

	"this hackish method, takes our method source, munges it into an anonymous block.
	to which we evaluate feeding the classEditor as a parameter, enabling the package
	the ability to add changes anywhere, before this method or even its class has been installed!!
	"

	| return code ourClassEditor |
	
	return := super edLoad: theEditor.
	
	ourClassEditor := theEditor at: className.
	
	code := '[ :', (source copyAfter: $: ), ' ]'.
	code at: (code indexOf: (Character cr)) put: $|.
	
	(self class compilerClass evaluate:  code) value: ourClassEditor.
		
	^ return
! !

!MCMethodBeforeCommitLoadDefinition methodsFor: 'as yet unclassified' stamp: 'kph 11/18/2007 03:42'!
preloadOver: thePrevious

	"this hackish method, takes our method source, munges it into an anonymous block.
	to which we evaluate feeding the Class as a parameter, enabling the package
	the ability to add changes anywhere,  
	"

	| return code ourClassEditor |
	
	return := super preLoadOver: thePrevious.
	
	ourClassEditor := Smalltalk at: className.
	
	code := '[ :', (source copyAfter: $: ), ' ]'.
	code at: (code indexOf: (Character cr)) put: $|.
	
	(self class compilerClass evaluate:  code) value: ourClassEditor.
		
	^ return
! !

!MCMethodBeforeCommitLoadDefinition methodsFor: 'as yet unclassified' stamp: 'kph 5/26/2007 05:20'!
sortKey
	^ 'zzz', super sortKey "force to the end so it gets loaded last"! !

MCMethodDefinition subclass: #MCMethodBeforeCommitUnloadDefinition
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Modeling'!

!MCMethodBeforeCommitUnloadDefinition methodsFor: 'as yet unclassified' stamp: 'kph 11/18/2007 03:52'!
edUnload: theEditor

	"this hackish method, takes our method source, munges it into an anonymous block.
	to which we evaluate feeding the classEditor as a parameter, enabling the package
	the ability to add changes anywhere, before this method or even its class has been installed!!
	"

	| return code ourClassEditor |
	
	return := super edUnload: theEditor.
	
	ourClassEditor := theEditor at: className.
	
	code := '[ :', (source copyAfter: $: ), ' ]'.
	code at: (code indexOf: (Character cr)) put: $|.
	
	(self class compilerClass evaluate:  code) value: ourClassEditor.
		
	^ return
! !

!MCMethodBeforeCommitUnloadDefinition methodsFor: 'as yet unclassified' stamp: 'kph 11/18/2007 03:55'!
sortKey
	^ 'aaa', super sortKey "force to the beginning so it gets unloaded last"! !

!MCMethodBeforeCommitUnloadDefinition methodsFor: 'as yet unclassified' stamp: 'kph 11/18/2007 03:52'!
unload
	"this hackish method, takes our method source, munges it into an anonymous block.
	to which we evaluate feeding the Class as a parameter, enabling the package
	the ability to add changes anywhere,  
	"

	| return code ourClassEditor |
	
	return := super unload.
	
	ourClassEditor := Smalltalk at: className.
	
	code := '[ :', (source copyAfter: $: ), ' ]'.
	code at: (code indexOf: (Character cr)) put: $|.
	
	(self class compilerClass evaluate:  code) value: ourClassEditor.
		
	^ return
! !

!MCMethodDefinition class methodsFor: 'as yet unclassified' stamp: 'kph 8/2/2007 15:02'!
cachedDefinitions
	Definitions ifNil: [Definitions := WeakIdentityKeyDictionary new.   
"WeakArray addWeakDependent: Definitions" "instead we call tidyCache on (un)loading a package" ].
	^ Definitions! !

!MCMethodDefinition class methodsFor: 'as yet unclassified' stamp: 'kph 1/18/2008 10:33'!
classForSelector: selectorString meta: isMeta

	isMeta ifFalse: [ ^ self ].
	
 	selectorString = 'initialize' ifTrue: [ ^ MCMethodInitializerDefinition ].
 	selectorString = 'unload' ifTrue: [ ^ MCMethodUnloaderDefinition ].
 	(selectorString beginsWith: self prefixAfterInstall) ifTrue: [ ^ MCMethodInitializerDefinition ].
 	(selectorString beginsWith: self prefixAfterRemoval) ifTrue: [ ^ MCMethodAfterRemovalActionDefinition ].

	(selectorString last = $:) ifTrue: [ 
	 	(selectorString beginsWith: self prefixBeforeCommitLoad) ifTrue: [ ^ MCMethodBeforeCommitLoadDefinition ].
 		(selectorString beginsWith: self prefixBeforeCommitUnload) ifTrue: [ ^ MCMethodBeforeCommitUnloadDefinition ].
	].
	selectorString = 'fields' ifTrue: [ ^ MCMethodExternalFieldDefinition ].
	! !

!MCMethodDefinition class methodsFor: 'as yet unclassified' stamp: 'kph 5/15/2007 17:05'!
className: classString
classIsMeta: metaBoolean
selector: selectorString
category: catString
timeStamp: timeString
source: sourceString
	^ self instanceLike:
		((self classForSelector: selectorString meta: metaBoolean) new
					initializeWithClassName: classString
					classIsMeta: metaBoolean
					selector: selectorString
					category: catString
					timeStamp: timeString
					source: sourceString)! !

!MCMethodDefinition class methodsFor: 'as yet unclassified' stamp: 'ab 4/1/2003 01:40'!
className: classString
selector: selectorString
category: catString
timeStamp: timeString
source: sourceString
	^ self	className: classString
			classIsMeta: false
			selector: selectorString
			category: catString
			timeStamp: timeString
			source: sourceString! !

!MCMethodDefinition class methodsFor: 'as yet unclassified' stamp: 'kph 8/2/2007 15:19'!
forMethodReference: aMethodReference
	| definition |
 
	definition := self cachedDefinitions at: aMethodReference compiledMethod ifAbsent: [].
	(definition isNil
		or: [definition selector ~= aMethodReference methodSymbol]
		or: [definition className ~= aMethodReference classSymbol]
		or: [definition classIsMeta ~= aMethodReference classIsMeta]
		or: [definition category ~= aMethodReference category])
			ifTrue: [definition := self 
						className: aMethodReference classSymbol
						classIsMeta: aMethodReference classIsMeta
						selector: aMethodReference methodSymbol
						category: aMethodReference category
						timeStamp: aMethodReference timeStamp
						source: aMethodReference source.
					self cachedDefinitions at: aMethodReference compiledMethod put: definition].
	^ definition
	! !

!MCMethodDefinition class methodsFor: 'as yet unclassified' stamp: 'kph 6/2/2008 17:26'!
freeSomeSpace
	Definitions := nil! !

!MCMethodDefinition class methodsFor: 'configuration' stamp: 'kph 5/30/2007 16:25'!
prefixAfterInstall

	^ 'mcAfterInstall'! !

!MCMethodDefinition class methodsFor: 'configuration' stamp: 'kph 5/30/2007 16:25'!
prefixAfterRemoval

	^ 'mcAfterRemoval'! !

!MCMethodDefinition class methodsFor: 'configuration' stamp: 'kph 11/18/2007 04:03'!
prefixBeforeCommitLoad

	^ 'mcOnInstall'! !

!MCMethodDefinition class methodsFor: 'configuration' stamp: 'kph 11/18/2007 04:03'!
prefixBeforeCommitUnload

	^ 'mcOnUninstall'! !

!MCMethodDefinition class methodsFor: 'as yet unclassified' stamp: 'kph 7/4/2007 16:38'!
shutDown
	
	Definitions := nil.! !

!MCMethodDefinition class methodsFor: 'file format' stamp: 'kph 5/19/2007 04:46'!
storeAsClassForMC1

^ MCMethodDefinition! !

!MCMethodDefinition class methodsFor: 'as yet unclassified' stamp: 'mtf 5/8/2008 10:01'!
tidyCache
	Smalltalk garbageCollect.
	self cachedDefinitions finalizeValues.! !

!MCMethodDefinition methodsFor: 'comparing' stamp: 'kph 10/25/2008 02:10'!
= aDefinition
	super = aDefinition ifFalse: [ ^ false ].
	aDefinition source = self source ifFalse: [ ^ false ].
	aDefinition category = self category ifFalse: [ ^ false ].
	aDefinition timeStamp = self timeStamp ifFalse: [ ^ false ].
	
	^ true! !

!MCMethodDefinition methodsFor: 'visiting' stamp: 'ab 7/18/2003 21:47'!
accept: aVisitor
	^ aVisitor visitMethodDefinition: self! !

!MCMethodDefinition methodsFor: 'accessing' stamp: 'mtf 6/12/2008 23:05'!
actualClass
	^ self actualClassIn: Smalltalk! !

!MCMethodDefinition methodsFor: 'accessing' stamp: 'mtf 10/7/2008 12:21'!
actualClassIn: anEnvironment
	^ (theClass := anEnvironment classNamed: className)
		ifNotNilDo: [:class | classIsMeta ifTrue: [class classSide] ifFalse: [class]]! !

!MCMethodDefinition methodsFor: 'obsolete' stamp: 'kph 8/1/2007 01:02'!
addMethodAdditionTo: a

	"dummy to enable loading into 3.10"! !

!MCMethodDefinition methodsFor: 'accessing' stamp: 'ab 11/13/2002 01:59'!
category
	^ category! !

!MCMethodDefinition methodsFor: 'accessing' stamp: 'cwp 11/25/2002 07:26'!
classIsMeta
	^ classIsMeta! !

!MCMethodDefinition methodsFor: 'accessing' stamp: 'ab 11/15/2002 01:12'!
className
	^className! !

!MCMethodDefinition methodsFor: 'compiling' stamp: 'mtf 10/8/2008 01:17'!
compileMethod
	methodAndNode := self actualClass 
							compile: self source 
							classified: self category 
							notifying: requestor
							trailer: self actualClass defaultMethodTrailer ifFail: [^nil].

	compiledSelector := methodAndNode selector.
	compiledMethod := methodAndNode method.
	
  ! !

!MCMethodDefinition methodsFor: 'accessing' stamp: 'kph 8/2/2007 11:19'!
compiledMethod
	^ compiledMethod! !

!MCMethodDefinition methodsFor: 'accessing' stamp: 'kph 10/24/2007 14:30'!
compiledMethod: aCM
 
	compiledSelector := aCM selector.
	compiledMethod := aCM.
	
  ! !

!MCMethodDefinition methodsFor: 'accessing' stamp: 'kph 8/2/2007 11:19'!
compiledSelector
	^ compiledSelector! !

!MCMethodDefinition methodsFor: 'compiling' stamp: 'kph 10/24/2007 14:33'!
createCompiledMethod

	"compiledSelector and selector should not be different, but just in case we use a searate instVar"
 
	self actualClass 
		ifNil: [ ^self orphanedBecause: ('class {1} not present' format: {className}) ].
		
	defnToInstall := self.
	
	((self isOverrideCategory: priorMethodCategory) and: [ (self isOverrideCategory: category) not ])
		ifTrue: [ "if we have been overriden, compile and add our source, then set defnToInstall to be the original"
		  
			defnToInstall := priorMethodOrNil.
			self compileMethod ].
	
	defnToInstall actualClass; compileMethod.
						 
	 ! !

!MCMethodDefinition methodsFor: 'printing' stamp: 'kph 5/23/2007 00:46'!
description
	^ Array	
		with: className
		with: selector
		with: classIsMeta

! !

!MCMethodDefinition methodsFor: 'system editor' stamp: 'mtf 6/12/2008 23:06'!
edLoad: editor

	"priorMethodOrNil := self actualClass compiledMethodAt: selector ifAbsent: [nil]."

	methodAndNode := (self actualClassIn: editor)
							compile: self source 
							classified: self category
							withStamp: timeStamp
							notifying: requestor.
							 
	^ false "dont ask for edPostLoad, switch our notifications off not sure if needed with SystemEditor-#commit"
 ! !

!MCMethodDefinition methodsFor: 'system editor' stamp: 'kph 10/24/2007 14:34'!
edPostload
	
	compiledMethod := self actualClass compiledMethodAt: selector ifAbsent: [ ^nil]. 

	SystemChangeNotifier uniqueInstance 
		doSilently: [self actualClass organization classify: selector under: category].
	
	priorMethodOrNil isNil
		ifTrue: [SystemChangeNotifier uniqueInstance methodAdded: compiledMethod selector: selector inProtocol: category class: self actualClass requestor: requestor]
		ifFalse: [SystemChangeNotifier uniqueInstance methodChangedFrom: priorMethodOrNil compiledMethod to: compiledMethod selector: selector inClass: self actualClass requestor: requestor].

	"The following code doesn't seem to do anything."
	self actualClass instanceSide noteCompilationOf: selector meta: self actualClass isClassSide.
! !

!MCMethodDefinition methodsFor: 'system editor' stamp: 'mtf 6/12/2008 23:18'!
edUnload: editor
	(self actualClassIn: editor) removeSelector: selector.
					 
	^ false   "we dont need an edPostUnload"! !

!MCMethodDefinition methodsFor: 'compiling' stamp: 'mtf 6/10/2008 20:21'!
ensuredCreateCompiledMethod
	"Create the compiled method from my source, or from user-provided source in the event of a syntax error"

	[ self createCompiledMethod ] on: SyntaxErrorNotification do: [ :ex |
		"Let the user install the fixed code into the system"
		ex defaultAction.
		"Now fetch and use that code instead"
		source := theClass sourceCodeAt: self selector ifAbsent: [^ nil].
		self createCompiledMethod]! !

!MCMethodDefinition methodsFor: 'converting' stamp: 'mtf 10/7/2008 12:38'!
fromImage
	| class |
	class := self actualClass ifNil: [^ nil].
	(class includesLocalSelector: selector) ifFalse: [^ nil].
	^(MethodReference new setStandardClass: class methodSymbol: selector) asMethodDefinition! !

!MCMethodDefinition methodsFor: 'printing' stamp: 'kph 5/24/2007 01:10'!
fullClassName
	"Using #class selector for classes for backwards compatibility"

	"The old implementation assumed that the definition is installed and refers to an existing class. This is not always the case, so we have to rely upon our self knowledge.
Note if we are a trait's method we probably need more state!!"
	
	^ self classIsMeta
		ifFalse: [self className]
		ifTrue: [ self isTraitMethod 
					ifFalse: [ self className, ' class']
					ifTrue: [ self className, ' classSide' ] ].! !

!MCMethodDefinition methodsFor: 'accessing' stamp: 'nk 10/21/2003 23:08'!
fullTimeStamp
	^TimeStamp fromMethodTimeStamp: timeStamp! !

!MCMethodDefinition methodsFor: 'comparing' stamp: 'stephaneducasse 2/4/2006 20:47'!
hash
	| hash |
	hash := String stringHash: classIsMeta asString initialHash: 0.
	hash := String stringHash: source initialHash: hash.
	hash := String stringHash: category initialHash: hash.
	hash := String stringHash: className initialHash: hash.
	^ hash! !

!MCMethodDefinition methodsFor: 'serializing' stamp: 'mtf 6/10/2008 21:48'!
initializeWithClassName: classString
classIsMeta: metaBoolean
selector: selectorString
category: catString
timeStamp: timeString
source: sourceString
	className := classString asSymbol.
	selector := selectorString asSymbol.
	category :=  (catString ifNil: [ '' ]) asSymbol.
	timeStamp := timeString.
	classIsMeta := metaBoolean.
	source := sourceString withSqueakLineEndings.
	requestor := nil. "Always handle syntax errors thru SyntaxErrorNotification, never directly"
! !

!MCMethodDefinition methodsFor: 'compiling' stamp: 'kph 10/24/2007 14:20'!
install

	compiledMethod ifNil: [ 	self protectedCreateCompiledMethod ].
	
	compiledMethod ifNotNil: [ self installMethod ].! !

!MCMethodDefinition methodsFor: 'installing' stamp: 'kph 8/6/2007 15:08'!
installCompiledMethod

	compiledMethod ifNil: [ ^ false ].
	
	self actualClass addSelectorSilently: compiledSelector withMethod: compiledMethod.
	 
	^ true! !

!MCMethodDefinition methodsFor: 'compiling' stamp: 'kph 8/6/2007 15:10'!
installMethod
 
	self writeSourceToLog.
	self installCompiledMethod.

	defnToInstall ifNil: [ ^ nil ].
	self ~= defnToInstall ifTrue: [ 
			defnToInstall writeSourceToLog.
			defnToInstall installCompiledMethod ]! !

!MCMethodDefinition methodsFor: 'testing' stamp: 'ab 5/24/2003 13:49'!
isCodeDefinition
	^ true! !

!MCMethodDefinition methodsFor: 'installing' stamp: 'avi 9/17/2003 22:27'!
isExtensionMethod
	^ category beginsWith: '*'! !

!MCMethodDefinition methodsFor: 'testing' stamp: 'kph 5/15/2007 17:19'!
isInitializer
	^ false! !

!MCMethodDefinition methodsFor: 'testing' stamp: 'ab 12/4/2002 21:52'!
isMethodDefinition
	^true! !

!MCMethodDefinition methodsFor: 'testing' stamp: 'kph 8/2/2007 13:28'!
isOverrideCategory: cat
 
	^ cat notNil and: [ (cat beginsWith: '*') and: [cat endsWith: '-override'] ]! !

!MCMethodDefinition methodsFor: 'installing' stamp: 'kph 8/2/2007 13:29'!
isOverrideMethod
	"this oughta check the package"
	^ self isOverrideCategory: category ! !

!MCMethodDefinition methodsFor: 'testing' stamp: 'kph 10/25/2008 01:34'!
isPartOfPackage: aPackageInfo or: orphanage

	| myOrphanedClassDefiniton |
	 
	(aPackageInfo isYourClassExtension: self category) ifTrue: [ ^ true ].
	
	myOrphanedClassDefiniton := orphanage orphans
				detect: [ :c | c isClassDefinition and: [ c className = self className ] ] ifNone: [ ^ false ].
	
	^ aPackageInfo includesSystemCategory: myOrphanedClassDefiniton category
	
	! !

!MCMethodDefinition methodsFor: 'printing' stamp: 'kph 5/24/2007 01:10'!
isTraitMethod

	"TODO support traits properly"
	^ false
			 ! !

!MCMethodDefinition methodsFor: 'testing' stamp: 'kph 1/18/2008 10:30'!
isUnloader
	^ false! !

!MCMethodDefinition methodsFor: 'obsolete' stamp: 'kph 5/24/2007 02:08'!
load

	"obsolete , only remaining so that old MC's can load this code"

	self actualClass
		compile: source
		classified: category
		withStamp: timeStamp
		notifying: (SyntaxError new category: category).
		
	(selector == #initialize and: [ self classIsMeta ]) 
		ifTrue: [
			self actualClass soleInstance initialize.	
		].		
	! !

!MCMethodDefinition methodsFor: 'compiling' stamp: 'kph 5/11/2007 16:49'!
logSource
	 ^ true! !

!MCMethodDefinition methodsFor: 'installing' stamp: 'kph 10/24/2007 14:40'!
notifyObservers
	SystemChangeNotifier uniqueInstance 
		doSilently: [self actualClass organization classify: selector under: category].
	priorMethodOrNil isNil
		ifTrue: [SystemChangeNotifier uniqueInstance methodAdded: compiledMethod selector: selector inProtocol: category class: self actualClass requestor: requestor]
		ifFalse: [SystemChangeNotifier uniqueInstance methodChangedFrom: priorMethodOrNil compiledMethod to: compiledMethod selector: selector inClass: self actualClass requestor: requestor].
	"The following code doesn't seem to do anything."
	self actualClass instanceSide noteCompilationOf: compiledSelector meta: self actualClass isClassSide.
! !

!MCMethodDefinition methodsFor: 'serializing' stamp: 'kph 8/2/2007 14:55'!
objectForDataStream: refStrm
    "nil out any compiled methods"
	methodAndNode := requestor := compiledMethod := priorMethodCategory := priorMethodOrNil := compiledSelector := isInstalled := theClass := defnToInstall :=  nil.
	
    ^ self! !

!MCMethodDefinition methodsFor: 'installing' stamp: 'mtf 6/10/2008 19:25'!
postloadOver: obs

	compiledMethod ifNil: [ 	
	  self ensuredCreateCompiledMethod.
	  self installMethod
	].

	defnToInstall ifNotNil: [ defnToInstall notifyObservers ].! !

!MCMethodDefinition methodsFor: 'installing' stamp: 'kph 10/24/2007 14:32'!
preloadOver: obs
	| |
 	"we trap errors the first time and retry later"
	super preloadOver: obs.	
		
	(obs notNil & self actualClass notNil) ifTrue: [ 
			theClass := self actualClass.		
		
			priorMethodOrNil :=  self priorMethod.			
			priorMethodCategory := theClass organization categoryOfElement: selector  .

			theClass removeSelector: obs selector 		
		 ].
				
	self protectedCreateCompiledMethod.
	
 ! !

!MCMethodDefinition methodsFor: 'annotations' stamp: 'nk 7/24/2003 16:06'!
printAnnotations: requests on: aStream
	"Add a string for an annotation pane, trying to fulfill the annotation requests.
	These might include anything that
		Preferences defaultAnnotationRequests 
	might return. Which includes anything in
		Preferences annotationInfo
	To edit these, use:"
	"Preferences editAnnotations"

	requests do: [ :aRequest |
		aRequest == #timeStamp ifTrue: [ aStream nextPutAll: self timeStamp ].
		aRequest == #messageCategory ifTrue: [ aStream nextPutAll: self category ].
		aRequest == #requirements ifTrue: [
			self requirements do: [ :req |
				aStream nextPutAll: req ] separatedBy: [ aStream space ]].
	] separatedBy: [ aStream space ].! !

!MCMethodDefinition methodsFor: 'installing' stamp: 'kph 10/24/2007 14:28'!
priorMethod

	| comp |
	
	comp :=	theClass compiledMethodAt: selector ifAbsent: [ ^nil ].
		
	^ (comp methodReference asMethodDefinition)
		compiledMethod: comp;
		yourself! !

!MCMethodDefinition methodsFor: 'compiling' stamp: 'kph 5/23/2007 00:55'!
protectedCreateCompiledMethod

	[ self createCompiledMethod ] on: Error, SyntaxErrorNotification do: [ :ex | ].
 ! !

!MCMethodDefinition methodsFor: 'comparing' stamp: 'kph 4/14/2008 14:29'!
readDataFrom: aDataStream size: varsOnDisk

	| read leClass |
	
	read := super readDataFrom: aDataStream size: varsOnDisk.
	
	read classIsMeta ifFalse: [ ^ read ].
	
	leClass := self class classForSelector: read selector meta: true.

	^ leClass new copyFrom: read	 ! !

!MCMethodDefinition methodsFor: 'accessing' stamp: 'kph 5/16/2007 05:13'!
requestor

	^ requestor! !

!MCMethodDefinition methodsFor: 'comparing' stamp: 'ab 5/24/2003 14:11'!
requirements
	^ Array with: className! !

!MCMethodDefinition methodsFor: 'installing' stamp: 'stephaneducasse 2/4/2006 20:47'!
scanForPreviousVersion
	| position prevPos prevFileIndex preamble tokens sourceFilesCopy stamp method file methodCategory |
	method := self actualClass compiledMethodAt: selector ifAbsent: [^ nil].
	position := method filePosition.
	sourceFilesCopy := SourceFiles collect:
		[:x | x isNil ifTrue: [ nil ]
				ifFalse: [x readOnlyCopy]].
	[method fileIndex == 0 ifTrue: [^ nil].
	file := sourceFilesCopy at: method fileIndex.
	[position notNil & file notNil]
		whileTrue:
		[file position: (0 max: position-150).  "Skip back to before the preamble"
		[file position < (position-1)]  "then pick it up from the front"
			whileTrue: [preamble := file nextChunk].

		"Preamble is likely a linked method preamble, if we're in
			a changes file (not the sources file).  Try to parse it
			for prior source position and file index"
		prevPos := nil.
		stamp := ''.
		(preamble findString: 'methodsFor:' startingAt: 1) > 0
			ifTrue: [tokens := Scanner new scanTokens: preamble]
			ifFalse: [tokens := Array new  "ie cant be back ref"].
		((tokens size between: 7 and: 8)
			and: [(tokens at: tokens size-5) = #methodsFor:])
			ifTrue:
				[(tokens at: tokens size-3) = #stamp:
				ifTrue: ["New format gives change stamp and unified prior pointer"
						stamp := tokens at: tokens size-2.
						prevPos := tokens last.
						prevFileIndex := sourceFilesCopy fileIndexFromSourcePointer: prevPos.
						prevPos := sourceFilesCopy filePositionFromSourcePointer: prevPos]
				ifFalse: ["Old format gives no stamp; prior pointer in two parts"
						prevPos := tokens at: tokens size-2.
						prevFileIndex := tokens last].
				(prevPos = 0 or: [prevFileIndex = 0]) ifTrue: [prevPos := nil]].
		((tokens size between: 5 and: 6)
			and: [(tokens at: tokens size-3) = #methodsFor:])
			ifTrue:
				[(tokens at: tokens size-1) = #stamp:
				ifTrue: ["New format gives change stamp and unified prior pointer"
						stamp := tokens at: tokens size]].
		methodCategory := tokens after: #methodsFor: ifAbsent: ['as yet unclassifed'].
		methodCategory = category ifFalse:
			[methodCategory = (Smalltalk 
									at: #Categorizer 
									ifAbsent: [Smalltalk at: #ClassOrganizer]) 
										default ifTrue: [methodCategory := methodCategory, ' '].
			^ ChangeRecord new file: file position: position type: #method
						class: className category: methodCategory meta: classIsMeta stamp: stamp].
		position := prevPos.
		prevPos notNil ifTrue:
			[file := sourceFilesCopy at: prevFileIndex]].
		^ nil]
			ensure: [sourceFilesCopy do: [:x | x notNil ifTrue: [x close]]]
	! !

!MCMethodDefinition methodsFor: 'accessing' stamp: 'ab 11/15/2002 01:11'!
selector
	^selector! !

!MCMethodDefinition methodsFor: 'comparing' stamp: 'kph 5/10/2007 04:55'!
sortKey
	^ self className, '.', (self classIsMeta ifTrue: ['meta'] ifFalse: ['nonmeta']), '.', self selector, '.' , self category! !

!MCMethodDefinition methodsFor: 'accessing' stamp: 'ab 11/13/2002 01:59'!
source
	^ source! !

!MCMethodDefinition methodsFor: 'visiting' stamp: 'kph 6/19/2008 17:33'!
storeDataOn: aDataStream

	"we only ignore trailing instVars that are nil"
 
	| lastNotNil |

	self propertyRemoveKey: #because.

	lastNotNil := (self storeMaxInstVars to: 1 by: -1) detect: [ :n |  (self instVarAt: n) notNil ].
	 
	aDataStream
		beginInstance: self class storeAsClassForMC1
		size: lastNotNil.
	1 to: lastNotNil do:
		[:i | aDataStream nextPut: (self instVarAt: i)].


! !

!MCMethodDefinition methodsFor: 'serializing' stamp: 'kph 6/5/2008 02:43'!
storeMaxInstVars

^ 9! !

!MCMethodDefinition methodsFor: 'printing' stamp: 'kph 8/2/2007 12:08'!
summary
	^ self fullClassName , '>>' , selector, '{',category,'}' ! !

!MCMethodDefinition methodsFor: 'accessing' stamp: 'ab 1/15/2003 13:42'!
timeStamp
	^ timeStamp! !

!MCMethodDefinition methodsFor: 'installing' stamp: 'kph 5/10/2007 21:15'!
unload
	| previousVersion |
 	
	self isOverrideMethod ifTrue: [previousVersion := self scanForPreviousVersion].
	previousVersion
		ifNil: [self actualClass ifNotNilDo: [:class | class removeSelector: selector]]
		ifNotNil: [previousVersion fileIn] ! !

!MCMethodDefinition methodsFor: 'compiling' stamp: 'kph 8/6/2007 15:10'!
writeSourceToLog
	self logSource ifTrue: [
		self actualClass logMethodSource: self source forMethodWithNode: methodAndNode 
			inCategory: self category withStamp: timeStamp notifying: self requestor.
	].
! !

MCMethodDefinition subclass: #MCMethodExternalFieldDefinition
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Modeling'!

!MCMethodExternalFieldDefinition methodsFor: 'as yet unclassified' stamp: 'kph 5/23/2007 00:58'!
postinstall 

	super postinstall.
	
	(Smalltalk at: #ExternalStructure ifPresent: [:externalStructure |
			self actualClass theNonMetaClass inheritsFrom: externalStructure]) == true
		
		 ifTrue: [ self actualClass theNonMetaClass compileFields ]
			
	! !

MCMethodDefinition subclass: #MCMethodInitializerDefinition
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Modeling'!

!MCMethodInitializerDefinition commentStamp: 'kph 1/16/2008 09:39' prior: 0!
MCMethodInitializerDefinition 

	is loaded as a normal method.
	at #postloadOver:, it runs itself.
	!

!MCMethodInitializerDefinition methodsFor: 'as yet unclassified' stamp: 'kph 5/26/2007 04:51'!
edLoad: editor

	super edLoad: editor.
	
	^ true
	
	 ! !

!MCMethodInitializerDefinition methodsFor: 'as yet unclassified' stamp: 'kph 5/26/2007 04:57'!
edPostload
	
	super edPostload.
	
	self actualClass isTrait ifTrue: [ ^ self ].
	 
	self actualClass theNonMetaClass perform: selector.! !

!MCMethodInitializerDefinition methodsFor: 'as yet unclassified' stamp: 'kph 5/15/2007 17:19'!
isInitializer
	
	^ true! !

!MCMethodInitializerDefinition methodsFor: 'as yet unclassified' stamp: 'kph 1/16/2008 09:38'!
postloadOver: old

	super postloadOver: old.
	
	self actualClass isTrait ifTrue: [ ^ self ].
	 
	self actualClass theNonMetaClass initialize.! !

!MCMethodInitializerDefinition methodsFor: '' stamp: 'kph 8/2/2007 01:10'!
sortKey
	^ '!!!!!!initialize'   ! !

MCMethodDefinition subclass: #MCMethodUnloaderDefinition
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Modeling'!

!MCMethodUnloaderDefinition methodsFor: 'as yet unclassified' stamp: 'kph 1/18/2008 10:31'!
isUnloader
	
	^ true! !

!MCMethodUnloaderDefinition methodsFor: 'as yet unclassified' stamp: 'kph 5/30/2008 02:53'!
sortKey
	^ 'unload'   ! !

!MCMethodUnloaderDefinition methodsFor: 'as yet unclassified' stamp: 'kph 1/18/2008 10:55'!
unload

	self actualClass soleInstance unload.
	super unload.! !

MCDefinition subclass: #MCOrganizationDefinition
	instanceVariableNames: 'categories properties'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Modeling'!

!MCOrganizationDefinition class methodsFor: 'as yet unclassified' stamp: 'ab 7/26/2003 02:06'!
categories: anArray
	^ self instanceLike: (self new categories: anArray)! !

!MCOrganizationDefinition methodsFor: 'as yet unclassified' stamp: 'ab 7/19/2003 18:25'!
= aDefinition
	^ (super = aDefinition)
		and: [categories = aDefinition categories]! !

!MCOrganizationDefinition methodsFor: 'as yet unclassified' stamp: 'ab 7/18/2003 21:47'!
accept: aVisitor
	^ aVisitor visitOrganizationDefinition: self! !

!MCOrganizationDefinition methodsFor: 'as yet unclassified' stamp: 'ab 5/24/2003 13:51'!
categories
	^ categories! !

!MCOrganizationDefinition methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
categories: anArray
	categories := anArray! !

!MCOrganizationDefinition methodsFor: 'as yet unclassified' stamp: 'kph 10/23/2008 23:00'!
commonPrefix
	| stream |
	categories isEmpty ifTrue: [^ ''].
	
	stream := String new writeStream.
	categories first keysAndValuesDo:
		[:i :c |
		categories do:
			[:ea |
			(ea at: i ifAbsent: []) = c ifFalse: [^ stream contents]].
		stream nextPut: c].
	^ stream contents! !

!MCOrganizationDefinition methodsFor: 'as yet unclassified' stamp: 'avi 9/28/2004 14:53'!
description
	^ Array with: #organization with: self commonPrefix! !

!MCOrganizationDefinition methodsFor: 'as yet unclassified' stamp: 'cwp 7/11/2003 01:33'!
isOrganizationDefinition
	^ true! !

!MCOrganizationDefinition methodsFor: 'as yet unclassified' stamp: 'avi 2/22/2004 13:46'!
postloadOver: oldDefinition
	SystemOrganization categories:
		(self
			reorderCategories: SystemOrganization categories
			original: (oldDefinition ifNil: [#()] ifNotNil: [oldDefinition categories]))! !

!MCOrganizationDefinition methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
reorderCategories: allCategories original: oldCategories
	| first |
	first := allCategories detect: [:ea | categories includes: ea] ifNone: [^ allCategories].
	^ 	((allCategories copyUpTo: first) copyWithoutAll: oldCategories, categories),
		categories,
		((allCategories copyAfter: first) copyWithoutAll: oldCategories, categories)
! !

!MCOrganizationDefinition methodsFor: 'as yet unclassified' stamp: 'ab 7/19/2003 18:01'!
sortKey
	^ '<organization>'! !

!MCOrganizationDefinition methodsFor: 'as yet unclassified' stamp: 'ab 7/22/2003 01:14'!
source
	^ String streamContents:
		[:s |
		categories do: [:ea | s nextPutAll: ea] separatedBy: [s cr]]! !

!MCOrganizationDefinition methodsFor: 'as yet unclassified' stamp: 'ab 5/24/2003 13:55'!
summary
	^ categories asArray printString! !

MCDefinition subclass: #MCScriptDefinition
	instanceVariableNames: 'script packageName properties'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Modeling'!

MCScriptDefinition subclass: #MCPostscriptDefinition
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Modeling'!

!MCPostscriptDefinition class methodsFor: 'as yet unclassified' stamp: 'avi 2/28/2005 17:04'!
scriptSelector
	^ #postscript! !

!MCPostscriptDefinition methodsFor: 'as yet unclassified' stamp: 'kph 6/5/2008 22:58'!
postloadOver: obs
	self evaluate! !

!MCPostscriptDefinition methodsFor: 'as yet unclassified' stamp: 'avi 2/28/2005 17:16'!
sortKey
	^ 'zzz' "force to the end so it gets loaded late"! !

MCScriptDefinition subclass: #MCPreambleDefinition
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Modeling'!

!MCPreambleDefinition class methodsFor: 'as yet unclassified' stamp: 'avi 2/28/2005 17:04'!
scriptSelector
	^ #preamble! !

!MCPreambleDefinition methodsFor: 'as yet unclassified' stamp: 'avi 2/28/2005 17:15'!
load
	super load.
	self evaluate! !

MCScriptDefinition subclass: #MCRemovalPostscriptDefinition
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Modeling'!

!MCRemovalPostscriptDefinition class methodsFor: 'as yet unclassified' stamp: 'avi 2/28/2005 17:04'!
scriptSelector
	^ #postscriptOfRemoval ! !

!MCRemovalPostscriptDefinition methodsFor: 'as yet unclassified' stamp: 'avi 2/28/2005 17:15'!
unload
	super unload.
	self evaluate! !

MCScriptDefinition subclass: #MCRemovalPreambleDefinition
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Modeling'!

!MCRemovalPreambleDefinition class methodsFor: 'as yet unclassified' stamp: 'avi 2/28/2005 17:04'!
scriptSelector
	^ #preambleOfRemoval! !

!MCRemovalPreambleDefinition methodsFor: 'as yet unclassified' stamp: 'avi 2/28/2005 17:14'!
sortKey
	^ 'zzz' "force to the end so it gets unloaded early"! !

!MCRemovalPreambleDefinition methodsFor: 'as yet unclassified' stamp: 'avi 2/28/2005 17:15'!
unload
	super unload.
	self evaluate! !

!MCScriptDefinition class methodsFor: 'as yet unclassified' stamp: 'kph 8/3/2008 03:02'!
from: aPackageInfo
	^ self script: (aPackageInfo propertyAt: self scriptSelector) contents asString packageName: aPackageInfo name! !

!MCScriptDefinition class methodsFor: 'as yet unclassified' stamp: 'kph 8/3/2008 03:22'!
from: aPackageInfo addTo: definitions
	
	(aPackageInfo respondsTo: #propertyAt:) ifFalse: [ ^ self ].
	
	(aPackageInfo propertyAt: self scriptSelector)
		ifNotNilDo: [ :v | definitions add: (self from: v) ]! !

!MCScriptDefinition class methodsFor: 'as yet unclassified' stamp: 'avi 2/28/2005 16:59'!
script: aString packageName: packageString
	^ self instanceLike: (self new initializeWithScript: aString packageName: packageString)! !

!MCScriptDefinition class methodsFor: 'as yet unclassified' stamp: 'avi 2/28/2005 17:00'!
scriptSelector
	self subclassResponsibility! !

!MCScriptDefinition methodsFor: 'as yet unclassified' stamp: 'avi 2/28/2005 16:55'!
= aDefinition
	^ (super = aDefinition)
		and: [script = aDefinition script]! !

!MCScriptDefinition methodsFor: 'as yet unclassified' stamp: 'avi 2/28/2005 17:19'!
accept: aVisitor
	"do nothing for now - this means it won't appear in the .st file"! !

!MCScriptDefinition methodsFor: 'as yet unclassified' stamp: 'avi 2/28/2005 17:12'!
description
	^ Array with: packageName with: self scriptSelector! !

!MCScriptDefinition methodsFor: 'as yet unclassified' stamp: 'avi 2/28/2005 17:03'!
evaluate
	Compiler evaluate: script! !

!MCScriptDefinition methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
initializeWithScript: aString packageName: packageString
	script := aString.
	packageName := packageString! !

!MCScriptDefinition methodsFor: 'as yet unclassified' stamp: 'avi 2/28/2005 17:11'!
installScript
	self installScript: script! !

!MCScriptDefinition methodsFor: 'as yet unclassified' stamp: 'bf 10/25/2005 18:55'!
installScript: aString
	| sel pi |
	sel := (self scriptSelector, ':') asSymbol.
	pi := self packageInfo.
	(pi respondsTo: sel)
		ifTrue: [pi perform: sel with: aString]! !

!MCScriptDefinition methodsFor: 'as yet unclassified' stamp: 'avi 2/28/2005 17:12'!
load
	self installScript! !

!MCScriptDefinition methodsFor: 'as yet unclassified' stamp: 'avi 2/28/2005 17:22'!
packageInfo
	^ PackageInfo named: packageName! !

!MCScriptDefinition methodsFor: '' stamp: 'kph 4/14/2008 13:18'!
preloadOver: aDefinition
 
	super preloadOver: aDefinition.
	self load! !

!MCScriptDefinition methodsFor: 'as yet unclassified' stamp: 'avi 2/28/2005 16:54'!
script
	^ script! !

!MCScriptDefinition methodsFor: 'as yet unclassified' stamp: 'avi 2/28/2005 17:10'!
scriptSelector
	^ self class scriptSelector! !

!MCScriptDefinition methodsFor: 'as yet unclassified' stamp: 'bf 10/25/2005 19:05'!
sortKey
	^ '!!', self scriptSelector "force to the front so it gets loaded first"! !

!MCScriptDefinition methodsFor: 'as yet unclassified' stamp: 'avi 2/28/2005 17:04'!
source
	^ script! !

!MCScriptDefinition methodsFor: 'as yet unclassified' stamp: 'avi 2/28/2005 17:12'!
summary
	^ packageName, ' ', self scriptSelector! !

!MCScriptDefinition methodsFor: 'as yet unclassified' stamp: 'avi 2/28/2005 17:12'!
unload
	self installScript: nil! !

MCDefinition subclass: #MCUnlinkedClassDefinition
	instanceVariableNames: 'name superclassName properties'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Orphanage'!

!MCUnlinkedClassDefinition commentStamp: 'kph 5/11/2007 04:32' prior: 0!
When a class has its super class removed, this definition is placed in the orphanage with the responsibility of relinking the class with its superclass if it is loaded.!

!MCUnlinkedClassDefinition class methodsFor: 'as yet unclassified' stamp: 'kph 5/11/2007 04:56'!
class: unlinkedClass
 
	^ self new class: unlinkedClass name superclass: unlinkedClass superclass name
	! !

!MCUnlinkedClassDefinition methodsFor: 'as yet unclassified' stamp: 'kph 5/11/2007 04:27'!
actualClass

	^Smalltalk classNamed: self className! !

!MCUnlinkedClassDefinition methodsFor: 'as yet unclassified' stamp: 'kph 5/11/2007 04:26'!
category

	^ self actualClass category ifNil: [ '' ]! !

!MCUnlinkedClassDefinition methodsFor: 'as yet unclassified' stamp: 'kph 5/11/2007 04:05'!
class: className superclass: scName

	name :=  className.
	superclassName := scName! !

!MCUnlinkedClassDefinition methodsFor: 'as yet unclassified' stamp: 'kph 5/11/2007 04:27'!
className

	^ name! !

!MCUnlinkedClassDefinition methodsFor: 'as yet unclassified' stamp: 'kph 5/11/2007 04:24'!
description
	^ Array with: name
! !

!MCUnlinkedClassDefinition methodsFor: 'as yet unclassified' stamp: 'kph 5/11/2007 04:26'!
hash
	| hash |
	
	hash := String stringHash: name initialHash: 0.
	hash := String stringHash: superclassName initialHash: hash.
	 
	hash := String stringHash: self category initialHash: hash.
	 
	^ hash! !

!MCUnlinkedClassDefinition methodsFor: 'as yet unclassified' stamp: 'kph 5/11/2007 04:33'!
isClassDefinitionExtension
	"Answer true if this definition extends the regular class definition"
	^true! !

!MCUnlinkedClassDefinition methodsFor: 'as yet unclassified' stamp: 'kph 5/15/2007 01:12'!
isPartOfPackage: aPackageInfo or: orphanage

	^ aPackageInfo includesSystemCategory: self category! !

!MCUnlinkedClassDefinition methodsFor: 'as yet unclassified' stamp: 'kph 5/11/2007 13:22'!
isUnlinkedClassDefinition
	 
	^true! !

!MCUnlinkedClassDefinition methodsFor: 'as yet unclassified' stamp: 'kph 5/11/2007 04:35'!
kindOfSubclass

	^ ' subclass: '! !

!MCUnlinkedClassDefinition methodsFor: 'as yet unclassified' stamp: 'kph 5/11/2008 01:23'!
preloadOver: obs

	super preloadOver: obs.
	self actualClass superclass: ((Smalltalk classNamed: self superclassName) ifNil: [ self error: 'not ready yet']).
	self actualClass superclass addSubclass: self actualClass.

	self actualClass class superclass: ((Smalltalk classNamed: self superclassName) class ifNil: [ self error: 'not ready yet']).
	
	self actualClass class superclass addSubclass: self actualClass class.
	
	(SystemNavigation default obsoleteBehaviors	detect: [ :obsClass | 
			
			([ obsClass nonObsoleteClass ] ifError: [ nil ]) = self actualClass ] ifNone: [ ^self ])
	in: [ :obsClass | 
		| metaClass |
		metaClass := obsClass metaclass.
		self actualClass updateInstancesFrom: obsClass. 
		obsClass becomeForward: self actualClass.
		metaClass becomeForward: self actualClass class ].
	
(SystemNavigation default obsoleteBehaviors select: [ :c | c isMeta not ])
	do: [ :obsClass | 
		| metaClass |
		metaClass := obsClass metaclass.
		self actualClass updateInstancesFrom: obsClass. 
		obsClass becomeForward: self actualClass.
		metaClass becomeForward: self actualClass class ]
	

	 
! !

!MCUnlinkedClassDefinition methodsFor: 'as yet unclassified' stamp: 'kph 5/22/2007 01:52'!
printDefinitionAndCommentOn: stream
		self printDefinitionOn: stream.

	  ! !

!MCUnlinkedClassDefinition methodsFor: 'as yet unclassified' stamp: 'kph 5/11/2007 04:40'!
printDefinitionOn: stream
		stream 
			nextPutAll: '"Class has an obsolete superclass, will relink..."';
			cr; cr;
			nextPutAll: self superclassName;
			nextPutAll: self kindOfSubclass;
			nextPut: $# ;
			nextPutAll: self className
			 ! !

!MCUnlinkedClassDefinition methodsFor: 'as yet unclassified' stamp: 'kph 5/11/2007 04:55'!
requirements
	^ (Array with: superclassName) ! !

!MCUnlinkedClassDefinition methodsFor: 'as yet unclassified' stamp: 'kph 5/11/2007 04:27'!
sortKey
	^ self className! !

!MCUnlinkedClassDefinition methodsFor: 'as yet unclassified' stamp: 'kph 5/11/2007 04:29'!
summary
	^ name! !

!MCUnlinkedClassDefinition methodsFor: 'as yet unclassified' stamp: 'kph 5/11/2007 04:37'!
superclassName

	^ superclassName! !

Object subclass: #MCDefinitionIndex
	instanceVariableNames: 'definitions'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Patching'!

!MCDefinitionIndex class methodsFor: 'as yet unclassified' stamp: 'ab 6/2/2003 01:29'!
definitions: aCollection
	^ self new addAll: aCollection! !

!MCDefinitionIndex methodsFor: 'as yet unclassified' stamp: 'ab 6/2/2003 00:38'!
add: aDefinition
	definitions at: aDefinition description put: aDefinition! !

!MCDefinitionIndex methodsFor: 'as yet unclassified' stamp: 'ab 6/2/2003 00:38'!
addAll: aCollection
	aCollection do: [:ea | self add: ea]! !

!MCDefinitionIndex methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
definitionLike: aDefinition ifPresent: foundBlock ifAbsent: errorBlock
	| definition |
	definition := definitions at: aDefinition description ifAbsent: [].
	^ definition
		ifNil: errorBlock
		ifNotNil: [foundBlock value: definition]! !

!MCDefinitionIndex methodsFor: 'as yet unclassified' stamp: 'ab 6/2/2003 00:42'!
definitions
	^ definitions values! !

!MCDefinitionIndex methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
initialize
	definitions := Dictionary new! !

!MCDefinitionIndex methodsFor: 'as yet unclassified' stamp: 'ab 6/2/2003 00:40'!
remove: aDefinition
	definitions removeKey: aDefinition description ifAbsent: []! !

MCDefinitionIndex subclass: #MCImageBackedDefinitionIndex
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Patching'!

!MCImageBackedDefinitionIndex commentStamp: '<historical>' prior: 0!
When using definitionLike: ... I go to the image before giving up!

!MCImageBackedDefinitionIndex methodsFor: 'as yet unclassified' stamp: 'mtf 10/7/2008 13:43'!
definitionLike: aDefinition ifPresent: foundBlock ifAbsent: errorBlock
	| definition |
	^ super definitionLike: aDefinition ifPresent: foundBlock ifAbsent: [
		definition := aDefinition fromImage.
		definition
			ifNil: [errorBlock value]
			ifNotNil: [foundBlock value: definition]]! !

Object subclass: #MCDependencySorter
	instanceVariableNames: 'required provided orderedItems'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Loading'!

!MCDependencySorter commentStamp: 'kph 5/3/2007 18:46' prior: 0!
MCDependencySorter

   * #orderedItems     has an OrderedCollection of the item Definitions that have
                   all of their requirements when added.
   * #required         A dictionary of Sets, keyed on the 'required' item, the Set
                   containing the items which require that item
   * #provided         A set of provided items, i.e. those which might be required.

As each item is added using #add, or #addAll, it obtains the items requirements, subtracting those items which it already has listed as #provided. Items with all of their requirements are added to the #orderedItems. In doing so each of those items is added to the #provided set. When adding to the provided set, a check is made in the #required items to see if the newly provided item is required. If so, it is cleared out of the required list and those items that were waiting for that requirement are then added via #add:/#addAll:

Finally you have:

   * #orderedItems                         - the collection of items which should load.
   * #itemsWithMissingRequirements     - the collection of items that will not yet load.
   * #externalRequirements             - the list of requirements still needed after
                                         all the internal provisions are accounted for.


Typical use (e.g. MCPackageLoader-#analyse) is to instantiate via #items, which adds all of the items and sorts them as they are added. Then calling #addExternalProvisions: with the set of class keys avalable in the image. This gives the ready to load result.

The #sortItems: instanciation method sorts the items as above, it assumes that all of the #externalProvisions are available so as to get the complete sorted order. This is used when writing the definitions.
!

!MCDependencySorter class methodsFor: 'as yet unclassified' stamp: 'ab 5/23/2003 14:17'!
items: aCollection
	^ self new addAll: aCollection! !

!MCDependencySorter class methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
sortItems: aCollection
	| sorter |
	sorter := self items: aCollection.
	sorter externalRequirements do: [:req  | sorter addProvision: req].
	^ sorter orderedItems.! !

!MCDependencySorter methodsFor: 'building' stamp: 'stephaneducasse 2/4/2006 20:47'!
add: anItem
	| requirements |
	requirements := self unresolvedRequirementsFor: anItem.
	requirements isEmpty
		ifTrue: [self addToOrder: anItem]
		ifFalse: [self addRequirements: requirements for: anItem]! !

!MCDependencySorter methodsFor: 'building' stamp: 'bf 11/12/2004 14:50'!
addAll: aCollection
	aCollection asArray sort do: [:ea | self add: ea]! !

!MCDependencySorter methodsFor: 'building' stamp: 'avi 10/7/2004 22:47'!
addExternalProvisions: aCollection
	(aCollection intersection: self externalRequirements)
		do: [:ea | self addProvision: ea]! !

!MCDependencySorter methodsFor: 'private' stamp: 'stephaneducasse 2/4/2006 20:47'!
addProvision: anObject
	| newlySatisfied |
	provided add: anObject.
	newlySatisfied := required removeKey: anObject ifAbsent: [#()].
	self addAll: newlySatisfied.! !

!MCDependencySorter methodsFor: 'private' stamp: 'ab 5/22/2003 23:24'!
addRequirement: reqObject for: itemObject
	(self itemsRequiring: reqObject) add: itemObject! !

!MCDependencySorter methodsFor: 'private' stamp: 'ab 5/22/2003 23:23'!
addRequirements: aCollection for: anObject
	aCollection do: [:ea | self addRequirement: ea for: anObject]! !

!MCDependencySorter methodsFor: 'private' stamp: 'ab 5/22/2003 23:15'!
addToOrder: anItem
	orderedItems add: anItem.
	anItem provisions do: [:ea | self addProvision: ea].! !

!MCDependencySorter methodsFor: 'accessing' stamp: 'dvf 9/8/2004 00:49'!
externalRequirements
	| unloaded providedByUnloaded |
	unloaded := self itemsWithMissingRequirements.
	providedByUnloaded := (unloaded gather: [:e | e provisions]) asSet.
	^ required keys reject: [:ea | providedByUnloaded includes: ea ]! !

!MCDependencySorter methodsFor: 'initialize-release' stamp: 'stephaneducasse 2/4/2006 20:47'!
initialize
	provided := Set new.
	required := Dictionary new.
	orderedItems := OrderedCollection new.! !

!MCDependencySorter methodsFor: 'private' stamp: 'ab 5/22/2003 23:24'!
itemsRequiring: anObject
	^ required at: anObject ifAbsentPut: [Set new]! !

!MCDependencySorter methodsFor: 'accessing' stamp: 'kph 5/15/2007 13:13'!
itemsWithMissingRequirements
	| items |
	items := Set new.
	required keysAndValuesDo: [ :ea :waiting | 
		waiting do: [ :eaWaiting | eaWaiting because: ' needs ', ea  ]. 
		items addAll: waiting
	].
	^ items
! !

!MCDependencySorter methodsFor: 'sorting' stamp: 'ab 5/22/2003 23:25'!
orderedItems
	^ orderedItems! !

!MCDependencySorter methodsFor: 'private' stamp: 'ab 5/22/2003 23:22'!
unresolvedRequirementsFor: anItem
	^ anItem requirements difference: provided! !

Object subclass: #MCDoItParser
	instanceVariableNames: 'source'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Chunk Format'!

MCDoItParser subclass: #MCClassTraitParser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Chunk Format'!

!MCClassTraitParser class methodsFor: 'as yet unclassified' stamp: 'al 10/9/2005 20:43'!
pattern
	^ '*classTrait*uses:*'! !

!MCClassTraitParser methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
addDefinitionsTo: aCollection
	| tokens  definition traitCompositionString |
	tokens := Scanner new scanTokens: source.
	traitCompositionString := ((ReadStream on: source)
		match: 'uses:';
		upToEnd) withBlanksTrimmed.
	definition := MCClassTraitDefinition
		baseTraitName: (tokens at: 1) 
		classTraitComposition: traitCompositionString.
	aCollection add: definition
! !

!MCDoItParser class methodsFor: 'as yet unclassified' stamp: 'avi 3/10/2004 12:29'!
concreteSubclasses
	^ self allSubclasses reject: [:c | c isAbstract]! !

!MCDoItParser class methodsFor: 'as yet unclassified' stamp: 'avi 3/10/2004 12:40'!
forDoit: aString
	^ (self subclassForDoit: aString) ifNotNilDo: [:c | c new source: aString]! !

!MCDoItParser class methodsFor: 'as yet unclassified' stamp: 'avi 3/10/2004 12:51'!
isAbstract
	^ self pattern isNil! !

!MCDoItParser class methodsFor: 'as yet unclassified' stamp: 'avi 3/10/2004 12:30'!
pattern
	^ nil! !

!MCDoItParser class methodsFor: 'as yet unclassified' stamp: 'avi 3/10/2004 12:30'!
subclassForDoit: aString
	^ self concreteSubclasses detect: [:ea | ea pattern match: aString] ifNone: []! !

!MCDoItParser methodsFor: 'as yet unclassified' stamp: 'avi 3/10/2004 12:40'!
addDefinitionsTo: aCollection
	self subclassResponsibility ! !

!MCDoItParser methodsFor: 'as yet unclassified' stamp: 'avi 3/10/2004 12:40'!
source
	^ source! !

!MCDoItParser methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
source: aString
	source := aString! !

MCDoItParser subclass: #MCSystemCategoryParser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Chunk Format'!

!MCSystemCategoryParser class methodsFor: 'as yet unclassified' stamp: 'avi 3/10/2004 12:41'!
pattern
	^ 'SystemOrganization*'! !

!MCSystemCategoryParser methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
addDefinitionsTo: aCollection
	| definition |
	definition := aCollection detect: [:ea | ea isOrganizationDefinition ] ifNone: [aCollection add: (MCOrganizationDefinition categories: #())].
	definition categories: (definition categories copyWith: self category).! !

!MCSystemCategoryParser methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
category
	| tokens  |
	tokens := Scanner new scanTokens: source.
	tokens size = 3 ifFalse: [self error: 'Unrecognized category definition'].
	^ tokens at: 3! !

MCDoItParser subclass: #MCTraitParser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Chunk Format'!

!MCTraitParser class methodsFor: 'as yet unclassified' stamp: 'al 10/9/2005 21:09'!
pattern
	^ 'Trait named:*'! !

!MCTraitParser methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
addDefinitionsTo: aCollection
	| tokens  definition traitCompositionString |
	tokens := Scanner new scanTokens: source.
	traitCompositionString := ((ReadStream on: source)
		match: 'uses:';
		upToAll: 'category:') withBlanksTrimmed.
	definition := MCTraitDefinition
		name: (tokens at: 3) 
		traitComposition: traitCompositionString
		category:  tokens last
		comment:  ''  
		commentStamp:   ''.
	aCollection add: definition.! !

Object subclass: #MCDocumentation
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Base'!

!MCDocumentation commentStamp: 'kph 5/3/2007 18:52' prior: 0!
Pulling MC enhancements together:

source.wiresong.ca/mv avi.276 !

!MCDocumentation methodsFor: 'as yet unclassified' stamp: 'kph 6/29/2007 00:37'!
releaseNotes! !

!MCDocumentation methodsFor: 'as yet unclassified' stamp: 'kph 6/29/2007 02:56'!
releaseNotes28062007

"
Clarified the parsing of the package name into its constituent parts, aiming to make things more resilient, and flexible.

also ensured more flexibility to version numbers

MonticelloTest-xxx.1.mcz - standard mc
MonticelloTest1.2.3-xxx.1.mcz - standard mc with dots in package name (must occur before first $-)
MonticelloTest-xxx.1.3.4b.mcz - traditional sw (note numberic #versionNumber only gives 1.3)
MonticelloTest-xxx.1bd9def2-25d1-11dc-909f-000a95edb42a.mcz - uuid version

"
! !

Object subclass: #MCFrontier
	instanceVariableNames: 'frontier bag'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Versioning'!

!MCFrontier class methodsFor: 'instance creation' stamp: 'avi 9/17/2005 22:07'!
frontierOn: aVersionInfo
	^ self frontierOnAll: (Array with: aVersionInfo)! !

!MCFrontier class methodsFor: 'instance creation' stamp: 'avi 9/17/2005 22:07'!
frontierOn: aVersionInfo and: otherVersionInfo
	^ self frontierOnAll: (Array with: aVersionInfo with: otherVersionInfo)! !

!MCFrontier class methodsFor: 'instance creation' stamp: 'stephaneducasse 2/4/2006 20:47'!
frontierOnAll: aCollection
	| remaining  allVersions |
	remaining := Bag new.
	allVersions := (aCollection gather: [:ea | ea withBreadthFirstAncestors]) asSet.
	allVersions do: [:ea | remaining addAll: ea ancestors].
	^self new frontier: aCollection bag: remaining! !

!MCFrontier methodsFor: 'accessing' stamp: 'avi 9/17/2005 22:02'!
frontier
	^frontier! !

!MCFrontier methodsFor: 'initialization' stamp: 'avi 9/17/2005 22:11'!
frontier: f bag: remaining
	frontier := f asOrderedCollection.
	bag := remaining! !

!MCFrontier methodsFor: 'advancing' stamp: 'avi 9/17/2005 22:13'!
remove: aVersionInfo
	frontier remove: aVersionInfo.
	aVersionInfo ancestors  do:
		[ :ancestor |
			bag remove: ancestor.
			(bag occurrencesOf: ancestor) = 0
				ifTrue: [frontier add: ancestor]].
	^aVersionInfo! !

!MCFrontier methodsFor: 'advancing' stamp: 'avi 9/17/2005 22:02'!
removeAll: collection
	collection do: [ :n | self remove: n]! !

Object subclass: #MCMergeRecord
	instanceVariableNames: 'version packageSnapshot ancestorInfo ancestor ancestorSnapshot imagePatch mergePatch'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Versioning'!

!MCMergeRecord class methodsFor: 'as yet unclassified' stamp: 'abc 2/13/2004 15:52'!
version: aVersion
	^ self basicNew initializeWithVersion: aVersion! !

!MCMergeRecord methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
ancestorInfo
	^ ancestorInfo ifNil: [ancestorInfo := version info commonAncestorWith: version workingCopy ancestry]! !

!MCMergeRecord methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
ancestorSnapshot
	^ ancestorSnapshot ifNil: [ancestorSnapshot := version workingCopy findSnapshotWithVersionInfo: self ancestorInfo]! !

!MCMergeRecord methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
imageIsClean
	| ancestors |
	ancestors := version workingCopy ancestors.
	^ ancestors size = 1
		and: [ancestors first = self ancestorInfo]	
		and: [self imagePatch isEmpty]! !

!MCMergeRecord methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
imagePatch
	^ imagePatch ifNil: [imagePatch := self packageSnapshot patchRelativeToBase: self ancestorSnapshot]! !

!MCMergeRecord methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
initializeWithVersion: aVersion
	version := aVersion! !

!MCMergeRecord methodsFor: 'as yet unclassified' stamp: 'abc 2/13/2004 17:14'!
isAncestorMerge
	^ version workingCopy ancestry hasAncestor: version info! !

!MCMergeRecord methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
mergePatch
	^ mergePatch ifNil: [mergePatch := version snapshot patchRelativeToBase: self ancestorSnapshot]! !

!MCMergeRecord methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
packageSnapshot
	^ packageSnapshot ifNil: [packageSnapshot := version package snapshot]! !

!MCMergeRecord methodsFor: 'as yet unclassified' stamp: 'abc 2/13/2004 17:14'!
updateWorkingCopy
	self isAncestorMerge ifFalse:
		[self imageIsClean
			ifTrue: [version workingCopy loaded: version]
			ifFalse: [version workingCopy merged: version]]! !

!MCMergeRecord methodsFor: 'as yet unclassified' stamp: 'abc 2/13/2004 15:52'!
version
	^ version! !

Object subclass: #MCMerger
	instanceVariableNames: 'conflicts'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Merging'!

!MCMerger methodsFor: 'as yet unclassified' stamp: 'ab 6/2/2003 01:10'!
addConflictWithOperation: anOperation
	self conflicts add: (MCConflict operation: anOperation)! !

!MCMerger methodsFor: 'as yet unclassified' stamp: 'ab 6/2/2003 01:01'!
applyTo: anObject
	self isMerged ifFalse: [self error: 'You must resolve all the conflicts first'].
	conflicts do: [:ea | ea applyTo: anObject]! !

!MCMerger methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
conflicts
	^ conflicts ifNil: [conflicts := OrderedCollection new]! !

!MCMerger methodsFor: 'as yet unclassified' stamp: 'ab 6/5/2003 19:09'!
isMerged
	^ self conflicts allSatisfy: [:ea | ea isResolved]! !

!MCMerger methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
load
	| loader |
	loader := MCPackageLoader new.
	loader provisions addAll: self provisions.
	self applyTo: loader.
	loader load! !

!MCMerger methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
loadWithNameLike: baseName
	| loader |
	loader := MCPackageLoader new.
	loader provisions addAll: self provisions.
	self applyTo: loader.
	loader loadWithNameLike: baseName! !

!MCMerger methodsFor: 'as yet unclassified' stamp: 'ab 6/2/2003 01:11'!
mergedSnapshot
	^ MCPatcher apply: self to: self baseSnapshot! !

!MCMerger methodsFor: 'as yet unclassified' stamp: 'ab 7/18/2003 16:34'!
operations
	^ #()! !

!MCMerger methodsFor: 'as yet unclassified' stamp: 'avi 10/6/2004 15:19'!
provisions
	^ #()! !

MCMerger subclass: #MCThreeWayMerger
	instanceVariableNames: 'index operations provisions redundantAdds'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Merging'!

!MCThreeWayMerger class methodsFor: 'as yet unclassified' stamp: 'avi 2/13/2004 01:53'!
base: aSnapshot patch: aPatch
	aPatch isEmpty ifTrue: [MCNoChangesException signal].
	^ self new
		addBaseSnapshot: aSnapshot;
		applyPatch: aPatch;
		yourself
		! !

!MCThreeWayMerger class methodsFor: 'as yet unclassified' stamp: 'ab 6/2/2003 01:09'!
base: aSnapshot target: targetSnapshot ancestor: ancestorSnapshot
	^ self base: aSnapshot patch: (targetSnapshot patchRelativeToBase: ancestorSnapshot)! !

!MCThreeWayMerger methodsFor: 'as yet unclassified' stamp: 'avi 10/6/2004 15:18'!
addBaseSnapshot: aSnapshot
	aSnapshot definitions do:
		[:ea |
		index add: ea.
		provisions addAll: ea provisions]! !

!MCThreeWayMerger methodsFor: 'as yet unclassified' stamp: 'mtf 10/7/2008 14:40'!
addDefinition: aDefinition
	index
		definitionLike: aDefinition
		ifPresent: [:other |
			(self removalForDefinition: aDefinition)
				ifNotNilDo:
					[:op |
					self addOperation: (MCModification of: other to: aDefinition).
					self removeOperation: op.
					^ self].
			other = aDefinition
				ifFalse: [(aDefinition isMethodDefinition and: [aDefinition isOverrideMethod])
					ifTrue: [self addOperation: (MCModification of: other to: aDefinition)]
					ifFalse: [self addConflictWithOperation: (MCModification of: other to: aDefinition)]]
				ifTrue: [self redundantAdds add: aDefinition]]
		ifAbsent: [self addOperation: (MCAddition of: aDefinition)]! !

!MCThreeWayMerger methodsFor: 'as yet unclassified' stamp: 'ab 6/2/2003 01:30'!
addOperation: anOperation
	self operations add: anOperation! !

!MCThreeWayMerger methodsFor: 'as yet unclassified' stamp: 'avi 2/13/2004 01:52'!
applyPatch: aPatch
	aPatch applyTo: self! !

!MCThreeWayMerger methodsFor: 'as yet unclassified' stamp: 'ab 6/2/2003 01:32'!
applyTo: anObject
	super applyTo: anObject.
	self operations do: [:ea | ea applyTo: anObject]! !

!MCThreeWayMerger methodsFor: 'as yet unclassified' stamp: 'avi 2/13/2004 01:49'!
baseSnapshot
	^ (MCSnapshot fromDefinitions: index definitions)! !

!MCThreeWayMerger methodsFor: 'as yet unclassified' stamp: 'mtf 10/7/2008 14:12'!
initialize
	index := MCImageBackedDefinitionIndex new.
	provisions := Set new! !

!MCThreeWayMerger methodsFor: 'as yet unclassified' stamp: 'avi 9/19/2005 02:22'!
modificationConflictForDefinition: aDefinition
	^ conflicts ifNotNil:
		[conflicts detect:
			[:ea | (ea definition isRevisionOf: aDefinition) and:
				[ea operation isModification]] ifNone: []]! !

!MCThreeWayMerger methodsFor: 'as yet unclassified' stamp: 'ab 6/2/2003 01:34'!
modifyDefinition: baseDefinition to: targetDefinition
	index
		definitionLike: baseDefinition
		ifPresent: [:other | other = baseDefinition
								ifTrue: [self addOperation: (MCModification of:  baseDefinition to: targetDefinition)]
								ifFalse: [other = targetDefinition
											ifFalse: [self addConflictWithOperation:
														(MCModification of: other to: targetDefinition)]]]
		ifAbsent: [self addConflictWithOperation: (MCAddition of: targetDefinition)]! !

!MCThreeWayMerger methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
operations
	^ operations ifNil: [operations := OrderedCollection new]! !

!MCThreeWayMerger methodsFor: 'as yet unclassified' stamp: 'avi 10/6/2004 15:19'!
provisions
	^ provisions! !

!MCThreeWayMerger methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
redundantAdds
	^ redundantAdds ifNil: [redundantAdds := Set new]! !

!MCThreeWayMerger methodsFor: 'as yet unclassified' stamp: 'avi 9/19/2005 02:40'!
removalForDefinition: aDefinition
	^ operations ifNotNil:
		[operations
			detect: [:ea | (ea definition isRevisionOf: aDefinition) and: [ea isRemoval]]
			ifNone: []]! !

!MCThreeWayMerger methodsFor: 'as yet unclassified' stamp: 'avi 9/19/2005 02:40'!
removeConflict: aConflict
	conflicts remove: aConflict! !

!MCThreeWayMerger methodsFor: 'as yet unclassified' stamp: 'avi 9/19/2005 03:23'!
removeDefinition: aDefinition
	index
		definitionLike: aDefinition
		ifPresent: [:other | other = aDefinition
								ifTrue:
									[(self modificationConflictForDefinition: aDefinition)
										ifNotNilDo:
											[:c |
											self addOperation: c operation.
											self removeConflict: c.
											^ self]. 
									(self redundantAdds includes: aDefinition)
										ifFalse: [self addOperation: (MCRemoval of: aDefinition)]]
								ifFalse:
									[self addConflictWithOperation: (MCRemoval of: other)]]
		ifAbsent: []! !

!MCThreeWayMerger methodsFor: 'as yet unclassified' stamp: 'avi 9/19/2005 02:40'!
removeOperation: anOperation
	operations remove: anOperation! !

Object subclass: #MCPackage
	instanceVariableNames: 'name'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Base'!

MCPackage subclass: #MCOrphanage
	instanceVariableNames: 'orphans unlinkedCount'
	classVariableNames: 'Current'
	poolDictionaries: ''
	category: 'Monticello-Orphanage'!

!MCOrphanage commentStamp: 'kph 5/15/2007 01:16' prior: 0!
MCOrphanage. supports out-of-order loading of packages, and safe unloading.

This enables a package to 'extend', and 'override' methods in not-yet-loaded packages. Not yet actually loaded ('orphaned') extensions should also save with the package so as not to be lost.
 
For Detailed Specification: OrphanageTest1 class comment, available in the OrphanageTest-Test1 package. 

Hooks:

MCPackageLoader calls #loading: before loading a package, and #loadComplete afterwards.
Orphans are added to the package prior to loading and afterwards.

#onClassRemoved: theClass is called by the environment notification system. Orphaned class extension methods are mopped up into the orphanage, and each subclass is given a MCUnlinkedClassDefinition poised to await the reloading of that class. (there may be a better way, but it is a good idicator of any unlinked classes having obsolete superclasses)

Each definition that is loaded without error sends us #successfullyLoaded: so we can remove that defn from the orphanage.

Design decision: packageInfo collates its info via #orphansFor: aPackage, which use polyMorphism to call #isPartOfPackage or: orphanage , which doesnt use any packageInfo 'monticello-orphanage' methods which call orphansFor: (think infinite loop).
!

!MCOrphanage class methodsFor: 'as yet unclassified' stamp: 'kph 5/15/2007 20:10'!
current
 
	^ Current ifNil: [ self initializeTheOrphanage ].! !

!MCOrphanage class methodsFor: 'as yet unclassified' stamp: 'kph 9/2/2008 03:50'!
initializeTheOrphanage

	Current := self named: '.Orphanage'.
	 
	MCWorkingOrphanage forPackage: Current.
	
	^ Current! !

!MCOrphanage methodsFor: 'accessing' stamp: 'kph 5/9/2007 06:24'!
addOrphans: some
	some ifNil: [ ^self ].
	orphans addAll: some! !

!MCOrphanage methodsFor: 'ui actions' stamp: 'kph 5/10/2007 17:24'!
clearOrphans

	self initialize! !

!MCOrphanage methodsFor: 'initialize-release' stamp: 'kph 7/16/2008 16:16'!
initialize

 	orphans := Set new.
	unlinkedCount := nil.! !

!MCOrphanage methodsFor: 'events' stamp: 'kph 5/11/2007 13:26'!
loadComplete

	unlinkedCount := nil! !

!MCOrphanage methodsFor: 'loading' stamp: 'kph 5/10/2007 23:42'!
loading: aPackageLoader

	"when the package loader is about to do a load the orphanage applys ALL orphans"
	
	self orphans do: [ :orphanDefn | (MCAddition of: orphanDefn) applyTo: aPackageLoader ].
! !

!MCOrphanage methodsFor: 'events' stamp: 'kph 1/18/2008 10:10'!
onClassRemoved: theClass
	
	| cat |
	orphans := orphans reject: [ :defn | defn className = theClass name ].
			
	theClass subclasses do: [ :aSubclass | orphans add: (MCUnlinkedClassDefinition class: aSubclass) ].
 
	(Array with: theClass with: theClass classSide ) do: [ :behavior |
		behavior selectorsAndMethodsDo: [ :selector :method |
			cat := behavior organization categoryOfElement: selector.
			
			cat ifNotNil: [ 
				(cat beginsWith: '*')
					ifTrue: [ orphans add: method methodReference asMethodDefinition ]
			]
		]
	].

	unlinkedCount := nil.! !

!MCOrphanage methodsFor: 'accessing' stamp: 'kph 10/25/2008 01:34'!
orphanClasses

	^ self orphans asOrderedCollection select: [ :o | o isClassDefinition ].
	
 ! !

!MCOrphanage methodsFor: 'accessing' stamp: 'kph 5/15/2007 01:31'!
orphanMethods

	^ self orphans select: [ :o | o isMethodDefinition ].! !

!MCOrphanage methodsFor: 'accessing' stamp: 'kph 4/30/2007 00:48'!
orphans
	"Answer the value of orphans, an MCPackageLoader"

	^ orphans! !

!MCOrphanage methodsFor: 'accessing' stamp: 'kph 5/15/2007 01:13'!
orphansFor: packageInfo 

	^ self orphans select: [ :defn  | defn isPartOfPackage: packageInfo or: self  ]! !

!MCOrphanage methodsFor: 'events' stamp: 'kph 5/15/2007 16:39'!
packageUnloaded: aPackageInfo 

	"while an unlinked class exists the power to relink it remains"
	((self orphansFor: aPackageInfo) difference: self unlinkedClasses)
		do: [ :ea | ea removeFromOrphanage: self ]! !

!MCOrphanage methodsFor: 'accessing' stamp: 'kph 4/30/2007 01:21'!
snapshot
	
	^ MCSnapshot fromDefinitions: self orphans
! !

!MCOrphanage methodsFor: 'accessing' stamp: 'kph 4/30/2007 01:07'!
storeOn: aStream
	self halt. "when is this used?"
	aStream
		nextPutAll: self class name;
		space; nextPutAll: 'named: '; store: name.! !

!MCOrphanage methodsFor: 'events' stamp: 'kph 5/10/2007 17:54'!
successfullyLoaded: aDefinition

	self orphans remove: aDefinition ifAbsent: [ ^ self ].
	
 ! !

!MCOrphanage methodsFor: 'accessing' stamp: 'kph 5/15/2007 01:18'!
unlinkedClasses

     ^ orphans select: [ :orphan | orphan isUnlinkedClassDefinition ] ! !

!MCOrphanage methodsFor: 'accessing' stamp: 'kph 5/15/2007 01:18'!
unlinkedCount

	^ unlinkedCount ifNil: [ unlinkedCount := self unlinkedClasses size ] ! !

!MCPackage class methodsFor: 'as yet unclassified' stamp: 'ab 7/10/2003 01:17'!
named: aString
	^ self new name: aString! !

!MCPackage methodsFor: 'as yet unclassified' stamp: 'ar 4/26/2005 21:57'!
= other
	^ other species = self species and: [other name sameAs: name]! !

!MCPackage methodsFor: 'as yet unclassified' stamp: 'kph 8/3/2008 04:16'!
hasWorkingCopy
	MCWorkingCopy organizer packageNamed: self name ifAbsent: [ ^ false ].
	
	^ true! !

!MCPackage methodsFor: 'as yet unclassified' stamp: 'ar 4/26/2005 21:57'!
hash
	^ name asLowercase hash! !

!MCPackage methodsFor: 'as yet unclassified' stamp: 'ab 7/7/2003 00:57'!
name
	^ name! !

!MCPackage methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
name: aString
	name := aString! !

!MCPackage methodsFor: 'as yet unclassified' stamp: 'kph 5/10/2007 05:36'!
orphanage

	^ MCOrphanage current! !

!MCPackage methodsFor: 'as yet unclassified' stamp: 'kph 8/3/2008 03:26'!
packageInfo
	^ PackageInfo named: name ! !

!MCPackage methodsFor: 'patching' stamp: 'mtf 10/7/2008 14:04'!
patchSnapshot
	^ self snapshot asPatchSnapshot! !

!MCPackage methodsFor: 'as yet unclassified' stamp: 'nk 7/28/2003 13:30'!
printOn: aStream
	super printOn: aStream.
	aStream
		nextPut: $(;
		nextPutAll: name;
		nextPut: $)! !

!MCPackage methodsFor: 'as yet unclassified' stamp: 'mtf 10/3/2008 23:18'!
snapshot
	| definitions categories packageInfo |
 
	packageInfo := self packageInfo.
	definitions := OrderedCollection new.
	categories := packageInfo systemCategories.
	
	definitions addAll: (self orphanage orphansFor: packageInfo). 
	definitions removeAllFoundIn: self orphanage unlinkedClasses.
	   
	categories isEmpty ifFalse: [ definitions add: (MCOrganizationDefinition categories: categories) ].

	packageInfo
		classesDo: [:ea | definitions addAll: ea classDefinitions]
		methodsDo: [:ea | definitions add: ea asMethodDefinition]
		displayingProgress: 'Snapshotting...'.
	
	MCScriptDefinition subclassesDo: [ :ea | ea from: packageInfo addTo: definitions ].
 
	^ MCSnapshot fromDefinitions: definitions
! !

!MCPackage methodsFor: 'as yet unclassified' stamp: 'ab 7/10/2003 01:13'!
storeOn: aStream
	aStream
		nextPutAll: 'MCPackage';
		space; nextPutAll: 'named: '; store: name.! !

!MCPackage methodsFor: 'as yet unclassified' stamp: 'cwp 11/13/2003 13:32'!
unload
	^ self workingCopy unload! !

!MCPackage methodsFor: 'as yet unclassified' stamp: 'cwp 11/13/2003 13:33'!
workingCopy
	^ MCWorkingCopy forPackage: self.! !

Object subclass: #MCPackageCache
	instanceVariableNames: 'sorter fileNames'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Repositories'!

!MCPackageCache class methodsFor: 'as yet unclassified' stamp: 'avi 1/22/2004 18:12'!
new
	^ self basicNew initialize! !

!MCPackageCache methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
initialize
	sorter := MCVersionSorter new.
	fileNames := Dictionary new.! !

!MCPackageCache methodsFor: 'as yet unclassified' stamp: 'avi 1/22/2004 18:25'!
recordVersionInfo: aVersionInfo forFileNamed: aString
	Transcript cr; show: aString.
	fileNames at: aVersionInfo put: aString.
	sorter addVersionInfo: aVersionInfo! !

!MCPackageCache methodsFor: 'as yet unclassified' stamp: 'avi 1/22/2004 18:21'!
versionInfos
	^ sorter sortedVersionInfos ! !

Object subclass: #MCPackageLoader
	instanceVariableNames: 'requirements unloadableDefinitions obsoletions additions removals errorDefinitions provisions isMultiplePackage methodAdditions'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Loading'!

!MCPackageLoader commentStamp: 'kph 5/26/2007 04:22' prior: 0!
This loader is the old MC1 code essentially untouched, thoug this class is now obsolete.

This is kept here so that when loading this new version of mc into a system with an older version of mc it is running this code and so we do not wish to disturb it. 

In loading this version, the class #new method is updated to switch future instanciations to the new package loader, this old one being obsolete.

This loader remains potentially usable but does not know about the orphanage. #installOrphanage is implemented here as a unused noop, just in case someone wants to revert to using this loader.



!

!MCPackageLoader class methodsFor: 'as yet unclassified' stamp: 'mtf 10/8/2008 00:36'!
initialize
	"MCPackageLoader initialize"
	Preferences addPreference: #useMonticelloAtomicLoader categories: #(monticello) default: false balloonHelp:
	'Use the experimental atomic loader for Monticello. You must have SystemEditor for this to work'! !

!MCPackageLoader class methodsFor: 'as yet unclassified' stamp: 'kph 5/26/2007 01:18'!
new

	^ (self theChosenLoaderClass basicNew) initialize; yourself! !

!MCPackageLoader class methodsFor: 'as yet unclassified' stamp: 'mtf 6/17/2008 09:21'!
theChosenLoaderClass

	^ (Preferences useMonticelloAtomicLoader and: [Smalltalk includesKey: #SystemEditor])
		ifTrue: [MCPackageLoader2]
		ifFalse: [MCPackageLoader1b]! !

!MCPackageLoader methodsFor: 'patch ops' stamp: 'ab 5/24/2003 16:13'!
addDefinition: aDefinition
	additions add: aDefinition! !

!MCPackageLoader methodsFor: 'private' stamp: 'kph 6/12/2007 09:36'!
analyze
	| sorter |
	
	self isMultiplePackage ifTrue: [ self analyzeMulti ].
	
	sorter := self sorterForItems: additions.
	additions := sorter orderedItems.
	requirements := sorter externalRequirements.
	unloadableDefinitions := sorter itemsWithMissingRequirements asSortedCollection.
	
	sorter := self sorterForItems: removals.
	removals := sorter orderedItems reversed.! !

!MCPackageLoader methodsFor: 'private' stamp: 'kph 6/12/2007 09:35'!
analyzeMulti
	| index |
	index := MCDefinitionIndex definitions: additions.
	removals removeAllSuchThat: [:removal |
		(index definitionLike: removal
			ifPresent: [:addition | obsoletions at: addition put: removal]
			ifAbsent: []) notNil].! !

!MCPackageLoader methodsFor: 'private' stamp: 'stephaneducasse 2/4/2006 20:47'!
basicLoad
	errorDefinitions := OrderedCollection new.
	[[additions do: [:ea | self tryToLoad: ea] displayingProgress: 'Loading...'.
	removals do: [:ea | ea unload] displayingProgress: 'Cleaning up...'.
	self shouldWarnAboutErrors ifTrue: [self warnAboutErrors].
	errorDefinitions do: [:ea | ea loadOver: (self obsoletionFor: ea)] displayingProgress: 'Reloading...'.
	additions do: [:ea | ea postloadOver: (self obsoletionFor: ea)] displayingProgress: 'Initializing...']
		on: InMidstOfFileinNotification 
		do: [:n | n resume: true]]
			ensure: [self flushChangesFile]! !

!MCPackageLoader methodsFor: 'public' stamp: 'kph 6/12/2007 09:33'!
beMultiplePackage

	isMultiplePackage := true! !

!MCPackageLoader methodsFor: 'public' stamp: 'kph 5/26/2007 01:12'!
beUnloading

	 "stub"! !

!MCPackageLoader methodsFor: 'private' stamp: 'ab 5/25/2003 01:24'!
dependencyWarning
	^ String streamContents:
		[:s |
		s nextPutAll: 'This package depends on the following classes:'; cr.
		requirements do: [:ea | s space; space; nextPutAll: ea; cr].
		s nextPutAll: 'You must resolve these dependencies before you will be able to load these definitions: '; cr.
		unloadableDefinitions do: [:ea | s space; space; nextPutAll: ea summary; cr]] ! !

!MCPackageLoader methodsFor: 'private' stamp: 'avi 1/24/2004 17:44'!
errorDefinitionWarning
	^ String streamContents:
		[:s |
		s nextPutAll: 'The following definitions had errors while loading.  Press Proceed to try to load them again (they may work on a second pass):'; cr.
		errorDefinitions do: [:ea | s space; space; nextPutAll: ea summary; cr]] ! !

!MCPackageLoader methodsFor: 'private' stamp: 'kph 5/15/2007 12:53'!
errorDefinitionWarning: erroredDefns
	^ String streamContents:
		[:s |
		s nextPutAll: 'The following definitions had errors while loading.  Press Proceed to try to load them again (they may work on a second pass):'; cr.
		erroredDefns do: [:ea | 
			s space; space; 
			  nextPutAll: ea summary;
			  space; 
			  nextPutAll: (ea because ifNil: [ '' ]); cr]] ! !

!MCPackageLoader methodsFor: 'private' stamp: 'cwp 11/13/2003 02:01'!
flushChangesFile
	"The changes file is second in the SourceFiles array"

	(SourceFiles at: 2) flush! !

!MCPackageLoader methodsFor: 'private' stamp: 'kph 12/18/2007 01:02'!
initialize
	additions := OrderedCollection new.
	errorDefinitions := OrderedCollection new.
	removals := OrderedCollection new.
	obsoletions := Dictionary new.
! !

!MCPackageLoader methodsFor: 'private' stamp: 'kph 5/26/2007 00:58'!
installOrphanage

	 ! !

!MCPackageLoader methodsFor: 'public' stamp: 'stephaneducasse 2/4/2006 20:47'!
installSnapshot: aSnapshot
	| patch |
	patch := aSnapshot patchRelativeToBase: MCSnapshot empty.
	patch applyTo: self.
! !

!MCPackageLoader methodsFor: 'public' stamp: 'kph 6/12/2007 09:33'!
isMultiplePackage

	^ isMultiplePackage == true! !

!MCPackageLoader methodsFor: 'public' stamp: 'kph 5/15/2007 15:39'!
isUnloading 

	^ isUnloading ifNil: [ false ]! !

!MCPackageLoader methodsFor: 'public' stamp: 'ab 8/24/2003 01:03'!
load
	self analyze.
	unloadableDefinitions isEmpty ifFalse: [self warnAboutDependencies].
	self useNewChangeSetDuring: [self basicLoad]! !

!MCPackageLoader methodsFor: 'private' stamp: 'ul 6/8/2007 01:17'!
loadComplete! !

!MCPackageLoader methodsFor: 'public' stamp: 'nk 2/23/2005 07:51'!
loadWithName: baseName
	self analyze.
	unloadableDefinitions isEmpty ifFalse: [self warnAboutDependencies].
	self useChangeSetNamed: baseName during: [self basicLoad]! !

!MCPackageLoader methodsFor: 'public' stamp: 'nk 8/30/2004 08:39'!
loadWithNameLike: baseName
	self analyze.
	unloadableDefinitions isEmpty ifFalse: [self warnAboutDependencies].
	self useNewChangeSetNamedLike: baseName during: [self basicLoad]! !

!MCPackageLoader methodsFor: 'patch ops' stamp: 'avi 2/17/2004 13:14'!
modifyDefinition: old to: new
	self addDefinition: new.
	obsoletions at: new put: old.! !

!MCPackageLoader methodsFor: 'private' stamp: 'avi 2/17/2004 13:15'!
obsoletionFor: aDefinition
	^ obsoletions at: aDefinition ifAbsent: [nil]! !

!MCPackageLoader methodsFor: 'private' stamp: 'ab 5/25/2003 01:19'!
orderDefinitionsForLoading: aCollection
	^ (self sorterForItems: aCollection) orderedItems! !

!MCPackageLoader methodsFor: 'private' stamp: 'ab 5/24/2003 16:52'!
orderedAdditions
	^ additions! !

!MCPackageLoader methodsFor: 'private' stamp: 'stephaneducasse 2/4/2006 20:47'!
provisions
	^ provisions ifNil: [provisions := Set withAll: Smalltalk keys]! !

!MCPackageLoader methodsFor: 'patch ops' stamp: 'ab 5/24/2003 16:14'!
removeDefinition: aDefinition
	removals add: aDefinition! !

!MCPackageLoader methodsFor: 'private' stamp: 'avi 1/25/2004 13:32'!
shouldWarnAboutErrors
	^ errorDefinitions isEmpty not and: [false "should make this a preference"]! !

!MCPackageLoader methodsFor: 'private' stamp: 'stephaneducasse 2/4/2006 20:47'!
sorterForItems: aCollection
	| sorter |
	sorter := MCDependencySorter items: aCollection.
	sorter addExternalProvisions: self provisions.
	^ sorter! !

!MCPackageLoader methodsFor: 'private' stamp: 'ul 6/8/2007 01:16'!
successfullyLoaded: ea 

	"dummy"

	 ! !

!MCPackageLoader methodsFor: 'private' stamp: 'avi 2/17/2004 13:15'!
tryToLoad: aDefinition
	[aDefinition loadOver: (self obsoletionFor: aDefinition)] on: Error do: [errorDefinitions add: aDefinition].! !

!MCPackageLoader methodsFor: 'private' stamp: 'kph 5/26/2007 01:01'!
tryToLoad: aDefinition errors: errs

	"a method used in some MC1 versions > md.308+"

	[aDefinition loadOver: (self obsoletionFor: aDefinition)] on: Error do: [errs add: aDefinition].! !

!MCPackageLoader methodsFor: 'private' stamp: 'kph 5/26/2007 01:01'!
tryToUnload: aDefinition errors: errs

	"a method used in some MC1 versions > md.308+"

	[aDefinition unload ] on: Error do: [errs add: aDefinition].! !

!MCPackageLoader methodsFor: 'public' stamp: 'kph 5/26/2007 10:02'!
unloadPackage: aPackage
 
	(self 
		updatePackage: aPackage withSnapshot: MCSnapshot empty;
		beUnloading;
		loadWithNameLike: aPackage name, '-unload';
		yourself)
		orphanage packageUnloaded: aPackage packageInfo.	! !

!MCPackageLoader methodsFor: 'public' stamp: 'stephaneducasse 2/4/2006 20:47'!
updatePackage: aPackage withSnapshot: aSnapshot
	|  patch packageSnap |
	packageSnap := aPackage snapshot.
	patch := aSnapshot patchRelativeToBase: packageSnap.
	patch applyTo: self.
	packageSnap definitions do: [:ea | self provisions addAll: ea provisions]
! !

!MCPackageLoader methodsFor: 'private' stamp: 'stephaneducasse 2/4/2006 20:47'!
useChangeSetNamed: baseName during: aBlock
	"Use the named change set, or create one with the given name."
	| changeHolder oldChanges newChanges |
	changeHolder := (ChangeSet respondsTo: #newChanges:)
						ifTrue: [ChangeSet]
						ifFalse: [Smalltalk].
	oldChanges := (ChangeSet respondsTo: #current)
						ifTrue: [ChangeSet current]
						ifFalse: [Smalltalk changes].

	newChanges := (ChangeSorter changeSetNamed: baseName) ifNil: [ ChangeSet new name: baseName ].
	changeHolder newChanges: newChanges.
	[aBlock value] ensure: [changeHolder newChanges: oldChanges].
! !

!MCPackageLoader methodsFor: 'private' stamp: 'nk 8/30/2004 08:38'!
useNewChangeSetDuring: aBlock
	^self useNewChangeSetNamedLike: 'MC' during: aBlock! !

!MCPackageLoader methodsFor: 'private' stamp: 'nk 2/23/2005 07:50'!
useNewChangeSetNamedLike: baseName during: aBlock
	^self useChangeSetNamed: (ChangeSet uniqueNameLike: baseName) during: aBlock! !

!MCPackageLoader methodsFor: 'private' stamp: 'ab 5/25/2003 01:22'!
warnAboutDependencies 
	self notify: self dependencyWarning! !

!MCPackageLoader methodsFor: 'private' stamp: 'avi 1/24/2004 17:42'!
warnAboutErrors
	self notify: self errorDefinitionWarning.
! !

!MCPackageLoader methodsFor: 'private' stamp: 'kph 5/4/2007 00:23'!
warnAboutErrors: errors
	errors isEmpty ifFalse: [ self notify: (self errorDefinitionWarning: errors) ]
! !

Object subclass: #MCPackageLoader1b
	instanceVariableNames: 'requirements unloadableDefinitions obsoletions additions removals errorDefinitions provisions isUnloading isMultiplePackage'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Loading'!

!MCPackageLoader1b commentStamp: 'kph 5/25/2007 04:43' prior: 0!
MCPackageLoader1b - Attempt to improve loading with a number of tricks

In #basicLoad the loading process starts with a collection of definitions 
in #additions, and another in #removals. #analyse should have sorted these so that 
dependencies shouldnt cause errors.

The load sequence sent to each addidion is: #preLoadOver: [ #install #postinstall ] #postloadOver:
The load sequence sent to each removal is                    [ #unload ]

The brackets above mark the fact that the 'load' itself which is perfromed by #install/#postinstall and #unload are performed in a tight loop and this is our attempt at atomicity.

For methods, #preloadOver:, and #postloadOver: obsoletions are supplied, being the original item being overwritten.

InstVar #defn is the currently loading definition. This allows error handlers to know what was being processed when the error occured.

-----
Note that to support loading this Monicello over a version which does not support atomic loading, soe things have to remain in place that are otherwise obsolete.

MCDefinition-#loadOver:
MCPackageLoader instVar's errorDefinitions, addtions, removals cant be renamed.
!

!MCPackageLoader1b class methodsFor: 'as yet unclassified' stamp: 'kph 5/25/2007 04:26'!
new
	^ self basicNew initialize! !

!MCPackageLoader1b methodsFor: 'patch ops' stamp: 'kph 5/25/2007 04:26'!
addDefinition: aDefinition
	additions add: aDefinition! !

!MCPackageLoader1b methodsFor: 'private' stamp: 'kph 6/12/2007 09:36'!
analyze

	| sorter |
	
	self isMultiplePackage ifTrue: [ self analyzeMulti ].
	
	sorter := self sorterForItems: additions.
	additions := sorter orderedItems.
	requirements := sorter externalRequirements.
	unloadableDefinitions := sorter itemsWithMissingRequirements asSortedCollection.
	
	sorter := self sorterForItems: removals.
	removals := sorter orderedItems reversed.! !

!MCPackageLoader1b methodsFor: 'private' stamp: 'kph 6/12/2007 09:35'!
analyzeMulti
	| index |
	index := MCDefinitionIndex definitions: additions.
	removals removeAllSuchThat: [:removal |
		(index definitionLike: removal
			ifPresent: [:addition | obsoletions at: addition put: removal]
			ifAbsent: []) notNil].! !

!MCPackageLoader1b methodsFor: 'private' stamp: 'kph 4/14/2008 13:18'!
basicLoad
  
	additions do: [:ea | ea preloadOver: (self obsoletionFor: ea).   ] displayingProgress: 'Loading...'.
		
	self shouldWarnAboutErrors ifTrue: [ self warnAboutErrors: errorDefinitions ].

	(1 to: 2) do: [:ea |  ] displayingProgress: (self isUnloading ifFalse: [ 'Installing...' ] 
																ifTrue: [ 'Unloading...' ]).
	"begin the psuedo atomic bit"		
	
	removals do: [:ea | ea unload ].
	additions do: [:ea | ea install ].
	additions do: [:ea | ea postinstall ].

	"end the psuedo atomic bit"
 
	 additions reverse do: [:ea | 
		
		ea postloadOver: (self obsoletionFor: ea).
		self successfullyLoaded: ea.

	] displayingProgress: 'Integration...'.
! !

!MCPackageLoader1b methodsFor: 'public' stamp: 'kph 6/12/2007 09:33'!
beMultiplePackage

	isMultiplePackage := true! !

!MCPackageLoader1b methodsFor: 'public' stamp: 'kph 5/25/2007 04:26'!
beUnloading

	isUnloading := true! !

!MCPackageLoader1b methodsFor: 'private' stamp: 'kph 5/25/2007 04:26'!
dependencyWarning
	^ String streamContents:
		[:s |
		s nextPutAll: 'This package, or the orphanage is looking for the following classes:'; cr.
		requirements do: [:ea | s space; space; nextPutAll: ea; cr].
		s nextPutAll: 'These definitions will be orphaned until the classes are loaded: '; cr.
		unloadableDefinitions do: [:ea | s space; space; nextPutAll: (ea summary, ea because); cr]] ! !

!MCPackageLoader1b methodsFor: 'private' stamp: 'kph 5/25/2007 04:26'!
errorDefinitionWarning: erroredDefns
	^ String streamContents:
		[:s |
		s nextPutAll: 'The following definitions had errors while loading.  Press Proceed to try to load them again (they may work on a second pass):'; cr.
		erroredDefns do: [:ea | 
			s space; space; 
			  nextPutAll: ea summary;
			  space; 
			  nextPutAll: (ea because ifNil: [ '' ]); cr]] ! !

!MCPackageLoader1b methodsFor: 'private' stamp: 'kph 5/25/2007 04:26'!
flushChangesFile
	"The changes file is second in the SourceFiles array"

	(SourceFiles at: 2) flush! !

!MCPackageLoader1b methodsFor: 'private' stamp: 'kph 5/26/2007 00:29'!
initialize
	additions := OrderedCollection new.
	removals := OrderedCollection new.
	obsoletions := Dictionary new.
 ! !

!MCPackageLoader1b methodsFor: 'private' stamp: 'kph 5/25/2007 04:26'!
installOrphanage

	self orphanage loading: self.! !

!MCPackageLoader1b methodsFor: 'public' stamp: 'kph 5/25/2007 04:26'!
installSnapshot: aSnapshot
	| patch |
	patch := aSnapshot patchRelativeToBase: MCSnapshot empty.
	patch applyTo: self.
	 ! !

!MCPackageLoader1b methodsFor: 'public' stamp: 'kph 6/12/2007 09:33'!
isMultiplePackage

	^ isMultiplePackage == true! !

!MCPackageLoader1b methodsFor: 'public' stamp: 'kph 5/25/2007 04:26'!
isUnloading 

	^ isUnloading ifNil: [ false ]! !

!MCPackageLoader1b methodsFor: 'public' stamp: 'kph 5/25/2007 04:26'!
load
	self analyze.
	unloadableDefinitions isEmpty ifFalse: [self warnAboutDependencies].
	self useNewChangeSetDuring: [self protectedLoad].
	self loadComplete.! !

!MCPackageLoader1b methodsFor: 'private' stamp: 'kph 6/7/2008 03:29'!
loadComplete
	MCMethodDefinition tidyCache.
	self orphanage 
		addOrphans: unloadableDefinitions;
	 	addOrphans: errorDefinitions;
		loadComplete.
	Behavior flushObsoleteSubclasses.! !

!MCPackageLoader1b methodsFor: 'public' stamp: 'kph 5/25/2007 04:26'!
loadWithName: baseName
 
	self analyze.
	unloadableDefinitions isEmpty ifFalse: [self warnAboutDependencies].
	self useChangeSetNamed: baseName during: [self protectedLoad].
	self loadComplete.! !

!MCPackageLoader1b methodsFor: 'public' stamp: 'kph 5/25/2007 04:26'!
loadWithNameLike: baseName
 
	self analyze.
	unloadableDefinitions isEmpty ifFalse: [self warnAboutDependencies].
	self useNewChangeSetNamedLike: baseName during: [self protectedLoad].
	self loadComplete.! !

!MCPackageLoader1b methodsFor: 'patch ops' stamp: 'kph 5/25/2007 04:26'!
modifyDefinition: old to: new
	self addDefinition: new.
	obsoletions at: new put: old.! !

!MCPackageLoader1b methodsFor: 'private' stamp: 'kph 5/25/2007 04:26'!
obsoletionFor: aDefinition
	^ obsoletions at: aDefinition ifAbsent: [nil]! !

!MCPackageLoader1b methodsFor: 'private' stamp: 'kph 5/25/2007 04:26'!
orderedAdditions
	^ additions! !

!MCPackageLoader1b methodsFor: 'private' stamp: 'kph 5/25/2007 04:26'!
orphanage

	^ MCOrphanage current! !

!MCPackageLoader1b methodsFor: 'private' stamp: 'kph 12/18/2007 01:05'!
protectedLoad
		errorDefinitions := OrderedCollection new.
 		[
			[
				[ self basicLoad ] on: InMidstOfFileinNotification do: [:n |  n resume: true]

			] ensure: [self flushChangesFile].
		
		]  on: MCOrphanedNotification do: [ :ex | additions remove: ex orphan ifAbsent: [].
											errorDefinitions add: ex orphan. 
									 		ex resume.					  		]
 ! !

!MCPackageLoader1b methodsFor: 'private' stamp: 'kph 5/25/2007 04:26'!
provisions
	^ provisions ifNil: [provisions := Set withAll: Smalltalk keys]! !

!MCPackageLoader1b methodsFor: 'patch ops' stamp: 'kph 5/25/2007 04:26'!
removeDefinition: aDefinition
	removals add: aDefinition! !

!MCPackageLoader1b methodsFor: 'private' stamp: 'kph 5/25/2007 04:26'!
shouldWarnAboutErrors
	^ false "should make this a preference"! !

!MCPackageLoader1b methodsFor: 'private' stamp: 'kph 5/25/2007 04:26'!
sorterForItems: aCollection
	| sorter |
	sorter := MCDependencySorter items: aCollection.
	sorter addExternalProvisions: self provisions.
	^ sorter! !

!MCPackageLoader1b methodsFor: 'private' stamp: 'kph 5/25/2007 04:26'!
successfullyLoaded: aDefinition

	self orphanage successfullyLoaded: aDefinition! !

!MCPackageLoader1b methodsFor: 'private' stamp: 'kph 5/25/2007 04:26'!
tryToLoad: aDefinition
	[aDefinition loadOver: (self obsoletionFor: aDefinition)] on: Error do: [errorDefinitions add: aDefinition].! !

!MCPackageLoader1b methodsFor: 'obsolete' stamp: 'kph 5/25/2007 04:26'!
tryToLoad: aDefinition errors: errs
 

	self tryToLoad: aDefinition errors: errorDefinitions
 ! !

!MCPackageLoader1b methodsFor: 'private' stamp: 'kph 5/26/2007 10:02'!
unloadPackage: aPackage
 
	(self 
		updatePackage: aPackage withSnapshot: MCSnapshot empty;
		beUnloading;
		loadWithNameLike: aPackage name, '-unload';
		yourself)
		orphanage packageUnloaded: aPackage packageInfo.	! !

!MCPackageLoader1b methodsFor: 'public' stamp: 'kph 8/2/2007 14:45'!
updatePackage: aPackage withSnapshot: aSnapshot
	|  patch packageSnap |
	packageSnap := aPackage snapshot.
	patch := aSnapshot patchRelativeToBase: packageSnap.
	patch applyTo: self.
	packageSnap definitions do: [:ea | self provisions addAll: ea provisions]
! !

!MCPackageLoader1b methodsFor: 'private' stamp: 'kph 5/25/2007 04:26'!
useChangeSetNamed: baseName during: aBlock
	"Use the named change set, or create one with the given name."
	| changeHolder oldChanges newChanges |
	changeHolder := (ChangeSet respondsTo: #newChanges:)
						ifTrue: [ChangeSet]
						ifFalse: [Smalltalk].
	oldChanges := (ChangeSet respondsTo: #current)
						ifTrue: [ChangeSet current]
						ifFalse: [Smalltalk changes].

	newChanges := (ChangeSorter changeSetNamed: baseName) ifNil: [ ChangeSet new name: baseName ].
	changeHolder newChanges: newChanges.
	[aBlock value] ensure: [changeHolder newChanges: oldChanges].
! !

!MCPackageLoader1b methodsFor: 'private' stamp: 'kph 5/25/2007 04:26'!
useNewChangeSetDuring: aBlock
	^self useNewChangeSetNamedLike: 'MC' during: aBlock! !

!MCPackageLoader1b methodsFor: 'private' stamp: 'kph 5/25/2007 04:26'!
useNewChangeSetNamedLike: baseName during: aBlock
	^self useChangeSetNamed: (ChangeSet uniqueNameLike: baseName) during: aBlock! !

!MCPackageLoader1b methodsFor: 'private' stamp: 'kph 8/2/2007 00:22'!
warnAboutDependencies 
	"self notify: self dependencyWarning"! !

!MCPackageLoader1b methodsFor: 'private' stamp: 'kph 5/25/2007 04:26'!
warnAboutErrors: errors
	errors isEmpty ifFalse: [ self notify: (self errorDefinitionWarning: errors) ]
! !

Object subclass: #MCPackageLoader2
	instanceVariableNames: 'editor requirements unloadableDefinitions obsoletions additions removals errorDefinitions provisions isUnloading isMultiplePackage'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Loading'!

!MCPackageLoader2 commentStamp: 'kph 5/26/2007 09:32' prior: 0!
MCPackageLoader2

ProgressBar has great improvements.

#setProgressBar: is used for clients to control how progress is displayed.

If no one indicates a preference we display our own progress bar.
Otherwise we use the one passed to us.

To not have any progress bar displayed give us an empty block.
#setProgressBar: [:v |  ] !

MCPackageLoader2 subclass: #MCMultiPackageLoader
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Loading'!

!MCMultiPackageLoader commentStamp: '<historical>' prior: 0!
A PackageLoader doing some additional cross-package checks!

!MCMultiPackageLoader methodsFor: 'private' stamp: 'bf 3/17/2006 15:51'!
analyze
	| index |
	index := MCDefinitionIndex definitions: additions.
	removals removeAllSuchThat: [:removal |
		(index definitionLike: removal
			ifPresent: [:addition | obsoletions at: addition put: removal]
			ifAbsent: []) notNil].
	super analyze! !

!MCPackageLoader2 class methodsFor: 'as yet unclassified' stamp: 'avi 9/13/2004 18:03'!
new
	^ self basicNew initialize! !

!MCPackageLoader2 methodsFor: 'patch ops' stamp: 'ab 5/24/2003 16:13'!
addDefinition: aDefinition
	additions add: aDefinition! !

!MCPackageLoader2 methodsFor: 'private' stamp: 'kph 6/12/2007 09:36'!
analyze
	| sorter |
	
	self isMultiplePackage ifTrue: [ self analyzeMulti ].
	
	sorter := self sorterForItems: additions.
	additions := sorter orderedItems.
	requirements := sorter externalRequirements.
	unloadableDefinitions := sorter itemsWithMissingRequirements asSortedCollection.
	
	sorter := self sorterForItems: removals.
	removals := sorter orderedItems reversed.! !

!MCPackageLoader2 methodsFor: 'private' stamp: 'kph 6/12/2007 09:35'!
analyzeMulti
	| index |
	index := MCDefinitionIndex definitions: additions.
	removals removeAllSuchThat: [:removal |
		(index definitionLike: removal
			ifPresent: [:addition | obsoletions at: addition put: removal]
			ifAbsent: []) notNil].! !

!MCPackageLoader2 methodsFor: 'private' stamp: 'mtf 8/6/2008 01:38'!
basicLoad

	| postloads postunloads |

	editor := SystemEditor new.
	postloads := OrderedCollection new.
	postunloads := OrderedCollection new.

	additions do: [:defn | (defn edLoad: editor) ifTrue: [postloads add: defn]]
		displayingProgress: 'Assembling Changes...'.
	removals do: [:defn | (defn edUnload: editor) ifTrue: [postunloads add: defn]]
		displayingProgress: 'Assembling Removals...'.

	"the business"
	editor commitWithProgress.

	postloads do: [:ea | ea edPostload] displayingProgress: 'Running post-load scripts...'.
	postunloads do: [ :ea |  ea edPostunload] displayingProgress: 'Running post-unload scripts...'.! !

!MCPackageLoader2 methodsFor: 'patch ops' stamp: 'kph 6/12/2007 09:32'!
beMultiplePackage

	isMultiplePackage := true! !

!MCPackageLoader2 methodsFor: 'public' stamp: 'kph 5/15/2007 15:38'!
beUnloading

	isUnloading := true! !

!MCPackageLoader2 methodsFor: 'private' stamp: 'kph 5/15/2007 13:38'!
dependencyWarning
	^ String streamContents:
		[:s |
		s nextPutAll: 'This package, or the orphanage is looking for the following classes:'; cr.
		requirements do: [:ea | s space; space; nextPutAll: ea; cr].
		s nextPutAll: 'These definitions will be orphaned until the classes are loaded: '; cr.
		unloadableDefinitions do: [:ea | s space; space; nextPutAll: (ea summary, ea because); cr]] ! !

!MCPackageLoader2 methodsFor: 'private' stamp: 'kph 5/15/2007 12:53'!
errorDefinitionWarning: erroredDefns
	^ String streamContents:
		[:s |
		s nextPutAll: 'The following definitions had errors while loading.  Press Proceed to try to load them again (they may work on a second pass):'; cr.
		erroredDefns do: [:ea | 
			s space; space; 
			  nextPutAll: ea summary;
			  space; 
			  nextPutAll: (ea because ifNil: [ '' ]); cr]] ! !

!MCPackageLoader2 methodsFor: 'private' stamp: 'cwp 11/13/2003 02:01'!
flushChangesFile
	"The changes file is second in the SourceFiles array"

	(SourceFiles at: 2) flush! !

!MCPackageLoader2 methodsFor: 'private' stamp: 'kph 7/4/2007 05:28'!
initialize
	additions := OrderedCollection new.
	removals := OrderedCollection new.
	obsoletions := Dictionary new.

! !

!MCPackageLoader2 methodsFor: 'private' stamp: 'kph 5/15/2007 12:12'!
installOrphanage

	self orphanage loading: self.! !

!MCPackageLoader2 methodsFor: 'public' stamp: 'kph 5/11/2007 07:28'!
installSnapshot: aSnapshot
	| patch |
	patch := aSnapshot patchRelativeToBase: MCSnapshot empty.
	patch applyTo: self.
	 ! !

!MCPackageLoader2 methodsFor: 'patch ops' stamp: 'kph 6/12/2007 09:33'!
isMultiplePackage

	^ isMultiplePackage == true! !

!MCPackageLoader2 methodsFor: 'public' stamp: 'kph 5/15/2007 15:39'!
isUnloading 

	^ isUnloading ifNil: [ false ]! !

!MCPackageLoader2 methodsFor: 'public' stamp: 'kph 5/15/2007 12:13'!
load
	self analyze.
	unloadableDefinitions isEmpty ifFalse: [self warnAboutDependencies].
	self useNewChangeSetDuring: [self protectedLoad].
	self loadComplete.! !

!MCPackageLoader2 methodsFor: 'private' stamp: 'kph 7/4/2007 15:41'!
loadComplete
	MCMethodDefinition tidyCache.
	self orphanage 
		addOrphans: unloadableDefinitions;
	 	addOrphans: errorDefinitions;
		loadComplete.
! !

!MCPackageLoader2 methodsFor: 'public' stamp: 'kph 5/15/2007 12:26'!
loadWithName: baseName
 
	self analyze.
	unloadableDefinitions isEmpty ifFalse: [self warnAboutDependencies].
	self useChangeSetNamed: baseName during: [self protectedLoad].
	self loadComplete.! !

!MCPackageLoader2 methodsFor: 'public' stamp: 'kph 5/15/2007 12:26'!
loadWithNameLike: baseName
 
	self analyze.
	unloadableDefinitions isEmpty ifFalse: [self warnAboutDependencies].
	self useNewChangeSetNamedLike: baseName during: [self protectedLoad].
	self loadComplete.! !

!MCPackageLoader2 methodsFor: 'patch ops' stamp: 'avi 2/17/2004 13:14'!
modifyDefinition: old to: new
	self addDefinition: new.
	obsoletions at: new put: old.! !

!MCPackageLoader2 methodsFor: 'private' stamp: 'avi 2/17/2004 13:15'!
obsoletionFor: aDefinition
	^ obsoletions at: aDefinition ifAbsent: [nil]! !

!MCPackageLoader2 methodsFor: 'private' stamp: 'ab 5/24/2003 16:52'!
orderedAdditions
	^ additions! !

!MCPackageLoader2 methodsFor: 'private' stamp: 'kph 4/30/2007 00:39'!
orphanage

	^ MCOrphanage current! !

!MCPackageLoader2 methodsFor: 'private' stamp: 'kph 5/26/2007 10:06'!
progressBar

	^ progressBar ! !

!MCPackageLoader2 methodsFor: 'private' stamp: 'mtf 8/6/2008 01:13'!
protectedLoad

 		[
			[
				[ self basicLoad ] on: InMidstOfFileinNotification do: [:n | n resume: true]

			] ensure: [self flushChangesFile].
		
		]  on: MCOrphanedNotification do: [ :ex | additions remove: ex orphan ifAbsent: [].
											errorDefinitions add: ex orphan. 
									 		ex resume.					  		]
 ! !

!MCPackageLoader2 methodsFor: 'private' stamp: 'kph 5/25/2007 03:43'!
provisions
	^ provisions ifNil: [provisions := Set withAll: Smalltalk keys]! !

!MCPackageLoader2 methodsFor: 'patch ops' stamp: 'ab 5/24/2003 16:14'!
removeDefinition: aDefinition
	removals add: aDefinition! !

!MCPackageLoader2 methodsFor: 'private' stamp: 'kph 5/26/2007 09:22'!
setProgressBar: aBar

	progressBar := aBar! !

!MCPackageLoader2 methodsFor: 'private' stamp: 'kph 5/4/2007 00:24'!
shouldWarnAboutErrors
	^ false "should make this a preference"! !

!MCPackageLoader2 methodsFor: 'private' stamp: 'stephaneducasse 2/4/2006 20:47'!
sorterForItems: aCollection
	| sorter |
	sorter := MCDependencySorter items: aCollection.
	sorter addExternalProvisions: self provisions.
	^ sorter! !

!MCPackageLoader2 methodsFor: 'private' stamp: 'kph 5/10/2007 17:30'!
successfullyLoaded: aDefinition

	self orphanage successfullyLoaded: aDefinition! !

!MCPackageLoader2 methodsFor: 'obsolete' stamp: 'avi 2/17/2004 13:15'!
tryToLoad: aDefinition
	[aDefinition loadOver: (self obsoletionFor: aDefinition)] on: Error do: [errorDefinitions add: aDefinition].! !

!MCPackageLoader2 methodsFor: 'obsolete' stamp: 'kph 5/22/2007 18:45'!
tryToLoad: aDefinition errors: errs
 

	self tryToLoad: aDefinition errors: errorDefinitions
 ! !

!MCPackageLoader2 methodsFor: 'private' stamp: 'kph 5/26/2007 10:02'!
unloadPackage: aPackage
 
	(self 
		updatePackage: aPackage withSnapshot: MCSnapshot empty;
		beUnloading;
		loadWithNameLike: aPackage name, '-unload';
		yourself)
		orphanage packageUnloaded: aPackage packageInfo.	! !

!MCPackageLoader2 methodsFor: 'public' stamp: 'stephaneducasse 2/4/2006 20:47'!
updatePackage: aPackage withSnapshot: aSnapshot
	|  patch packageSnap |
	packageSnap := aPackage snapshot.
	patch := aSnapshot patchRelativeToBase: packageSnap.
	patch applyTo: self.
	packageSnap definitions do: [:ea | self provisions addAll: ea provisions]
! !

!MCPackageLoader2 methodsFor: 'private' stamp: 'stephaneducasse 2/4/2006 20:47'!
useChangeSetNamed: baseName during: aBlock
	"Use the named change set, or create one with the given name."
	| changeHolder oldChanges newChanges |
	changeHolder := (ChangeSet respondsTo: #newChanges:)
						ifTrue: [ChangeSet]
						ifFalse: [Smalltalk].
	oldChanges := (ChangeSet respondsTo: #current)
						ifTrue: [ChangeSet current]
						ifFalse: [Smalltalk changes].

	newChanges := (ChangeSorter changeSetNamed: baseName) ifNil: [ ChangeSet new name: baseName ].
	changeHolder newChanges: newChanges.
	[aBlock value] ensure: [changeHolder newChanges: oldChanges].
! !

!MCPackageLoader2 methodsFor: 'private' stamp: 'nk 8/30/2004 08:38'!
useNewChangeSetDuring: aBlock
	^self useNewChangeSetNamedLike: 'MC' during: aBlock! !

!MCPackageLoader2 methodsFor: 'private' stamp: 'nk 2/23/2005 07:50'!
useNewChangeSetNamedLike: baseName during: aBlock
	^self useChangeSetNamed: (ChangeSet uniqueNameLike: baseName) during: aBlock! !

!MCPackageLoader2 methodsFor: 'private' stamp: 'ab 5/25/2003 01:22'!
warnAboutDependencies 
	self notify: self dependencyWarning! !

!MCPackageLoader2 methodsFor: 'private' stamp: 'kph 5/4/2007 00:23'!
warnAboutErrors: errors
	errors isEmpty ifFalse: [ self notify: (self errorDefinitionWarning: errors) ]
! !

Object subclass: #MCPackageManager
	instanceVariableNames: 'package modified'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Versioning'!

MCPackageManager class
	instanceVariableNames: 'registry'!

MCPackageManager class
	instanceVariableNames: 'registry'!

!MCPackageManager class methodsFor: 'as yet unclassified' stamp: 'kph 8/3/2008 04:43'!
allManagers
	^ self packageInfos collect:[ :pi | pi manager ifNil: [ self forPackage: (MCPackage named: pi name) ] ]! !

!MCPackageManager class methodsFor: 'system changes' stamp: 'ar 4/27/2005 14:28'!
classModified: anEvent
	self managersForClass: anEvent item do:[:mgr| mgr modified: true].! !

!MCPackageManager class methodsFor: 'system changes' stamp: 'ar 4/26/2005 21:31'!
classMoved: anEvent
	self classModified: anEvent.
	self managersForCategory: anEvent oldCategory do:[:mgr| mgr modified: true].! !

!MCPackageManager class methodsFor: 'system changes' stamp: 'kph 5/11/2007 03:17'!
classRemoved: anEvent
	MCOrphanage current onClassRemoved: anEvent item.
	self classModified: anEvent! !

!MCPackageManager class methodsFor: 'as yet unclassified' stamp: 'kph 8/3/2008 04:02'!
forPackage: aPackage

	^ (PackageInfo named: aPackage name) in: [ :pi | 
		
		pi manager ifNil: [ pi manager: (self new initializeWithPackage: aPackage).
							self changed: #allManagers.
							pi manager ]
		
		] ! !

!MCPackageManager class methodsFor: 'system changes' stamp: 'kph 9/2/2008 04:33'!
managersForCategory: aSystemCategory do: aBlock
	"Got to be careful here - we might get method categories where capitalization is problematic."
	| cat foundOne index |
	foundOne := false.
	cat := aSystemCategory ifNil:[^nil]. "yes this happens; for example in eToy projects"
	"first ask PackageInfos, their package name might not match the category"
	self packageInfos do: [:pi | 
		((pi includesSystemCategory: aSystemCategory) and: [ pi manager notNil ])	ifTrue: [
			aBlock value: pi manager.
			foundOne := true.
		]
	].
	foundOne ifTrue: [^self].
	["Loop over categories until we found a matching one"
	
	(self organizer packageNamed: cat ifAbsent: nil) ifNotNilDo:[:pi |
		aBlock value: pi manager.
		foundOne := true.
	].

	index := cat lastIndexOf: $-.
	index > 0]whileTrue:[
		"Step up to next level package"
		cat := cat copyFrom: 1 to: index-1.
	].
	foundOne ifFalse:[
		"Create a new (but only top-level)"
		aBlock value: (MCWorkingCopy forPackage: (MCPackage named: (aSystemCategory copyUpTo: $-) capitalized)).
	].! !

!MCPackageManager class methodsFor: 'system changes' stamp: 'ar 4/27/2005 14:11'!
managersForClass: aClass category: methodCategory do: aBlock
	(methodCategory isEmptyOrNil or:[methodCategory first ~= $*]) ifTrue:[
		"Not an extension method"
		^self managersForClass: aClass do: aBlock.
	].
	self managersForCategory: methodCategory allButFirst do: aBlock.! !

!MCPackageManager class methodsFor: 'system changes' stamp: 'kph 9/2/2008 04:34'!
managersForClass: aClass do: aBlock

	self packageInfos do: [:pi |
		((pi includesClass: aClass) and: [ pi manager notNil ])
			ifTrue: [aBlock value: pi manager]]! !

!MCPackageManager class methodsFor: 'system changes' stamp: 'ar 4/26/2005 21:40'!
managersForClass: aClass selector: aSelector do: aBlock
	^self managersForClass: aClass category: (aClass organization categoryOfElement: aSelector) do: aBlock! !

!MCPackageManager class methodsFor: 'system changes' stamp: 'ar 4/26/2005 21:40'!
methodModified: anEvent
	^self managersForClass: anEvent itemClass selector: anEvent itemSelector do:[:mgr| mgr modified: true].! !

!MCPackageManager class methodsFor: 'system changes' stamp: 'ar 4/26/2005 21:40'!
methodMoved: anEvent
	self managersForClass: anEvent itemClass category: anEvent oldCategory do:[:mgr| mgr modified: true].
	self methodModified: anEvent.! !

!MCPackageManager class methodsFor: 'system changes' stamp: 'ar 4/26/2005 22:12'!
methodRemoved: anEvent
	self managersForClass: anEvent itemClass category: anEvent itemProtocol do:[:mgr| mgr modified: true].
! !

!MCPackageManager class methodsFor: 'as yet unclassified' stamp: 'kph 8/3/2008 04:07'!
organizer
	^ PackageOrganizer default  ! !

!MCPackageManager class methodsFor: 'as yet unclassified' stamp: 'kph 8/3/2008 04:07'!
packageInfos
	^ self organizer packageInfos! !

!MCPackageManager class methodsFor: 'system changes' stamp: 'ar 4/26/2005 22:18'!
registerForNotifications
	Smalltalk at: #SystemChangeNotifier ifPresent:[:cls|
	(cls uniqueInstance)
		noMoreNotificationsFor: self;
		notify: self ofSystemChangesOfItem: #class change: #Added using: #classModified:;
		notify: self ofSystemChangesOfItem: #class change: #Modified using: #classModified:;
		notify: self ofSystemChangesOfItem: #class change: #Renamed using: #classModified:;
		notify: self ofSystemChangesOfItem: #class change: #Commented using: #classModified:;
		notify: self ofSystemChangesOfItem: #class change: #Recategorized using: #classMoved:;
		notify: self ofSystemChangesOfItem: #class change: #Removed using: #classRemoved:;
		notify: self ofSystemChangesOfItem: #method change: #Added using: #methodModified:;
		notify: self ofSystemChangesOfItem: #method change: #Modified using: #methodModified:;
		notify: self ofSystemChangesOfItem: #method change: #Recategorized using: #methodMoved:;
		notify: self ofSystemChangesOfItem: #method change: #Removed using: #methodRemoved:
	].! !

!MCPackageManager class methodsFor: 'as yet unclassified' stamp: 'kph 9/2/2008 04:05'!
registry
	 
	"if we have a registry migrate to using PackageOrganizer"
 
	^ registry
	! !

!MCPackageManager methodsFor: 'initialize-release' stamp: 'kph 5/10/2007 18:01'!
initialize
	modified := false.
	 ! !

!MCPackageManager methodsFor: 'initialize-release' stamp: 'stephaneducasse 2/4/2006 20:47'!
initializeWithPackage: aPackage
	package := aPackage.
	self initialize.! !

!MCPackageManager methodsFor: 'accessing' stamp: 'cwp 11/13/2003 14:12'!
modified
	^ modified! !

!MCPackageManager methodsFor: 'accessing' stamp: 'stephaneducasse 2/4/2006 20:47'!
modified: aBoolean
     modified = aBoolean ifTrue: [^ self].
	modified := aBoolean.
	self changed: #modified.
	
	modified ifFalse:
		[(((Smalltalk classNamed: 'SmalltalkImage') ifNotNilDo: [:si | si current]) ifNil: [Smalltalk])
			logChange: '"', self packageName, '"'].! !

!MCPackageManager methodsFor: 'accessing' stamp: 'ab 7/7/2003 16:47'!
package
	^ package! !

!MCPackageManager methodsFor: 'accessing' stamp: 'ab 7/7/2003 13:33'!
packageInfo
	^ package packageInfo! !

!MCPackageManager methodsFor: 'accessing' stamp: 'ab 7/7/2003 12:18'!
packageName
	^ package name! !

!MCPackageManager methodsFor: 'accessing' stamp: 'ab 7/5/2003 23:18'!
packageNameWithStar
	^ modified
		ifTrue: ['* ', self packageName]
		ifFalse: [self packageName]! !

!MCPackageManager methodsFor: 'system changes' stamp: 'ar 4/26/2005 22:17'!
systemChange: anEvent
	"obsolete - remove this later"! !

!MCPackageManager methodsFor: 'operations' stamp: 'kph 8/3/2008 04:18'!
unregister
	self class organizer unregisterPackageNamed: self package name.
	self class changed: #allManagers! !

!MCPackageManager methodsFor: 'system changes' stamp: 'avi 11/11/2003 12:06'!
update: aSymbol
	InMidstOfFileinNotification signal ifFalse: [
	[((aSymbol = #recentMethodSubmissions)
		and: [self packageInfo
				includesMethodReference: Utilities recentMethodSubmissions last])
					ifTrue: [self modified: true]]
		on: Error do: []]! !

MCPackageManager subclass: #MCWorkingCopy
	instanceVariableNames: 'ancestry counter repositoryGroup requiredPackages'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Versioning'!

!MCWorkingCopy class methodsFor: 'as yet unclassified' stamp: 'kph 9/3/2008 15:41'!
initialize
	Smalltalk 
		at: #MczInstaller
		ifPresent: [:installer | self adoptVersionInfoFrom: installer].

	registry ifNotNil:[registry rehash]. "changed #="
	self allInstancesDo:[:each| "moved notifications"
		Smalltalk at: #SystemChangeNotifier ifPresent:[:cls|
			cls uniqueInstance noMoreNotificationsFor: each.
		].
	].
	self registerForNotifications.
	
	registry ifNotNil: [ 
		PackageOrganizer forcefullyClearCache.
		registry values collect: [ :mgr | mgr package packageInfo manager: mgr  ].
		registry := nil.
	].! !

!MCWorkingCopy class methodsFor: 'as yet unclassified' stamp: 'kph 11/13/2007 22:55'!
adoptVersionInfoFrom: anInstaller
	|viCache|
	viCache := Dictionary new.
	anInstaller versionInfo keysAndValuesDo: [:packageName :info |
		
	[(self forPackage: (MCPackage named: packageName))
			versionInfo: (self infoFromDictionary:  info cache: viCache) ] ifError: ["backwards compat"] ].
		
		
	[anInstaller clearVersionInfo] on: Error do: ["backwards compat"].! !

!MCWorkingCopy class methodsFor: 'as yet unclassified' stamp: 'avi 2/17/2004 01:23'!
ancestorsFromArray: anArray cache: aDictionary
	^ anArray ifNotNil: [anArray collect: [:dict | self infoFromDictionary: dict cache: aDictionary]]! !

!MCWorkingCopy class methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
infoFromDictionary: aDictionary cache: cache
	| id |
	id := aDictionary at: #id.
	^ cache at: id ifAbsentPut:
		[MCVersionInfo
			name: (aDictionary at: #name)
			id: (aDictionary at: #id)
			message: (aDictionary at: #message)
			date: (aDictionary at: #date)
			time: (aDictionary at: #time)
			author: (aDictionary at: #author)
			ancestors: (self ancestorsFromArray: (aDictionary at: #ancestors) cache: cache)]! !

!MCWorkingCopy class methodsFor: 'as yet unclassified' stamp: 'avi 2/17/2004 02:36'!
updateInstVars
	self allInstances do: [:ea | ea updateInstVars]! !

!MCWorkingCopy methodsFor: 'operations' stamp: 'avi 2/13/2004 01:07'!
adopt: aVersion
	ancestry addAncestor: aVersion info.
	self changed.! !

!MCWorkingCopy methodsFor: 'accessing' stamp: 'avi 2/13/2004 01:07'!
ancestors
	^ ancestry ancestors! !

!MCWorkingCopy methodsFor: 'accessing' stamp: 'avi 2/13/2004 01:13'!
ancestry
	^ ancestry! !

!MCWorkingCopy methodsFor: 'operations' stamp: 'kph 5/15/2007 12:18'!
backportChangesTo: aVersionInfo
	| baseVersion fullPatch currentVersionInfo currentVersion newSnapshot newAncestry |
	currentVersionInfo := self currentVersionInfo.
	baseVersion := self repositoryGroup versionWithInfo: aVersionInfo.
	currentVersion := self repositoryGroup versionWithInfo: currentVersionInfo.
	fullPatch := currentVersion snapshot patchRelativeToBase: baseVersion snapshot.
	(MCChangeSelectionRequest new
		patch: fullPatch;
		label: 'Changes to Backport';
		signal ) ifNotNilDo:
		[:partialPatch |
		newSnapshot := MCPatcher apply: partialPatch to: baseVersion snapshot.
		newAncestry := MCWorkingAncestry new
							addAncestor: aVersionInfo;
							addStepChild: currentVersionInfo;
							yourself.

		(MCPackageLoader new) 
			updatePackage: package withSnapshot: newSnapshot;
			installOrphanage.
			
		ancestry := newAncestry.
		self modified: false; modified: true]! !

!MCWorkingCopy methodsFor: 'operations' stamp: 'kph 5/11/2007 05:08'!
canBackport

	^ true! !

!MCWorkingCopy methodsFor: 'operations' stamp: 'stephaneducasse 2/4/2006 20:47'!
changesRelativeToRepository: aRepository
	| ancestorVersion ancestorSnapshot |
	ancestorVersion := aRepository closestAncestorVersionFor: ancestry ifNone: [].
	ancestorSnapshot := ancestorVersion ifNil: [MCSnapshot empty] ifNotNil: [ancestorVersion snapshot].
	^ package snapshot patchRelativeToBase: ancestorSnapshot! !

!MCWorkingCopy methodsFor: 'obsolete' stamp: 'kph 5/19/2007 14:14'!
classModified: aMethod

	"this is a dummy to enable MC to be loaded in by older MC's who have Working copies registered for events for some reason"
	
 Smalltalk at: #SystemChangeNotifier ifPresent:[:cls|
	
		self class allInstances do: [ :i | 
			(cls uniqueInstance) noMoreNotificationsFor: i.
		].
	].! !

!MCWorkingCopy methodsFor: 'accessing' stamp: 'stephaneducasse 2/4/2006 20:47'!
clearRequiredPackages
	requiredPackages := nil! !

!MCWorkingCopy methodsFor: 'accessing' stamp: 'avi 2/13/2004 20:01'!
currentVersionInfo
	^ (self needsSaving or: [ancestry ancestors isEmpty])
		ifTrue: [self newVersion info]
		ifFalse: [ancestry ancestors first]! !

!MCWorkingCopy methodsFor: 'accessing' stamp: 'avi 2/13/2004 01:05'!
description
	^ self packageNameWithStar, ' (', ancestry ancestorString, ')'! !

!MCWorkingCopy methodsFor: 'private' stamp: 'avi 9/24/2004 12:15'!
findSnapshotWithVersionInfo: aVersionInfo
	^ aVersionInfo
		ifNil: [MCSnapshot empty]
		ifNotNil: [(self repositoryGroup versionWithInfo: aVersionInfo) snapshot]! !

!MCWorkingCopy methodsFor: 'private' stamp: 'stephaneducasse 2/4/2006 20:47'!
initialize
	super initialize.
	ancestry := MCWorkingAncestry new! !

!MCWorkingCopy methodsFor: 'testing' stamp: 'kph 5/13/2007 01:05'!
isOrphanage

	^ false! !

!MCWorkingCopy methodsFor: 'operations' stamp: 'stephaneducasse 2/4/2006 20:47'!
loaded: aVersion
	ancestry := MCWorkingAncestry new addAncestor: aVersion info.
	requiredPackages := OrderedCollection withAll: (aVersion dependencies collect: [:ea | ea package]).
	self modified: false.
	self changed! !

!MCWorkingCopy methodsFor: 'operations' stamp: 'stephaneducasse 2/4/2006 20:47'!
merge: targetVersion
	| ancestorInfo merger ancestorSnapshot packageSnapshot |
	targetVersion dependencies do: [:ea | ea resolve merge].
	ancestorInfo := targetVersion info commonAncestorWith: ancestry.
	
	ancestorInfo = targetVersion info ifTrue: [^ MCNoChangesException signal].
	
	packageSnapshot := package snapshot.
	ancestorSnapshot := ancestorInfo
							ifNotNil: [(self findSnapshotWithVersionInfo: ancestorInfo)]
							ifNil: [self notifyNoCommonAncestorWith: targetVersion.  MCSnapshot empty].
	
	(ancestry ancestors size = 1
		and: [ancestry ancestors first = ancestorInfo]
		and: [(packageSnapshot patchRelativeToBase: ancestorSnapshot) isEmpty])
				ifTrue: [^ targetVersion load].
	
	merger := MCThreeWayMerger 
				base: packageSnapshot
				target: targetVersion snapshot
				ancestor: ancestorSnapshot.
	((MCMergeResolutionRequest new merger: merger)
		signal: 'Merging ', targetVersion info name) = true ifTrue:
			[merger loadWithNameLike: targetVersion info name.
			ancestry addAncestor: targetVersion info].
	self changed! !

!MCWorkingCopy methodsFor: 'operations' stamp: 'abc 2/13/2004 15:57'!
merged: aVersion
	ancestry addAncestor: aVersion info.
	self changed! !

!MCWorkingCopy methodsFor: 'obsolete' stamp: 'kph 5/19/2007 14:14'!
methodModified: aMethod

	"this is a dummy to enable MC to be loaded in by older MC's who have Working copies registered for events for some reason"
	
	Smalltalk at: #SystemChangeNotifier ifPresent:[:cls|
	
		self class allInstances do: [ :i | 
			(cls uniqueInstance) noMoreNotificationsFor: i.
		].
	].! !

!MCWorkingCopy methodsFor: 'obsolete' stamp: 'kph 5/19/2007 14:14'!
methodRemoved: aMethod

	"this is a dummy to enable MC to be loaded in by older MC's who have Working copies registered for events for some reason"
	
	Smalltalk at: #SystemChangeNotifier ifPresent:[:cls|
	
		self class allInstances do: [ :i | 
			(cls uniqueInstance) noMoreNotificationsFor: i.
		].
	].! !

!MCWorkingCopy methodsFor: 'menu' stamp: 'kph 6/8/2008 03:42'!
myMenu: aMenu inBrowser: browser
 
	browser fillMenu: aMenu fromSpecs:
		#(
			('add required package' #addRequiredPackage)
			('clear required packages' #clearRequiredPackages)
			('add repository...' #addPackageRepository)
			"('browse package' #browseWorkingCopy)
			('view changes' #viewChanges)
			('view history' #viewHistory)"
			('recompile package' #recompilePackage)
			('recompile all packages' #recompileAllPackages)		
			('memory use for all packages' #viewMemoryUseAll)		
			('memory use' #viewMemoryUse)		
			('revert package...' #revertPackage)
			('unload package code' #unloadPackage)
			('trim ancestry' #trimAncestry)
			('remove package name from list' #deleteWorkingCopy)
			('explore working copy' #exploreWorkingCopy)
		).
	(Smalltalk includesKey: #SARMCPackageDumper) ifTrue: [
		aMenu add: 'make SAR' target: self selector: #fileOutAsSAR
	].
 
	^ aMenu
! !

!MCWorkingCopy methodsFor: 'accessing' stamp: 'avi 2/13/2004 20:02'!
needsSaving
	^ self modified or: [self requiredPackages anySatisfy: [:ea | ea workingCopy needsSaving]]! !

!MCWorkingCopy methodsFor: 'private' stamp: 'kph 6/29/2007 04:12'!
newMcFilenameUUID

	| uuid |

	uuid := (UUID new asString copyFrom: 1 to: 18) copyWithout: $-
! !

!MCWorkingCopy methodsFor: 'operations' stamp: 'avi 1/19/2004 16:18'!
newVersion
	^ (self requestVersionNameAndMessageWithSuggestion: self uniqueVersionName) ifNotNilDo:
		[:pair |
		self newVersionWithName: pair first message: pair last].
! !

!MCWorkingCopy methodsFor: 'operations' stamp: 'stephaneducasse 2/4/2006 20:47'!
newVersionWithName: nameString message: messageString
	| info deps |
	info := ancestry infoWithName: nameString message: messageString.
	ancestry := MCWorkingAncestry new addAncestor: info.
	self modified: true; modified: false.
	
	deps := self requiredPackages collect:
		[:ea | 
		MCVersionDependency
			package: ea
			info: ea workingCopy currentVersionInfo].

	^ MCVersion
		package: package
		info: info
		snapshot: package snapshot
		dependencies: deps! !

!MCWorkingCopy methodsFor: 'private' stamp: 'kph 6/29/2007 03:49'!
nextVersionNameHaveAlreadyTried: previousAttempt	 

	previousAttempt ifNotNil:[ ^ previousAttempt nextVersion ].

	ancestry ancestors isEmpty
		ifTrue: [ ^ MCVersionInfo firstVersionForPackage: package name ].

	^ ancestry ancestors first nextVersionConsidering: ancestry.
 ! !

!MCWorkingCopy methodsFor: 'operations' stamp: 'avi 2/13/2004 01:07'!
notifyNoCommonAncestorWith: aVersion
	self notify:
'Could not find a common ancestor between (',
aVersion info name,
') and (',
ancestry ancestorString, ').
Proceeding with this merge may cause spurious conflicts.'! !

!MCWorkingCopy methodsFor: 'private' stamp: 'bf 9/8/2005 10:58'!
possiblyNewerVersions

	^Array streamContents: [:strm |
		self repositoryGroup repositories do: [:repo |
			strm nextPutAll: (self possiblyNewerVersionsIn: repo)]]! !

!MCWorkingCopy methodsFor: 'private' stamp: 'bf 9/8/2005 10:58'!
possiblyNewerVersionsIn: aRepository

	^aRepository possiblyNewerVersionsOfAnyOf: self ancestors! !

!MCWorkingCopy methodsFor: 'printing' stamp: 'kph 2/14/2007 11:54'!
printOn: stream

super printOn: stream.
stream space.
self description printOn: stream.! !

!MCWorkingCopy methodsFor: 'operations' stamp: 'kph 5/12/2007 01:22'!
recompilePackage

	self packageInfo methods
		do: [:ea | ea actualClass recompile: ea methodSymbol]
		displayingProgress: 'Recompiling...'! !

!MCWorkingCopy methodsFor: 'repositories' stamp: 'stephaneducasse 2/4/2006 20:47'!
repositoryGroup
	^ repositoryGroup ifNil: [repositoryGroup := MCRepositoryGroup new]! !

!MCWorkingCopy methodsFor: 'repositories' stamp: 'stephaneducasse 2/4/2006 20:47'!
repositoryGroup: aRepositoryGroup
	repositoryGroup := aRepositoryGroup! !

!MCWorkingCopy methodsFor: 'private' stamp: 'ab 8/24/2003 20:38'!
requestVersionNameAndMessageWithSuggestion: aString
	^ (MCVersionNameAndMessageRequest new suggestedName: aString) signal! !

!MCWorkingCopy methodsFor: 'accessing' stamp: 'avi 1/20/2004 16:04'!
requirePackage: aPackage
	(self requiredPackages includes: aPackage) ifFalse: [requiredPackages add: aPackage]! !

!MCWorkingCopy methodsFor: 'accessing' stamp: 'stephaneducasse 2/4/2006 20:47'!
requiredPackages
	^ requiredPackages ifNil: [requiredPackages := OrderedCollection new]! !

!MCWorkingCopy methodsFor: 'private' stamp: 'kph 5/30/2008 14:38'!
trimAncestry

	self ancestry trimAfterVersionInfo: versionInfo! !

!MCWorkingCopy methodsFor: 'private' stamp: 'kph 1/28/2008 02:23'!
uniqueVersionName
	|version |
 	 
	version := nil.
	[version  := self nextVersionNameHaveAlreadyTried: version .
	self repositoryGroup includesVersionNamed: version name ] whileTrue.
	^ version name! !

!MCWorkingCopy methodsFor: 'operations' stamp: 'kph 5/26/2007 10:04'!
unload
	MCPackageLoader new unloadPackage: self package.
	self unregister.! !

!MCWorkingCopy methodsFor: 'accessing' stamp: 'stephaneducasse 2/4/2006 20:47'!
versionInfo: aVersionInfo
	ancestry := MCWorkingAncestry new addAncestor: aVersionInfo! !

MCWorkingCopy subclass: #MCWorkingOrphanage
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Orphanage'!

!MCWorkingOrphanage methodsFor: 'as yet unclassified' stamp: 'kph 5/11/2007 05:04'!
adopt: aVersion! !

!MCWorkingOrphanage methodsFor: 'as yet unclassified' stamp: 'kph 5/11/2007 05:03'!
backportChangesTo: aVersionInfo! !

!MCWorkingOrphanage methodsFor: 'as yet unclassified' stamp: 'kph 5/11/2007 05:08'!
canBackport

	^ false! !

!MCWorkingOrphanage methodsFor: 'as yet unclassified' stamp: 'kph 5/11/2007 15:40'!
description
	^ self packageNameWithStar, ' ( ', self package orphans size printString, ' orphans ', (self package unlinkedCount > 0 ifTrue: [ self package unlinkedCount printString, ' unlinked )' ] ifFalse: [ ')' ])! !

!MCWorkingOrphanage methodsFor: 'as yet unclassified' stamp: 'kph 5/13/2007 01:05'!
isOrphanage

	^ true! !

!MCWorkingOrphanage methodsFor: 'as yet unclassified' stamp: 'kph 5/11/2007 05:05'!
merge: targetVersion! !

!MCWorkingOrphanage methodsFor: 'as yet unclassified' stamp: 'kph 12/8/2007 07:19'!
myMenu: aMenu inBrowser: browser

	 	
	browser fillMenu: aMenu fromSpecs:
		#(
			 
			('browse orphans' #browseWorkingCopy)
 			('reload orphans' #recompilePackage)
			('clearOrphans' 	  #clearOrphans)
			('explore orphans' #exploreWorkingCopy)
			('recompile all packages' #recompileAllPackages)
			('fix all obsolete references' #fixAllObsoleteReferences )).			
	 
	^ aMenu
! !

!MCWorkingOrphanage methodsFor: 'as yet unclassified' stamp: 'kph 5/13/2007 00:57'!
recompilePackage
	
	"for the orphanage this means attempt to re-home them"
	
	 
	self package snapshot install.! !

!MCWorkingOrphanage methodsFor: 'as yet unclassified' stamp: 'kph 5/11/2007 05:04'!
unload! !

Object subclass: #MCPasswordManager
	instanceVariableNames: 'directory'
	classVariableNames: 'Default'
	poolDictionaries: ''
	category: 'Monticello-Repositories'!

!MCPasswordManager commentStamp: '<historical>' prior: 0!
Simple password manager to avoid storing passwords in the image.!

!MCPasswordManager class methodsFor: 'accessing' stamp: 'ar 11/4/2005 15:23'!
default
	^Default ifNil:[Default := self new]! !

!MCPasswordManager class methodsFor: 'accessing' stamp: 'ar 11/4/2005 16:53'!
editPasswords
	^self default editPasswords! !

!MCPasswordManager class methodsFor: 'accessing' stamp: 'ar 11/4/2005 16:14'!
passwordAt: location user: user
	^self default passwordAt: location user: user! !

!MCPasswordManager class methodsFor: 'accessing' stamp: 'ar 11/4/2005 16:14'!
passwordAt: location user: user put: password
	^self default passwordAt: location user: user put: password! !

!MCPasswordManager class methodsFor: 'accessing' stamp: 'ar 11/4/2005 16:37'!
queryPasswordAt: location user: user
	^self default queryPasswordAt: location user: user! !

!MCPasswordManager methodsFor: 'accessing' stamp: 'bf 11/7/2005 14:58'!
directory
	"Answer the directory in which to find the passwords"
	^directory ifNil:[ExternalSettings assuredPreferenceDirectory]! !

!MCPasswordManager methodsFor: 'accessing' stamp: 'ar 11/4/2005 15:01'!
directory: aDirectory
	"Indicate the directory in which to find the passwords"
	directory := aDirectory! !

!MCPasswordManager methodsFor: 'editing' stamp: 'ar 11/4/2005 16:48'!
editPasswords
	"Edit the passwords"
	(self directory fileNamesMatching: '*.pwd') do:[:fName|
		self editPasswordsIn: (self directory fullNameFor: fName).
	].! !

!MCPasswordManager methodsFor: 'editing' stamp: 'ar 11/4/2005 16:53'!
editPasswordsIn: pwdFile
	"Edit the passwords"
	| file data |
	file := FileStream readOnlyFileNamed: pwdFile.
	data := file contents.
	file close.
	UIManager default edit: data label: pwdFile accept:[:text|
		file := FileStream forceNewFileNamed: pwdFile.
		file nextPutAll: text asString.
		file close.
	].

! !

!MCPasswordManager methodsFor: 'queries' stamp: 'ar 11/4/2005 17:14'!
passwordAt: location user: user
	"Answer the password stored under the given key, or nil if none can be found"
	| file stream tokens pwdFile |
	(location indexOf: Character space) = 0 
		ifFalse:[^self error: 'Location must not contain spaces'].
	(user indexOf: Character space) = 0 
		ifFalse:[^self error: 'User name must not contain spaces'].
	pwdFile := user,'.pwd'.
	file := [self directory readOnlyFileNamed: pwdFile] on: Error do:[:ex| ex return: nil].
	file ifNil:[^nil].
	[stream := file contents readStream] ensure:[file close].
	[stream atEnd] whileFalse:[
		tokens := stream nextLine findTokens: ' '.
		(tokens size = 2 and:[tokens first match: location]) ifTrue:[
			^(Base64MimeConverter mimeDecode: tokens last as: String)
		].
	].
	^nil! !

!MCPasswordManager methodsFor: 'queries' stamp: 'kph 6/3/2008 11:36'!
passwordAt: location user: user put: password
	"Store the password under the given key"
	| file tokens pwd in out done sz pwdFile |
	user ifNil:[ ^ self ].
	(location indexOf: Character space) = 0 
		ifFalse:[^self error: 'Key must not contain spaces'].
	(user indexOf: Character space) = 0 
		ifFalse:[^self error: 'Key must not contain spaces'].
	password ifNotNil:[
		pwd := (Base64MimeConverter mimeEncode: password readStream) contents.
	].
	pwdFile := user,'.pwd'.
	file := [self directory readOnlyFileNamed: pwdFile] on: Error do:[:ex| ex return: nil].
	file ifNotNil:[
		[in := file contents readStream] ensure:[file close].
	] ifNil:[in := String new readStream].
	out := WriteStream on: (String new: 1000).

	done := pwd == nil. "if clearing passwords, we're done already"
	[in atEnd] whileFalse:[
		tokens := in nextLine findTokens: ' '.
		tokens size = 2 ifTrue:[
			(tokens first match: location) 
				ifTrue:[pwd ifNotNil:[out nextPutAll: location; space; nextPutAll: pwd; cr. done := true]]
				ifFalse:[out nextPutAll: tokens first; space; nextPutAll: tokens last; cr]]].
	done ifFalse:[out nextPutAll: location; space; nextPutAll: pwd; cr].

	file := [self directory forceNewFileNamed: pwdFile] on: Error do:[:ex| ex return: nil].
	file ifNil:[^nil].
	[file nextPutAll: out contents.
	sz := file size] ensure:[file close].
	sz = 0 ifTrue:[self directory deleteFileNamed: pwdFile ifAbsent:[]].
! !

!MCPasswordManager methodsFor: 'queries' stamp: 'kph 12/19/2007 00:18'!
queryPasswordAt: location user: user
	"Answer the password for the given user/location. 
	If the password is absent, query the user if interactive."
	| pwd |
	"search for existing password"
 
	pwd := self passwordAt: location user: user.
	pwd ifNotNil:[^pwd].

	user = 'squeak' ifTrue: [ ^ 'squeak' ].
	user = '' ifTrue: [ ^ 'squeak' ].

	pwd := FillInTheBlank requestPassword: 'Password for "', user, '" at ', location.
	pwd isEmptyOrNil ifTrue:[^nil].
	(self confirm: 'Remember password for "', user, '" at ', location,'?') ifTrue:[
		self passwordAt: location user: user put: pwd.
	].
	^pwd
	
 ! !

Object subclass: #MCPatch
	instanceVariableNames: 'operations'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Patching'!

!MCPatch class methodsFor: 'as yet unclassified' stamp: 'mtf 10/7/2008 13:01'!
fromBase: baseSnapshot target: targetSnapshot
	^ self new initializeWithBase: baseSnapshot definitionIndex
		target: targetSnapshot definitionIndex! !

!MCPatch class methodsFor: 'as yet unclassified' stamp: 'avi 9/11/2004 15:50'!
operations: aCollection
	^ self basicNew initializeWithOperations: aCollection! !

!MCPatch methodsFor: 'applying' stamp: 'ab 5/24/2003 16:12'!
applyTo: anObject
	operations do: [:ea | ea applyTo: anObject].
! !

!MCPatch methodsFor: 'ui' stamp: 'cwp 8/2/2003 13:34'!
browse
	^ (MCPatchBrowser forPatch: self) show! !

!MCPatch methodsFor: 'intializing' stamp: 'mtf 10/7/2008 11:39'!
initializeWithBase: base target: target
"Generates operation from the given MCDefinitionIndexes"

	operations := OrderedCollection new.
	
	target definitions do:
		[:t |
		base
			definitionLike: t
			ifPresent: [:b | (b isSameRevisionAs: t) ifFalse: [operations add: (MCModification of: b to: t)]]
			ifAbsent: [operations add: (MCAddition of: t)]]
		displayingProgress: 'Diffing...'.
		
	base definitions do:
		[:b |
		target
			definitionLike: b
			ifPresent: [:t]
			ifAbsent: [operations add: (MCRemoval of: b)]]		! !

!MCPatch methodsFor: 'intializing' stamp: 'stephaneducasse 2/4/2006 20:47'!
initializeWithOperations: aCollection
	operations := aCollection! !

!MCPatch methodsFor: 'querying' stamp: 'cwp 6/9/2003 11:53'!
isEmpty
	^ operations isEmpty! !

!MCPatch methodsFor: 'accessing' stamp: 'ab 5/13/2003 12:18'!
operations
	^ operations! !

Object subclass: #MCPatchOperation
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Patching'!

MCPatchOperation subclass: #MCAddition
	instanceVariableNames: 'definition'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Patching'!

!MCAddition class methodsFor: 'as yet unclassified' stamp: 'cwp 11/27/2002 10:03'!
of: aDefinition
	^ self new intializeWithDefinition: aDefinition! !

!MCAddition methodsFor: 'accessing' stamp: 'ab 5/24/2003 16:11'!
applyTo: anObject
	anObject addDefinition: definition! !

!MCAddition methodsFor: 'accessing' stamp: 'ab 6/1/2003 13:09'!
baseDefinition
	^ nil! !

!MCAddition methodsFor: 'accessing' stamp: 'cwp 11/27/2002 09:32'!
definition
	^ definition! !

!MCAddition methodsFor: 'accessing' stamp: 'ab 7/18/2003 16:45'!
fromSource
	^ ''! !

!MCAddition methodsFor: 'initializing' stamp: 'stephaneducasse 2/4/2006 20:47'!
intializeWithDefinition: aDefinition
	definition := aDefinition! !

!MCAddition methodsFor: 'as yet unclassified' stamp: 'ab 8/22/2003 02:26'!
inverse
	^ MCRemoval of: definition! !

!MCAddition methodsFor: 'testing' stamp: 'cwp 11/28/2002 07:22'!
isAddition
	^ true! !

!MCAddition methodsFor: 'as yet unclassified' stamp: 'nk 2/25/2005 17:28'!
isClassPatch
	^definition isClassDefinition! !

!MCAddition methodsFor: 'accessing' stamp: 'nk 2/23/2005 18:39'!
sourceString
	^(self toSource asText)
		addAttribute: TextColor red;
		yourself! !

!MCAddition methodsFor: 'accessing' stamp: 'ab 5/13/2003 12:18'!
summary
	^ definition summary! !

!MCAddition methodsFor: 'accessing' stamp: 'nk 2/23/2005 18:17'!
targetClass
	^definition actualClass ! !

!MCAddition methodsFor: 'accessing' stamp: 'ab 6/1/2003 13:09'!
targetDefinition
	^ definition! !

!MCAddition methodsFor: 'accessing' stamp: 'ab 7/18/2003 16:45'!
toSource
	^ definition source! !

MCPatchOperation subclass: #MCModification
	instanceVariableNames: 'obsoletion modification'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Patching'!

!MCModification class methodsFor: 'as yet unclassified' stamp: 'cwp 11/28/2002 07:19'!
of: base to: target
	^ self new initializeWithBase: base target: target! !

!MCModification methodsFor: 'accessing' stamp: 'ab 5/24/2003 16:12'!
applyTo: anObject
	anObject modifyDefinition: obsoletion to: modification! !

!MCModification methodsFor: 'accessing' stamp: 'ab 6/1/2003 13:10'!
baseDefinition
	^ obsoletion! !

!MCModification methodsFor: 'accessing' stamp: 'cwp 11/28/2002 06:55'!
definition
	^ modification! !

!MCModification methodsFor: 'accessing' stamp: 'ab 7/18/2003 16:44'!
fromSource
	^ obsoletion source! !

!MCModification methodsFor: 'initializing' stamp: 'stephaneducasse 2/4/2006 20:47'!
initializeWithBase: base target: target
	obsoletion := base.
	modification := target.! !

!MCModification methodsFor: 'as yet unclassified' stamp: 'ab 8/22/2003 02:27'!
inverse
	^ MCModification of: modification to: obsoletion! !

!MCModification methodsFor: 'as yet unclassified' stamp: 'nk 2/25/2005 17:29'!
isClassPatch
	^obsoletion isClassDefinition! !

!MCModification methodsFor: 'testing' stamp: 'cwp 11/28/2002 07:23'!
isModification
	^ true! !

!MCModification methodsFor: 'accessing' stamp: 'cwp 11/27/2002 09:46'!
modification
	^ modification! !

!MCModification methodsFor: 'accessing' stamp: 'cwp 11/27/2002 09:48'!
obsoletion
	^ obsoletion! !

!MCModification methodsFor: 'as yet unclassified' stamp: 'nk 11/10/2003 21:44'!
printAnnotations: request on: aStream
	aStream nextPutAll: 'old: '.
	obsoletion printAnnotations: request on: aStream.
	aStream cr.
	aStream nextPutAll: 'new: '.
	modification printAnnotations: request on: aStream.! !

!MCModification methodsFor: 'accessing' stamp: 'nk 10/21/2003 22:54'!
summarySuffix
	^self fromSource = self toSource
		ifTrue: [ ' (source same but rev changed)' ]
		ifFalse: [ ' (changed)' ]! !

!MCModification methodsFor: 'accessing' stamp: 'nk 2/23/2005 18:18'!
targetClass
	^ obsoletion actualClass! !

!MCModification methodsFor: 'accessing' stamp: 'ab 6/1/2003 13:10'!
targetDefinition
	^ modification! !

!MCModification methodsFor: 'accessing' stamp: 'ab 7/18/2003 16:44'!
toSource
	^ modification source! !

!MCPatchOperation methodsFor: 'comparing' stamp: 'ab 7/19/2003 18:11'!
<= other
	^ self definition <= other definition! !

!MCPatchOperation methodsFor: 'accessing' stamp: 'nk 11/10/2003 21:38'!
annotations
	^self annotations: Preferences defaultAnnotationRequests! !

!MCPatchOperation methodsFor: 'accessing' stamp: 'nk 11/10/2003 21:39'!
annotations: requests
	"Answer a string for an annotation pane, trying to fulfill the annotation requests.
	These might include anything that
		Preferences defaultAnnotationRequests 
	might return. Which includes anything in
		Preferences annotationInfo
	To edit these, use:"
	"Preferences editAnnotations"

	^String streamContents: [ :s | self printAnnotations: requests on: s ].! !

!MCPatchOperation methodsFor: 'accessing' stamp: 'cwp 11/28/2002 06:59'!
definition
	^ self subclassResponsibility ! !

!MCPatchOperation methodsFor: 'accessing' stamp: 'cwp 11/27/2002 09:26'!
inverse
	self subclassResponsibility! !

!MCPatchOperation methodsFor: 'testing' stamp: 'cwp 11/27/2002 09:30'!
isAddition
	^ false! !

!MCPatchOperation methodsFor: 'as yet unclassified' stamp: 'nk 2/25/2005 17:28'!
isClassPatch
	^false! !

!MCPatchOperation methodsFor: 'testing' stamp: 'cwp 11/27/2002 09:30'!
isModification
	^ false! !

!MCPatchOperation methodsFor: 'testing' stamp: 'cwp 11/27/2002 09:30'!
isRemoval
	^ false! !

!MCPatchOperation methodsFor: 'accessing' stamp: 'avi 8/31/2003 17:53'!
prefixForOperation: aSymbol
	aSymbol == #insert ifTrue: [^ '+'].
	aSymbol == #remove ifTrue: [^ '-'].
	^ ' '! !

!MCPatchOperation methodsFor: 'accessing' stamp: 'nk 11/10/2003 21:40'!
printAnnotations: requests on: aStream
	"Add a string for an annotation pane, trying to fulfill the annotation requests.
	These might include anything that
		Preferences defaultAnnotationRequests 
	might return. Which includes anything in
		Preferences annotationInfo
	To edit these, use:"
	"Preferences editAnnotations"

	self definition printAnnotations: requests on: aStream.! !

!MCPatchOperation methodsFor: 'accessing' stamp: 'avi 8/31/2003 17:55'!
source
	^ self sourceText! !

!MCPatchOperation methodsFor: 'accessing' stamp: 'nk 2/25/2005 17:26'!
sourceString
	^self sourceText asString! !

!MCPatchOperation methodsFor: 'accessing' stamp: 'nk 2/25/2005 17:29'!
sourceText
	| builder |
	builder := (Preferences diffsWithPrettyPrint and: [ self targetClass notNil and: [ self isClassPatch not ] ])
				ifTrue: 
					[PrettyTextDiffBuilder 
						from: self fromSource
						to: self toSource
						inClass: self targetClass]
				ifFalse: [TextDiffBuilder from: self fromSource to: self toSource].
	^builder buildDisplayPatch.! !

!MCPatchOperation methodsFor: 'accessing' stamp: 'ab 7/6/2003 00:06'!
summary
	^ self definition summary, self summarySuffix! !

!MCPatchOperation methodsFor: 'accessing' stamp: 'ab 7/6/2003 00:06'!
summarySuffix
	^ ''! !

!MCPatchOperation methodsFor: 'as yet unclassified' stamp: 'nk 2/23/2005 18:17'!
targetClass
	self subclassResponsibility.! !

MCPatchOperation subclass: #MCRemoval
	instanceVariableNames: 'definition'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Patching'!

!MCRemoval class methodsFor: 'as yet unclassified' stamp: 'cwp 11/27/2002 10:03'!
of: aDefinition
	^ self new intializeWithDefinition: aDefinition! !

!MCRemoval methodsFor: 'accessing' stamp: 'ab 5/24/2003 16:11'!
applyTo: anObject
	anObject removeDefinition: definition! !

!MCRemoval methodsFor: 'accessing' stamp: 'ab 6/1/2003 13:10'!
baseDefinition
	^ definition! !

!MCRemoval methodsFor: 'accessing' stamp: 'cwp 11/27/2002 10:02'!
definition
	^ definition! !

!MCRemoval methodsFor: 'accessing' stamp: 'ab 7/18/2003 16:44'!
fromSource
	^ definition source! !

!MCRemoval methodsFor: 'initializing' stamp: 'stephaneducasse 2/4/2006 20:47'!
intializeWithDefinition: aDefinition
	definition := aDefinition! !

!MCRemoval methodsFor: 'as yet unclassified' stamp: 'ab 8/22/2003 02:26'!
inverse
	^ MCAddition of: definition! !

!MCRemoval methodsFor: 'as yet unclassified' stamp: 'nk 2/25/2005 17:28'!
isClassPatch
	^definition isClassDefinition! !

!MCRemoval methodsFor: 'testing' stamp: 'cwp 11/28/2002 07:24'!
isRemoval
	^ true! !

!MCRemoval methodsFor: 'accessing' stamp: 'nk 2/23/2005 18:38'!
sourceString
	^self fromSource asText
		addAttribute: TextEmphasis struckOut;
		addAttribute: TextColor blue;
		yourself! !

!MCRemoval methodsFor: 'accessing' stamp: 'ab 5/13/2003 12:22'!
summary
	^ definition summary, ' (removed)'! !

!MCRemoval methodsFor: 'accessing' stamp: 'ab 7/6/2003 00:05'!
summarySuffix
	^ ' (removed)'! !

!MCRemoval methodsFor: 'accessing' stamp: 'nk 2/25/2005 17:23'!
targetClass
	^ definition actualClass! !

!MCRemoval methodsFor: 'accessing' stamp: 'ab 6/1/2003 13:10'!
targetDefinition
	^ nil! !

!MCRemoval methodsFor: 'accessing' stamp: 'ab 7/18/2003 16:44'!
toSource
	^ ''! !

Object subclass: #MCPatcher
	instanceVariableNames: 'definitions'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Patching'!

!MCPatcher class methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
apply: aPatch to: aSnapshot
	| loader |
	loader := self snapshot: aSnapshot.
	aPatch applyTo: loader.
	^ loader patchedSnapshot! !

!MCPatcher class methodsFor: 'as yet unclassified' stamp: 'ab 6/1/2003 14:22'!
snapshot: aSnapshot
	^ self new initializeWithSnapshot: aSnapshot! !

!MCPatcher methodsFor: 'as yet unclassified' stamp: 'ab 6/2/2003 00:46'!
addDefinition: aDefinition
	definitions add: aDefinition! !

!MCPatcher methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
initializeWithSnapshot: aSnapshot
	definitions := MCDefinitionIndex definitions: aSnapshot definitions! !

!MCPatcher methodsFor: 'as yet unclassified' stamp: 'ab 6/1/2003 14:23'!
modifyDefinition: baseDefinition to: targetDefinition
	self addDefinition: targetDefinition! !

!MCPatcher methodsFor: 'as yet unclassified' stamp: 'ab 7/6/2003 23:48'!
patchedSnapshot
	^ MCSnapshot fromDefinitions: definitions definitions! !

!MCPatcher methodsFor: 'as yet unclassified' stamp: 'ab 6/2/2003 00:46'!
removeDefinition: aDefinition
	definitions remove: aDefinition! !

Object subclass: #MCReader
	instanceVariableNames: 'stream'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Storing'!

!MCReader class methodsFor: 'testing' stamp: 'avi 1/21/2004 19:00'!
canReadFileNamed: fileName
	^ (fileName endsWith: self extension)! !

!MCReader class methodsFor: 'testing' stamp: 'avi 1/21/2004 19:01'!
concreteSubclasses
	^ self allSubclasses reject: [:c | c isAbstract]! !

!MCReader class methodsFor: 'testing' stamp: 'avi 1/21/2004 19:01'!
isAbstract
	^ (self respondsTo: #extension) not! !

!MCReader class methodsFor: 'instance creation' stamp: 'avi 1/21/2004 19:02'!
on: aStream
	^ self new stream: aStream! !

!MCReader class methodsFor: 'instance creation' stamp: 'stephaneducasse 2/4/2006 20:47'!
on: aStream name: aFileName
	| class |
	class := self readerClassForFileNamed: aFileName.
	^ class
		ifNil: [self error: 'Unsupported format: ', aFileName]
		ifNotNil: [class on: aStream]! !

!MCReader class methodsFor: 'testing' stamp: 'avi 1/21/2004 19:03'!
readerClassForFileNamed: fileName
	^ self concreteSubclasses
		detect: [:c | c canReadFileNamed: fileName]
		ifNone: [nil]! !

!MCReader methodsFor: 'accessing' stamp: 'stephaneducasse 2/4/2006 20:47'!
stream: aStream
	stream := aStream! !

MCReader subclass: #MCSnapshotReader
	instanceVariableNames: 'definitions'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Storing'!

!MCSnapshotReader class methodsFor: 'as yet unclassified' stamp: 'avi 1/21/2004 22:56'!
snapshotFromStream: aStream
	^ (self on: aStream) snapshot! !

!MCSnapshotReader methodsFor: 'as yet unclassified' stamp: 'avi 1/21/2004 23:09'!
definitions
	definitions ifNil: [self loadDefinitions].
	^ definitions! !

!MCSnapshotReader methodsFor: 'as yet unclassified' stamp: 'avi 1/21/2004 23:10'!
snapshot
	^ MCSnapshot fromDefinitions: self definitions! !

MCSnapshotReader subclass: #MCStReader
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Chunk Format'!

MCStReader subclass: #MCCsReader
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Chunk Format'!

!MCCsReader commentStamp: 'abc 7/4/2007 01:25' prior: 0!
In order to designate a changeset for loading with Monticello use the extension .mcs 
Monticello does not execute preambles or postscripts.!

!MCCsReader class methodsFor: 'as yet unclassified' stamp: 'abc 7/4/2007 01:17'!
extension
	^ 'mcs'! !

!MCStReader class methodsFor: 'as yet unclassified' stamp: 'avi 1/20/2004 00:17'!
extension
	^ 'st'! !

!MCStReader methodsFor: 'as yet unclassified' stamp: 'avi 3/10/2004 12:52'!
addDefinitionsFromDoit: aString
	(MCDoItParser forDoit: aString) ifNotNilDo:
		[:parser |
		parser addDefinitionsTo: definitions]! !

!MCStReader methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
categoryFromDoIt: aString
	| tokens  |
	tokens := Scanner new scanTokens: aString.
	tokens size = 3 ifFalse: [self error: 'Unrecognized category definition'].
	^ tokens at: 3! !

!MCStReader methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
classDefinitionFrom: aPseudoClass
	| tokens traitCompositionString lastIndex classTraitCompositionString |
	tokens := Scanner new scanTokens: aPseudoClass definition.
	traitCompositionString := ((ReadStream on: aPseudoClass definition)
		match: 'uses:';
		upToAll: 'instanceVariableNames:') withBlanksTrimmed.
	classTraitCompositionString := ((ReadStream on: aPseudoClass metaClass definition asString)
		match: 'uses:';
		upToAll: 'instanceVariableNames:') withBlanksTrimmed.
	traitCompositionString isEmpty ifTrue: [traitCompositionString := '{}'].
	classTraitCompositionString isEmpty ifTrue: [classTraitCompositionString := '{}'].
	lastIndex := tokens size.
	^ MCClassDefinition
		name: (tokens at: 3)
		superclassName: (tokens at: 1)
		traitComposition: traitCompositionString
		classTraitComposition: classTraitCompositionString
		category: (tokens at: lastIndex)
		instVarNames: ((tokens at: lastIndex - 6) findTokens: ' ')
		classVarNames: ((tokens at: lastIndex - 4) findTokens: ' ')
		poolDictionaryNames: ((tokens at: lastIndex - 2) findTokens: ' ')
		classInstVarNames: (self classInstVarNamesFor: aPseudoClass)
		type: (self typeOfSubclass: (tokens at: 2))
		comment: (self commentFor: aPseudoClass)
		commentStamp: (self commentStampFor: aPseudoClass)! !

!MCStReader methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
classInstVarNamesFor: aPseudoClass
	| tokens |
	
	self flag: #traits.
	aPseudoClass metaClass hasDefinition ifFalse: [^ #()].
	tokens := Scanner new scanTokens: aPseudoClass metaClass definition.
	"tokens size = 4 ifFalse: [self error: 'Unrecognized metaclass definition']."
	^ tokens last findTokens: ' '! !

!MCStReader methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
commentFor: aPseudoClass
	| comment |
	comment := aPseudoClass organization classComment.
	^ comment asString = ''
		ifTrue: [comment]
		ifFalse: [comment string]! !

!MCStReader methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
commentStampFor: aPseudoClass
	| comment |
	comment := aPseudoClass organization classComment.
	^  [comment stamp] on: MessageNotUnderstood do: [nil]! !

!MCStReader methodsFor: 'evaluating' stamp: 'stephaneducasse 2/4/2006 20:47'!
loadDefinitions
	| filePackage |
	filePackage :=
		FilePackage new
			fullName: 'ReadStream';
			fileInFrom: self readStream.
	definitions := OrderedCollection new.
	filePackage classes do:
		[:pseudoClass |
		pseudoClass hasDefinition
			ifTrue: [definitions add:
					(self classDefinitionFrom: pseudoClass)].
		definitions addAll: (self methodDefinitionsFor: pseudoClass).
		definitions addAll: (self methodDefinitionsFor: pseudoClass metaClass)].
	filePackage doIts do:
		[:ea |
		self addDefinitionsFromDoit: ea string].
	! !

!MCStReader methodsFor: 'as yet unclassified' stamp: 'avi 3/3/2004 15:23'!
methodDefinitionsFor: aPseudoClass
	^ aPseudoClass selectors collect: 
		[:ea |
		 MCMethodDefinition
			className: aPseudoClass name
			classIsMeta: aPseudoClass isMeta
			selector: ea
			category: (aPseudoClass organization categoryOfElement: ea)
			timeStamp: (aPseudoClass stampAt: ea)
			source: (aPseudoClass sourceCodeAt: ea)]! !

!MCStReader methodsFor: 'evaluating' stamp: 'avi 1/21/2004 14:21'!
readStream
	^ ('!!!!

', stream contents) readStream! !

!MCStReader methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
systemOrganizationFromRecords: changeRecords
	| categories |
	categories := changeRecords
					select: [:ea | 'SystemOrganization*' match: ea string]
					thenCollect: [:ea | (self categoryFromDoIt: ea string)].
	^ categories isEmpty ifFalse: [MCOrganizationDefinition categories: categories asArray]! !

!MCStReader methodsFor: 'as yet unclassified' stamp: 'avi 1/19/2004 21:56'!
typeOfSubclass: aSymbol
	#(
		(subclass: normal)
		(variableSubclass: variable)
		(variableByteSubclass: bytes)
		(variableWordSubclass: words)
		(weakSubclass: weak)
		) do: [:ea | ea first = aSymbol ifTrue: [^ ea second]].
	self error: 'Unrecognized class definition'! !

MCReader subclass: #MCVersionReader
	instanceVariableNames: 'package info definitions dependencies stepChildren'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Storing'!

!MCVersionReader class methodsFor: 'file services' stamp: 'avi 1/24/2004 19:01'!
initialize
	"MCVersionReader initialize"
	Smalltalk 
		at: #MczInstaller
		ifPresent: [:installer | FileList unregisterFileReader: installer].
	self concreteSubclasses do: [:aClass | FileList registerFileReader: aClass].

	"get rid of AnObsoleteMCMcReader and AnObsoleteMCMcvReader"
	(FileList registeredFileReaderClasses  select: [ :ea | ea isObsolete ]) do: 
		[ :ea | FileList unregisterFileReader: ea ]
! !

MCVersionReader subclass: #MCMczReader
	instanceVariableNames: 'zip infoCache'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Storing'!

MCMczReader subclass: #MCMcdReader
	instanceVariableNames: 'baseInfo patch'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Storing'!

!MCMcdReader class methodsFor: 'as yet unclassified' stamp: 'avi 2/13/2004 23:09'!
extension
	^ 'mcd'! !

!MCMcdReader methodsFor: 'as yet unclassified' stamp: 'avi 2/14/2004 21:33'!
baseInfo
	^ baseInfo ifNil: [self loadBaseInfo]! !

!MCMcdReader methodsFor: 'as yet unclassified' stamp: 'avi 2/13/2004 23:41'!
basicVersion
	^ MCDiffyVersion
		package: self package
		info: self info
		dependencies: self dependencies
		baseInfo: self baseInfo
		patch: self patch! !

!MCMcdReader methodsFor: 'as yet unclassified' stamp: 'avi 2/14/2004 21:37'!
buildPatchFrom: oldDefinitions to: newDefinitions
	^ MCPatch
		fromBase: (MCSnapshot fromDefinitions: oldDefinitions)
		target: (MCSnapshot fromDefinitions: newDefinitions)! !

!MCMcdReader methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
loadBaseInfo
	^ baseInfo := self extractInfoFrom: (self parseMember: 'base')! !

!MCMcdReader methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
loadPatch
	| old new |
	(self zip memberNamed: 'patch.bin') ifNotNilDo:
		[:m | [^ patch := (DataStream on: m contentStream) next ]
			on: Error do: [:fallThrough ]].
	definitions := OrderedCollection new.
	(self zip membersMatching: 'old/*')
		do: [:m | self extractDefinitionsFrom: m].
	old := definitions asArray.
	definitions := OrderedCollection new.
	(self zip membersMatching: 'new/*')
		do: [:m | self extractDefinitionsFrom: m].
	new := definitions asArray.
	^ patch := self buildPatchFrom: old to: new.
	! !

!MCMcdReader methodsFor: 'as yet unclassified' stamp: 'avi 2/14/2004 21:34'!
patch
	^ patch ifNil: [self loadPatch]! !

!MCMczReader class methodsFor: 'accessing' stamp: 'cwp 8/1/2003 14:59'!
extension
	^ 'mcz'! !

!MCMczReader class methodsFor: 'testing' stamp: 'avi 1/19/2004 14:48'!
supportsDependencies
	^ true! !

!MCMczReader class methodsFor: 'testing' stamp: 'cwp 8/1/2003 12:19'!
supportsVersions
	^ true! !

!MCMczReader methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
associate: tokens
	| result |
	result := Dictionary new.
	tokens pairsDo: [:key :value | 
					value isString ifFalse: [value := value collect: [:ea | self associate: ea]].
					result at: key put: value].
	^ result! !

!MCMczReader methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
extractDefinitionsFrom: member
	| reader |
	(MCSnapshotReader readerClassForFileNamed: member fileName)
		ifNotNilDo: [:rc | reader := rc on: member contentStream text.
					definitions addAll: reader definitions]
! !

!MCMczReader methodsFor: 'as yet unclassified' stamp: 'avi 1/19/2004 16:11'!
extractDependencyFrom: zipMember
	^ MCVersionDependency
		package: (MCPackage named: (zipMember fileName copyAfterLast: $/))
		info: (self extractInfoFrom: (self parseMember: zipMember fileName))! !

!MCMczReader methodsFor: 'as yet unclassified' stamp: 'mtf 1/18/2008 10:11'!
extractInfoFrom: dict
	^ self infoCache at: (dict at: #id) ifAbsentPut:
		[MCVersionInfo
			name: (dict at: #name ifAbsent: [''])
			id: (UUID fromString: (dict at: #id))
			message: (dict at: #message ifAbsent: [''])
			date: ([Date fromString: (dict at: #date) ] on: Error do: [ :ex | ex return: nil ])
			time: ([ Time fromString:(dict at: #time)] on: Error do: [ :ex | ex return: nil ])
			author: (dict at: #author ifAbsent: [''])
			ancestors: (dict at: #ancestors ifPresent: [:ancestors |
				ancestors collect: [:ea | self extractInfoFrom: ea]])
			stepChildren: ((dict at: #stepChildren ifAbsent: [#()]) collect: [:ea | self extractInfoFrom: ea])]! !

!MCMczReader methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
infoCache
	^ infoCache ifNil: [infoCache := Dictionary new]! !

!MCMczReader methodsFor: 'as yet unclassified' stamp: 'kph 6/5/2008 00:51'!
loadDefinitions
	definitions := OrderedCollection new.
	(self zip memberNamed: 'snapshot.bin') ifNotNilDo:
		[:m | [^ definitions := (DataStream on: m contentStream) next definitions]
			on: Error do: [:fallThrough |  self halt ]].
	"otherwise"
	(self zip membersMatching: 'snapshot/*')
		do: [:m | self extractDefinitionsFrom: m].
! !

!MCMczReader methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
loadDependencies
	dependencies := (self zip membersMatching: 'dependencies/*') collect: [:m | self extractDependencyFrom: m].
	dependencies := dependencies asArray.
! !

!MCMczReader methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
loadPackage
	| dict |
	dict := self parseMember: 'package'.
	package := MCPackage named: (dict at: #name)! !

!MCMczReader methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
loadVersionInfo
	info := self extractInfoFrom: (self parseMember: 'version')! !

!MCMczReader methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
parseMember: fileName
	| tokens |
	tokens := (self scanner scanTokens: (self zip contentsOf: fileName)) first.
	^ self associate: tokens! !

!MCMczReader methodsFor: 'as yet unclassified' stamp: 'avi 1/22/2004 20:33'!
scanner
	^ MCScanner! !

!MCMczReader methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
zip
	zip ifNil:
		[zip := ZipArchive new.
		zip readFrom: stream].
	^ zip! !

!MCVersionReader class methodsFor: 'reading' stamp: 'stephaneducasse 2/4/2006 20:47'!
file: fileName streamDo: aBlock
	| file |
	^ 	[file := FileStream readOnlyFileNamed: fileName.
		aBlock value: file]
			ensure: [file close]! !

!MCVersionReader class methodsFor: 'file services' stamp: 'avi 10/15/2003 02:01'!
fileReaderServicesForFile: fullName suffix: suffix
	self isAbstract ifTrue: [^ #()].
	^ ((suffix = self extension) or: [ suffix = '*' ])
		ifTrue: [self services]
		ifFalse: [Array new: 0]
		! !

!MCVersionReader class methodsFor: 'file services' stamp: 'kph 8/16/2007 17:43'!
installFileNamed: filename

 "this method makes invocation of MczInstaller and MczReader similar"
 "fix for mc < 1.5"

 ^ self loadVersionFile: filename! !

!MCVersionReader class methodsFor: 'file services' stamp: 'stephaneducasse 2/4/2006 20:47'!
loadVersionFile: fileName
	| version |
	version := self versionFromFile: fileName.
	version workingCopy repositoryGroup addRepository:
		(MCDirectoryRepository new directory:
			(FileDirectory on: (FileDirectory dirPathFor: fileName))).
	version load.
! !

!MCVersionReader class methodsFor: '*monticello-file services' stamp: 'stephaneducasse 2/4/2006 20:47'!
loadVersionStream: stream fromDirectory: directory
	| version |
	version := self versionFromStream: stream.
	directory isRemoteDirectory ifFalse: [
	version workingCopy repositoryGroup addRepository:
		(MCDirectoryRepository new directory: directory). ].
	version load.
! !

!MCVersionReader class methodsFor: 'file services' stamp: 'cwp 8/1/2003 14:46'!
mergeVersionFile: fileName
	(self versionFromFile: fileName) merge! !

!MCVersionReader class methodsFor: '*monticello-file services' stamp: 'nk 2/25/2005 11:17'!
mergeVersionStream: stream
	(self versionFromStream: stream) merge! !

!MCVersionReader class methodsFor: 'reading' stamp: 'bf 3/23/2005 01:20'!
on: s fileName: f
	^ self on: s! !

!MCVersionReader class methodsFor: 'file services' stamp: 'cwp 8/1/2003 14:46'!
openVersionFile: fileName
	(self versionFromFile: fileName) open! !

!MCVersionReader class methodsFor: '*monticello-file services' stamp: 'nk 2/25/2005 11:12'!
openVersionFromStream: stream
	(self versionFromStream: stream) open! !

!MCVersionReader class methodsFor: '*monticello-file services-override-override' stamp: 'nk 2/25/2005 11:15'!
serviceLoadVersion
	^ (SimpleServiceEntry
		provider: self
		label: 'load version'
		selector: #loadVersionStream:fromDirectory:
		description: 'load a package version'
		buttonLabel: 'load')
		argumentGetter: [ :fileList | { fileList readOnlyStream . fileList directory } ]! !

!MCVersionReader class methodsFor: '*monticello-file services-override-override' stamp: 'nk 2/25/2005 11:16'!
serviceMergeVersion
	^ (SimpleServiceEntry
		provider: self
		label: 'merge version'
		selector: #mergeVersionStream:
		description: 'merge a package version into the image'
		buttonLabel: 'merge')
		argumentGetter: [ :fileList | fileList readOnlyStream ]! !

!MCVersionReader class methodsFor: '*monticello-file services-override-override' stamp: 'nk 2/25/2005 11:16'!
serviceOpenVersion
	^ (SimpleServiceEntry
		provider: self
		label: 'open version'
		selector: #openVersionFromStream:
		description: 'open a package version'
		buttonLabel: 'open')
		argumentGetter: [ :fileList | fileList readOnlyStream ]! !

!MCVersionReader class methodsFor: 'file services' stamp: 'avi 1/21/2004 22:55'!
services
	^ Array 
		with: self serviceLoadVersion
		with: self serviceMergeVersion
		with: self serviceOpenVersion! !

!MCVersionReader class methodsFor: 'reading' stamp: 'cwp 7/31/2003 23:03'!
versionFromFile: fileName
	^ self file: fileName streamDo: [:stream | self versionFromStream: stream]! !

!MCVersionReader class methodsFor: 'reading' stamp: 'avi 1/21/2004 22:58'!
versionFromStream: aStream
	^ (self on: aStream) version! !

!MCVersionReader class methodsFor: 'reading' stamp: 'avi 1/21/2004 22:59'!
versionInfoFromStream: aStream
	^ (self on: aStream) info! !

!MCVersionReader methodsFor: 'accessing' stamp: 'avi 1/19/2004 16:52'!
basicVersion
	^ MCVersion
		package: self package
		info: self info
		snapshot: self snapshot
		dependencies: self dependencies! !

!MCVersionReader methodsFor: 'accessing' stamp: 'avi 1/21/2004 23:10'!
definitions
	definitions ifNil: [self loadDefinitions].
	^ definitions! !

!MCVersionReader methodsFor: 'accessing' stamp: 'avi 1/19/2004 14:50'!
dependencies
	dependencies ifNil: [self loadDependencies].
	^ dependencies! !

!MCVersionReader methodsFor: 'accessing' stamp: 'ab 8/20/2003 19:53'!
info
	info ifNil: [self loadVersionInfo].
	^ info! !

!MCVersionReader methodsFor: 'loading' stamp: 'ab 8/20/2003 19:54'!
loadDefinitions
	self subclassResponsibility ! !

!MCVersionReader methodsFor: 'loading' stamp: 'avi 1/19/2004 14:50'!
loadDependencies
	self subclassResponsibility ! !

!MCVersionReader methodsFor: 'loading' stamp: 'ab 8/20/2003 19:54'!
loadPackage
	self subclassResponsibility ! !

!MCVersionReader methodsFor: 'loading' stamp: 'ab 8/20/2003 19:54'!
loadVersionInfo
	self subclassResponsibility! !

!MCVersionReader methodsFor: 'accessing' stamp: 'ab 8/20/2003 19:53'!
package
	package ifNil: [self loadPackage].
	^ package! !

!MCVersionReader methodsFor: 'accessing' stamp: 'kph 8/2/2007 15:21'!
snapshot
 
	^ MCSnapshot fromDefinitions: self definitions! !

!MCVersionReader methodsFor: 'accessing' stamp: 'avi 10/9/2003 12:38'!
version
	^ self basicVersion! !

!MCVersionReader class methodsFor: 'file services' stamp: 'cwp 8/1/2003 14:33'!
unload
	FileList unregisterFileReader: self ! !

Object subclass: #MCRepository
	instanceVariableNames: 'creationTemplate storeDiffs'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Repositories'!

MCRepository class
	instanceVariableNames: 'ui'!

!MCRepository commentStamp: 'kph 5/17/2007 14:31' prior: 0!
MCRepository subclasses appear in the UI list of avaiable repository types if they return #description.!

MCRepository class
	instanceVariableNames: 'ui'!

!MCRepository class methodsFor: 'class initialization' stamp: 'ar 11/4/2005 17:30'!
initialize
	"self initialize"
	ExternalSettings registeredClients remove: self ifAbsent:[].! !

MCRepository subclass: #MCDictionaryRepository
	instanceVariableNames: 'description dictionary'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Repositories'!

!MCDictionaryRepository class methodsFor: 'as yet unclassified' stamp: 'kph 5/16/2007 12:29'!
description

^ 'In Memory Cache'! !

!MCDictionaryRepository class methodsFor: 'as yet unclassified' stamp: 'kph 5/16/2007 12:44'!
morphicConfigure
	| nameIt |
	nameIt := FillInTheBlankMorph request: 'Name this cache:'.
	^ nameIt isEmpty ifFalse: [self new description: nameIt]! !

!MCDictionaryRepository methodsFor: 'as yet unclassified' stamp: 'ab 8/21/2003 12:56'!
= other
	^ self == other! !

!MCDictionaryRepository methodsFor: 'baseclass required' stamp: 'kph 5/16/2007 22:18'!
basicStoreVersion: aVersion
	dictionary at: aVersion info put: aVersion! !

!MCDictionaryRepository methodsFor: 'baseclass overrides' stamp: 'stephaneducasse 2/4/2006 20:47'!
closestAncestorVersionFor: anAncestry ifNone: errorBlock
	| info |
	info := anAncestry breadthFirstAncestors
			detect: [:ea | self includesVersionWithInfo: ea]
			ifNone: [^ errorBlock value].
	^ self versionWithInfo: info! !

!MCDictionaryRepository methodsFor: 'accessing' stamp: 'kph 5/16/2007 12:28'!
description

	^ description ifNil: [ self class description ]! !

!MCDictionaryRepository methodsFor: 'accessing' stamp: 'stephaneducasse 2/4/2006 20:47'!
description: aString

	description := aString ! !

!MCDictionaryRepository methodsFor: 'baseclass overrides' stamp: 'kph 5/16/2007 22:51'!
descriptionDisplay

	^ 'In Memory: ', self description! !

!MCDictionaryRepository methodsFor: 'accessing' stamp: 'kph 5/16/2007 22:17'!
dictionary

	^ dictionary! !

!MCDictionaryRepository methodsFor: 'accessing' stamp: 'kph 5/16/2007 22:17'!
dictionary: aDictionary

	dictionary := aDictionary! !

!MCDictionaryRepository methodsFor: 'baseclass overrides' stamp: 'kph 5/17/2007 00:23'!
heldVersionInfos
	^ dictionary keys! !

!MCDictionaryRepository methodsFor: 'baseclass required' stamp: 'kph 5/17/2007 00:20'!
heldVersions 

	^ dictionary values! !

!MCDictionaryRepository methodsFor: 'baseclass overrides' stamp: 'kph 5/16/2007 23:12'!
includesVersionWithInfo: aVersionInfo

	"quicker implementation than inherited"
	^ dictionary includesKey: aVersionInfo! !

!MCDictionaryRepository methodsFor: 'initialization' stamp: 'kph 5/16/2007 22:42'!
initialize

	dictionary := Dictionary new.
! !

!MCDictionaryRepository methodsFor: 'initialization' stamp: 'kph 5/16/2007 12:55'!
openAndEditTemplateCopy
 
	| nameIt |
	nameIt := FillInTheBlankMorph request: 'Rename In Memory Cache: ' initialAnswer: self description.
	^ nameIt isEmpty ifFalse: [self class new description: nameIt]! !

!MCDictionaryRepository methodsFor: 'baseclass required' stamp: 'kph 6/12/2008 23:01'!
storeVersion: aVersion
	aVersion info name: aVersion info name withBlanksTrimmed.
	aVersion info allAncestorsDo: [ :each | each name: each name withBlanksTrimmed ].

	^ super storeVersion: aVersion! !

!MCDictionaryRepository methodsFor: 'baseclass required' stamp: 'kph 5/16/2007 22:20'!
versionWithInfo: aVersionInfo ifAbsent: errorBlock
	^ dictionary at: aVersionInfo ifAbsent: errorBlock! !

MCDictionaryRepository subclass: #MCSystemChangeSetsRepository
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Repositories'!

!MCSystemChangeSetsRepository class methodsFor: 'as yet unclassified' stamp: 'kph 5/17/2007 14:30'!
description

	^ " 'System Change Sets'" nil! !

!MCSystemChangeSetsRepository class methodsFor: 'as yet unclassified' stamp: 'kph 5/17/2007 13:21'!
ignoreMonticelloChangeSets

	^ true! !

!MCSystemChangeSetsRepository class methodsFor: 'as yet unclassified' stamp: 'kph 5/17/2007 15:06'!
isValid

^ false! !

!MCSystemChangeSetsRepository class methodsFor: 'as yet unclassified' stamp: 'kph 5/17/2007 11:45'!
morphicConfigure
	 
	^ self new! !

!MCSystemChangeSetsRepository methodsFor: 'as yet unclassified' stamp: 'kph 5/17/2007 14:04'!
addCSFrom: cs 
	
	|   info |
	  
	info := MCSystemChangeSetVersionInfo on: cs.
	
	dictionary 	at: info 
				put: info 
	 ! !

!MCSystemChangeSetsRepository methodsFor: 'as yet unclassified' stamp: 'kph 5/17/2007 13:55'!
addMCVersionFrom: cs 
	
	| filename wc info |
	
	self class ignoreMonticelloChangeSets ifTrue: [ ^self ].
	
	filename := MCVersionInfoFilename filename: cs name.
	
	wc := (MCPackage named: filename packageName) workingCopy.
	info := wc ancestry infoWithName: cs name message: 'testing'.
	
	dictionary 	at: (wc ancestry infoWithName: cs name message: 'testing') 
				put: (MCVersion package: wc package info: info) 
	 ! !

!MCSystemChangeSetsRepository methodsFor: 'as yet unclassified' stamp: 'kph 5/17/2007 12:49'!
initialize
 
	self loadFromChangeSet.
	! !

!MCSystemChangeSetsRepository methodsFor: 'as yet unclassified' stamp: 'kph 5/17/2007 12:51'!
isChangeSetAMonticelloPackage: name

	| number pa author |	

	number := name copyAfterLast: $..
	
	number isEmpty ifTrue: [ ^ false ].
	
	(number allSatisfy: [ :n | n isDigit ]) ifFalse: [ ^ false ].
	
	pa := name copyUpToLast: $..
	
	author := pa copyAfterLast: $-.
	
	(author allSatisfy: [ :n | n isLetter ]) ifFalse: [ ^ false ].
	
	^ true! !

!MCSystemChangeSetsRepository methodsFor: 'testing' stamp: 'kph 5/17/2007 15:06'!
isValid

	^ false! !

!MCSystemChangeSetsRepository methodsFor: 'as yet unclassified' stamp: 'kph 5/17/2007 13:56'!
loadFromChangeSet

	| |

	dictionary := Dictionary new.
	
	ChangeSet allChangeSets do: [ :cs |
	 
		( self isChangeSetAMonticelloPackage: cs name )
				ifTrue: [ self addMCVersionFrom: cs ]
				ifFalse: [ self addCSFrom: cs ]
	
	]
	! !

!MCSystemChangeSetsRepository methodsFor: 'as yet unclassified' stamp: 'kph 5/17/2007 12:40'!
loadFromChangeSetTmp

	| |

	dictionary := Dictionary new.
	
	ChangeSet allChangeSets do: [ :cs |
		
		(self isChangeSetAMonticelloPackage: cs name)
						ifTrue: [ self addMCVersionFrom: cs name ]
	
	
						ifFalse: [ 
		 		
			dictionary at: (MCVersionInfo 
								name: (self createMCNameFromCSName: cs name) 
								id: nil "id added on copy into a proper repository" 
								message: 'System Change Set: ', (cs name) 
								date: Date today 
								time: Time now
								author: 'system' 
								ancestors: #() 
							 )
						put: MCSystemChangeSetVersion new ]
	]
	! !

!MCSystemChangeSetsRepository methodsFor: 'as yet unclassified' stamp: 'kph 5/17/2007 11:43'!
openAndEditTemplateCopy! !

!MCSystemChangeSetsRepository methodsFor: 'as yet unclassified' stamp: 'mtf 6/27/2008 23:22'!
sortedHeldVersionInfos

	^ self heldVersionInfos asSortedCollection: [ :a :b | a > b ]! !

MCRepository subclass: #MCFileBasedRepository
	instanceVariableNames: 'cache allFileNames heldVersionInfos'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Repositories'!

!MCFileBasedRepository commentStamp: 'kph 5/17/2007 04:42' prior: 0!
For the FileBased repository we use versionInfo's as before, 
but they are based on only what info we can obtain from the file system and fileName.
 
We are using instances of MCVersionInfo to represent fileNames in order that the UI
doesnt see anything special about a file based repository. 
This saves us special UI implementations for File based repositories."
	!

MCFileBasedRepository subclass: #MCDirectoryRepository
	instanceVariableNames: 'directory'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Repositories'!

MCDirectoryRepository subclass: #MCCacheRepository
	instanceVariableNames: 'packageCaches seenFiles'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Repositories'!

MCCacheRepository class
	instanceVariableNames: 'default'!

MCCacheRepository class
	instanceVariableNames: 'default'!

!MCCacheRepository class methodsFor: 'as yet unclassified' stamp: 'kph 5/19/2007 00:27'!
initialize
	default := nil! !

!MCCacheRepository class methodsFor: 'as yet unclassified' stamp: 'avi 10/9/2003 12:46'!
cacheDirectory
	^ (FileDirectory default directoryNamed: 'package-cache')
		assureExistence;
		yourself! !

!MCCacheRepository class methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
checkCacheDirectory
	default notNil and: [default directory exists ifFalse: [default := nil]]! !

!MCCacheRepository class methodsFor: 'as yet unclassified' stamp: 'kph 5/16/2007 12:38'!
creationTemplate
	^
'In Memory Cache'! !

!MCCacheRepository class methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
default
	self checkCacheDirectory.
	^ default ifNil: [default := self new directory: self cacheDirectory]! !

!MCCacheRepository class methodsFor: 'as yet unclassified' stamp: 'avi 10/9/2003 12:56'!
description
	^ nil! !

!MCCacheRepository methodsFor: 'as yet unclassified' stamp: 'bf 3/23/2005 00:52'!
basicStoreVersion: aVersion
	(aVersion isCacheable and: [self allFileNames includes: aVersion fileName])
		ifFalse: [super basicStoreVersion: aVersion]
! !

!MCCacheRepository methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
cacheForPackage: aPackage
	packageCaches ifNil: [packageCaches := Dictionary new].
	^ packageCaches at: aPackage ifAbsentPut: [MCPackageCache new]! !

!MCCacheRepository methodsFor: 'as yet unclassified' stamp: 'avi 1/22/2004 18:15'!
newFileNames 
	^ self allFileNames difference: self seenFileNames! !

!MCCacheRepository methodsFor: 'as yet unclassified' stamp: 'avi 1/22/2004 15:13'!
packageForFileNamed: aString
	^ self packageCache at: aString ifAbsentPut: [self versionReaderForFileNamed: aString do: [:r | r package]]! !

!MCCacheRepository methodsFor: 'as yet unclassified' stamp: 'avi 1/22/2004 18:29'!
rescan
	self newFileNames do:
		[:ea |
		self versionReaderForFileNamed: ea do:
			[:reader |
			(self cacheForPackage: reader package)
				recordVersionInfo: reader info
				forFileNamed: ea.
			self seenFileNames add: ea]]
		displayingProgress: 'Scanning cache...'! !

!MCCacheRepository methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
seenFileNames
	^ seenFiles ifNil: [seenFiles := OrderedCollection new]! !

!MCCacheRepository methodsFor: 'as yet unclassified' stamp: 'avi 1/22/2004 15:05'!
versionInfoForFileNamed: aString
	^ self infoCache at: aString ifAbsentPut: [self versionReaderForFileNamed: aString do: [:r | r info]]! !

!MCDirectoryRepository class methodsFor: 'instance creation' stamp: 'ab 7/24/2003 21:20'!
description
	^ 'directory'! !

!MCDirectoryRepository class methodsFor: 'instance creation' stamp: 'ab 7/24/2003 21:01'!
morphicConfigure
	^ FileList2 modalFolderSelector ifNotNilDo:
		[:directory |
		self new directory: directory]! !

!MCDirectoryRepository methodsFor: 'as yet unclassified' stamp: 'avi 8/31/2003 15:28'!
allFileNames
	^ (directory entries sortBy: [:a :b | a modificationTime >= b modificationTime]) collect: [:ea | ea name]! !

!MCDirectoryRepository methodsFor: 'as yet unclassified' stamp: 'mtf 6/28/2008 12:32'!
creationTemplate
	^ self class name, ' new directory: (FileDirectory on: ''', self description, ''')'.! !

!MCDirectoryRepository methodsFor: 'as yet unclassified' stamp: 'kph 5/14/2007 15:27'!
delete: versionFileName
	
	[ self directory deleteFileNamed: versionFileName ifAbsent: [ ^false ] ]
		 on: Error do: [ :ex | ^ false ].
	
	^ true! !

!MCDirectoryRepository methodsFor: 'as yet unclassified' stamp: 'avi 10/9/2003 12:57'!
description
	^ directory pathName! !

!MCDirectoryRepository methodsFor: 'as yet unclassified' stamp: 'ab 7/6/2003 17:49'!
directory
	^ directory! !

!MCDirectoryRepository methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
directory: aDirectory
	directory := aDirectory! !

!MCDirectoryRepository methodsFor: 'comparing' stamp: 'ab 7/19/2003 21:40'!
hash
	^ directory pathName hash! !

!MCDirectoryRepository methodsFor: 'as yet unclassified' stamp: 'mtf 6/28/2008 13:52'!
heldVersionInfos
	" If you manipulate the underlying file system the caching will get you. If for example, you store your Monticello repo in a Git repo"
	^ heldVersionInfos := self getHeldVersionInfos! !

!MCDirectoryRepository methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
initialize
	directory := FileDirectory default! !

!MCDirectoryRepository methodsFor: 'as yet unclassified' stamp: 'kph 5/14/2007 14:17'!
isDeletingAllowed

	^ true! !

!MCDirectoryRepository methodsFor: 'as yet unclassified' stamp: 'nk 11/2/2003 10:55'!
isValid
	^directory exists! !

!MCDirectoryRepository methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
readStreamForFileNamed: aString do: aBlock
	| file val |
	file := FileStream readOnlyFileNamed: (directory fullNameFor: aString).
	val := aBlock value: file.
	file close.
	^ val! !

!MCDirectoryRepository methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
writeStreamForFileNamed: aString replace: aBoolean do: aBlock
	| file sel |
	sel := aBoolean ifTrue: [#forceNewFileNamed:] ifFalse: [#newFileNamed:].
	file := FileStream perform: sel with: (directory fullNameFor: aString).
	aBlock value: file.
	file close.! !

MCDirectoryRepository subclass: #MCSubDirectoryRepository
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Repositories'!

!MCSubDirectoryRepository commentStamp: 'nk 6/11/2004 18:56' prior: 0!
A MCDirectoryRepository that looks in subdirectories too.!

!MCSubDirectoryRepository class methodsFor: 'user interface' stamp: 'nk 6/11/2004 18:48'!
description
	^ 'directory with subdirectories'! !

!MCSubDirectoryRepository methodsFor: 'enumeration' stamp: 'nk 6/11/2004 18:55'!
allDirectories
	| remaining dir dirs |
	remaining := OrderedCollection new.
	dirs := OrderedCollection new.
	remaining addLast: directory.
	[remaining isEmpty]
		whileFalse: [dir := remaining removeFirst.
			dirs add: dir.
			dir entries
				do: [:ent | ent isDirectory
						ifTrue: [remaining
								addLast: (dir directoryNamed: ent name)]]].
	^ dirs! !

!MCSubDirectoryRepository methodsFor: 'enumeration' stamp: 'nk 6/11/2004 20:25'!
allFileNames
	"sorting {entry. dirName. name}"

	| sorted |
	sorted := SortedCollection sortBlock: [:a :b |
		a first modificationTime >= b first modificationTime ].
	self allDirectories
		do: [:dir | dir entries
				do: [:ent | ent isDirectory
						ifFalse: [sorted add: {ent. dir fullName. ent name}]]].
	^ sorted
		collect: [:ea | ea third ]! !

!MCSubDirectoryRepository methodsFor: 'user interface' stamp: 'nk 6/11/2004 18:23'!
description
	^ directory pathName, '/*'! !

!MCSubDirectoryRepository methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
findFullNameForReading: aBaseName
	"Answer the latest version of aBaseName"
	| possible |
	possible := SortedCollection sortBlock: [ :a :b | b first modificationTime < a first modificationTime ].
	self allDirectories
		do: [:dir | dir entries
				do: [:ent | ent isDirectory
						ifFalse: [
							(ent name = aBaseName) ifTrue: [ possible add: {ent. dir fullNameFor: ent name}]]]].
	^(possible at: 1 ifAbsent: [ ^nil ]) second
! !

!MCSubDirectoryRepository methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
findFullNameForWriting: aBaseName
	| possible split dirScore fileScore prefix fpattern parts now |
	split := directory splitNameVersionExtensionFor: aBaseName.
	fpattern := split first, '*'.
	possible := SortedCollection sortBlock: [ :a :b |
		a first = b first
			ifTrue: [ a second = b second
					ifFalse: [ a second < b second ]
					ifTrue: [ a third fullName size < b third fullName size ]]
			ifFalse: [ a first > b first ] ].
	now := Time totalSeconds.
	prefix := directory pathParts size.
	self allDirectories do: [:dir |
		parts := dir pathParts allButFirst: prefix.
		dirScore := (parts select: [ :part | fpattern match: part ]) size.
		fileScore := (dir entries collect: [ :ent |
			(ent isDirectory not and: [ fpattern match: ent name ])
				ifFalse: [ SmallInteger maxVal ]
				ifTrue: [ now - ent modificationTime ]]).	"minimum age"
		fileScore := fileScore isEmpty ifTrue: [ SmallInteger maxVal  ]
			ifFalse: [ fileScore min ].
		possible add: { dirScore. fileScore. dir } ].
	^ (possible first third) fullNameFor: aBaseName! !

!MCSubDirectoryRepository methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
readStreamForFileNamed: aString do: aBlock
	| file val |
	file := FileStream readOnlyFileNamed: (self findFullNameForReading: aString).
	val := aBlock value: file.
	file close.
	^ val! !

!MCSubDirectoryRepository methodsFor: 'as yet unclassified' stamp: 'nk 6/11/2004 20:34'!
writeStreamForFileNamed: aString replace: aBoolean do: aBlock 
	| file |
	file := aBoolean
				ifTrue: [FileStream
						forceNewFileNamed: (self findFullNameForReading: aString)]
				ifFalse: [FileStream
						newFileNamed: (self findFullNameForWriting: aString)].
	aBlock value: file.
	file close! !

!MCFileBasedRepository class methodsFor: 'as yet unclassified' stamp: 'avi 2/3/2005 00:43'!
flushAllCaches
	self allSubInstancesDo: [:ea | ea flushCache]! !

!MCFileBasedRepository class methodsFor: 'as yet unclassified' stamp: 'kph 5/16/2007 12:35'!
freeSomeSpace
	"system request for memory cleanup (e.g. prior to an image publication)"
	self flushAllCaches! !

!MCFileBasedRepository methodsFor: 'filesystem-private' stamp: 'ab 8/21/2003 00:35'!
allFileNames
	self subclassResponsibility! !

!MCFileBasedRepository methodsFor: 'filesystem' stamp: 'ab 8/21/2003 20:01'!
allFileNamesForVersionNamed: aString
	^ self filterFileNames: self readableFileNames forVersionNamed: aString! !

!MCFileBasedRepository methodsFor: 'filesystem-private' stamp: 'kph 9/28/2007 10:29'!
allFileNamesOrCache
	^ allFileNames ifNil: [ self allFileNames]! !

!MCFileBasedRepository methodsFor: 'baseclass required' stamp: 'kph 5/17/2007 02:30'!
basicStoreVersion: aVersion
	self
		writeStreamForFileNamed: aVersion fileName
		do: [:s | aVersion fileOutOn: s].
	aVersion isCacheable ifTrue: [ self cache at: aVersion fileName put: aVersion].
! !

!MCFileBasedRepository methodsFor: 'cache' stamp: 'stephaneducasse 2/4/2006 20:47'!
cache
	^ cache ifNil: [cache := Dictionary new]! !

!MCFileBasedRepository methodsFor: 'cache' stamp: 'stephaneducasse 2/4/2006 20:47'!
cacheAllFileNamesDuring: aBlock
	allFileNames := self allFileNames.
	^ aBlock ensure: [allFileNames := nil]! !

!MCFileBasedRepository methodsFor: 'cache' stamp: 'bf 6/9/2005 15:47'!
cachedFileNames
	^cache == nil
		ifTrue: [#()]
		ifFalse: [cache keys]! !

!MCFileBasedRepository methodsFor: 'filesystem-private' stamp: 'stephaneducasse 2/4/2006 20:47'!
canReadFileNamed: aString
	| reader |
	reader := MCVersionReader readerClassForFileNamed: aString.
	^ reader notNil! !

!MCFileBasedRepository methodsFor: 'as yet unclassified' stamp: 'avi 9/17/2005 16:40'!
closestAncestorVersionFor: anAncestry ifNone: errorBlock
	^ self cacheAllFileNamesDuring:
		[super closestAncestorVersionFor: anAncestry ifNone: errorBlock]! !

!MCFileBasedRepository methodsFor: 'filesystem' stamp: 'ab 8/21/2003 20:01'!
filterFileNames: aCollection forVersionNamed: aString
	^ aCollection select: [:ea | (self versionNameFromFileName: ea) = aString] ! !

!MCFileBasedRepository methodsFor: 'cache' stamp: 'kph 9/28/2007 10:53'!
flushCache
	cache := nil.
	heldVersionInfos := nil! !

!MCFileBasedRepository methodsFor: 'baseclass required' stamp: 'kph 9/28/2007 10:30'!
getHeldVersionInfos

	^ self readableFileNames collect: [:ea | MCVersionInfoFilename filename: ea]! !

!MCFileBasedRepository methodsFor: 'baseclass required' stamp: 'kph 9/28/2007 10:43'!
heldVersionInfos

	^ heldVersionInfos ifNil: [ heldVersionInfos := self getHeldVersionInfos ]! !

!MCFileBasedRepository methodsFor: 'baseclass required' stamp: 'kph 9/28/2007 10:31'!
heldVersionInfosClearCache

heldVersionInfos := nil! !

!MCFileBasedRepository methodsFor: 'baseclass required' stamp: 'kph 5/17/2007 02:32'!
heldVersions

	self error: 'I dont plan to load every version in from disk'! !

!MCFileBasedRepository methodsFor: 'as yet unclassified' stamp: 'kph 5/17/2007 06:03'!
includesVersionNamed: aString
	^ self heldVersionNames includes: aString! !

!MCFileBasedRepository methodsFor: 'filesystem-private' stamp: 'avi 1/22/2004 13:34'!
loadVersionFromFileNamed: aString
	^ self versionReaderForFileNamed: aString do: [:r | r version]! !

!MCFileBasedRepository methodsFor: 'filesystem-private' stamp: 'kph 8/1/2007 01:51'!
loadVersionInfoFromFileNamed: aString
	^ [ self versionReaderForFileNamed: aString do: [:r | r info]
	 	] on: Error 
			do: [ :ex | Transcript cr; show: 'Could not read ', aString. 
		].! !

!MCFileBasedRepository methodsFor: 'cache' stamp: 'avi 9/17/2005 16:52'!
maxCacheSize
	^ 8! !

!MCFileBasedRepository methodsFor: 'notify' stamp: 'avi 8/26/2004 14:34'!
notifyList
	| list |
	(self allFileNames includes: 'notify') ifFalse: [^ #()].
	^ self readStreamForFileNamed: 'notify' do:
		[:s |
		s upToEnd withSqueakLineEndings findTokens: (String with: Character cr)]! !

!MCFileBasedRepository methodsFor: 'as yet unclassified' stamp: 'mtf 6/27/2008 23:17'!
possiblyNewerVersionsOfAnyOf: someVersions
	| pkgs |
	pkgs := Dictionary new.

	someVersions do: [:aVersionInfo |
		pkgs at: aVersionInfo packageName
			put: aVersionInfo].
 
	^[self heldVersionInfos select: [:got |
		pkgs at: (got packageName) ifPresent: [:verInfo | got > verInfo]		
	]] on: Error do: [:ex | ex return: #()]! !

!MCFileBasedRepository methodsFor: 'filesystem-private' stamp: 'avi 9/17/2005 16:39'!
readableFileNames
	| all cached new |
	all := self allFileNamesOrCache.	"from repository"
	cached := self cachedFileNames.	"in memory"
	new := all difference: cached.
	^ (cached asArray, new)
		select: [:ea | self canReadFileNamed: ea]! !

!MCFileBasedRepository methodsFor: 'baseclass required' stamp: 'kph 5/17/2007 03:15'!
realizeFullVersionInfo: aStubVersionInfoInstance

	^ self versionInfoFromFileNamed: aStubVersionInfoInstance name! !

!MCFileBasedRepository methodsFor: 'cache' stamp: 'avi 9/18/2005 22:43'!
resizeCache: aDictionary
	[aDictionary size <= self maxCacheSize] whileFalse:
		[aDictionary removeKey: aDictionary keys atRandom]! !

!MCFileBasedRepository methodsFor: 'baseclass required' stamp: 'mtf 6/27/2008 23:22'!
sortedHeldVersionInfos

	^ self heldVersionInfos asSortedCollection: [ :a :b | a > b ]! !

!MCFileBasedRepository methodsFor: 'baseclass required' stamp: 'mtf 6/27/2008 23:22'!
sortedVersionInfosAvailableForPackageNamed: aPackageName

	^ (self versionInfosAvailableForPackageNamed: aPackageName) asSortedCollection: [ :a :b | a > b ]! !

!MCFileBasedRepository methodsFor: 'filesystem' stamp: 'avi 9/17/2005 23:09'!
versionFromFileNamed: aString
	| v |
	v := self cache at: aString ifAbsent: [self loadVersionFromFileNamed: aString].
	self resizeCache: cache.
	(v notNil and: [v isCacheable]) ifTrue: [cache at: aString put: v].
	^ v! !

!MCFileBasedRepository methodsFor: 'filesystem' stamp: 'avi 9/17/2005 18:37'!
versionInfoFromFileNamed: aString
	self cache at: aString ifPresent: [:v | ^ v info].
	^ self loadVersionInfoFromFileNamed: aString! !

!MCFileBasedRepository methodsFor: 'as yet unclassified' stamp: 'bf 5/30/2005 22:52'!
versionNameFromFileName: aString
	^ (aString copyUpToLast: $.) copyUpTo: $(! !

!MCFileBasedRepository methodsFor: 'filesystem-private' stamp: 'kph 8/1/2007 01:50'!
versionReaderForFileNamed: aString do: aBlock
	^ self
		readStreamForFileNamed: aString
		do: [:s |
			(MCVersionReader readerClassForFileNamed: aString) 
				ifNil:  [ MCNoVersionInfoAvailable signal ]
				ifNotNilDo: [:class | aBlock value: (class on: s fileName: aString)]]
! !

!MCFileBasedRepository methodsFor: 'baseclass required' stamp: 'kph 5/17/2007 04:56'!
versionWithInfo: aStubOrRealVersionInfo ifAbsent: errorBlock

	| realVersionInfo version |
	 
	"aVersionInfo could be a real one or a stub"
	
	realVersionInfo := aStubOrRealVersionInfo realizeFrom: self.
	
	(self allFileNamesForVersionNamed: realVersionInfo name) do:
		[:fileName |
		version := self versionFromFileNamed: fileName.
		version info = realVersionInfo ifTrue: [^ version]].
	^ errorBlock value! !

!MCFileBasedRepository methodsFor: 'filesystem-private' stamp: 'avi 10/31/2003 14:32'!
writeStreamForFileNamed: aString do: aBlock
	^ self writeStreamForFileNamed: aString replace: false do: aBlock! !

MCFileBasedRepository subclass: #MCFtpRepository
	instanceVariableNames: 'host directory user connection'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Repositories'!

!MCFtpRepository class methodsFor: 'as yet unclassified' stamp: 'avi 9/16/2003 13:57'!
creationTemplate
	^
'MCFtpRepository
	host: ''modules.squeakfoundation.org''
	directory: ''mc''
	user: ''squeak''
	password: ''squeak'''
	! !

!MCFtpRepository class methodsFor: 'as yet unclassified' stamp: 'avi 9/16/2003 13:57'!
description
	^ 'FTP'! !

!MCFtpRepository class methodsFor: 'as yet unclassified' stamp: 'bkv 2/18/2004 21:02'!
fillInTheBlankRequest
	^ 'FTP Repository:'

	! !

!MCFtpRepository class methodsFor: 'as yet unclassified' stamp: 'avi 9/16/2003 13:57'!
host: host directory: directory user: user password: password
	^ self new
		host: host;
		directory: directory;
		user: user;
		password: password! !

!MCFtpRepository class methodsFor: 'as yet unclassified' stamp: 'bkv 2/18/2004 21:02'!
morphicConfigure
	^ self fillInTheBlankConfigure! !

!MCFtpRepository methodsFor: 'required' stamp: 'avi 9/16/2003 14:04'!
allFileNames
	^ self clientDo:
		[:client |
		self parseDirectoryListing: client getDirectory]! !

!MCFtpRepository methodsFor: 'as yet unclassified' stamp: 'kph 5/10/2007 01:52'!
clientDo: aBlock
	| client |
	client := FTPClient openOnHostNamed: host.
	client loginUser: user password: self password.
	directory isEmpty ifFalse: [client changeDirectoryTo: directory].
	^ [aBlock value: client] ensure: [client close]! !

!MCFtpRepository methodsFor: 'as yet unclassified' stamp: 'kph 5/14/2007 15:13'!
delete: versionFileName

	self clientDo: [ :ftp |
					 [ ftp deleteFileNamed: versionFileName ] on: Error do: [ :ex | ^ false ].
	].

	^ true! !

!MCFtpRepository methodsFor: 'required' stamp: 'avi 9/17/2003 12:52'!
description
	^ 'ftp://', user, '@', host, '/', directory! !

!MCFtpRepository methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
directory: dirPath
	directory := dirPath! !

!MCFtpRepository methodsFor: 'required' stamp: 'ar 11/4/2005 17:26'!
flushPasswords
	MCPasswordManager default passwordAt: 'ftp://',host user: user put: nil.! !

!MCFtpRepository methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
host: hostname
	host := hostname! !

!MCFtpRepository methodsFor: 'as yet unclassified' stamp: 'kph 12/19/2007 00:14'!
isDeletingAllowed

	| p |
	p := self password.

	^ p notEmpty and: [ p ~= 'squeak' ].! !

!MCFtpRepository methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
parseDirectoryListing: aString
	| stream files line tokens |
	stream := aString readStream.
	files := OrderedCollection new.
	[stream atEnd] whileFalse:
		[line := stream nextLine.
		tokens := line findTokens: ' '.
		tokens size > 2 ifTrue: [files add: tokens last]].
	^ files! !

!MCFtpRepository methodsFor: 'as yet unclassified' stamp: 'ar 11/4/2005 17:26'!
password
	^(MCPasswordManager default queryPasswordAt: 'ftp://',host user: user) ifNil:[^''].! !

!MCFtpRepository methodsFor: 'as yet unclassified' stamp: 'ar 11/4/2005 17:26'!
password: passwordString
	| pwd |
	passwordString isEmpty ifTrue:[pwd := nil] ifFalse:[pwd := passwordString].
	MCPasswordManager default passwordAt: 'ftp://',host user: user put: pwd.
! !

!MCFtpRepository methodsFor: 'required' stamp: 'stephaneducasse 2/4/2006 20:47'!
readStreamForFileNamed: aString do: aBlock
	| stream |
	^ self clientDo:
		[:client |
		client binary.
		stream := RWBinaryOrTextStream on: String new.
		stream nextPutAll: (client getFileNamed: aString).
		aBlock value: stream reset]! !

!MCFtpRepository methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
user: userString
	user := userString! !

!MCFtpRepository methodsFor: 'required' stamp: 'stephaneducasse 2/4/2006 20:47'!
writeStreamForFileNamed: aString replace: ignoreBoolean do: aBlock
	| stream |
	stream := RWBinaryOrTextStream on: String new.
	aBlock value: stream.
	self clientDo:
		[:client |
		client binary.
		client putFileStreamContents: stream reset as: aString]! !

MCFileBasedRepository subclass: #MCHttpRepository
	instanceVariableNames: 'location user readerCache'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Repositories'!

!MCHttpRepository class methodsFor: 'as yet unclassified' stamp: 'mtf 7/21/2008 18:13'!
creationTemplate
	^self creationTemplateLocation: 'http://www.squeaksource.com/mc'
		user: ''
		password: ''
! !

!MCHttpRepository class methodsFor: 'as yet unclassified' stamp: 'bf 4/14/2005 15:27'!
creationTemplateLocation: location user: user password: password
	^
'MCHttpRepository
	location: {1}
	user: {2}
	password: {3}' format: {location printString. user printString. password printString}! !

!MCHttpRepository class methodsFor: 'as yet unclassified' stamp: 'ab 7/24/2003 21:20'!
description
	^ 'HTTP'! !

!MCHttpRepository class methodsFor: 'as yet unclassified' stamp: 'bkv 2/18/2004 21:00'!
fillInTheBlankRequest
	^ 'HTTP Repository:'
			! !

!MCHttpRepository class methodsFor: 'as yet unclassified' stamp: 'ab 7/24/2003 21:32'!
location: location user: user password: password
	^ self new
		location: location;
		user: user;
		password: password! !

!MCHttpRepository class methodsFor: 'as yet unclassified' stamp: 'bkv 2/18/2004 21:01'!
morphicConfigure
	^ self fillInTheBlankConfigure! !

!MCHttpRepository methodsFor: 'required' stamp: 'stephaneducasse 2/4/2006 20:47'!
allFileNames
	| index |
	index := HTTPSocket httpGet: self locationWithTrailingSlash, '?C=M;O=D' args: nil user: self user passwd: self password.
	index isString ifTrue: [self error: 'Could not access ', location].
	^ self parseFileNamesFromStream: index	! !

!MCHttpRepository methodsFor: 'as yet unclassified' stamp: 'ar 11/4/2005 17:26'!
asCreationTemplate
	^self class creationTemplateLocation: location user: user password: (
		(MCPasswordManager default passwordAt: location user: user) ifNil:['']
	)! !

!MCHttpRepository methodsFor: 'as yet unclassified' stamp: 'bf 11/7/2005 17:28'!
creationTemplate
	^ self asCreationTemplate
! !

!MCHttpRepository methodsFor: 'as yet unclassified' stamp: 'kph 5/10/2007 02:17'!
creationTemplate: ignored
	creationTemplate := nil.! !

!MCHttpRepository methodsFor: 'required' stamp: 'ab 7/24/2003 21:10'!
description
	^ location! !

!MCHttpRepository methodsFor: 'required' stamp: 'al 12/12/2005 11:06'!
flushCache
	super flushCache.
	readerCache := nil.! !

!MCHttpRepository methodsFor: 'required' stamp: 'kph 5/10/2007 03:43'!
flushPasswords
	MCPasswordManager default passwordAt: self locationHost user: user put: nil.! !

!MCHttpRepository methodsFor: 'as yet unclassified' stamp: 'kph 5/11/2007 07:49'!
location: aUrlString

	location := (aUrlString beginsWith: 'http://') 
		ifTrue: [ aUrlString ]
		ifFalse: [ 'http://', aUrlString ]! !

!MCHttpRepository methodsFor: 'as yet unclassified' stamp: 'kph 5/11/2007 07:50'!
locationHost

	| end |
	
	(end := (location lastIndexOf: $/) - 1) < 9 ifTrue: [ ^ location ].
  	
	^ location copyFrom: 1 to: end.
! !

!MCHttpRepository methodsFor: 'as yet unclassified' stamp: 'ab 7/24/2003 22:17'!
locationWithTrailingSlash
	^ (location endsWith: '/')
		ifTrue: [location]
		ifFalse: [location, '/']! !

!MCHttpRepository methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
parseFileNamesFromStream: aStream
	| names fullName |
	names := OrderedCollection new.
	[aStream atEnd] whileFalse:
		[[aStream upTo: $<. {$a. $A. nil} includes: aStream next] whileFalse.
		aStream upTo: $".
		aStream atEnd ifFalse: [
			fullName := aStream upTo: $".
			names add: fullName unescapePercents]].
	^ names! !

!MCHttpRepository methodsFor: 'as yet unclassified' stamp: 'kph 4/25/2008 00:42'!
password
	self user isEmpty ifTrue: [^''].
	^(MCPasswordManager default queryPasswordAt: self locationHost user: self user) ifNil:[^'']! !

!MCHttpRepository methodsFor: 'as yet unclassified' stamp: 'kph 5/10/2007 03:42'!
password: passwordString
	| pwd |
	passwordString isEmpty ifTrue:[pwd := nil] ifFalse:[pwd := passwordString].
	MCPasswordManager default passwordAt: self locationHost user: self user put: pwd.! !

!MCHttpRepository methodsFor: 'required' stamp: 'stephaneducasse 2/4/2006 20:47'!
readStreamForFileNamed: aString do: aBlock
	| contents |
	contents := HTTPSocket httpGet: (self urlForFileNamed: aString) args: nil user: self user passwd: self password.
	^ contents isString ifFalse: [aBlock value: contents]! !

!MCHttpRepository methodsFor: 'as yet unclassified' stamp: 'ab 8/21/2003 13:08'!
urlForFileNamed: aString
	^ self locationWithTrailingSlash, aString encodeForHTTP! !

!MCHttpRepository methodsFor: 'as yet unclassified' stamp: 'kph 4/25/2008 00:42'!
user
	^user ifNil: [ '' ]! !

!MCHttpRepository methodsFor: 'as yet unclassified' stamp: 'mtf 6/9/2008 20:56'!
user: userString
	user := userString! !

!MCHttpRepository methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
versionReaderForFileNamed: aString
	readerCache ifNil: [readerCache := Dictionary new].
	^ readerCache at: aString ifAbsent:
		[self resizeCache: readerCache.
		super versionReaderForFileNamed: aString do:
			[:r |
			r ifNotNil: [readerCache at: aString put: r]]]
	! !

!MCHttpRepository methodsFor: 'as yet unclassified' stamp: 'avi 9/17/2005 23:06'!
versionReaderForFileNamed: aString do: aBlock
	^ (self versionReaderForFileNamed: aString) ifNotNilDo: aBlock! !

!MCHttpRepository methodsFor: 'required' stamp: 'cds 9/19/2008 07:40'!
writeStreamForFileNamed: aString replace: ignoreBoolean do: aBlock
	| stream response statusLine code |
	stream := RWBinaryOrTextStream on: String new.
	aBlock value: stream.
	response := HTTPSocket
					httpPut: stream contents
					to: (self urlForFileNamed: aString)
					user: self user
					passwd: self password.

	statusLine := response copyUpTo: Character cr.
	code := (statusLine findTokens: ' ') second asInteger.
	(code >= 200 and: [code < 300])
			ifFalse: [self error: response].! !

MCFileBasedRepository subclass: #MCSMCacheRepository
	instanceVariableNames: 'smCache'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Repositories'!

!MCSMCacheRepository commentStamp: 'nk 1/23/2004 09:57' prior: 0!
I am a Monticello repository that reflects the caching of SqueakMap v2.

I refer write attempts to the default MCCacheRepository.!

!MCSMCacheRepository class methodsFor: 'instance creation' stamp: 'nk 1/23/2004 10:04'!
description
	^ 'SqueakMap Cache'! !

!MCSMCacheRepository methodsFor: 'accessing' stamp: 'nk 1/23/2004 09:46'!
allFileNames
	^self allFullFileNames collect: [ :ea | self directory localNameFor: ea ]! !

!MCSMCacheRepository methodsFor: 'accessing' stamp: 'stephaneducasse 2/4/2006 20:47'!
allFullFileNames
	| cachedPackages |
	cachedPackages := smCache map installedPackages select: [ :ea | ea isCached ].
	^Array streamContents: [ :s |
		cachedPackages do: [ :ea | | d |
			d := ea cacheDirectory.
			(d fileNamesMatching: '*.mcz') do: [ :fn | s nextPut: (d fullNameFor: fn) ]]]! !

!MCSMCacheRepository methodsFor: 'accessing' stamp: 'nk 1/23/2004 09:40'!
description
	^ smCache directory pathName! !

!MCSMCacheRepository methodsFor: 'accessing' stamp: 'nk 1/23/2004 09:40'!
directory
	^ smCache directory! !

!MCSMCacheRepository methodsFor: 'accessing' stamp: 'nk 1/23/2004 09:40'!
directory: aDirectory
! !

!MCSMCacheRepository methodsFor: 'accessing' stamp: 'nk 1/23/2004 09:55'!
fullNameFor: aFileName
	^self allFullFileNames detect: [ :ffn | (self directory localNameFor: ffn) = aFileName ] ifNone: []! !

!MCSMCacheRepository methodsFor: 'comparing' stamp: 'nk 1/23/2004 09:55'!
hash
	^ smCache hash! !

!MCSMCacheRepository methodsFor: 'initialize-release' stamp: 'stephaneducasse 2/4/2006 20:47'!
initialize
	super initialize.
	smCache := SMSqueakMap default cache.! !

!MCSMCacheRepository methodsFor: 'testing' stamp: 'nk 1/23/2004 09:47'!
isValid
	^smCache notNil and: [ self directory exists ]! !

!MCSMCacheRepository methodsFor: 'file streaming' stamp: 'stephaneducasse 2/4/2006 20:47'!
readStreamForFileNamed: aString do: aBlock
	| file fileName |
	fileName := self fullNameFor: aString.
	fileName ifNil: [
		"assume that this will come from the cache."
		^MCCacheRepository default readStreamForFileNamed: aString do: aBlock ].
	file := FileStream readOnlyFileNamed: fileName.
	^[ aBlock value: file ] ensure: [ file close ].
! !

!MCSMCacheRepository methodsFor: 'accessing' stamp: 'nk 1/23/2004 09:37'!
smCache
	^smCache! !

!MCSMCacheRepository methodsFor: 'accessing' stamp: 'nk 1/23/2004 09:45'!
smCache: aSMFileCache
	| |
	smCache := aSMFileCache.
	self directory: aSMFileCache directory.
! !

!MCSMCacheRepository methodsFor: 'file streaming' stamp: 'nk 1/23/2004 09:57'!
writeStreamForFileNamed: aString replace: aBoolean do: aBlock
	"Can't write into the SM cache, so..."
	^MCCacheRepository default writeStreamForFileNamed: aString replace: aBoolean do: aBlock! !

MCRepository subclass: #MCGOODSRepository
	instanceVariableNames: 'hostname port connection'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Repositories'!

!MCGOODSRepository class methodsFor: 'as yet unclassified' stamp: 'avi 2/28/2004 20:36'!
creationTemplate
	^
'MCGOODSRepository
	host: ''localhost''
	port: 6100'! !

!MCGOODSRepository class methodsFor: 'as yet unclassified' stamp: 'avi 2/28/2004 14:33'!
description
	^ 'GOODS'! !

!MCGOODSRepository class methodsFor: 'as yet unclassified' stamp: 'avi 2/28/2004 20:33'!
fillInTheBlankRequest
	^ 'GOODS Repository:'! !

!MCGOODSRepository class methodsFor: 'as yet unclassified' stamp: 'avi 2/28/2004 20:36'!
host: hostname port: portNumber
	^ self new
		host: hostname;
		port: portNumber! !

!MCGOODSRepository class methodsFor: 'as yet unclassified' stamp: 'avi 2/28/2004 20:35'!
morphicConfigure
	^ self fillInTheBlankConfigure! !

!MCGOODSRepository methodsFor: 'baseclass required' stamp: 'avi 8/26/2004 14:20'!
basicStoreVersion: aVersion
	self root at: aVersion info put: aVersion.
	self db commit.! !

!MCGOODSRepository methodsFor: 'as yet unclassified' stamp: 'mtf 6/9/2008 20:57'!
db
	Smalltalk at: #KKDatabase ifPresent: [:classKKDatabase |
	(connection isNil or: [connection isConnected not]) ifTrue: [
		connection := classKKDatabase onHost:hostname port: port].
	].
	^ connection! !

!MCGOODSRepository methodsFor: 'baseclass overides' stamp: 'avi 2/28/2004 20:35'!
description
	^ 'goods://', hostname asString, ':', port asString! !

!MCGOODSRepository methodsFor: 'baseclass required' stamp: 'kph 5/17/2007 01:44'!
heldVersionInfos
	^ self root keys  ! !

!MCGOODSRepository methodsFor: 'baseclass required' stamp: 'kph 5/17/2007 00:23'!
heldVersions
	^ self root asArray  ! !

!MCGOODSRepository methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
host: aString
	hostname := aString! !

!MCGOODSRepository methodsFor: 'baseclass overides' stamp: 'avi 2/28/2004 20:17'!
packages
	^ (self root collect: [:ea | ea package]) asSet asSortedCollection! !

!MCGOODSRepository methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
port: aNumber
	port := aNumber! !

!MCGOODSRepository methodsFor: 'as yet unclassified' stamp: 'avi 2/28/2004 14:35'!
root
	self db root ifNil: [self db root: Dictionary new].
	^ self db root! !

!MCGOODSRepository methodsFor: 'baseclass required' stamp: 'avi 2/28/2004 20:21'!
versionWithInfo: aVersionInfo ifAbsent: errorBlock
	^ self root at: aVersionInfo ifAbsent: errorBlock! !

!MCRepository class methodsFor: 'as yet unclassified' stamp: 'ab 8/21/2003 00:30'!
allConcreteSubclasses
	^ self withAllSubclasses reject: [:ea | ea isAbstract]! !

!MCRepository class methodsFor: 'as yet unclassified' stamp: 'kph 5/3/2007 20:22'!
clearPasswords
	self allSubInstancesDo: [:ea | (ea respondsTo: #flushPasswords) ifTrue:[ ea flushPasswords ] ].! !

!MCRepository class methodsFor: 'as yet unclassified' stamp: 'bkv 2/18/2004 20:59'!
creationTemplate
	self subclassResponsibility.! !

!MCRepository class methodsFor: 'as yet unclassified' stamp: 'ab 8/21/2003 00:29'!
description
	^ nil! !

!MCRepository class methodsFor: 'obsolete' stamp: 'kph 5/15/2007 20:16'!
fetchExternalSettingsIn: dir

	"no longer used"! !

!MCRepository class methodsFor: 'as yet unclassified' stamp: 'bkv 2/18/2004 21:05'!
fillInTheBlankConfigure
	^ self fillInTheBlankConfigure: self creationTemplate
			! !

!MCRepository class methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
fillInTheBlankConfigure: aTemplateString
	| chunk repo |
	
	aTemplateString ifNil: [ ^ false ].
	chunk := FillInTheBlankMorph 
			request: self fillInTheBlankRequest
			initialAnswer: aTemplateString
			centerAt: Sensor cursorPoint
			inWorld: World
			onCancelReturn: nil
			acceptOnCR: false
			answerExtent: 400@120.
			
	chunk 
		ifNotNil: [ 
			repo := self readFrom: chunk readStream.
			repo creationTemplate: chunk. 
	].

	^ repo! !

!MCRepository class methodsFor: 'as yet unclassified' stamp: 'bkv 2/18/2004 20:58'!
fillInTheBlankRequest
	self subclassResponsibility.! !

!MCRepository class methodsFor: 'as yet unclassified' stamp: 'ab 8/21/2003 12:59'!
isAbstract
	^ self description isNil! !

!MCRepository class methodsFor: 'as yet unclassified' stamp: 'ab 7/24/2003 21:01'!
morphicConfigure
	^ self new! !

!MCRepository class methodsFor: 'as yet unclassified' stamp: 'kph 5/17/2007 09:04'!
uiClass

	^ ui ifNil: [ MCRepositoryPackagesVersionsInspector ]! !

!MCRepository class methodsFor: 'as yet unclassified' stamp: 'kph 5/17/2007 09:05'!
uiClassUseOneColumn

	^ ui := MCRepositoryVersionsInspector ! !

!MCRepository class methodsFor: 'as yet unclassified' stamp: 'kph 5/17/2007 09:05'!
uiClassUseTwoColumn

	^ ui := nil! !

!MCRepository methodsFor: 'testing' stamp: 'ab 8/21/2003 12:36'!
= other
	^ other species = self species and: [other description = self description]! !

!MCRepository methodsFor: '' stamp: 'kph 8/1/2007 12:47'!
allVersionNames 
	"method for comapibility with MCConfigurations"
	^ self heldVersionNames! !

!MCRepository methodsFor: 'accessing' stamp: 'avi 8/31/2004 01:08'!
alwaysStoreDiffs
	^ storeDiffs ifNil: [false]! !

!MCRepository methodsFor: 'initialization' stamp: 'bkv 2/18/2004 20:48'!
asCreationTemplate
	^ self creationTemplate! !

!MCRepository methodsFor: 'required in subclasses' stamp: 'avi 8/26/2004 14:20'!
basicStoreVersion: aVersion
	self subclassResponsibility! !

!MCRepository methodsFor: 'misc' stamp: 'kph 5/16/2007 23:31'!
cacheAllFileNamesDuring: aBlock
 
	^ aBlock value! !

!MCRepository methodsFor: 'storing' stamp: 'avi 9/17/2005 16:12'!
closestAncestorVersionFor: anAncestry ifNone: errorBlock
	anAncestry breadthFirstAncestorsDo:
		[:ancestorInfo |
		(self versionWithInfo: ancestorInfo) ifNotNilDo: [:v | ^ v]].
	^ errorBlock value! !

!MCRepository methodsFor: 'initialization' stamp: 'bkv 2/18/2004 20:46'!
creationTemplate
	^ creationTemplate! !

!MCRepository methodsFor: 'initialization' stamp: 'stephaneducasse 2/4/2006 20:47'!
creationTemplate: aString
	self creationTemplate ifNotNil: [ self error: 'Creation template already set for this MCRepository instance.' ].
	
	creationTemplate := aString.! !

!MCRepository methodsFor: 'accessing' stamp: 'avi 10/9/2003 12:53'!
description
	^ self class name! !

!MCRepository methodsFor: 'accessing' stamp: 'kph 5/16/2007 22:48'!
descriptionDisplay
	^ self description! !

!MCRepository methodsFor: 'actions' stamp: 'stephaneducasse 2/4/2006 20:47'!
doAlwaysStoreDiffs
	storeDiffs := true! !

!MCRepository methodsFor: 'actions' stamp: 'stephaneducasse 2/4/2006 20:47'!
doNotAlwaysStoreDiffs
	storeDiffs := false! !

!MCRepository methodsFor: 'actions' stamp: 'ar 11/4/2005 16:58'!
flushPasswords! !

!MCRepository methodsFor: 'testing' stamp: 'ab 8/21/2003 12:36'!
hash
	^ self description hash! !

!MCRepository methodsFor: 'required in subclasses' stamp: 'kph 5/17/2007 00:21'!
heldVersionInfos
	^ self heldVersions collect: [ :ver | ver info ]! !

!MCRepository methodsFor: '' stamp: 'kph 6/2/2008 19:35'!
heldVersionInfosClearCache! !

!MCRepository methodsFor: '' stamp: 'kph 8/1/2007 12:46'!
heldVersionNames
	^ self heldVersionInfos collect: [:v |  v versionName ]! !

!MCRepository methodsFor: 'required in subclasses' stamp: 'kph 5/17/2007 00:22'!
heldVersions
	self subclassResponsibility! !

!MCRepository methodsFor: 'required in subclasses' stamp: 'kph 1/28/2008 02:17'!
includesVersionNamed: aString
	^ self heldVersionInfos anySatisfy: [:ea | ea versionName = aString]! !

!MCRepository methodsFor: 'accessing' stamp: 'kph 5/17/2007 00:21'!
includesVersionWithInfo: aVersionInfo
	^ self heldVersionInfos includes: aVersionInfo! !

!MCRepository methodsFor: 'testing' stamp: 'kph 5/14/2007 13:56'!
isDeletingAllowed

	^ false! !

!MCRepository methodsFor: 'testing' stamp: 'nk 11/2/2003 10:55'!
isValid
	^true! !

!MCRepository methodsFor: 'interface' stamp: 'lr 9/26/2003 20:03'!
morphicOpen
	self morphicOpen: nil! !

!MCRepository methodsFor: 'required in subclasses' stamp: 'mtf 6/7/2008 12:28'!
morphicOpen: aWorkingCopy
	(self class uiClass 
		repository: self 
		selectedPackage: (aWorkingCopy ifNotNilDo: [ :wc |  wc package ]) 
		selectedVersion: nil ) show! !

!MCRepository methodsFor: 'notifications' stamp: 'avi 8/26/2004 14:27'!
notificationForVersion: aVersion
	^ MCVersionNotification version: aVersion repository: self! !

!MCRepository methodsFor: 'notifications' stamp: 'avi 8/26/2004 14:23'!
notifyList
	^ #()! !

!MCRepository methodsFor: 'initialization' stamp: 'bf 4/14/2005 17:30'!
openAndEditTemplateCopy
	^ self class fillInTheBlankConfigure: (self asCreationTemplate ifNil: [^nil])! !

!MCRepository methodsFor: 'accessing' stamp: 'kph 5/17/2007 01:50'!
packageNames
	^ (self heldVersionInfos collect: [:ea | ea packageName]) asSet asSortedCollection! !

!MCRepository methodsFor: 'accessing' stamp: 'kph 5/17/2007 00:21'!
packages
	^ (self heldVersions collect: [:ea | ea package]) asSet asSortedCollection! !

!MCRepository methodsFor: 'storing' stamp: 'bf 3/10/2005 23:01'!
possiblyNewerVersionsOfAnyOf: someVersions
	^#()! !

!MCRepository methodsFor: 'storing' stamp: 'avi 8/31/2004 01:06'!
prepareVersionForStorage: aVersion
	^ self alwaysStoreDiffs
		ifTrue: [aVersion asDiffAgainst:
				 (self closestAncestorVersionFor: aVersion info ifNone: [^ aVersion])]
		ifFalse: [aVersion]! !

!MCRepository methodsFor: 'printing' stamp: 'mas 9/24/2003 04:21'!
printOn: aStream
	super printOn: aStream.
	aStream
		nextPut: $(;
		nextPutAll: self description;
		nextPut: $).! !

!MCRepository methodsFor: 'required in subclasses' stamp: 'kph 5/17/2007 03:14'!
realizeFullVersionInfo: aStubVersionInfoInstance

	"we use full versionInfos anyway"
	
	^ aStubVersionInfoInstance! !

!MCRepository methodsFor: 'notifications' stamp: 'stephaneducasse 2/4/2006 20:47'!
sendNotificationsForVersion: aVersion
	| notification notifyList |
	notifyList := self notifyList.
	notifyList isEmpty ifFalse:
		[notification := self notificationForVersion: aVersion.
		notifyList do: [:ea | notification notify: ea]]! !

!MCRepository methodsFor: 'accessing' stamp: 'kph 5/17/2007 00:28'!
sortedHeldVersionInfos 
	 
	^ self sortedVersionInfos select: [:ea | self heldVersionInfos includes: ea]! !

!MCRepository methodsFor: 'accessing' stamp: 'kph 5/17/2007 03:51'!
sortedVersionInfos 
	| sorter |
	sorter := MCVersionSorter new.
	self heldVersionInfos do: [:ea | sorter addVersionInfo: ea].
	^ sorter sortedVersionInfos
! !

!MCRepository methodsFor: 'accessing' stamp: 'kph 5/17/2007 02:03'!
sortedVersionInfosAvailableForPackageNamed: aPackageName 
	 
	^ self sortedVersionInfos 
		select: [:ea | (self versionInfosAvailableForPackageNamed: aPackageName)  includes: ea]! !

!MCRepository methodsFor: 'storing' stamp: 'avi 8/31/2004 01:05'!
storeVersion: aVersion
	self basicStoreVersion: (self prepareVersionForStorage: aVersion).
	self sendNotificationsForVersion: aVersion! !

!MCRepository methodsFor: 'accessing' stamp: 'kph 5/17/2007 02:02'!
versionInfosAvailableForPackageNamed: aPackageName
	^ self heldVersionInfos select: [:ea | ea packageName = aPackageName] ! !

!MCRepository methodsFor: 'interface' stamp: 'avi 10/9/2003 12:42'!
versionWithInfo: aVersionInfo
	^ self versionWithInfo: aVersionInfo ifAbsent: [nil]! !

!MCRepository methodsFor: 'required in subclasses' stamp: 'ab 8/16/2003 18:22'!
versionWithInfo: aVersionInfo ifAbsent: aBlock
	self subclassResponsibility ! !

!MCRepository methodsFor: 'required in subclasses' stamp: 'kph 5/16/2007 22:56'!
versions
	self subclassResponsibility! !

MCRepository subclass: #MCWriteOnlyRepository
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Repositories'!

MCWriteOnlyRepository subclass: #MCSMReleaseRepository
	instanceVariableNames: 'packageName user'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Repositories'!

!MCSMReleaseRepository class methodsFor: 'as yet unclassified' stamp: 'avi 2/10/2004 14:15'!
creationTemplate
	^
'MCSMReleaseRepository
	package: ''mypackage''
	user: ''squeak''
	password: ''squeak'''
	! !

!MCSMReleaseRepository class methodsFor: 'as yet unclassified' stamp: 'avi 2/10/2004 13:42'!
description
	^ 'SqueakMap Release'! !

!MCSMReleaseRepository class methodsFor: 'as yet unclassified' stamp: 'bkv 2/18/2004 21:03'!
fillInTheBlankRequest
	^  'SqueakMap Release Repository:'
		! !

!MCSMReleaseRepository class methodsFor: 'as yet unclassified' stamp: 'bkv 2/18/2004 21:03'!
morphicConfigure
	^ self fillInTheBlankConfigure! !

!MCSMReleaseRepository class methodsFor: 'as yet unclassified' stamp: 'avi 2/10/2004 14:14'!
package: packageString user: userString password: passString
	^ self basicNew initializeWithPackage: packageString user: userString password: passString! !

!MCSMReleaseRepository methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
basicStoreVersion: aVersion
	| url |
	url := self uploadVersion: aVersion.
	self releaseVersion: aVersion url: url! !

!MCSMReleaseRepository methodsFor: 'as yet unclassified' stamp: 'avi 2/10/2004 14:11'!
checkResult: resultString
(#( 'HTTP/1.1 201 ' 'HTTP/1.1 200 ' 'HTTP/1.0 201 ' 'HTTP/1.0 200 ')
		anySatisfy: [:code | resultString beginsWith: code ])
			ifFalse: [self error: resultString].
! !

!MCSMReleaseRepository methodsFor: 'as yet unclassified' stamp: 'avi 2/10/2004 14:15'!
description
	^ 'sm://', packageName! !

!MCSMReleaseRepository methodsFor: 'as yet unclassified' stamp: 'ar 11/4/2005 17:26'!
flushPasswords
	MCPasswordManager default passwordAt: self squeakMapUrl user: user put: nil! !

!MCSMReleaseRepository methodsFor: 'as yet unclassified' stamp: 'kph 5/3/2007 19:56'!
initializeWithPackage: packageString user: userString password: passString
	packageName := packageString.
	user := userString.
	self password: passString.
! !

!MCSMReleaseRepository methodsFor: 'as yet unclassified' stamp: 'ar 11/4/2005 17:26'!
password
	^(MCPasswordManager default queryPasswordAt: self squeakMapUrl user: user) ifNil:[^''].! !

!MCSMReleaseRepository methodsFor: 'as yet unclassified' stamp: 'ar 11/4/2005 17:26'!
password: passwordString
	| pwd |
	passwordString isEmpty ifTrue:[pwd := nil] ifFalse:[pwd := passwordString].
	MCPasswordManager default passwordAt: self squeakMapUrl user: user put: pwd.! !

!MCSMReleaseRepository methodsFor: 'as yet unclassified' stamp: 'kph 5/3/2007 19:55'!
releaseVersion: aVersion url: urlString
	| result |
	result := HTTPSocket
		httpPost: self squeakMapUrl, '/packagebyname/', packageName, '/newrelease'
		args: {'version' -> {(aVersion info name copyAfter: $.) extractNumber asString}.
			   'note' -> {aVersion info message}.
			   'downloadURL' -> {urlString}}
		user: user
		passwd: self password.
	result contents size > 4 ifTrue: [self error: result contents]
! !

!MCSMReleaseRepository methodsFor: 'as yet unclassified' stamp: 'kph 6/9/2008 00:46'!
squeakMapUrl 
	^ 'http://map.squeak.org/sm'

 ! !

!MCSMReleaseRepository methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
stringForVersion: aVersion
	| stream |
	stream := RWBinaryOrTextStream on: String new.
	aVersion fileOutOn: stream.
	^ stream contents! !

!MCSMReleaseRepository methodsFor: 'as yet unclassified' stamp: 'kph 5/3/2007 19:55'!
uploadVersion: aVersion
	| result stream |
	result := HTTPSocket
		httpPut: (self stringForVersion: aVersion)
		to: self squeakMapUrl, '/upload/', aVersion fileName
		user: user
		passwd: self password.
	self checkResult: result.
	stream := result readStream.
	stream upToAll: 'http://'.
	^ 'http://', stream upToEnd! !

MCWriteOnlyRepository subclass: #MCSmtpRepository
	instanceVariableNames: 'email'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Repositories'!

!MCSmtpRepository class methodsFor: 'as yet unclassified' stamp: 'avi 10/9/2003 12:56'!
description
	^ 'SMTP'! !

!MCSmtpRepository class methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
morphicConfigure
	| address |
	address := FillInTheBlankMorph request: 'Email address:'.
	^ address isEmpty ifFalse: [self new emailAddress: address]! !

!MCSmtpRepository methodsFor: 'as yet unclassified' stamp: 'avi 8/26/2004 14:21'!
basicStoreVersion: aVersion
	MailSender sendMessage: (self messageForVersion: aVersion)! !

!MCSmtpRepository methodsFor: 'as yet unclassified' stamp: 'avi 10/9/2003 13:11'!
bodyForVersion: aVersion
	^ String streamContents:
		[ :s |
		s nextPutAll: 'from version info:'; cr; cr.
		s nextPutAll:  aVersion info summary]! !

!MCSmtpRepository methodsFor: 'as yet unclassified' stamp: 'avi 10/9/2003 12:56'!
description
	^ 'mailto://', email! !

!MCSmtpRepository methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
emailAddress: aString
	email := aString	! !

!MCSmtpRepository methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
messageForVersion: aVersion
	| message data |
	message := MailMessage empty.
	message setField: 'from' toString: MailSender userName.
	message setField: 'to' toString: email.
	message setField: 'subject' toString: (self subjectForVersion: aVersion). 

	message body:
		(MIMEDocument
			contentType: 'text/plain'
			content: (self bodyForVersion: aVersion)).

	"Prepare the gzipped data"
	data := RWBinaryOrTextStream on: String new.
	aVersion fileOutOn: data.
	message addAttachmentFrom: data reset withName: aVersion fileName.
	^ message! !

!MCSmtpRepository methodsFor: 'as yet unclassified' stamp: 'avi 10/9/2003 13:14'!
subjectForVersion: aVersion
	^ '[Package] ', aVersion info name! !

!MCWriteOnlyRepository methodsFor: 'as yet unclassified' stamp: 'avi 10/9/2003 12:52'!
includesVersionNamed: aString
	^ false! !

!MCWriteOnlyRepository methodsFor: 'as yet unclassified' stamp: 'avi 10/9/2003 12:53'!
morphicOpen: aWorkingCopy
	self inform: 'This repository is write-only'! !

!MCWriteOnlyRepository methodsFor: 'as yet unclassified' stamp: 'avi 10/9/2003 12:52'!
versionWithInfo: aVersionInfo ifAbsent: aBlock
	^ aBlock value! !

Object subclass: #MCRepositoryGroup
	instanceVariableNames: 'repositories useCache'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Versioning'!

MCRepositoryGroup class
	instanceVariableNames: 'default'!

!MCRepositoryGroup commentStamp: '<historical>' prior: 0!
A singleton class, holds the list of repositories. Can look for a requested VersionInfo among its repositories.!

MCRepositoryGroup class
	instanceVariableNames: 'default'!

!MCRepositoryGroup class methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
default
	^ default ifNil: [default := self new]! !

!MCRepositoryGroup methodsFor: 'as yet unclassified' stamp: 'avi 1/27/2004 00:29'!
addRepository: aRepository
	((repositories includes: aRepository) or: [aRepository == MCCacheRepository default])
		ifFalse: [repositories add: aRepository.
				self class default addRepository: aRepository].
	self changed: #repositories! !

!MCRepositoryGroup methodsFor: 'as yet unclassified' stamp: 'NorbertHartl 6/20/2008 10:11'!
disableCache
	useCache := false! !

!MCRepositoryGroup methodsFor: 'as yet unclassified' stamp: 'avi 11/7/2003 00:20'!
includes: aRepository
	^ self repositories includes: aRepository! !

!MCRepositoryGroup methodsFor: 'as yet unclassified' stamp: 'abc 11/6/2004 20:32'!
includesVersionNamed: aString
	self repositoriesDo: [:ea | (ea includesVersionNamed: aString) ifTrue: [^ true]].
	^ false	! !

!MCRepositoryGroup methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
initialize
	repositories := OrderedCollection new! !

!MCRepositoryGroup methodsFor: 'as yet unclassified' stamp: 'ab 7/22/2003 00:11'!
removeRepository: aRepository
	repositories remove: aRepository ifAbsent: [].
	self changed: #repositories! !

!MCRepositoryGroup methodsFor: 'as yet unclassified' stamp: 'mtf 7/21/2008 18:34'!
repositories
	^ (self useCache 
		ifTrue: [Array with: MCCacheRepository default] 
		ifFalse: [Array new]) , repositories select: [ :ea | ea isValid ]
! !

!MCRepositoryGroup methodsFor: 'as yet unclassified' stamp: 'avi 11/7/2003 00:51'!
repositoriesDo: aBlock
	self repositories do: [:ea | [aBlock value: ea] on: Error do: []]! !

!MCRepositoryGroup methodsFor: 'as yet unclassified' stamp: 'abc 6/20/2008 10:02'!
useCache
	^ useCache ifNil: [
		useCache := true
	]! !

!MCRepositoryGroup methodsFor: 'as yet unclassified' stamp: 'dvf 8/10/2004 23:02'!
versionWithInfo: aVersionInfo
	^self versionWithInfo: aVersionInfo ifNone: [ self error: 'Could not find version ', aVersionInfo name printString,'. Maybe you need to add a repository?' ]! !

!MCRepositoryGroup methodsFor: 'as yet unclassified' stamp: 'nk 1/23/2004 18:15'!
versionWithInfo: aVersionInfo ifNone: aBlock
	self repositoriesDo: [:ea | (ea versionWithInfo: aVersionInfo) ifNotNilDo: [:v | ^ v]].
	^aBlock value! !

Object subclass: #MCScanner
	instanceVariableNames: 'stream'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Chunk Format'!

!MCScanner class methodsFor: 'as yet unclassified' stamp: 'avi 1/22/2004 20:14'!
scan: aStream
	^ (self new stream: aStream) next! !

!MCScanner class methodsFor: 'as yet unclassified' stamp: 'avi 1/22/2004 20:32'!
scanTokens: aString
	"compatibility"
	^ Array with: (self scan: aString readStream)! !

!MCScanner methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
next
	| c |
	stream skipSeparators.
	c := stream peek.
	c = $# ifTrue: [c := stream next; peek].
	c = $' ifTrue: [^ self nextString].
	c = $( ifTrue: [^ self nextArray].
	c isAlphaNumeric ifTrue: [^ self nextSymbol].
	self error: 'Unknown token type'.	! !

!MCScanner methodsFor: 'as yet unclassified' stamp: 'avi 1/22/2004 20:27'!
nextArray
	stream next. "("
	^ Array streamContents:
		[:s |
		[stream skipSeparators.
		(stream peek = $)) or: [stream atEnd]] whileFalse: [s nextPut: self next].
		stream next = $) ifFalse: [self error: 'Unclosed array']]! !

!MCScanner methodsFor: 'as yet unclassified' stamp: 'avi 1/22/2004 20:09'!
nextString
	^ stream nextDelimited: $'! !

!MCScanner methodsFor: 'as yet unclassified' stamp: 'avi 1/22/2004 20:16'!
nextSymbol
	^ (String streamContents:
		[:s |
		[stream peek isAlphaNumeric] whileTrue: [s nextPut: stream next]]) asSymbol
			! !

!MCScanner methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
stream: aStream
	stream := aStream! !

Object subclass: #MCSnapshot
	instanceVariableNames: 'definitions'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Base'!

MCSnapshot subclass: #MCImageBackedSnapshot
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Patching'!

!MCImageBackedSnapshot methodsFor: 'as yet unclassified' stamp: 'mtf 10/7/2008 17:05'!
asPatchSnapshot
	^ self! !

!MCImageBackedSnapshot methodsFor: 'as yet unclassified' stamp: 'mtf 10/7/2008 12:58'!
definitionIndex
	^ MCImageBackedDefinitionIndex definitions: definitions! !

!MCSnapshot class methodsFor: 'as yet unclassified' stamp: 'ab 7/6/2003 23:48'!
empty
	^ self fromDefinitions: #()! !

!MCSnapshot class methodsFor: 'as yet unclassified' stamp: 'kph 5/15/2007 01:43'!
fromDefinitions: aCollection
 
	^ self new initializeWithDefinitions: aCollection! !

!MCSnapshot methodsFor: 'accessing' stamp: 'ab 7/10/2003 01:05'!
= other
	^ definitions asArray = other definitions asArray! !

!MCSnapshot methodsFor: 'patching' stamp: 'mtf 10/7/2008 14:02'!
asPatchSnapshot
	^ MCImageBackedSnapshot fromDefinitions: definitions! !

!MCSnapshot methodsFor: 'patching' stamp: 'mtf 10/7/2008 12:57'!
definitionIndex
	^ MCDefinitionIndex definitions: definitions! !

!MCSnapshot methodsFor: 'accessing' stamp: 'ab 12/4/2002 18:09'!
definitions
	^ definitions! !

!MCSnapshot methodsFor: 'accessing' stamp: 'ab 7/10/2003 01:05'!
hash
	^ definitions asArray hash! !

!MCSnapshot methodsFor: 'initializing' stamp: 'stephaneducasse 2/4/2006 20:47'!
initializeWithDefinitions: aCollection
	definitions := aCollection.! !

!MCSnapshot methodsFor: 'loading' stamp: 'kph 5/15/2007 13:10'!
install
	(MCPackageLoader new)
		installSnapshot: self;
		installOrphanage;
		load! !

!MCSnapshot methodsFor: 'accessing' stamp: 'kph 5/10/2007 18:10'!
isEmpty
	^ definitions isEmpty! !

!MCSnapshot methodsFor: 'patching' stamp: 'ab 7/7/2003 00:37'!
patchRelativeToBase: aSnapshot
	^ MCPatch fromBase: aSnapshot target: self! !

!MCSnapshot methodsFor: 'loading' stamp: 'kph 5/15/2007 12:14'!
updatePackage: aPackage
	(MCPackageLoader new)
		updatePackage: aPackage withSnapshot: self;
		installOrphanage;
		load
		! !

Object subclass: #MCTool
	instanceVariableNames: 'morph label modal modalValue'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-UI'!

MCTool subclass: #MCCodeTool
	instanceVariableNames: 'items'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-UI'!

!MCCodeTool commentStamp: 'nk 11/10/2003 22:00' prior: 0!
MCCodeTool is an abstract superclass for those Monticello browsers that display code.
It contains copies of the various CodeHolder methods that perform the various menu operations in the method list.
!

!MCCodeTool methodsFor: 'menus' stamp: 'nk 11/10/2003 21:00'!
adoptMessageInCurrentChangeset
	"Add the receiver's method to the current change set if not already there"

	self selectedClassOrMetaClass ifNotNilDo: [ :cl |
		self selectedMessageName ifNotNilDo: [ :sel |
			ChangeSet current adoptSelector: sel forClass: cl.
			self changed: #annotations ]]
! !

!MCCodeTool methodsFor: 'subclassResponsibility' stamp: 'nk 11/10/2003 22:01'!
annotations
	"Build an annotations string for the various browsers"
	^''! !

!MCCodeTool methodsFor: 'menus' stamp: 'sd 5/10/2008 17:31'!
browseFullProtocol
	"Open up a protocol-category browser on the value of the
	receiver's current selection."
	
	^ self spawnFullProtocol! !

!MCCodeTool methodsFor: 'menus' stamp: 'nk 11/10/2003 21:26'!
browseMessages
	"Present a menu of the currently selected message, as well as all messages sent by it.  Open a message set browser of all implementors of the selector chosen."

	self systemNavigation browseAllImplementorsOf: (self selectedMessageName ifNil: [ ^nil ])! !

!MCCodeTool methodsFor: 'menus' stamp: 'stephaneducasse 2/4/2006 20:47'!
browseMethodFull
	"Create and schedule a full Browser and then select the current class and message."

	| myClass |
	(myClass := self selectedClassOrMetaClass) ifNotNil:
		[Browser fullOnClass: myClass selector: self selectedMessageName]! !

!MCCodeTool methodsFor: 'menus' stamp: 'nk 11/10/2003 20:54'!
browseSendersOfMessages
	"Present a menu of the currently selected message, as well as all messages sent by it.  Open a message set browser of all senders of the selector chosen."

	self systemNavigation browseAllCallsOn: (self selectedMessageName ifNil: [ ^nil ])! !

!MCCodeTool methodsFor: 'menus' stamp: 'stephaneducasse 2/4/2006 20:47'!
browseVersions
	"Create and schedule a message set browser on all versions of the 
	currently selected message selector."

	| class selector compiledMethod |
	class := self selectedClassOrMetaClass.
	selector := self selectedMessageName.
	compiledMethod := class compiledMethodAt: selector ifAbsent: [ ^self ].
	VersionsBrowser
		browseVersionsOf: compiledMethod
		class: class theNonMetaClass
		meta: class isMeta
		category: self selectedMessageCategoryName
		selector: selector! !

!MCCodeTool methodsFor: 'menus' stamp: 'nk 7/30/2004 17:56'!
classHierarchy
	"Create and schedule a class list browser on the receiver's hierarchy."

	self systemNavigation  spawnHierarchyForClass: self selectedClassOrMetaClass
		selector: self selectedMessageName	"OK if nil"! !

!MCCodeTool methodsFor: 'menus' stamp: 'nk 6/12/2004 14:01'!
classListMenu: aMenu 

	aMenu addList: #(
		-
		('browse full (b)'			browseMethodFull)
		('browse hierarchy (h)'		classHierarchy)
		('browse protocol (p)'		browseFullProtocol)
"		-
		('printOut'					printOutClass)
		('fileOut'					fileOutClass)
"		-
		('show hierarchy'			methodHierarchy)
"		('show definition'			editClass)
		('show comment'			editComment)
"
"		-
		('inst var refs...'			browseInstVarRefs)
		('inst var defs...'			browseInstVarDefs)
		-
		('class var refs...'			browseClassVarRefs)
		('class vars'					browseClassVariables)
		('class refs (N)'				browseClassRefs)
		-
		('rename class ...'			renameClass)
		('copy class'				copyClass)
		('remove class (x)'			removeClass)
"
		-
		('find method...'				findMethodInChangeSets)).
							
	^aMenu! !

!MCCodeTool methodsFor: 'menus' stamp: 'jl 11/1/2006 10:40'!
copyClassAndSelector
	"Copy the selected selector to the clipboard"

	Clipboard clipboardText: '',self selectedClassOrMetaClass,'>>',self selectedMessageName! !

!MCCodeTool methodsFor: 'menus' stamp: 'stephaneducasse 2/4/2006 20:47'!
copySelector
	"Copy the selected selector to the clipboard"

	| selector |
	(selector := self selectedMessageName) ifNotNil:
		[Clipboard clipboardText: selector asString]! !

!MCCodeTool methodsFor: 'menus' stamp: 'nk 11/10/2003 20:55'!
fileOutMessage
	"Put a description of the selected message on a file"

	self selectedMessageName ifNotNil:
		[Cursor write showWhile:
			[self selectedClassOrMetaClass fileOutMethod: self selectedMessageName]]! !

!MCCodeTool methodsFor: 'menus' stamp: 'stephaneducasse 2/4/2006 20:47'!
findMethodInChangeSets
	"Find and open a changeSet containing the current method."

	| aName |
	(aName := self selectedMessageName) ifNotNil: [
		ChangeSorter browseChangeSetsWithClass: self selectedClassOrMetaClass
					selector: aName]! !

!MCCodeTool methodsFor: 'menus' stamp: 'nk 7/30/2004 17:56'!
methodHierarchy
	"Create and schedule a method browser on the hierarchy of implementors."

	self systemNavigation methodHierarchyBrowserForClass: self selectedClassOrMetaClass
		selector: self selectedMessageName! !

!MCCodeTool methodsFor: 'menus' stamp: 'nk 2/16/2004 17:00'!
methodListKey: aKeystroke from: aListMorph 
	aKeystroke caseOf: {
		[$b] -> [self browseMethodFull].
		[$h] -> [self classHierarchy].
		[$O] -> [self openSingleMessageBrowser].
		[$p] -> [self browseFullProtocol].
		[$o] -> [self fileOutMessage].
		[$c] -> [self copySelector].
		[$n] -> [self browseSendersOfMessages].
		[$m] -> [self browseMessages].
		[$i] -> [self methodHierarchy].
		[$v] -> [self browseVersions]}
		 otherwise: []! !

!MCCodeTool methodsFor: 'menus' stamp: 'md 4/30/2008 15:39'!
methodListMenu: aMenu
	"Build the menu for the selected method, if any."
	
	self selectedMessageName ifNotNil: [
	aMenu addList:#(
			('browse full (b)' 						browseMethodFull)
			('browse hierarchy (h)'					classHierarchy)
			('browse method (O)'					openSingleMessageBrowser)
			('browse protocol (p)'					browseFullProtocol)
			-
			('fileOut (o)'							fileOutMessage)
			('copy selector (c)'						copySelector)).
		aMenu addList: #(
			-
			('browse senders (n)'						browseSendersOfMessages)
			('browse implementors (m)'					browseMessages)
			('inheritance (i)'						methodHierarchy)
			('versions (v)'							browseVersions)
		('change sets with this method'			findMethodInChangeSets)
"		('x revert to previous version'				revertToPreviousVersion)"
		('remove from current change set'		removeFromCurrentChanges)
"		('x revert & remove from changes'		revertAndForget)"
		('add to current change set'				adoptMessageInCurrentChangeset)
"		('x copy up or copy down...'				copyUpOrCopyDown)"
"		('x remove method (x)'					removeMessage)"
		"-"
		).
	].
"	aMenu addList: #(
			('x inst var refs...'						browseInstVarRefs)
			('x inst var defs...'						browseInstVarDefs)
			('x class var refs...'						browseClassVarRefs)
			('x class variables'						browseClassVariables)
			('x class refs (N)'							browseClassRefs)
	).
"
	^ aMenu
! !

!MCCodeTool methodsFor: 'menus' stamp: 'stephaneducasse 2/4/2006 20:47'!
openSingleMessageBrowser
	| msgName mr |
	"Create and schedule a message list browser populated only by the currently selected message"

	(msgName := self selectedMessageName) ifNil: [^ self].

	mr := MethodReference new
		setStandardClass: self selectedClassOrMetaClass
		methodSymbol: msgName.

	self systemNavigation 
		browseMessageList: (Array with: mr)
		name: mr asStringOrText
		autoSelect: nil! !

!MCCodeTool methodsFor: 'menus' stamp: 'nk 11/10/2003 20:55'!
perform: selector orSendTo: otherTarget 

	"Selector was just chosen from a menu by a user. If can respond, then  
	perform it on myself. If not, send it to otherTarget, presumably the  
	editPane from which the menu was invoked."

	(self respondsTo: selector)
		ifTrue: [^ self perform: selector]
		ifFalse: [^ super perform: selector orSendTo: otherTarget]! !

!MCCodeTool methodsFor: 'menus' stamp: 'nk 11/10/2003 21:00'!
removeFromCurrentChanges
	"Tell the changes mgr to forget that the current msg was changed."

	ChangeSet current removeSelectorChanges: self selectedMessageName 
			class: self selectedClassOrMetaClass.
	self changed: #annotations! !

!MCCodeTool methodsFor: 'subclassResponsibility' stamp: 'nk 11/10/2003 22:02'!
selectedClass
	"Answer the class that is selected, or nil"
	self subclassResponsibility! !

!MCCodeTool methodsFor: 'subclassResponsibility' stamp: 'nk 11/10/2003 22:02'!
selectedClassOrMetaClass
	"Answer the class that is selected, or nil"
	self subclassResponsibility! !

!MCCodeTool methodsFor: 'subclassResponsibility' stamp: 'nk 11/10/2003 22:02'!
selectedMessageCategoryName
	"Answer the method category of the method that is selected, or nil"
	self subclassResponsibility! !

!MCCodeTool methodsFor: 'subclassResponsibility' stamp: 'nk 11/10/2003 22:02'!
selectedMessageName
	"Answer the name of the selected message"
	self subclassResponsibility! !

MCCodeTool subclass: #MCPatchBrowser
	instanceVariableNames: 'selection'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-UI'!

MCPatchBrowser subclass: #MCChangeSelector
	instanceVariableNames: 'kept'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-UI'!

!MCChangeSelector methodsFor: 'as yet unclassified' stamp: 'avi 9/11/2004 16:07'!
buttonSpecs
	^ #((Select select 'Select these changes')
		 (Cancel cancel 'Cancel the operation')
		)! !

!MCChangeSelector methodsFor: 'as yet unclassified' stamp: 'avi 9/11/2004 16:26'!
cancel
	self answer: nil! !

!MCChangeSelector methodsFor: 'as yet unclassified' stamp: 'avi 9/14/2004 15:01'!
defaultLabel
	^ 'Change Selector'! !

!MCChangeSelector methodsFor: 'as yet unclassified' stamp: 'avi 9/11/2004 16:13'!
innerButtonRow
	^ self buttonRow:
		#(('Select All' selectAll 'select all changes')
		  ('Select None' selectNone 'select no changes'))! !

!MCChangeSelector methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
kept
	^ kept ifNil: [kept := Set new]! !

!MCChangeSelector methodsFor: 'as yet unclassified' stamp: 'avi 9/11/2004 16:22'!
listSelectionAt: aNumber
	^ self kept includes: (self items at: aNumber)! !

!MCChangeSelector methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
listSelectionAt: aNumber put: aBoolean
	| item |
	item := self items at: aNumber.
	aBoolean
		ifTrue: [self kept add: item ]
		ifFalse: [self kept remove: item ifAbsent: []]! !

!MCChangeSelector methodsFor: 'as yet unclassified' stamp: 'avi 9/11/2004 16:26'!
select
	self answer: (MCPatch operations: kept)! !

!MCChangeSelector methodsFor: 'as yet unclassified' stamp: 'avi 9/14/2004 15:00'!
selectAll
	kept addAll: self items.
	self changed: #list! !

!MCChangeSelector methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
selectNone
	kept := Set new.
	self changed: #list! !

!MCChangeSelector methodsFor: 'as yet unclassified' stamp: 'avi 9/11/2004 16:25'!
widgetSpecs
	Preferences annotationPanes ifFalse: [ ^#(
		((buttonRow) (0 0 1 0) (0 0 0 30))
		((multiListMorph:selection:listSelection:menu: list selection listSelectionAt: methodListMenu:) (0 0 1 0.4) (0 30 0 0))
		((innerButtonRow) (0 0.4 1 0.4) (0 0 0 30))
		((textMorph: text) (0 0.4 1 1) (0 30 0 0))
		)].

	^ #(
		((buttonRow) (0 0 1 0) (0 0 0 30))
		((multiListMorph:selection:listSelection:menu: list selection listSelectionAt: methodListMenu:) (0 0 1 0.4) (0 30 0 0))
		((innerButtonRow) (0 0.4 1 0.4) (0 0 0 30))
		((textMorph: annotations) (0 0.4 1 0.4) (0 30 0 60))
		((textMorph: text) (0 0.4 1 1) (0 60 0 0))
		)! !

MCPatchBrowser subclass: #MCMergeBrowser
	instanceVariableNames: 'conflicts merger ok'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-UI'!

!MCMergeBrowser class methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
resolveConflictsInMerger: aMerger
	| inst |
	inst := self new merger: aMerger.
	^ inst showModally ifNil: [false]! !

!MCMergeBrowser methodsFor: 'as yet unclassified' stamp: 'nk 10/21/2003 23:35'!
buttonSpecs
	^ #((Merge merge 'Proceed with the merge' canMerge)
		 (Cancel cancel 'Cancel the merge')
		('All Newer' chooseAllNewerConflicts 'Choose all newer conflict versions')
		('All Older' chooseAllOlderConflicts 'Choose all older conflict versions')
		('Rest Local' chooseAllUnchosenLocal 'Choose local versions of all remaining conflicts')
		('Rest Remote' chooseAllUnchosenRemote 'Choose remote versions of all remaining conflicts')
)! !

!MCMergeBrowser methodsFor: 'as yet unclassified' stamp: 'ab 7/22/2003 00:51'!
canMerge
	^ merger isMerged! !

!MCMergeBrowser methodsFor: 'as yet unclassified' stamp: 'ab 7/18/2003 17:52'!
cancel
	self answer: false! !

!MCMergeBrowser methodsFor: 'as yet unclassified' stamp: 'nk 10/21/2003 23:21'!
chooseAllNewerConflicts
	conflicts do: [ :ea | ea chooseNewer ].
	self changed: #text; changed: #list.! !

!MCMergeBrowser methodsFor: 'as yet unclassified' stamp: 'nk 10/21/2003 23:22'!
chooseAllOlderConflicts
	conflicts do: [ :ea | ea chooseOlder ].
	self changed: #text; changed: #list.! !

!MCMergeBrowser methodsFor: 'as yet unclassified' stamp: 'nk 10/21/2003 23:34'!
chooseAllUnchosenLocal
	conflicts do: [ :ea | ea isResolved ifFalse: [ ea chooseLocal ] ].
	self changed: #text; changed: #list.! !

!MCMergeBrowser methodsFor: 'as yet unclassified' stamp: 'nk 10/21/2003 23:34'!
chooseAllUnchosenRemote
	conflicts do: [ :ea | ea isResolved ifFalse: [ ea chooseRemote ] ].
	self changed: #text; changed: #list.! !

!MCMergeBrowser methodsFor: 'as yet unclassified' stamp: 'ab 7/18/2003 18:41'!
chooseLocal
	self conflictSelectionDo:
		[selection chooseLocal.
		self changed: #text; changed: #list]! !

!MCMergeBrowser methodsFor: 'as yet unclassified' stamp: 'ab 7/18/2003 18:41'!
chooseRemote
	self conflictSelectionDo:
		[selection chooseRemote.
		self changed: #text; changed: #list]! !

!MCMergeBrowser methodsFor: 'as yet unclassified' stamp: 'ab 7/18/2003 18:41'!
clearChoice
	self conflictSelectionDo:
		[selection clearChoice.
		self changed: #text; changed: #list]! !

!MCMergeBrowser methodsFor: 'as yet unclassified' stamp: 'ab 7/18/2003 18:42'!
conflictSelectionDo: aBlock
	self selectionIsConflicted
		ifTrue: aBlock
		ifFalse: [self inform: 'You must have a conflict selected']! !

!MCMergeBrowser methodsFor: 'as yet unclassified' stamp: 'ab 7/19/2003 21:31'!
defaultLabel
	^ 'Merge Browser'! !

!MCMergeBrowser methodsFor: 'as yet unclassified' stamp: 'ab 7/18/2003 18:04'!
getConflictMenu: aMenu
	selection remoteChosen
		ifTrue: [aMenu add: 'undo keep change' target: self selector: #clearChoice]
		ifFalse: [aMenu add: 'keep change' target: self selector: #chooseRemote].
	selection localChosen
		ifTrue: [aMenu add: 'undo reject change' target: self selector: #clearChoice]	
		ifFalse: [aMenu add: 'reject change' target: self selector: #chooseLocal].
	^ aMenu! !

!MCMergeBrowser methodsFor: 'as yet unclassified' stamp: 'ab 7/18/2003 18:07'!
getMenu: aMenu
	selection ifNil: [^ aMenu].
	^ self selectionIsConflicted
		ifTrue: [self getConflictMenu: aMenu]
		ifFalse: [self getOperationMenu: aMenu]! !

!MCMergeBrowser methodsFor: 'as yet unclassified' stamp: 'ab 7/18/2003 16:37'!
getOperationMenu: aMenu
	^ aMenu! !

!MCMergeBrowser methodsFor: 'as yet unclassified' stamp: 'ab 7/22/2003 00:49'!
innerButtonRow
	^ self buttonRow:
		#((Keep chooseRemote 'keep the selected change' selectionIsConflicted)
		  (Reject chooseLocal 'reject the selected change' selectionIsConflicted))! !

!MCMergeBrowser methodsFor: 'as yet unclassified' stamp: 'ab 7/18/2003 16:32'!
items
	^ conflicts, items! !

!MCMergeBrowser methodsFor: 'as yet unclassified' stamp: 'ab 7/18/2003 17:52'!
merge
	merger isMerged
		ifFalse: [self inform: 'You must resolve all the conflicts first']
		ifTrue: [self answer: true] ! !

!MCMergeBrowser methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
merger: aMerger
	merger := aMerger.
	items := aMerger operations asSortedCollection.
	conflicts := aMerger conflicts.! !

!MCMergeBrowser methodsFor: 'as yet unclassified' stamp: 'ab 7/18/2003 18:07'!
selectionIsConflicted
	^ selection isKindOf: MCConflict! !

!MCMergeBrowser methodsFor: 'as yet unclassified' stamp: 'nk 11/10/2003 22:08'!
widgetSpecs
	Preferences annotationPanes ifFalse: [ ^#(
		((buttonRow) (0 0 1 0) (0 0 0 30))
		((listMorph:selection:menu: list selection methodListMenu:) (0 0 1 0.4) (0 30 0 0))
		((innerButtonRow) (0 0.4 1 0.4) (0 0 0 30))
		((textMorph: text) (0 0.4 1 1) (0 30 0 0))
		)].

	^ #(
		((buttonRow) (0 0 1 0) (0 0 0 30))
		((listMorph:selection:menu: list selection methodListMenu:) (0 0 1 0.4) (0 30 0 0))
		((innerButtonRow) (0 0.4 1 0.4) (0 0 0 30))
		((textMorph: annotations) (0 0.4 1 0.4) (0 30 0 60))
		((textMorph: text) (0 0.4 1 1) (0 60 0 0))
		)! !

!MCPatchBrowser class methodsFor: 'as yet unclassified' stamp: 'ab 7/16/2003 14:35'!
forPatch: aPatch
	^ self new patch: aPatch! !

!MCPatchBrowser methodsFor: 'as yet unclassified' stamp: 'nk 11/10/2003 21:41'!
annotations
	^selection ifNil: [ super annotations ]
		ifNotNil: [ selection annotations ]! !

!MCPatchBrowser methodsFor: 'morphic ui' stamp: 'ab 8/22/2003 02:21'!
buttonSpecs
	^ #((Invert invert 'Show the reverse set of changes')
		 (Export export 'Export the changes as a change set'))! !

!MCPatchBrowser methodsFor: 'as yet unclassified' stamp: 'nk 2/23/2005 08:04'!
changeSetNameForInstall
	"Answer the name of the change set into which my selection will be installed.
	Derive this from my label.
	If I have no label, use the current change set."

	| tokens |
	label ifNil: [ ^ChangeSet current name ].
	tokens := label findTokens: ' '.
	tokens removeAllFoundIn: { 'changes'. 'between'. 'and' }.
	(tokens size = 3 and: [ tokens second = '<working' ]) ifTrue: [ ^tokens first, '-to-working' ].
	tokens size = 2 ifFalse: [ ^'InstalledPatches' ].
	^'{1}-to-{2}' format: tokens ! !

!MCPatchBrowser methodsFor: 'morphic ui' stamp: 'ab 7/19/2003 21:31'!
defaultLabel
	^ 'Patch Browser'! !

!MCPatchBrowser methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
installSelection
	| loader |
	selection ifNotNil:
		[loader := MCPackageLoader new.
		selection applyTo: loader.
		loader loadWithName: self changeSetNameForInstall ]! !

!MCPatchBrowser methodsFor: 'selecting' stamp: 'stephaneducasse 2/4/2006 20:47'!
invert
	items := items collect: [:ea | ea inverse].
	self changed: #list; changed: #text; changed: #selection! !

!MCPatchBrowser methodsFor: 'accessing' stamp: 'ab 7/16/2003 14:36'!
items
	^ items! !

!MCPatchBrowser methodsFor: 'accessing' stamp: 'ab 7/16/2003 14:39'!
list
	^ self items collect: [:ea | ea summary]! !

!MCPatchBrowser methodsFor: 'menus' stamp: 'bf 3/5/2006 14:10'!
methodListMenu: aMenu
	selection ifNotNil:
		[aMenu addList:#(
			('install'	 installSelection)
			('revert'	 revertSelection)
			-)].
	super methodListMenu: aMenu.
	^ aMenu
! !

!MCPatchBrowser methodsFor: 'accessing' stamp: 'stephaneducasse 2/4/2006 20:47'!
patch: aPatch
	items := aPatch operations asSortedCollection! !

!MCPatchBrowser methodsFor: 'morphic ui' stamp: 'nk 11/10/2003 20:55'!
perform: selector orSendTo: otherTarget
	"Selector was just chosen from a menu by a user.  If can respond, then
perform it on myself. If not, send it to otherTarget, presumably the
editPane from which the menu was invoked."

	(self respondsTo: selector)
		ifTrue: [^ self perform: selector]
		ifFalse: [^ otherTarget perform: selector]! !

!MCPatchBrowser methodsFor: 'as yet unclassified' stamp: 'mtf 6/9/2008 20:57'!
revertSelection
	| loader |
	selection ifNotNil:
		[loader := MCPackageLoader new.
		selection inverse applyTo: loader.
		loader loadWithName: self changeSetNameForInstall ]! !

!MCPatchBrowser methodsFor: 'subclassResponsibility' stamp: 'stephaneducasse 2/4/2006 20:47'!
selectedClass
	| definition |
	selection ifNil: [ ^nil ].
	(definition := selection definition) ifNil: [ ^nil ].
	definition isMethodDefinition ifFalse: [ ^nil ].
	^Smalltalk at: definition className ifAbsent: [ ]! !

!MCPatchBrowser methodsFor: 'subclassResponsibility' stamp: 'stephaneducasse 2/4/2006 20:47'!
selectedClassOrMetaClass
	| definition |
	selection ifNil: [ ^nil ].
	(definition := selection definition) ifNil: [ ^nil ].
	definition isMethodDefinition ifFalse: [ ^nil ].
	^definition actualClass! !

!MCPatchBrowser methodsFor: 'subclassResponsibility' stamp: 'stephaneducasse 2/4/2006 20:47'!
selectedMessageCategoryName
	| definition |
	selection ifNil: [ ^nil ].
	(definition := selection definition) ifNil: [ ^nil ].
	definition isMethodDefinition ifFalse: [ ^nil ].
	^definition category! !

!MCPatchBrowser methodsFor: 'subclassResponsibility' stamp: 'stephaneducasse 2/4/2006 20:47'!
selectedMessageName
	| definition |
	selection ifNil: [ ^nil ].
	(definition := selection definition) ifNil: [ ^nil ].
	definition isMethodDefinition ifFalse: [ ^nil ].
	^definition  selector! !

!MCPatchBrowser methodsFor: 'selecting' stamp: 'ab 7/16/2003 14:30'!
selection
	^ selection 
		ifNil: [0]
		ifNotNil: [self items indexOf: selection]! !

!MCPatchBrowser methodsFor: 'selecting' stamp: 'stephaneducasse 2/4/2006 20:47'!
selection: aNumber
	selection := aNumber = 0 ifFalse: [self items at: aNumber].
	self changed: #selection; changed: #text; changed: #annotations! !

!MCPatchBrowser methodsFor: 'text' stamp: 'ab 7/16/2003 14:40'!
text
	^ selection ifNil: [''] ifNotNil: [selection source]! !

!MCPatchBrowser methodsFor: 'text' stamp: 'ab 7/16/2003 14:27'!
text: aTextOrString
	self changed: #text! !

!MCPatchBrowser methodsFor: 'morphic ui' stamp: 'mtf 10/7/2008 18:32'!
widgetSpecs
	Preferences annotationPanes ifFalse: [ ^#(
		((listMorph:selection:menu:keystroke: list selection methodListMenu: methodListKey:from: ) (0 0 1 0.4) (0 0 0 0))
		((textMorph: text) (0 0.4 1 1))
		) ].

	^ {
		#((listMorph:selection:menu:keystroke: list selection methodListMenu: methodListKey:from: ) (0 0 1 0.4) (0 0 0 0)).
		{ #(textMorph: annotations). #(0 0.4 1 0.4). { 0. 0. 0. self defaultAnnotationPaneHeight. } }.
		{ #(textMorph: text). #(0 0.4 1 1). { 0. self defaultAnnotationPaneHeight. 0. 0. } }.
		}! !

MCCodeTool subclass: #MCSnapshotBrowser
	instanceVariableNames: 'categorySelection classSelection protocolSelection methodSelection switch'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-UI'!

!MCSnapshotBrowser class methodsFor: 'as yet unclassified' stamp: 'ab 7/19/2003 18:03'!
forSnapshot: aSnapshot
	^ self new snapshot: aSnapshot! !

!MCSnapshotBrowser methodsFor: 'accessing' stamp: 'ab 7/18/2003 15:47'!
allClassNames
	^ (items 
		select: [:ea | ea isOrganizationDefinition not] 
		thenCollect: [:ea | ea className]) asSet.
! !

!MCSnapshotBrowser methodsFor: 'text' stamp: 'nk 7/24/2003 13:40'!
annotations
	methodSelection ifNotNil: [^ methodSelection annotations ].
	^ ''! !

!MCSnapshotBrowser methodsFor: 'text' stamp: 'nk 7/24/2003 13:41'!
annotations: stuff
	self changed: #annotations! !

!MCSnapshotBrowser methodsFor: 'morphic ui' stamp: 'bf 5/27/2005 19:33'!
buttonSpecs
	^ #(('instance' switchBeInstance 'show instance' buttonEnabled switchIsInstance)
		('?' switchBeComment 'show comment' buttonEnabled switchIsComment)
		('class' switchBeClass 'show class' buttonEnabled switchIsClass))! !

!MCSnapshotBrowser methodsFor: 'listing' stamp: 'cwp 7/10/2003 18:33'!
categoryList
	^ self visibleCategories! !

!MCSnapshotBrowser methodsFor: 'menus' stamp: 'nk 4/17/2004 09:52'!
categoryListMenu: aMenu 
	categorySelection
		ifNotNil: [aMenu
				add: (categorySelection = '*Extensions'
						ifTrue: ['load all extension methods' translated]
						ifFalse: ['load class category {1}' translated format: {categorySelection}])
				action: #loadCategorySelection].
	^ aMenu! !

!MCSnapshotBrowser methodsFor: 'selecting' stamp: 'cwp 7/10/2003 18:33'!
categorySelection
	^ categorySelection ifNil: [0] ifNotNil: [self visibleCategories indexOf: categorySelection]! !

!MCSnapshotBrowser methodsFor: 'selecting' stamp: 'stephaneducasse 2/4/2006 20:47'!
categorySelection: aNumber
	categorySelection := aNumber = 0 ifFalse: [self visibleCategories at: aNumber].
	self classSelection: 0.
	self changed: #categorySelection;
		changed: #annotations;
		changed: #classList.
! !

!MCSnapshotBrowser methodsFor: 'text' stamp: 'ab 7/18/2003 15:48'!
classCommentString
	^ (items 
		detect: [:ea | ea isClassDefinition and: [ea className = classSelection]]
		ifNone: [^ '']) comment.! !

!MCSnapshotBrowser methodsFor: 'text' stamp: 'kph 5/21/2007 16:29'!
classDefinitionString
	| defs |
	defs := items select: [:ea | (ea isClassDefinition or: [ea isClassDefinitionExtension])
			and: [ea className = classSelection]].

	defs isEmpty ifTrue: [^ 'This class is defined elsewhere.'].

	^ String streamContents: [:stream | 
		defs asArray sort 
			do: [:ea | ea printDefinitionAndCommentOn: stream]
			separatedBy: [stream nextPut: $.; cr]
	].! !

!MCSnapshotBrowser methodsFor: 'listing' stamp: 'cwp 7/10/2003 20:20'!
classList
	^ self visibleClasses! !

!MCSnapshotBrowser methodsFor: 'accessing' stamp: 'kph 5/13/2007 04:29'!
classListMenu: aMenu 
	classSelection ifNil: [ ^aMenu ].

	super classListMenu: aMenu.

	aMenu
		addLine;
				add: ('load class {1} (all)' translated format: {classSelection})
				action: #loadClassSelection;
				add: ('load class {1} (defn only) '  format: {classSelection})
				action: #loadClassSelectionDefinition.
	^ aMenu! !

!MCSnapshotBrowser methodsFor: 'selecting' stamp: 'cwp 7/10/2003 18:28'!
classSelection
	^ classSelection ifNil: [0] ifNotNil: [self visibleClasses indexOf: classSelection]! !

!MCSnapshotBrowser methodsFor: 'selecting' stamp: 'stephaneducasse 2/4/2006 20:47'!
classSelection: aNumber
	classSelection := aNumber = 0 ifFalse: [self visibleClasses at: aNumber].
	self protocolSelection: 0.
	self changed: #classSelection; 
		changed: #protocolList;
		changed: #annotations;
		changed: #methodList.
! !

!MCSnapshotBrowser methodsFor: 'morphic ui' stamp: 'ab 7/18/2003 17:29'!
defaultExtent
	^ 650@400.! !

!MCSnapshotBrowser methodsFor: 'morphic ui' stamp: 'ab 7/19/2003 21:31'!
defaultLabel
	^ 'Snapshot Browser'! !

!MCSnapshotBrowser methodsFor: 'text' stamp: 'kph 5/13/2007 04:29'!
definitionsForSelectedClassCategory
	| visibleClasses |
	visibleClasses := self visibleClasses.
	^ items select: [:ea | visibleClasses includes: ea className ]
								! !

!MCSnapshotBrowser methodsFor: 'accessing' stamp: 'kph 5/11/2007 04:45'!
extensionClassNames
 
	^ (self allClassNames difference: self packageClassNames) asSortedCollection! !

!MCSnapshotBrowser methodsFor: 'accessing' stamp: 'ab 7/5/2003 23:41'!
extensionsCategory
	^ '*Extensions'! !

!MCSnapshotBrowser methodsFor: 'menus' stamp: 'cwp 7/10/2003 18:03'!
inspectSelection
	^ self methodSelection inspect! !

!MCSnapshotBrowser methodsFor: 'menus' stamp: 'kph 5/13/2007 04:13'!
loadCategorySelection
 
	categorySelection ifNil: [ ^self ]. 

	(MCSnapshot fromDefinitions: self definitionsForSelectedClassCategory ) install! !

!MCSnapshotBrowser methodsFor: 'text' stamp: 'kph 5/15/2007 11:59'!
loadClassSelection

	| defns |

	classSelection ifNil: [ ^self ].
	
	"defns := self methodsForSelectedClass.
	
	defns add: (self packageClasses 
						detect: [ :ea | ea className = classSelection ] 
						ifNone: [ ^self ])."
	"easier implementation"
	
	defns := items select: [ :d | (d respondsTo: #className) and: [ d className = classSelection ] ].
	
	(MCSnapshot fromDefinitions: defns) install! !

!MCSnapshotBrowser methodsFor: 'listing' stamp: 'kph 5/15/2007 13:11'!
loadClassSelectionDefinition

	| classDefinition |
	
	classSelection ifNil: [ ^self ].

	classDefinition := items 
		detect: [ :defn | defn isClassDefinition and: [ defn className = classSelection ] ].
	
	(MCPackageLoader new) 
		addDefinition: classDefinition;
		load
	! !

!MCSnapshotBrowser methodsFor: 'menus' stamp: 'kph 5/15/2007 13:09'!
loadMethodSelection
	methodSelection ifNil: [ ^self ].
	(MCPackageLoader new) 
		addDefinition: methodSelection;
		load! !

!MCSnapshotBrowser methodsFor: 'menus' stamp: 'kph 5/13/2007 04:07'!
loadProtocolSelection

	protocolSelection ifNil: [ ^self ].
	
	(MCSnapshot fromDefinitions: self methodsForSelectedProtocol) install! !

!MCSnapshotBrowser methodsFor: 'listing' stamp: 'cwp 7/13/2003 02:11'!
methodList
	^ self visibleMethods collect: [:ea | ea selector]! !

!MCSnapshotBrowser methodsFor: 'menus' stamp: 'kph 5/13/2007 04:23'!
methodListMenu: aMenu 
	super methodListMenu: aMenu.
	self selectedMessageName
		ifNotNilDo: [:msgName | aMenu addLine; add: 'debug/load method' translated action: #loadMethodSelection].
	^ aMenu! !

!MCSnapshotBrowser methodsFor: 'selecting' stamp: 'cwp 7/10/2003 20:26'!
methodSelection
	^ methodSelection
			ifNil: [0] 
			ifNotNil: [self visibleMethods indexOf: methodSelection]! !

!MCSnapshotBrowser methodsFor: 'selecting' stamp: 'stephaneducasse 2/4/2006 20:47'!
methodSelection: aNumber
	methodSelection := aNumber = 0 ifFalse: [self visibleMethods at: aNumber].
	self changed: #methodSelection; changed: #text; changed: #annotations! !

!MCSnapshotBrowser methodsFor: 'accessing' stamp: 'kph 5/3/2007 19:17'!
methodsForSelectedClass
 
	^ items select: [:ea | (ea className = classSelection) 
									and: [ea isMethodDefinition] ].! !

!MCSnapshotBrowser methodsFor: 'accessing' stamp: 'kph 5/3/2007 19:17'!
methodsForSelectedClassSide
 
	^  self methodsForSelectedClass select: [:ea | ea classIsMeta = self switchIsClass ].! !

!MCSnapshotBrowser methodsFor: 'accessing' stamp: 'kph 5/3/2007 19:17'!
methodsForSelectedProtocol
	| methods |
	protocolSelection ifNil: [^ Array new].
	methods := self methodsForSelectedClassSide asOrderedCollection.
	(protocolSelection = '-- all --') 
		ifFalse: [methods removeAllSuchThat: [:ea | ea category ~= protocolSelection]].
	^ methods 
	
								! !

!MCSnapshotBrowser methodsFor: 'accessing' stamp: 'ab 7/18/2003 15:48'!
packageClassNames
	^ self packageClasses collect: [:ea | ea className]! !

!MCSnapshotBrowser methodsFor: 'accessing' stamp: 'kph 5/11/2007 04:47'!
packageClasses
	^ items select: [:ea | (ea isClassDefinition or: [ea isClassDefinitionExtension]) ]! !

!MCSnapshotBrowser methodsFor: 'listing' stamp: 'cwp 7/10/2003 19:07'!
protocolList
	^ self visibleProtocols! !

!MCSnapshotBrowser methodsFor: 'menus' stamp: 'nk 4/17/2004 09:43'!
protocolListMenu: aMenu 
	protocolSelection
		ifNotNil: [aMenu
				add: ('load protocol ''{1}''' translated format: {protocolSelection})
				action: #loadProtocolSelection ].
	^ aMenu! !

!MCSnapshotBrowser methodsFor: 'selecting' stamp: 'cwp 7/10/2003 19:35'!
protocolSelection
	^ protocolSelection 
		ifNil: [0]
		ifNotNil: [self visibleProtocols indexOf: protocolSelection]! !

!MCSnapshotBrowser methodsFor: 'selecting' stamp: 'stephaneducasse 2/4/2006 20:47'!
protocolSelection: anInteger
	protocolSelection := (anInteger = 0 ifFalse: [self visibleProtocols at: anInteger]).
	self methodSelection: 0.
	self changed: #protocolSelection;
		changed: #methodList;	
		changed: #annotations! !

!MCSnapshotBrowser methodsFor: 'accessing' stamp: 'nk 10/11/2003 16:53'!
selectedClass
	classSelection ifNil: [ ^nil ].
	^Smalltalk at: classSelection ifAbsent: [ nil ].
! !

!MCSnapshotBrowser methodsFor: 'accessing' stamp: 'stephaneducasse 2/4/2006 20:47'!
selectedClassOrMetaClass
	| class |
	classSelection ifNil: [ ^nil ].
	class := Smalltalk at: classSelection ifAbsent: [ ^nil ].
	^self switchIsClass ifTrue: [ class class ]
		ifFalse: [ class ].! !

!MCSnapshotBrowser methodsFor: 'accessing' stamp: 'nk 11/10/2003 21:29'!
selectedMessageCategoryName
	^protocolSelection! !

!MCSnapshotBrowser methodsFor: 'accessing' stamp: 'nk 10/11/2003 16:45'!
selectedMessageName
	^methodSelection ifNotNil: [^ methodSelection selector ].
! !

!MCSnapshotBrowser methodsFor: 'switch' stamp: 'cwp 7/12/2003 18:57'!
signalSwitchChanged
	self protocolSelection: 0.
	self 
		changed: #switchIsInstance;
		changed: #switchIsComment;
		changed: #switchIsClass;
		changed: #protocolList;
		changed: #methodList;
		changed: #text.! !

!MCSnapshotBrowser methodsFor: 'accessing' stamp: 'stephaneducasse 2/4/2006 20:47'!
snapshot: aSnapshot
	items := aSnapshot definitions asSortedCollection.
	self categorySelection: 0.! !

!MCSnapshotBrowser methodsFor: 'switch' stamp: 'stephaneducasse 2/4/2006 20:47'!
switchBeClass
	switch := #class.
	self signalSwitchChanged.! !

!MCSnapshotBrowser methodsFor: 'switch' stamp: 'stephaneducasse 2/4/2006 20:47'!
switchBeComment
	switch := #comment.
	self signalSwitchChanged.! !

!MCSnapshotBrowser methodsFor: 'switch' stamp: 'stephaneducasse 2/4/2006 20:47'!
switchBeInstance
	switch := #instance.
	self signalSwitchChanged.! !

!MCSnapshotBrowser methodsFor: 'switch' stamp: 'cwp 7/12/2003 18:05'!
switchIsClass
	^ switch = #class! !

!MCSnapshotBrowser methodsFor: 'switch' stamp: 'cwp 7/12/2003 18:04'!
switchIsComment
	^ switch = #comment.! !

!MCSnapshotBrowser methodsFor: 'switch' stamp: 'stephaneducasse 2/4/2006 20:47'!
switchIsInstance
	switch ifNil: [switch := #instance].
	^ switch = #instance.! !

!MCSnapshotBrowser methodsFor: 'text' stamp: 'cwp 7/13/2003 08:55'!
text
	self switchIsComment ifTrue: [^ self classCommentString].
	methodSelection ifNotNil: [^ methodSelection source].
	protocolSelection ifNotNil: [^ ''].
	classSelection ifNotNil: [^ self classDefinitionString].
	^ ''! !

!MCSnapshotBrowser methodsFor: 'text' stamp: 'cwp 7/11/2003 00:30'!
text: aTextOrString
	self changed: #text! !

!MCSnapshotBrowser methodsFor: 'listing' stamp: 'ab 7/18/2003 15:48'!
visibleCategories
	^ (self packageClasses collect: [:ea | ea category]) 
			asSet asSortedCollection add: self extensionsCategory; yourself.! !

!MCSnapshotBrowser methodsFor: 'listing' stamp: 'ab 7/18/2003 15:48'!
visibleClasses
	^ categorySelection = self extensionsCategory
		ifTrue: [self extensionClassNames]
		ifFalse: [self packageClasses
					select: [:ea | ea category = categorySelection]
					thenCollect: [:ea | ea className]].! !

!MCSnapshotBrowser methodsFor: 'listing' stamp: 'cwp 7/10/2003 19:46'!
visibleMethods
	^ classSelection 
		ifNil: [#()]
		ifNotNil: [self methodsForSelectedProtocol]! !

!MCSnapshotBrowser methodsFor: 'listing' stamp: 'kph 5/3/2007 19:18'!
visibleProtocols
	| methods protocols |
	self switchIsComment ifTrue: [^ Array new].
	methods := self methodsForSelectedClassSide.
	protocols := (methods collect: [:ea | ea category]) asSet asSortedCollection.
	(protocols size > 1) ifTrue: [protocols add: '-- all --'].
	^ protocols ! !

!MCSnapshotBrowser methodsFor: 'morphic ui' stamp: 'bf 5/27/2005 19:13'!
widgetSpecs

	Preferences annotationPanes ifFalse: [ ^#(
		((listMorph: category) (0 0 0.25 0.4))
		((listMorph: class) (0.25 0 0.50 0.4) (0 0 0 -30))
		((listMorph: protocol) (0.50 0 0.75 0.4))
		((listMorph:selection:menu:keystroke:  methodList methodSelection methodListMenu: methodListKey:from:) (0.75 0 1 0.4))
		((buttonRow) (0.25 0.4 0.5 0.4) (0 -30 0 0))
		((textMorph: text) (0 0.4 1 1))
		) ].

	^#(
		((listMorph: category) (0 0 0.25 0.4))
		((listMorph: class) (0.25 0 0.50 0.4) (0 0 0 -30))
		((listMorph: protocol) (0.50 0 0.75 0.4))
		((listMorph:selection:menu:keystroke:  methodList methodSelection methodListMenu: methodListKey:from:) (0.75 0 1 0.4))

		((buttonRow) (0.25 0.4 0.5 0.4) (0 -30 0 0))

		((textMorph: annotations) (0 0.4 1 0.4) (0 0 0 30))
		((textMorph: text) (0 0.4 1 1) (0 30 0 0))
		)! !

MCTool subclass: #MCSaveVersionDialog
	instanceVariableNames: 'name message'
	classVariableNames: 'LastComments'
	poolDictionaries: ''
	category: 'Monticello-UI'!

!MCSaveVersionDialog methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 8/1/2008 12:17'!
accept
	self addAsLastComment: (self findTextMorph: #logMessage) text asString.
	self answer:
		(Array
			with: (self findTextMorph: #versionName) text asString
			with: (self findTextMorph: #logMessage) text asString)
	! !

!MCSaveVersionDialog methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 8/1/2008 12:22'!
addAsLastComment: aString
	((self lastComments size > 0) and: [self lastComments first = aString])
		ifTrue: [ ^ self ].
		
	self lastComments addFirst: aString.
	(self lastComments size > self maxCommentHistory)
		ifTrue: [LastComments := LastComments copyFrom: 1 to: self maxCommentHistory]! !

!MCSaveVersionDialog methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 8/1/2008 11:59'!
buttonSpecs
	^ #((Accept accept 'accept version name and log message')
		(Cancel cancel 'cancel saving version')
		('Comments...' comment 'use previous comment')
		) ! !

!MCSaveVersionDialog methodsFor: 'as yet unclassified' stamp: 'ab 8/24/2003 20:41'!
cancel
	self answer: nil! !

!MCSaveVersionDialog methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 8/11/2008 21:23'!
comment
	| list index |
	list := self lastComments collect: [:s | s copyFrom: 1 to: (s size min: 20)].
	list ifEmpty: [UIManager default inform: 'No previous comments was entered'. ^ self].
	index := UIManager default chooseFrom: list.
	
	"no comment was selected"
	index isZero ifTrue: [ ^ self ].
	
	self logMessage: (self lastComments at: index)! !

!MCSaveVersionDialog methodsFor: 'as yet unclassified' stamp: 'ab 8/24/2003 20:07'!
defaultExtent 
	^ 400@300! !

!MCSaveVersionDialog methodsFor: 'as yet unclassified' stamp: 'ab 8/24/2003 20:43'!
defaultLabel
	^ 'Edit Version Name and Message:'! !

!MCSaveVersionDialog methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 8/1/2008 12:01'!
lastComments
	LastComments ifNil: [ LastComments := OrderedCollection new].
	^ LastComments! !

!MCSaveVersionDialog methodsFor: 'as yet unclassified' stamp: 'ab 8/24/2003 20:41'!
logMessage
	^ message ifNil: ['empty log message']! !

!MCSaveVersionDialog methodsFor: 'as yet unclassified' stamp: 'mtf 7/22/2008 11:00'!
logMessage: aString
	message := aString.
	self changed: #logMessage.
	self accept.! !

!MCSaveVersionDialog methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 8/1/2008 12:21'!
maxCommentHistory
	^ 15! !

!MCSaveVersionDialog methodsFor: 'as yet unclassified' stamp: 'ab 8/24/2003 20:37'!
versionName
	^ name! !

!MCSaveVersionDialog methodsFor: 'as yet unclassified' stamp: 'kph 6/2/2008 19:41'!
versionName: aString
	name := aString withBlanksTrimmed.
	self changed: #versionName! !

!MCSaveVersionDialog methodsFor: 'as yet unclassified' stamp: 'jrp 7/2/2005 10:33'!
widgetSpecs
	^ #(	
		((textMorph: versionName) (0 0 1 0) (0 0 0 30))
		((textMorph: logMessage) (0 0 1 1) (0 30 0 -30))
		((buttonRow) (0 1 1 1) (0 -40 0 0))
		)! !

!MCTool methodsFor: 'morphic ui' stamp: 'stephaneducasse 2/4/2006 20:47'!
answer: anObject
	modalValue := anObject.
	self close.! !

!MCTool methodsFor: 'morphic ui' stamp: 'nk 2/16/2004 16:50'!
arrowKey: aCharacter from: aPluggableListMorph 
	"backstop"! !

!MCTool methodsFor: 'morphic ui' stamp: 'kph 5/19/2007 00:29'!
buildPanelWith: builder
	|  panelBuilder |

	panelBuilder := MCToolWindowBuilder builder: builder tool: self.
	panelBuilder startWithWindow.
	self widgetSpecs do:
		[:spec | | send fractions offsets origin corner |
		send := spec first.
		fractions := spec at: 2 ifAbsent: [#(0 0 1 1)].
		offsets := spec at: 3 ifAbsent: [#(0 0 0 0)].
		origin := (offsets first @ offsets second) 
			/ self defaultExtent asFloatPoint
			+ (fractions first @ fractions second).
		corner := (offsets third @ offsets fourth) 
			/ self defaultExtent asFloatPoint
			+ (fractions third @ fractions fourth).
		panelBuilder frame: (origin corner: corner).
		panelBuilder perform: send first withArguments: send allButFirst].

	^ panelBuilder build submorphs! !

!MCTool methodsFor: 'morphic ui' stamp: 'stephaneducasse 2/4/2006 20:47'!
buildWindow
	| window |
	window := SystemWindow labelled: self label.
	window model: self.
	self widgetSpecs do:
		[:pair | |send fractions offsets|
		send := pair first.
		fractions := pair at: 2 ifAbsent: [#(0 0 1 1)].
		offsets := pair at: 3 ifAbsent: [#(0 0 0 0)].
		window
			addMorph: (self perform: send first withArguments: send allButFirst )
			fullFrame:
				(LayoutFrame
					fractions: 
					((fractions first)@(fractions second) corner: 
						(fractions third)@(fractions fourth))
					offsets:
						((offsets first)@(offsets second)  corner:
							(offsets third)@(offsets fourth)))].
	^ window! !

!MCTool methodsFor: 'morphic ui' stamp: 'kph 5/18/2007 12:49'!
buildWith: builder
	|  windowBuilder |

	windowBuilder := MCToolWindowBuilder builder: builder tool: self.
	windowBuilder startWithWindow.
	self widgetSpecs do:
		[:spec | | send fractions offsets origin corner |
		send := spec first.
		fractions := spec at: 2 ifAbsent: [#(0 0 1 1)].
		offsets := spec at: 3 ifAbsent: [#(0 0 0 0)].
		origin := (offsets first @ offsets second) 
			/ self defaultExtent asFloatPoint
			+ (fractions first @ fractions second).
		corner := (offsets third @ offsets fourth) 
			/ self defaultExtent asFloatPoint
			+ (fractions third @ fractions fourth).
		windowBuilder frame: (origin corner: corner).
		windowBuilder perform: send first withArguments: send allButFirst].
 
	^ windowBuilder build
! !

!MCTool methodsFor: 'morphic ui' stamp: 'bf 5/27/2005 19:19'!
buttonEnabled
	^ true! !

!MCTool methodsFor: 'morphic ui' stamp: 'ab 7/18/2003 18:34'!
buttonRow
	^ self buttonRow: self buttonSpecs! !

!MCTool methodsFor: 'morphic ui' stamp: 'al 6/21/2008 23:45'!
buttonRow: specArray
	| aRow aButton state |
	aRow := AlignmentMorph newRow.
	aRow 
		color: (Display depth <= 8 ifTrue: [Color transparent] ifFalse: [Color gray alpha: 0.2]);
		borderWidth: 0.

	aRow hResizing: #spaceFill; vResizing: #spaceFill; rubberBandCells: true.
	aRow clipSubmorphs: true.
	aRow layoutInset: 2@2; cellInset: 1; color: Color white.
	aRow wrapCentering: #center; cellPositioning: #leftCenter.
	specArray do:
		[:triplet |
			state := triplet at: 5 ifAbsent: [#buttonState].
			aButton := PluggableButtonMorph
				on: self
				getState: state
				action: #performButtonAction:enabled:.
			aButton
				hResizing: #spaceFill;
				vResizing: #spaceFill;
				label: triplet first asString;
				arguments: (Array with: triplet second with: (triplet at: 4 ifAbsent: [#buttonEnabled])); 
				onColor: Color white offColor: Color white.
			aRow addMorphBack: aButton.
			aButton setBalloonText: triplet third].
	^ aRow! !

!MCTool methodsFor: 'morphic ui' stamp: 'bf 5/27/2005 19:22'!
buttonSelected
	^ false! !

!MCTool methodsFor: 'morphic ui' stamp: 'ab 7/17/2003 15:23'!
buttonSpecs
	^ #()! !

!MCTool methodsFor: 'morphic ui' stamp: 'ab 7/22/2003 00:45'!
buttonState
	^ true! !

!MCTool methodsFor: 'morphic ui' stamp: 'ab 7/18/2003 17:42'!
close
	self window delete! !

!MCTool methodsFor: 'morphic ui' stamp: 'nk 7/24/2003 13:31'!
defaultAnnotationPaneHeight 
	"Answer the receiver's preferred default height for new annotation panes."
	^ Preferences parameterAt: #defaultAnnotationPaneHeight ifAbsentPut: [25]! !

!MCTool methodsFor: 'morphic ui' stamp: 'avi 2/18/2004 19:56'!
defaultBackgroundColor 
	^ (Color r: 0.627 g: 0.69 b: 0.976)! !

!MCTool methodsFor: 'morphic ui' stamp: 'ab 7/17/2003 15:22'!
defaultButtonPaneHeight
	"Answer the user's preferred default height for new button panes."

	^ Preferences parameterAt: #defaultButtonPaneHeight ifAbsentPut: [25]! !

!MCTool methodsFor: 'morphic ui' stamp: 'ab 7/17/2003 15:10'!
defaultExtent
	^ 500@500! !

!MCTool methodsFor: 'morphic ui' stamp: 'ab 7/17/2003 15:11'!
defaultLabel
	^ self class name! !

!MCTool methodsFor: 'morphic ui' stamp: 'ab 7/19/2003 22:33'!
fillMenu: aMenu fromSpecs: anArray
	anArray do:
		[:pair |
		aMenu add: pair first target: self selector: pair second].
	^ aMenu! !

!MCTool methodsFor: 'morphic ui' stamp: 'bf 3/16/2005 14:48'!
findListMorph: aSymbol
	^ morph submorphs detect: [:ea | (ea respondsTo: #getListSelector) and: [ea getListSelector = aSymbol]] ifNone: []! !

!MCTool methodsFor: 'morphic ui' stamp: 'ab 8/24/2003 20:15'!
findTextMorph: aSymbol
	^ morph submorphs detect: [:ea | (ea respondsTo: #getTextSelector) and: [ea getTextSelector = aSymbol]] ifNone: []! !

!MCTool methodsFor: 'morphic ui' stamp: 'ab 7/18/2003 17:23'!
getMenu: aMenu
	^aMenu! !

!MCTool methodsFor: 'morphic ui' stamp: 'ab 7/18/2003 17:36'!
label
	^ label ifNil: [self defaultLabel]! !

!MCTool methodsFor: 'morphic ui' stamp: 'stephaneducasse 2/4/2006 20:47'!
label: aString
	label := aString! !

!MCTool methodsFor: 'morphic ui' stamp: 'lr 9/26/2003 17:30'!
listMorph: listSymbol
	^ self
		listMorph: (listSymbol, 'List') asSymbol
		selection: (listSymbol, 'Selection') asSymbol
		menu: (listSymbol, 'ListMenu:') asSymbol! !

!MCTool methodsFor: 'morphic ui' stamp: 'nk 2/16/2004 17:03'!
listMorph: listSymbol keystroke: keystrokeSymbol
	^ (self
		listMorph: (listSymbol, 'List') asSymbol
		selection: (listSymbol, 'Selection') asSymbol
		menu: (listSymbol, 'ListMenu:') asSymbol)
		keystrokeActionSelector: keystrokeSymbol;
		yourself! !

!MCTool methodsFor: 'morphic ui' stamp: 'ab 7/17/2003 15:46'!
listMorph: listSymbol selection: selectionSymbol
	^ PluggableListMorph
		on: self
		list: listSymbol
		selected: selectionSymbol
		changeSelected: (selectionSymbol, ':') asSymbol! !

!MCTool methodsFor: 'morphic ui' stamp: 'ab 7/17/2003 15:46'!
listMorph: listSymbol selection: selectionSymbol menu: menuSymbol
	^ PluggableListMorph
		on: self
		list: listSymbol
		selected: selectionSymbol
		changeSelected: (selectionSymbol, ':') asSymbol
		menu: menuSymbol! !

!MCTool methodsFor: 'morphic ui' stamp: 'nk 2/16/2004 16:50'!
listMorph: listSymbol selection: selectionSymbol menu: menuSymbol keystroke: keystrokeSymbol
	^ (PluggableListMorph
		on: self
		list: listSymbol
		selected: selectionSymbol
		changeSelected: (selectionSymbol, ':') asSymbol
		menu: menuSymbol)
		keystrokeActionSelector: keystrokeSymbol;
		yourself! !

!MCTool methodsFor: 'morphic ui' stamp: 'avi 9/11/2004 16:19'!
multiListMorph: listSymbol selection: selectionSymbol listSelection: listSelectionSymbol menu: menuSymbol
	^ PluggableListMorphOfMany
		on: self
		list: listSymbol
		primarySelection: selectionSymbol
		changePrimarySelection: (selectionSymbol, ':') asSymbol
		listSelection: listSelectionSymbol
		changeListSelection: (listSelectionSymbol, 'put:') asSymbol
		menu: menuSymbol! !

!MCTool methodsFor: 'morphic ui' stamp: 'ab 7/18/2003 17:18'!
perform: selector orSendTo: otherTarget
	"Selector was just chosen from a menu by a user.  If can respond, then
perform it on myself. If not, send it to otherTarget, presumably the
editPane from which the menu was invoked."

	(self respondsTo: selector)
		ifTrue: [^ self perform: selector]
		ifFalse: [^ otherTarget perform: selector]! !

!MCTool methodsFor: 'morphic ui' stamp: 'lr 10/5/2003 09:09'!
performButtonAction: anActionSelector enabled: anEnabledSelector
	(self perform: anEnabledSelector) 
		ifTrue: [ self perform: anActionSelector ]! !

!MCTool methodsFor: 'morphic ui' stamp: 'kph 5/13/2007 16:53'!
show
	modal := false.
	Smalltalk at: #ToolBuilder ifPresent: [:tb |   ^ tb open: self.  ].
	^self window openInWorldExtent: self defaultExtent; yourself! !

!MCTool methodsFor: 'morphic ui' stamp: 'stephaneducasse 2/4/2006 20:47'!
showLabelled: labelString
	modal := false.
	self label: labelString.
	^(self window)
		openInWorldExtent: self defaultExtent;
		yourself! !

!MCTool methodsFor: 'morphic ui' stamp: 'stephaneducasse 2/4/2006 20:47'!
showModally
	modal := true.
	self window openInWorldExtent: (400@400).
	[self window world notNil] whileTrue: [
		self window outermostWorldMorph doOneCycle.
	].
	morph := nil.
	^ modalValue! !

!MCTool methodsFor: 'morphic ui' stamp: 'nk 6/12/2004 14:11'!
step
! !

!MCTool methodsFor: 'morphic ui' stamp: 'ab 7/17/2003 15:36'!
textMorph: aSymbol
	^ PluggableTextMorph on: self text: aSymbol accept: (aSymbol, ':') asSymbol! !

!MCTool methodsFor: 'morphic ui' stamp: 'kph 5/21/2007 12:58'!
toolPanel: selectorForTool

	| subTool |
	
	subTool := self perform: selectorForTool.	

	subTool ifNil:[ ^#() ].
	
	^ subTool buildPanelWith: ToolBuilder default. 
! !

!MCTool methodsFor: 'morphic ui' stamp: 'ar 2/14/2004 02:27'!
treeMorph: listSymbol
	^ self
		treeMorph: (listSymbol, 'Tree') asSymbol
		selection: (listSymbol, 'SelectionWrapper') asSymbol
		menu: (listSymbol, 'TreeMenu:') asSymbol! !

!MCTool methodsFor: 'morphic ui' stamp: 'ar 2/14/2004 02:43'!
treeMorph: listSymbol selection: selectionSymbol menu: menuSymbol
	^ SimpleHierarchicalListMorph
		on: self
		list: listSymbol
		selected: selectionSymbol
		changeSelected: (selectionSymbol, ':') asSymbol
		menu: menuSymbol
		keystroke: nil! !

!MCTool methodsFor: 'morphic ui' stamp: 'avi 3/6/2005 22:31'!
treeOrListMorph: aSymbol
	^ self treeMorph: aSymbol! !

!MCTool methodsFor: 'morphic ui' stamp: 'ab 7/17/2003 15:40'!
widgetSpecs
	^ #()! !

!MCTool methodsFor: 'morphic ui' stamp: 'stephaneducasse 2/4/2006 20:47'!
window
	^ morph ifNil: [morph := self buildWindow]! !

MCTool subclass: #MCVersionHistoryBrowser
	instanceVariableNames: 'ancestry index repositoryGroup package infos'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-UI'!

MCVersionHistoryBrowser subclass: #MCChangeLogBrowser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-UI'!

!MCVersionHistoryBrowser methodsFor: 'accessing' stamp: 'stephaneducasse 2/4/2006 20:47'!
ancestry: anAncestry
	ancestry := anAncestry! !

!MCVersionHistoryBrowser methodsFor: 'accessing' stamp: 'avi 2/13/2004 01:10'!
baseSnapshot
	^ self snapshotForInfo: ancestry! !

!MCVersionHistoryBrowser methodsFor: 'morphic ui' stamp: 'ab 7/17/2003 15:41'!
defaultExtent
	^ 440@169.
	! !

!MCVersionHistoryBrowser methodsFor: 'morphic ui' stamp: 'avi 2/13/2004 01:09'!
defaultLabel
	^ ancestry name, ' History'! !

!MCVersionHistoryBrowser methodsFor: 'morphic ui' stamp: 'avi 2/13/2004 01:09'!
getMenu: aMenu
	index < 2 ifTrue: [^ aMenu].
	self fillMenu: aMenu fromSpecs: 
		(Array
			with: (Array with: 'view changes -> ', ancestry name with: #viewChanges)
			with: #('spawn history' spawnHistory)).
	^ aMenu! !

!MCVersionHistoryBrowser methodsFor: 'accessing' stamp: 'ab 8/22/2003 01:41'!
index
	"Answer the value of index"

	^ index! !

!MCVersionHistoryBrowser methodsFor: 'accessing' stamp: 'stephaneducasse 2/4/2006 20:47'!
index: anObject
	"Set the value of index"

	index := anObject! !

!MCVersionHistoryBrowser methodsFor: 'accessing' stamp: 'stephaneducasse 2/4/2006 20:47'!
infos
	^ infos ifNil: [infos := ancestry withBreadthFirstAncestors]! !

!MCVersionHistoryBrowser methodsFor: 'accessing' stamp: 'avi 9/17/2005 16:10'!
list
	^ self infos collect: [:ea | ea name]! !

!MCVersionHistoryBrowser methodsFor: 'accessing' stamp: 'stephaneducasse 2/4/2006 20:47'!
package: aMCPackage
	package := aMCPackage! !

!MCVersionHistoryBrowser methodsFor: 'accessing' stamp: 'ab 8/7/2003 21:27'!
repositoryGroup
	^ MCRepositoryGroup default! !

!MCVersionHistoryBrowser methodsFor: 'accessing' stamp: 'avi 9/17/2005 16:09'!
selectedInfo
	^ self infos at: self selection ifAbsent: [nil]! !

!MCVersionHistoryBrowser methodsFor: 'accessing' stamp: 'ab 8/22/2003 01:39'!
selectedSnapshot
	^ self snapshotForInfo: self selectedInfo! !

!MCVersionHistoryBrowser methodsFor: 'accessing' stamp: 'ab 7/11/2003 23:24'!
selection
	^ index ifNil: [0]! !

!MCVersionHistoryBrowser methodsFor: 'accessing' stamp: 'stephaneducasse 2/4/2006 20:47'!
selection: aNumber
	index := aNumber.
	self changed: #selection; changed: #summary! !

!MCVersionHistoryBrowser methodsFor: 'accessing' stamp: 'ab 8/22/2003 01:38'!
snapshotForInfo: aVersionInfo
	^ (self repositoryGroup versionWithInfo: aVersionInfo) snapshot! !

!MCVersionHistoryBrowser methodsFor: 'morphic ui' stamp: 'avi 2/13/2004 01:10'!
spawnHistory
	MCVersionHistoryBrowser new
		ancestry: self selectedInfo;
		package: package;
		show! !

!MCVersionHistoryBrowser methodsFor: 'accessing' stamp: 'stephaneducasse 2/4/2006 20:47'!
summary
	| selInfo |
	selInfo := self selectedInfo.
	^ selInfo 
		ifNil: ['']
		ifNotNil: [selInfo summary]! !

!MCVersionHistoryBrowser methodsFor: 'morphic ui' stamp: 'stephaneducasse 2/4/2006 20:47'!
viewChanges
	"Note that the patchLabel will be parsed in MCPatchBrowser>>installSelection, so don't translate it!!"
	| patch patchLabel |
	patchLabel := 'changes between {1} and {2}' format: { self selectedInfo name. ancestry name }.
	patch := self baseSnapshot patchRelativeToBase: self selectedSnapshot.
	(MCPatchBrowser forPatch: patch) label: patchLabel; show! !

!MCVersionHistoryBrowser methodsFor: 'morphic ui' stamp: 'nk 7/28/2003 18:05'!
widgetSpecs
	^ #(
		((listMorph:selection:menu: list selection getMenu:) (0 0 0.3 1))
		((textMorph: summary) (0.3 0 1 1))
	 	)! !

MCVersionHistoryBrowser subclass: #MCWorkingHistoryBrowser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-UI'!

!MCWorkingHistoryBrowser methodsFor: 'as yet unclassified' stamp: 'ab 8/22/2003 01:37'!
baseSnapshot
	^ package snapshot! !

MCTool subclass: #MCVersionInspector
	instanceVariableNames: 'version'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-UI'!

MCVersionInspector subclass: #MCChangeLogEntry
	instanceVariableNames: 'ancestors repository'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-UI'!

!MCChangeLogEntry class methodsFor: 'as yet unclassified' stamp: 'kph 5/16/2007 10:36'!
forVersion: aVersion in: repository
	^self basicNew
		version: aVersion;
		repository: repository;
		show;
		yourself.
		! !

!MCChangeLogEntry methodsFor: 'as yet unclassified' stamp: 'kb 3/23/2007 12:26'!
ancestors
	^ancestors ifNil: [ |  infosOfAncestors |
		infosOfAncestors := self version info ancestors.
		ancestors := infosOfAncestors collect: [:each | self versionForName: each name ] ]
	! !

!MCChangeLogEntry methodsFor: 'as yet unclassified' stamp: 'kb 3/23/2007 12:17'!
changesAccordingTo: anAncestor
	^ anAncestor snapshot patchRelativeToBase: version snapshot! !

!MCChangeLogEntry methodsFor: 'as yet unclassified' stamp: 'kb 3/23/2007 12:47'!
fileNameForVersionName: aVersionName
	| fileNames |
	fileNames := [(repository allFileNamesForVersionNamed: aVersionName)]
		on: Error do: [self error: 'No such version in this repository'].
	(fileNames size = 0) ifTrue: [self error: 'No file for this version'].
	(fileNames size = 1) ifFalse: [self error: 'What to do with multiple files?' ].
	^fileNames first	! !

!MCChangeLogEntry methodsFor: 'as yet unclassified' stamp: 'kph 5/16/2007 10:54'!
report

	^ String streamContents: [ :str | self writeOn: str ].! !

!MCChangeLogEntry methodsFor: 'as yet unclassified' stamp: 'kb 3/23/2007 12:18'!
repository: aRepository
	repository := aRepository! !

!MCChangeLogEntry methodsFor: 'as yet unclassified' stamp: 'kph 5/16/2007 10:43'!
versionForName: aVersionName
	^ repository versionFromFileNamed: (self fileNameForVersionName: aVersionName)! !

!MCChangeLogEntry methodsFor: 'as yet unclassified' stamp: 'kph 5/16/2007 10:56'!
widgetSpecs
	^ #(
		((buttonRow) (0 0 1 0) (0 0 0 30))
		((textMorph: summary) (0 0 1 0.5) (0 30 0 0))
		((textMorph: report) (0 0.5 1 1) (0 0 0 0))
		)! !

!MCChangeLogEntry methodsFor: 'as yet unclassified' stamp: 'kph 5/16/2007 10:39'!
writeChanges: anMCPatch  on: aStream
	anMCPatch operations asSortedCollection do: [:each |
		aStream nextPutAll: each summary; cr ]! !

!MCChangeLogEntry methodsFor: 'as yet unclassified' stamp: 'kph 5/16/2007 10:57'!
writeOn: aStream
	"aStream 
		nextPutAll: version info summary; cr."
	self ancestors do: [:each |
		aStream 
			cr; 
			nextPutAll: 'Changes from: ';
			nextPutAll: each info name;
			cr.
		self writeChanges: (self changesAccordingTo: each) on: aStream]! !

MCVersionInspector subclass: #MCRepositoryVersionsInspector
	instanceVariableNames: 'loaded loadedNames inheritedNames newer inherited latest repository versionInfos selectedVersionInfo order'
	classVariableNames: 'Order'
	poolDictionaries: ''
	category: 'Monticello-UI'!

MCRepositoryVersionsInspector class
	instanceVariableNames: 'loaded'!

!MCRepositoryVersionsInspector commentStamp: 'kph 5/16/2007 23:22' prior: 0!
Customize the MCVersionInspector to show a list of versions from a repository.!

MCRepositoryVersionsInspector class
	instanceVariableNames: 'loaded'!

MCRepositoryVersionsInspector subclass: #MCDictionaryRepositoryInspector
	instanceVariableNames: 'repositories'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-UI'!

!MCDictionaryRepositoryInspector class methodsFor: 'as yet unclassified' stamp: 'kph 5/16/2007 14:44'!
initialOpenOnLeft: aDictRepository right: aWorkingCopy
	 
	^ (self new) initialSetLeft: aDictRepository andRight: (self workingCopyRepositoryWith: aWorkingCopy)
! !

!MCDictionaryRepositoryInspector class methodsFor: 'as yet unclassified' stamp: 'kph 5/16/2007 14:49'!
workingCopyRepositoryWith: aWorkingCopy 

	| dict |
	
	dict :=  (MCDictionaryRepository new)
		 		description: 'Working Copy (In Memory)'.
				
	aWorkingCopy ifNotNil: [ 
	
		dict	 at: aWorkingCopy name put: aWorkingCopy 
	].

	^ dict! !

!MCDictionaryRepositoryInspector methodsFor: 'as yet unclassified' stamp: 'kph 5/16/2007 14:45'!
defaultLabel
	^'Compare: ' , repositories first description, ' vs ', repositories second description! !

!MCDictionaryRepositoryInspector methodsFor: 'as yet unclassified' stamp: 'kph 5/16/2007 14:23'!
initialSetLeft: l andRight: r

repositories := Array with: l with: r ! !

!MCDictionaryRepositoryInspector methodsFor: 'as yet unclassified' stamp: 'kph 5/16/2007 15:21'!
sortedAndHighlightedVersions: someVersions
	 
	^ (self sortedVersions: (someVersions)) collect: [ :each | self versionHighlight: each fourth ]! !

!MCDictionaryRepositoryInspector methodsFor: 'as yet unclassified' stamp: 'kph 5/16/2007 15:23'!
sortedVersions: someVersions
	| sorter |
	sorter := MCVersionSorter new.
	sorter addAllVersionInfos: someVersions.
	^ sorter sortedVersionInfos select: [:ea | someVersions includes: ea]! !

!MCDictionaryRepositoryInspector methodsFor: 'as yet unclassified' stamp: 'kph 5/16/2007 14:40'!
swapSides

	repositories := repositories reverse.
	
! !

!MCDictionaryRepositoryInspector methodsFor: 'as yet unclassified' stamp: 'kph 5/17/2007 00:20'!
versionsLeftList
	 
	^ self sortedAndHighlightedVersions: repositories first heldVersions! !

!MCDictionaryRepositoryInspector methodsFor: 'as yet unclassified' stamp: 'kph 5/17/2007 00:20'!
versionsRightList
	 
	^ self sortedAndHighlightedVersions: repositories last heldVersions! !

!MCDictionaryRepositoryInspector methodsFor: 'as yet unclassified' stamp: 'kph 5/16/2007 14:34'!
widgetSpecs

	^ #(
		((buttonRow) (0 0 1 0) (0 0 0 30))
		((listMorph: versionsLeft) (0 0 0.30 0.6) (0 30 0 0))
		((listMorph: middle) (0.30 0 0.70 1) (0 30 0 0))
		((textMorph: versionsRight) (0.70 0 1 0.6) (0 30 0 0)) )
		
	! !

MCRepositoryVersionsInspector subclass: #MCRepositoryDualVersionsInspector
	instanceVariableNames: 'repository2 versionInfos2 selectedVersionInfo2 order2 centerPanelModel thePatch'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-UI'!

!MCRepositoryDualVersionsInspector commentStamp: 'kph 6/4/2008 14:42' prior: 0!
Known Issues

Some crosstalk between the two versions list... very mystifying!

!MCRepositoryDualVersionsInspector class methodsFor: 'as yet unclassified' stamp: 'kph 5/17/2007 15:18'!
repository: r repository2: r2 

	^(self new) setRepository: r  repository2: r2;
				yourself! !

!MCRepositoryDualVersionsInspector methodsFor: 'as yet unclassified' stamp: 'kph 5/17/2007 16:36'!
adopt2
	(self confirm:
'Modifying ancestry can be dangerous unless you know
what you are doing.  Are you sure you want to adopt
',self version info name, ' as an ancestor of your working copy?')
		ifTrue: [self version2 adopt]! !

!MCRepositoryDualVersionsInspector methodsFor: 'as yet unclassified' stamp: 'kph 5/17/2007 17:41'!
browse2


	^ self version2 browse ! !

!MCRepositoryDualVersionsInspector methodsFor: 'THE BUSINESS' stamp: 'kph 5/18/2007 13:51'!
browseDiffAB
 
	| diff |
	
	diff := self version asDiffAgainst: self version2.
     centerPanelModel := MCSnapshotBrowser forSnapshot: diff snapshot.
	 
	self changed: #buildCenterPanel.! !

!MCRepositoryDualVersionsInspector methodsFor: 'panel' stamp: 'kph 5/18/2007 13:35'!
buildCenterPanel
	centerPanelModel ifNil:[ ^#() ].
	
	^ centerPanelModel buildPanelWith: ToolBuilder default. 
! !

!MCRepositoryDualVersionsInspector methodsFor: 'as yet unclassified' stamp: 'kph 5/18/2007 13:47'!
buttonSpecs
	^#(
	
		('Refresh' refresh 'refresh the version-list') 
         
		 ('Browse A' browse 'Browse this version' hasVersion)
           
		 ('Changes A-B' changesAB 'Browse Changes From A to B' hasVersions)	
		 ('Diff A-B' diffAB 'Diff From A to B' hasVersions)	
		 ('Browse A-B' browseDiffAB 'Browse Diff From A to B' hasVersions)	
		 ('Stages A..B' stagesAB 'Browse Steps from A to B' hasVersions	)
	 	          
		('Browse B' browse2 'Browse the second version' hasVersion2)
         

	)
	
	
	"
	 (Changes changes 'Browse the changes F' hasVersion)
            
               (Merge merge 'Merge this version into the image' hasVersion)
              (Adopt adopt 'Adopt this version as an ancestor of your working copy'
hasVersion)
               (Copy copy 'Copy this version to another repository' hasVersion)
              (Diff diff 'Create an equivalent version based on an earlier release'
hasVersion)" 
	 
	! !

!MCRepositoryDualVersionsInspector methodsFor: 'panel' stamp: 'kph 5/17/2007 19:37'!
centerPanelSpecs

	^ centerPanelModel widgetSpecs
	
! !

!MCRepositoryDualVersionsInspector methodsFor: 'as yet unclassified' stamp: 'kph 5/17/2007 16:30'!
changes2
	(MCPatchBrowser forPatch: self version2 changes)
		showLabelled: 'Changes from ', self version2 info name! !

!MCRepositoryDualVersionsInspector methodsFor: 'THE BUSINESS' stamp: 'kph 5/18/2007 13:32'!
changesAB
 
	thePatch := MCPatch fromBase: (self version snapshot) target: (self version2 snapshot).
	centerPanelModel := MCPatchBrowser forPatch: thePatch.
	self changed: #buildCenterPanel.! !

!MCRepositoryDualVersionsInspector methodsFor: 'panel' stamp: 'kph 5/17/2007 19:05'!
changesPanel

! !

!MCRepositoryDualVersionsInspector methodsFor: 'as yet unclassified' stamp: 'kph 5/17/2007 18:27'!
copy2
	self pickRepository ifNotNilDo:
		[:ea |
		ea storeVersion: self version2]! !

!MCRepositoryDualVersionsInspector methodsFor: 'as yet unclassified' stamp: 'kph 5/17/2007 10:48'!
defaultExtent
	^550@350! !

!MCRepositoryDualVersionsInspector methodsFor: 'as yet unclassified' stamp: 'kph 5/17/2007 10:50'!
defaultLabel
	^'Dual Change Log:  ' , repository description , ' & ',  repository2 description! !

!MCRepositoryDualVersionsInspector methodsFor: 'as yet unclassified' stamp: 'kph 5/17/2007 14:34'!
delete2

	self isVersionDeletable ifFalse: [ ^ self ].
	
	(self confirm: 'Are you sure you wish to delete ', selectedVersionInfo2 name, '
from ', repository description)
		ifFalse: [ ^ self ].

	 (repository delete: selectedVersionInfo2)
		ifTrue: [ self inform: ('Successfully deleted ', selectedVersionInfo2 name) ]
		ifFalse: [ self inform: ('Sorry, could not delete, ', selectedVersionInfo2 name) ].! !

!MCRepositoryDualVersionsInspector methodsFor: 'as yet unclassified' stamp: 'kph 5/17/2007 16:35'!
diff2
	| ancestorVersion |
	self pickAncestor ifNotNilDo:
		[:ancestor |
		ancestorVersion := self version2 workingCopy repositoryGroup versionWithInfo: ancestor.
		(self version2 asDiffAgainst: ancestorVersion) open]! !

!MCRepositoryDualVersionsInspector methodsFor: 'THE BUSINESS' stamp: 'kph 5/18/2007 13:49'!
diffAB
 
	| diff |
	
	diff := self version asDiffAgainst: self version2.
     centerPanelModel :=   (MCVersionInspector new version: diff)  .
	self changed: #buildCenterPanel.! !

!MCRepositoryDualVersionsInspector methodsFor: 'as yet unclassified' stamp: 'kph 5/18/2007 13:19'!
hasVersion2
 
	^ selectedVersionInfo2 notNil! !

!MCRepositoryDualVersionsInspector methodsFor: 'as yet unclassified' stamp: 'kph 5/18/2007 13:19'!
hasVersions
 
	^ self hasVersion and: [ self hasVersion2 ]! !

!MCRepositoryDualVersionsInspector methodsFor: 'as yet unclassified' stamp: 'kph 5/18/2007 13:33'!
initialize
! !

!MCRepositoryDualVersionsInspector methodsFor: 'as yet unclassified' stamp: 'kph 5/17/2007 14:36'!
isVersionDeletable2

"disable version deleting until sure its ok"
	^ false 
	"^ self repository2 ifNotNilDo:[ :r | r isDeletingAllowed ] ifNil: [ false ]"! !

!MCRepositoryDualVersionsInspector methodsFor: 'as yet unclassified' stamp: 'kph 5/17/2007 17:41'!
load2
 
	self version2 isCacheable
			ifTrue: [ self version2 workingCopy repositoryGroup addRepository: repository2].
		
	Cursor wait showWhile: [ self version2 load]
	
	self refresh
	! !

!MCRepositoryDualVersionsInspector methodsFor: 'as yet unclassified' stamp: 'kph 5/17/2007 18:20'!
merge2
	self version2 merge.
	self refresh.! !

!MCRepositoryDualVersionsInspector methodsFor: 'as yet unclassified' stamp: 'kph 5/18/2007 12:54'!
obtainPatch

	^ MCPatch operations: #()! !

!MCRepositoryDualVersionsInspector methodsFor: 'as yet unclassified' stamp: 'kph 5/17/2007 10:45'!
order2: anInteger
 	order2 := anInteger.
	self refresh! !

!MCRepositoryDualVersionsInspector methodsFor: 'as yet unclassified' stamp: 'kph 5/17/2007 10:48'!
order2String: anIndex
	^ self order: order2 string: anIndex! !

!MCRepositoryDualVersionsInspector methodsFor: 'as yet unclassified' stamp: 'mtf 6/27/2008 23:18'!
orderSpecs
	^{
		'unchanged' -> nil.
		'order by package' -> [ :x :y | x <= y ].	
	}! !

!MCRepositoryDualVersionsInspector methodsFor: 'as yet unclassified' stamp: 'mtf 6/27/2008 23:18'!
orderSpecs2
	^{
		'unchanged' -> nil.
		'order by package' -> [ :x :y | x <= y ].	
	}! !

!MCRepositoryDualVersionsInspector methodsFor: 'as yet unclassified' stamp: 'kph 5/17/2007 14:39'!
refresh

	self versionListObtain.
	self version2ListObtain.

	self versionListSort.
	self version2ListSort.
	 
	self collateHighlightingData.
	self changed: #versionList.
	self changed: #version2List.
 ! !

!MCRepositoryDualVersionsInspector methodsFor: 'as yet unclassified' stamp: 'kph 5/17/2007 10:42'!
repository

	^ repository ! !

!MCRepositoryDualVersionsInspector methodsFor: 'as yet unclassified' stamp: 'kph 5/17/2007 10:43'!
repository2

	^ repository ! !

!MCRepositoryDualVersionsInspector methodsFor: 'as yet unclassified' stamp: 'kph 5/17/2007 15:13'!
setRepository: r1 repository2: r2   
	order := order2 := self class order.
	repository := r1.
	repository2 := r2.
    	self refresh! !

!MCRepositoryDualVersionsInspector methodsFor: 'THE BUSINESS' stamp: 'kph 5/17/2007 16:37'!
stagesAB! !

!MCRepositoryDualVersionsInspector methodsFor: 'as yet unclassified' stamp: 'kph 5/17/2007 17:42'!
summary2
	^self hasVersion2
		ifTrue: [ self version2Summary ]
		ifFalse: [ String new ]! !

!MCRepositoryDualVersionsInspector methodsFor: 'as yet unclassified' stamp: 'kph 5/17/2007 17:33'!
version2
 
	^ self repository2 versionWithInfo: selectedVersionInfo2! !

!MCRepositoryDualVersionsInspector methodsFor: 'as yet unclassified' stamp: 'kph 6/4/2008 14:44'!
version2List
	^ versionInfos2 collect: [:ea | self versionHighlight: ea ]! !

!MCRepositoryDualVersionsInspector methodsFor: 'as yet unclassified' stamp: 'kph 5/17/2007 18:25'!
version2ListMenu: aMenu
	 	
	self hasVersion ifNotNil: [
		self fillMenu: aMenu fromSpecs:
			#(
			('changes B to image' #changes2)
			('merge B to image' #merge2)
			('load B into image' #load2)
			('adopt B' #adopt2)
			('copy B' #copy2)
			('diff B to image' #diff2)  
			('change log entry' #openChangeLogEntry2)
			
			).
		aMenu add:  'copy all' target: self selector: #copyAll2.
		aMenu add:  'copy all (forked)' target: self selector: #copyAllFork2.
		aMenu addLine.
		self isVersionDeletable ifTrue: [   
			aMenu add: 'delete' target: self selector: #delete2.
		].
		 
	].
	aMenu addLine.
	
	self version2ListOrderMenu: aMenu.
	
	^ aMenu
! !

!MCRepositoryDualVersionsInspector methodsFor: 'as yet unclassified' stamp: 'kph 5/17/2007 15:19'!
version2ListObtain
 
	 versionInfos2 := self repository2 sortedHeldVersionInfos.
	! !

!MCRepositoryDualVersionsInspector methodsFor: 'as yet unclassified' stamp: 'kph 6/4/2008 14:18'!
version2ListOrderMenu: aMenu
 
	1 to: self orderSpecs2 size do: [ :index |
		aMenu addUpdating: #order2String: target: self selector: #order2: argumentList: { index } ].
	 
	^aMenu! !

!MCRepositoryDualVersionsInspector methodsFor: 'as yet unclassified' stamp: 'kph 6/4/2008 14:46'!
version2ListSort

	 ^ self version2SortBlock ifNotNilDo: [ :sb | sb fixTemps. versionInfos2 := versionInfos2 asSortedCollection: [:b :a | [sb value: a value: b] on: Error do: [true]]].
	
	 ! !

!MCRepositoryDualVersionsInspector methodsFor: 'as yet unclassified' stamp: 'kph 5/17/2007 14:46'!
version2Selection  
	^ versionInfos2 indexOf: selectedVersionInfo2! !

!MCRepositoryDualVersionsInspector methodsFor: 'as yet unclassified' stamp: 'kph 5/19/2007 01:05'!
version2Selection: aNumber
	aNumber isZero 
		ifTrue: [ selectedVersionInfo2 := nil ]
		ifFalse: [ 
			selectedVersionInfo2 := versionInfos2 at: aNumber].
	self changed:#version2Selection; 
		changed: #summary2 ;
		changed: #hasVersion2;
		changed: #hasVersions.! !

!MCRepositoryDualVersionsInspector methodsFor: 'as yet unclassified' stamp: 'kph 6/4/2008 14:17'!
version2SortBlock

	^ (self orderSpecs2 at: order2) value ! !

!MCRepositoryDualVersionsInspector methodsFor: 'as yet unclassified' stamp: 'kph 5/17/2007 15:28'!
version2Summary

	selectedVersionInfo2 ifNil: [ ^ '' ].

	^ (inherited detect: [ :info | info versionName = selectedVersionInfo2 versionName ] ifNone: [ self version2 ]) summary! !

!MCRepositoryDualVersionsInspector methodsFor: 'as yet unclassified' stamp: 'kph 5/17/2007 18:24'!
versionListMenu: aMenu
	 	
	self hasVersion ifNotNil: [
		self fillMenu: aMenu fromSpecs:
			#(
			('changes A to image' #changes)
			('merge A to image' #merge)
			('load A into image' #load)
			('adopt A' #adopt)
			('copy A' #copy)
	
			('diff A to image' #diff)  
			('change log entry' #openChangeLogEntry)
			('spawn dual changes browser' #openDual)
			
			).
		aMenu add:  'copy all' target: self selector: #copyAll.
		aMenu add:  'copy all (forked)' target: self selector: #copyAllFork.
		aMenu addLine.
		self isVersionDeletable ifTrue: [   
			aMenu add: 'delete' target: self selector: #delete.
		].
		 
	].
	aMenu addLine.
	
	self versionListOrderMenu: aMenu.

	(self class = MCRepositoryVersionsInspector) 
		ifTrue: [
			aMenu addLine.
			aMenu add: 'open two column display' target: self selector: #twoColumnOpen.
			aMenu add: 'always use two column display' target: self selector: #twoColumnAlways.
		].
	
	^ aMenu
! !

!MCRepositoryDualVersionsInspector methodsFor: 'as yet unclassified' stamp: 'kph 5/18/2007 13:20'!
versionSelection: aNumber
	super versionSelection: aNumber.
	self changed: #hasVersions.! !

!MCRepositoryDualVersionsInspector methodsFor: 'as yet unclassified' stamp: 'kph 5/21/2007 13:02'!
widgetSpecs

"left top right bottom"

	^#(	((buttonRow) (0 0 1 0) (0 0 0 30))
		((listMorph: version ) (0 0 0.25 0.6) (0 30 0 0))
		((textMorph: summary) (0 0.6 0.25 1) (0 0 0 0)) 
		
		((panel: buildCenterPanel) (0.25 0 0.75 1) (0 30 0 0))

		((textMorph: summary2) (0.75 0.6 1 1) (0 0 0 0)) 
		((listMorph: version2 ) (0.75 0 1 0.6) (0 30 0 0))
													)! !

MCRepositoryVersionsInspector subclass: #MCRepositoryPackagesVersionsInspector
	instanceVariableNames: 'selectedPackageName packageNames'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-UI'!

!MCRepositoryPackagesVersionsInspector methodsFor: 'morphic ui' stamp: 'kph 5/17/2007 08:41'!
hasPackage

	^ selectedPackageName notNil! !

!MCRepositoryPackagesVersionsInspector methodsFor: 'actions' stamp: 'kph 5/17/2007 10:12'!
oneColumnAlways

	repository class uiClassUseOneColumn.
 ! !

!MCRepositoryPackagesVersionsInspector methodsFor: 'actions' stamp: 'kph 5/17/2007 10:20'!
oneColumnOpen

	| newUI |
	
	newUI := MCRepositoryVersionsInspector
				repository: repository 
				selectedPackage: selectedPackageName
	 			selectedVersion: selectedVersionInfo.
	
	 newUI show.! !

!MCRepositoryPackagesVersionsInspector methodsFor: 'actions' stamp: 'kph 5/17/2007 10:11'!
oneColumnSwitch

	| newUI newWindow |
	
	newUI := repository class uiClassUseOneColumn 
				repository: repository 
				selectedPackage: selectedPackageName
	 			selectedVersion: selectedVersionInfo.
	
	Smalltalk at: #ToolBuilder ifPresent: [:tb |  
		newWindow := (tb default build: newUI)
		 
	].
	
	newWindow ifNil: [ 
		newWindow := self window  openInWorldExtent: self defaultExtent; yourself 
		
	].
	"newWindow bounds: self window fullBounds.."
	 
	newWindow openInWorldExtent: self window extent; yourself. 
	
	 
	self  window closeBoxHit..! !

!MCRepositoryPackagesVersionsInspector methodsFor: 'morphic ui' stamp: 'kph 5/17/2007 08:35'!
packageHighlight: aPackageName

	newer ifNil: [newer := #()].
	
	^(loaded anySatisfy: [:each | each packageName = aPackageName])
		ifTrue: [
			Text string: (MCPackage named: aPackageName)
					workingCopy packageNameWithStar
				attribute: (TextEmphasis new emphasisCode: (
					((newer includes: aPackageName)
						ifTrue: [5] ifFalse: [4])))]
		ifFalse: [aPackageName]! !

!MCRepositoryPackagesVersionsInspector methodsFor: 'morphic ui' stamp: 'kph 5/17/2007 08:38'!
packageList
	^ packageNames collect: [ :n | self packageHighlight: n ]! !

!MCRepositoryPackagesVersionsInspector methodsFor: 'morphic ui' stamp: 'kph 5/17/2007 10:18'!
packageListMenu: aMenu

			aMenu add: 'open to one column display' target: self selector: #oneColumnOpen.
			aMenu add: 'always use one column display' target: self selector: #oneColumnAlways.
	
	^ aMenu! !

!MCRepositoryPackagesVersionsInspector methodsFor: 'accessing-model' stamp: 'kph 5/17/2007 01:55'!
packageListObtain
	^ packageNames := repository packageNames ! !

!MCRepositoryPackagesVersionsInspector methodsFor: 'morphic ui' stamp: 'kph 5/17/2007 01:54'!
packageSelection
	^ packageNames indexOf: selectedPackageName! !

!MCRepositoryPackagesVersionsInspector methodsFor: 'accessing-model' stamp: 'kph 9/28/2007 10:39'!
packageSelection: aNumber
	selectedPackageName := aNumber isZero ifFalse: [ packageNames at: aNumber ].
	selectedVersionInfo := nil.
	self update.
	self changed: #packageSelection; changed: #versionList ; changed: #versionSelection ; changed: #summary
	
! !

!MCRepositoryPackagesVersionsInspector methodsFor: 'morphic ui' stamp: 'kph 5/17/2007 01:53'!
packagesNames
	^  packageNames! !

!MCRepositoryPackagesVersionsInspector methodsFor: 'accessing-model' stamp: 'kph 5/17/2007 04:25'!
setSelectedPackage: aPackage

	selectedPackageName := aPackage name! !

!MCRepositoryPackagesVersionsInspector methodsFor: 'as yet unclassified' stamp: 'kph 9/28/2007 10:38'!
update

	self packageListObtain.
	self changed: #packageList.
 	
	super update.
  	! !

!MCRepositoryPackagesVersionsInspector methodsFor: 'accessing-model' stamp: 'kph 5/17/2007 02:03'!
versionListObtain

	^ versionInfos := self repository 
		sortedVersionInfosAvailableForPackageNamed: selectedPackageName! !

!MCRepositoryPackagesVersionsInspector methodsFor: 'accessing-model' stamp: 'kph 5/17/2007 01:57'!
widgetSpecs

	^#(	((buttonRow) (0 0 1 0) (0 0 0 30))
		((listMorph: package) (0 0 0.5 0.6) (0 30 0 0))
		((listMorph: version) (0.5 0 1 0.6) (0 30 0 0))
		((textMorph: summary) (0 0.6 1 1) (0 0 0 0)) )! !

!MCRepositoryVersionsInspector class methodsFor: 'as yet unclassified' stamp: 'mtf 6/7/2008 11:34'!
order
	Order isNil
		ifTrue: [ Order := 2 ].
	^Order! !

!MCRepositoryVersionsInspector class methodsFor: 'as yet unclassified' stamp: 'kph 5/17/2007 03:46'!
order: anInteger
	Order := anInteger! !

!MCRepositoryVersionsInspector class methodsFor: 'as yet unclassified' stamp: 'kph 5/17/2007 08:56'!
repository: r selectedPackage: p selectedVersion: v

	^(self new) setRepository: r selectedPackage: p selectedVersion: v! !

!MCRepositoryVersionsInspector class methodsFor: 'as yet unclassified' stamp: 'kph 5/16/2007 14:13'!
repository: r workingCopy: wc

	^(self new) setRepository: r workingCopy: wc! !

!MCRepositoryVersionsInspector methodsFor: '' stamp: 'kph 8/2/2007 15:37'!
browse

	self hasVersion ifTrue: [ self versionInfo browseVersionFrom: repository ].
	 
	self refresh
	! !

!MCRepositoryVersionsInspector methodsFor: 'morphic ui' stamp: 'kph 5/14/2007 14:47'!
buttonSpecs
	^#(('Refresh' refresh 'refresh the version-list')) , super buttonSpecs! !

!MCRepositoryVersionsInspector methodsFor: 'highlighting' stamp: 'mtf 6/27/2008 23:20'!
collateHighlightingData

loaded := Set new.
inherited := Set new.
newer := Set new.
loadedNames := nil.
inheritedNames := nil.

	(MCWorkingCopy allManagers 
	   	"select: [ :each | self packageNames includes: each packageName]")
			do: [:each |
				each ancestors do: [ :ancestor |
					loaded add: ancestor  .
					ancestor ancestorsDoWhileTrue: [:heir | 
						(inherited includes: heir )
							ifTrue: [false]
							ifFalse: [inherited add: heir . true]]].
					
				latest := (versionInfos select: [:v | v packageName = each package name])	
					detectMax: [:v | v ].
					
				(latest notNil and: [
					each ancestors allSatisfy: [:ancestor | latest > ancestor]])
					ifTrue: [newer add: each package name ]
			].

		"self changed: #packageList; changed: #versionList"
	 ! !

!MCRepositoryVersionsInspector methodsFor: 'morphic ui' stamp: 'kph 6/2/2008 19:31'!
copyAll
		| versionsToCopy   aVersion gotAlready |
 
	self pickRepository ifNotNilDo: [:aRepository |.
	
	gotAlready := aRepository heldVersionInfos collect: [ :v | v versionName ].

	versionsToCopy := versionInfos removeAllSuchThat: [ :v | gotAlready includes: v versionName ].
				 
		Transcript cr; show: 'versions: ', versionsToCopy asArray printString.
		Cursor wait showWhile: [
			versionsToCopy do: [:each |
				[ aVersion := repository versionWithInfo: each.
				Transcript cr; show: '',aRepository description,' storeVersion: ', aVersion description.

 		aVersion info name: aVersion info name withBlanksTrimmed.
		aVersion info allAncestorsDo: [ :ea | ea name: ea name withBlanksTrimmed ].
 
				aRepository storeVersion: aVersion ] ifError: [ Transcript cr; show: each description, ' not readable'. ]
			
			]
		]
	]! !

!MCRepositoryVersionsInspector methodsFor: 'morphic ui' stamp: 'kph 11/16/2007 23:24'!
copyAllFork
		| versionsToCopy   aVersion gotAlready |
 
	self pickRepository ifNotNilDo: [:aRepository |.
	
	gotAlready := aRepository heldVersionInfos collect: [ :v | v versionName ].

	versionsToCopy := versionInfos removeAllSuchThat: [ :v | gotAlready includes: v versionName ].

		Transcript cr; show: 'versions: ', versionsToCopy asArray printString.
	[   [
		versionsToCopy do: [:each |
		
				 [ aVersion := repository versionWithInfo: each. 
				Transcript cr; show: '',aRepository description,' storeVersion: ', aVersion printString.
				aRepository storeVersion: aVersion ] ifError: [ Transcript cr; show: each description, ' not readable'. ]
			
			]
		] on: Notification do: [ :ex | ex resume ].
	] forkAt: Processor userBackgroundPriority.
]
	! !

!MCRepositoryVersionsInspector methodsFor: 'morphic ui' stamp: 'kph 5/14/2007 14:46'!
defaultExtent
	^450@300! !

!MCRepositoryVersionsInspector methodsFor: 'morphic ui' stamp: 'kph 5/14/2007 14:48'!
defaultLabel
	^'Repository: ' , repository description! !

!MCRepositoryVersionsInspector methodsFor: 'actions' stamp: 'kph 5/14/2007 15:06'!
delete

	self isVersionDeletable ifFalse: [ ^ self ].
	
	(self confirm: 'Are you sure you wish to delete ', selectedVersion, '
from ', repository description)
		ifFalse: [ ^ self ].

	 (repository delete: selectedVersion)
		ifTrue: [ self inform: ('Successfully deleted ', selectedVersion) ]
		ifFalse: [ self inform: ('Sorry, could not delete, ', selectedVersion) ].
		
	! !

!MCRepositoryVersionsInspector methodsFor: '' stamp: 'kph 8/7/2007 14:09'!
hasSnapshot
	 
	^ selectedVersionInfo notNil and: [ selectedVersionInfo hasSnapshot ]! !

!MCRepositoryVersionsInspector methodsFor: 'testing' stamp: 'kph 5/17/2007 00:09'!
hasVersion
	^ selectedVersionInfo notNil! !

!MCRepositoryVersionsInspector methodsFor: 'highlighting' stamp: 'kph 5/17/2007 07:24'!
inheritedNames

	^ inheritedNames ifNil: [ inheritedNames := inherited collect: [ :v | v name ]] ! !

!MCRepositoryVersionsInspector methodsFor: 'testing' stamp: 'kph 5/14/2007 16:21'!
isVersionDeletable

"disable version deleting until sure its ok"
	^ false 
	"^ self repository ifNotNilDo:[ :r | r isDeletingAllowed ] ifNil: [ false ]"! !

!MCRepositoryVersionsInspector methodsFor: 'actions' stamp: 'kph 8/1/2007 12:11'!
load
 
	self hasVersion ifTrue: [ self versionInfo loadVersionFrom: repository ].
	 
	self refresh
	! !

!MCRepositoryVersionsInspector methodsFor: 'highlighting' stamp: 'kph 5/17/2007 07:24'!
loadedNames

	^ loadedNames ifNil: [ loadedNames := loaded collect: [ :v | v name ]] ! !

!MCRepositoryVersionsInspector methodsFor: '' stamp: 'kph 8/1/2007 12:51'!
merge
 
	self hasVersion ifTrue: [ self versionInfo mergeVersionFrom: repository ].
	 
	self refresh
	! !

!MCRepositoryVersionsInspector methodsFor: 'actions' stamp: 'kph 5/16/2007 21:02'!
openChangeLogEntry

	(MCChangeLogEntry new version: self version; repository: repository) show.! !

!MCRepositoryVersionsInspector methodsFor: 'morphic ui' stamp: 'kph 5/17/2007 15:23'!
openDual

   |  dual r2 |

	r2 := self pickRepository.
	
	dual :=	MCRepositoryDualVersionsInspector repository: self repository repository2: r2.
		
	dual versionSelection: self versionSelection.
	
	dual show.! !

!MCRepositoryVersionsInspector methodsFor: 'ordering' stamp: 'kph 5/17/2007 06:18'!
order: anInteger
	self class order: (order := anInteger).
	self refresh! !

!MCRepositoryVersionsInspector methodsFor: 'ordering' stamp: 'kph 5/17/2007 10:46'!
order: setting string: anIndex
	^String streamContents: [ :stream |
		setting = anIndex
			ifTrue: [ stream nextPutAll: '<yes>' ]
			ifFalse: [ stream nextPutAll: '<no>' ].
		stream nextPutAll: (self orderSpecs at: anIndex) key ]! !

!MCRepositoryVersionsInspector methodsFor: 'ordering' stamp: 'mtf 6/27/2008 23:20'!
orderSpecs
	^{
		'unchanged' -> nil.
		'order by package' -> [ :x :y | x <= y ].	
	}! !

!MCRepositoryVersionsInspector methodsFor: 'ordering' stamp: 'kph 5/17/2007 10:47'!
orderString: anIndex
	^ self order: order string: anIndex! !

!MCRepositoryVersionsInspector methodsFor: 'morphic ui' stamp: 'kph 9/28/2007 11:02'!
refresh

	self repository heldVersionInfosClearCache. 
	self update
  	! !

!MCRepositoryVersionsInspector methodsFor: 'accessing-model' stamp: 'kph 5/16/2007 23:38'!
repository

	^ repository ! !

!MCRepositoryVersionsInspector methodsFor: 'initialize-release' stamp: 'mtf 6/7/2008 12:27'!
setRepository: aRepository selectedPackage: package selectedVersion: versionInfo
	order := self class order.
	repository := aRepository.
     package isNil ifFalse: [ self setSelectedPackage: package].
	self refresh! !

!MCRepositoryVersionsInspector methodsFor: 'initialize-release' stamp: 'kph 5/17/2007 04:24'!
setRepository: aRepository workingCopy: aWorkingCopy
	order := self class order.
	repository := aRepository.
     aWorkingCopy isNil ifFalse: [ self setSelectedPackage: aWorkingCopy package].
	self refresh! !

!MCRepositoryVersionsInspector methodsFor: 'initialize-release' stamp: 'kph 5/17/2007 04:25'!
setSelectedPackage: aPackage
! !

!MCRepositoryVersionsInspector methodsFor: 'actions' stamp: 'kph 5/17/2007 10:20'!
twoColumnOpen

	| newUI |
	
	newUI := MCRepositoryPackagesVersionsInspector
				repository: repository 
				selectedPackage: nil
	 			selectedVersion: selectedVersionInfo.
	
	 newUI show.
	! !

!MCRepositoryVersionsInspector methodsFor: 'actions' stamp: 'kph 5/17/2007 10:15'!
twoColumnSwitch

	 ! !

!MCRepositoryVersionsInspector methodsFor: 'actions' stamp: 'kph 5/17/2007 09:28'!
twoColumnsAlways

	repository class uiClassUseTwoColumn.
	self twoColumnSwitch! !

!MCRepositoryVersionsInspector methodsFor: 'morphic ui' stamp: 'kph 9/28/2007 10:41'!
update

	self versionListObtain.
	self versionListSort.
	 
	self collateHighlightingData.
	self changed: #versionList.
 ! !

!MCRepositoryVersionsInspector methodsFor: 'accessing-model' stamp: 'kph 5/17/2007 00:55'!
version
 
	^ self repository versionWithInfo: selectedVersionInfo! !

!MCRepositoryVersionsInspector methodsFor: 'highlighting' stamp: 'kph 5/17/2007 07:26'!
versionHighlight: anInfo

	^Text
		string: anInfo name
		attribute: (TextEmphasis new emphasisCode: (
			(self loadedNames includes: anInfo versionName) ifTrue: [ 4 "underlined" ]
				ifFalse: [ (self inheritedNames includes: anInfo versionName)
					ifTrue: [ 0 ]
					ifFalse: [ 1 "bold" ] ]))
				
			! !

!MCRepositoryVersionsInspector methodsFor: 'accessing-model' stamp: 'kph 5/22/2007 16:56'!
versionInfo
 
	^ selectedVersionInfo realizeFrom: repository! !

!MCRepositoryVersionsInspector methodsFor: 'morphic ui' stamp: 'kph 5/17/2007 16:18'!
versionList
	^ versionInfos collect: [:ea | (self versionHighlight: ea) ]! !

!MCRepositoryVersionsInspector methodsFor: 'morphic ui' stamp: 'kph 8/1/2007 12:34'!
versionListMenu: aMenu
	 	
	self hasSnapshot ifTrue: [
		self fillMenu: aMenu fromSpecs:
			#(
			('adopt' #adopt)
			('diff' #diff)  
			('change log entry' #openChangeLogEntry)
			('open dual changes browser' #openDual)
			
			).
		aMenu add:  'copy all' target: self selector: #copyAll.
		aMenu add:  'copy all (forked)' target: self selector: #copyAllFork.
		aMenu addLine.
		self isVersionDeletable ifTrue: [   
			aMenu add: 'delete' target: self selector: #delete.
		].
		 
	].
	aMenu addLine.
	
	self versionListOrderMenu: aMenu.

	(self class = MCRepositoryVersionsInspector) 
		ifTrue: [
			aMenu addLine.
			aMenu add: 'open two column display' target: self selector: #twoColumnOpen.
			aMenu add: 'always use two column display' target: self selector: #twoColumnAlways.
		].
	
	^ aMenu
! !

!MCRepositoryVersionsInspector methodsFor: 'accessing-model' stamp: 'kph 5/17/2007 08:08'!
versionListObtain
 
	 versionInfos := self repository sortedHeldVersionInfos.
	! !

!MCRepositoryVersionsInspector methodsFor: 'morphic ui' stamp: 'kph 5/17/2007 03:43'!
versionListOrderMenu: aMenu
 
	1 to: self orderSpecs size do: [ :index |
		aMenu addUpdating: #orderString: target: self selector: #order: argumentList: { index } ].
	 
	^aMenu! !

!MCRepositoryVersionsInspector methodsFor: 'morphic ui' stamp: 'kph 6/4/2008 14:46'!
versionListSort

	 self versionsSortBlock ifNotNilDo: [ :sb | versionInfos := versionInfos asSortedCollection: [:b :a | [sb value: a value: b] on: Error do: [true]]].
	
	 ! !

!MCRepositoryVersionsInspector methodsFor: 'morphic ui' stamp: 'kph 5/17/2007 06:17'!
versionSelection  
	^ versionInfos indexOf: selectedVersionInfo! !

!MCRepositoryVersionsInspector methodsFor: 'morphic ui' stamp: 'kph 8/1/2007 13:18'!
versionSelection: aNumber
	aNumber isZero 
		ifTrue: [ selectedVersionInfo := nil ]
		ifFalse: [ 
			selectedVersionInfo := versionInfos at: aNumber].
		
		 
	self changed:#versionSelection; changed: #summary ; changed: #hasVersion; changed: #hasSnapshot.! !

!MCRepositoryVersionsInspector methodsFor: 'highlighting' stamp: 'kph 5/22/2007 16:48'!
versionSummary

	^ (inherited detect: [ :info | info versionName = selectedVersionInfo versionName ] ifNone: [ self versionInfo ]) summary! !

!MCRepositoryVersionsInspector methodsFor: 'morphic ui' stamp: 'kph 6/4/2008 14:06'!
versionsSortBlock

	^ (self orderSpecs at: order) value ! !

!MCRepositoryVersionsInspector methodsFor: 'morphic ui' stamp: 'kph 5/16/2007 23:23'!
widgetSpecs
	^#(	((buttonRow) (0 0 1 0) (0 0 0 30))
		((listMorph: version) (0 0 1 0.6) (0 30 0 0))
		((textMorph: summary) (0 0.6 1 1) (0 0 0 0)) )! !

!MCVersionInspector methodsFor: 'as yet unclassified' stamp: 'avi 2/28/2004 20:19'!
adopt
	(self confirm:
'Modifying ancestry can be dangerous unless you know
what you are doing.  Are you sure you want to adopt
',self version info name, ' as an ancestor of your working copy?')
		ifTrue: [self version adopt]! !

!MCVersionInspector methodsFor: 'as yet unclassified' stamp: 'kph 8/2/2007 15:22'!
browse
 
	self version browse! !

!MCVersionInspector methodsFor: 'morphic ui' stamp: 'kph 8/7/2007 14:10'!
buttonSpecs
       ^ #((Browse browse 'Browse this version' hasVersion)
               (History history 'Browse the history of this version' hasVersion)
               (Changes changes 'Browse the changes this version would make to the
image' hasVersion)
               (Load load 'Load this version into the image' hasVersion)
               (Merge merge 'Merge this version into the image' hasVersion)
             "  (Adopt adopt 'Adopt this version as an ancestor of your working copy'
hasVersion)"
               (Copy copy 'Copy this version to another repository' hasVersion)
            "   (Diff diff 'Create an equivalent version based on an earlier release'
hasVersion)" )! !

!MCVersionInspector methodsFor: 'as yet unclassified' stamp: 'kph 8/7/2007 14:13'!
changes
 
	(MCPatchBrowser forPatch: self version changes)
		showLabelled: 'Changes from ', self version info name! !

!MCVersionInspector methodsFor: 'as yet unclassified' stamp: 'kph 6/12/2008 23:02'!
copy
 
	self pickRepository ifNotNilDo: [:ea | ea storeVersion: self version ]
	 
 ! !

!MCVersionInspector methodsFor: 'as yet unclassified' stamp: 'kph 6/2/2008 19:38'!
copyAll
	self halt.! !

!MCVersionInspector methodsFor: 'morphic ui' stamp: 'avi 8/31/2003 00:45'!
defaultExtent
	^ 400@200! !

!MCVersionInspector methodsFor: 'morphic ui' stamp: 'avi 2/28/2004 20:18'!
defaultLabel
	^ 'Version: ', self version info name! !

!MCVersionInspector methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
diff
	| ancestorVersion |
	self pickAncestor ifNotNilDo:
		[:ancestor |
		ancestorVersion := self version workingCopy repositoryGroup versionWithInfo: ancestor.
		(self version asDiffAgainst: ancestorVersion) open]! !

!MCVersionInspector methodsFor: '' stamp: 'kph 8/7/2007 14:08'!
hasSnapshot
	^ self version notNil! !

!MCVersionInspector methodsFor: 'as yet unclassified' stamp: 'kph 5/17/2007 00:07'!
hasVersion
	^ self version notNil! !

!MCVersionInspector methodsFor: 'as yet unclassified' stamp: 'avi 9/17/2005 17:14'!
history
	(MCVersionHistoryBrowser new ancestry: self versionInfo) show! !

!MCVersionInspector methodsFor: 'as yet unclassified' stamp: 'kph 5/15/2007 15:46'!
load
	Cursor wait showWhile: [ self version load]! !

!MCVersionInspector methodsFor: 'as yet unclassified' stamp: 'kph 5/17/2007 18:19'!
merge
	self version merge.
	self refresh.! !

!MCVersionInspector methodsFor: 'morphic ui' stamp: 'stephaneducasse 2/4/2006 20:47'!
pickAncestor
	| index versions |
	versions := self version info breadthFirstAncestors.
	index := (PopUpMenu labelArray: (versions collect: [:ea | ea name]))
				startUpWithCaption: 'Ancestor:'.
	^ index = 0 ifFalse: [versions at: index]! !

!MCVersionInspector methodsFor: 'morphic ui' stamp: 'stephaneducasse 2/4/2006 20:47'!
pickRepository
	| index |
	index := (PopUpMenu labelArray: (self repositories collect: [:ea | ea description]))
				startUpWithCaption: 'Repository:'.
	^ index = 0 ifFalse: [self repositories at: index]! !

!MCVersionInspector methodsFor: 'morphic ui' stamp: 'avi 8/31/2003 00:44'!
repositories
	^ MCRepositoryGroup default repositories! !

!MCVersionInspector methodsFor: 'as yet unclassified' stamp: 'kph 5/14/2007 14:32'!
repository 

	^ nil! !

!MCVersionInspector methodsFor: 'as yet unclassified' stamp: 'avi 9/17/2005 17:16'!
summary
	^self hasVersion
		ifTrue: [ self versionSummary ]
		ifFalse: [ String new ]! !

!MCVersionInspector methodsFor: 'as yet unclassified' stamp: 'avi 2/28/2004 20:19'!
version
	^ version! !

!MCVersionInspector methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
version: aVersion
	version := aVersion! !

!MCVersionInspector methodsFor: 'as yet unclassified' stamp: 'avi 9/17/2005 17:16'!
versionInfo
	^ self version info! !

!MCVersionInspector methodsFor: 'as yet unclassified' stamp: 'avi 9/17/2005 17:16'!
versionSummary
	^ self version summary! !

!MCVersionInspector methodsFor: 'morphic ui' stamp: 'ab 7/18/2003 18:43'!
widgetSpecs
	^ #(
		((buttonRow) (0 0 1 0) (0 0 0 30))
		((textMorph: summary) (0 0 1 1) (0 30 0 0))
		)! !

MCTool subclass: #MCWorkingCopyBrowser
	instanceVariableNames: 'workingCopy workingCopyWrapper repository defaults'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-UI'!

!MCWorkingCopyBrowser class methodsFor: 'as yet unclassified' stamp: 'ab 7/19/2003 23:38'!
initialize
	 (TheWorldMenu respondsTo: #registerOpenCommand:)
         ifTrue: [TheWorldMenu registerOpenCommand: {'Monticello Browser'. {self. #open}}]! !

!MCWorkingCopyBrowser class methodsFor: 'as yet unclassified' stamp: 'kph 5/13/2007 16:47'!
open
	^ self new show! !

!MCWorkingCopyBrowser methodsFor: 'actions' stamp: 'bf 11/4/2005 17:46'!
addPackageRepository

	workingCopy ifNotNil:
		[
		(self pickRepositorySatisfying: [ :repos | (workingCopy repositoryGroup includes: repos) not ])
			ifNotNilDo:
				[:repos |
					workingCopy repositoryGroup addRepository: repos.
					self repository: repos.	
					self
						changed: #repositoryList;
						changed: #repositorySelection.
					self changedButtons]]! !

!MCWorkingCopyBrowser methodsFor: 'actions' stamp: 'bkv 2/18/2004 21:22'!
addRepository
	self newRepository ifNotNilDo:
		[:repos | self addRepository: repos ].
! !

!MCWorkingCopyBrowser methodsFor: 'actions' stamp: 'bf 6/21/2005 15:58'!
addRepository: aRepository
	self repository: aRepository.
	self repositoryGroup addRepository: aRepository.
	self changed: #repositoryList; changed: #repositorySelection.
	self changedButtons.! !

!MCWorkingCopyBrowser methodsFor: 'actions' stamp: 'stephaneducasse 2/4/2006 20:47'!
addRepositoryToPackage
	self repository ifNotNilDo:
		[:repos |
		(self pickWorkingCopySatisfying: [ :p | (p repositoryGroup includes: repos) not ]) ifNotNilDo:
			[:wc |
			workingCopy := wc.
			workingCopy repositoryGroup addRepository: repos.
			self repository: repos.	
			self
				changed: #workingCopySelection;
				changed: #repositoryList;
				changed: #repositorySelection.
			self changedButtons]]! !

!MCWorkingCopyBrowser methodsFor: 'actions' stamp: 'kph 6/28/2007 22:58'!
addRepositoryToPackagesMatching

	| result |
	result := FillInTheBlank request: 'packages matching: ' initialAnswer: 'My*'.
	result size > 0 ifTrue: [
		self addRepositoryToPackagesMatching: result.
	]! !

!MCWorkingCopyBrowser methodsFor: 'actions' stamp: 'kph 5/17/2007 00:34'!
addRepositoryToPackagesMatching: aString

	self repository ifNotNilDo:  
		[:repos |
			(self workingCopies select: [:each | aString match: each package name ]) 
				do: [ :wc | 
					Transcript cr; show: 'add ', wc package name, ' to ', repos description.
					wc repositoryGroup addRepository: repos]]! !

!MCWorkingCopyBrowser methodsFor: 'actions' stamp: 'bf 6/21/2005 15:57'!
addRepositoryToWorkingCopy
	workingCopy ifNotNilDo:
		[:wc |
			workingCopy repositoryGroup addRepository: self repository.
			self
				changed: #workingCopySelection;
				changed: #repositoryList;
				changed: #repositorySelection.
			self changedButtons]! !

!MCWorkingCopyBrowser methodsFor: 'actions' stamp: 'AlexandreBergel 7/30/2008 13:47'!
addRequiredDirtyPackage
	| dirtyPackages |
	dirtyPackages := self workingCopies select: [:copy | copy needsSaving].

	workingCopy ifNotNilDo:
		[:wc |
		dirtyPackages do:
			[:required |
			wc requirePackage: required package]].
	
	self workingCopyListChanged! !

!MCWorkingCopyBrowser methodsFor: 'actions' stamp: 'ar 2/14/2004 02:45'!
addRequiredPackage
	workingCopy ifNotNilDo:
		[:wc |
		self pickWorkingCopy ifNotNilDo:
			[:required |
			wc requirePackage: required package.
			self workingCopyListChanged]]! !

!MCWorkingCopyBrowser methodsFor: 'actions' stamp: 'stephaneducasse 2/4/2006 20:47'!
addWorkingCopy
	|name|
	name := FillInTheBlankMorph request: 'Name of package:'.
	name isEmptyOrNil ifFalse:
		[PackageInfo registerPackageName: name.
		workingCopy := MCWorkingCopy forPackage: (MCPackage new name: name).
		workingCopyWrapper := nil.
		self repositorySelection: 0].
	self workingCopyListChanged; changed: #workingCopySelection; changed: #repositoryList.
	self changedButtons.! !

!MCWorkingCopyBrowser methodsFor: 'actions' stamp: 'bf 6/21/2005 15:38'!
backportChanges
	self canBackport ifFalse: [^self].
	workingCopy ifNotNil:
		[workingCopy needsSaving ifTrue: [^ self inform: 'You must save the working copy before backporting.'].
		self pickAncestorVersionInfo ifNotNilDo:
			[:baseVersionInfo |
			workingCopy backportChangesTo: baseVersionInfo]]! !

!MCWorkingCopyBrowser methodsFor: 'actions' stamp: 'ab 7/19/2003 22:58'!
browseWorkingCopy
	workingCopy ifNotNil:
		[(MCSnapshotBrowser forSnapshot: workingCopy package snapshot)
			label: 'Snapshot Browser: ', workingCopy packageName;
			show]! !

!MCWorkingCopyBrowser methodsFor: 'morphic ui' stamp: 'avi 2/28/2005 16:28'!
buttonSpecs
       ^ #(
               ('+Package' addWorkingCopy 'Add a new package and make it the working copy')
               (Browse browseWorkingCopy 'Browse the working copy of the selected package' hasWorkingCopy)
               (Scripts editLoadScripts 'Edit the load/unload scripts of this package' hasWorkingCopy)
               (History viewHistory 'View the working copy''s history' hasWorkingCopy)
               (Changes viewChanges 'View the working copy''s changes relative to the installed version from the repository' canSave)
               (Backport backportChanges 'Backport the working copy''s changes to an ancestor' canBackport)
               (Save saveVersion 'Save the working copy as a new version to the selected repository' canSave)
             ('+Repository' addRepository 'Add an existing repository to the list of those visible')
               (Open openRepository 'Open a browser on the selected repository' hasRepository)
               )! !

!MCWorkingCopyBrowser methodsFor: 'actions' stamp: 'kph 5/11/2007 05:07'!
canBackport
	^ self hasWorkingCopy and: [ workingCopy canBackport ]! !

!MCWorkingCopyBrowser methodsFor: 'morphic ui' stamp: 'ab 8/7/2003 21:22'!
canSave
	^ self hasWorkingCopy and: [self hasRepository]! !

!MCWorkingCopyBrowser methodsFor: 'morphic ui' stamp: 'bf 6/21/2005 15:50'!
changedButtons
	self changed: #hasWorkingCopy.
	self changed: #canSave.
	self changed: #canBackport.
	self changed: #hasRepository.
! !

!MCWorkingCopyBrowser methodsFor: 'actions' stamp: 'kph 8/1/2007 11:42'!
checkForNewerVersions
	| newer |
 
	newer := workingCopy possiblyNewerVersionsIn: self repository.
	^ newer isEmpty or: [
		self confirm: 'CAUTION!! These versions in the repository may be newer:', 
			String cr, newer asString, String cr,
			'Do you really want to save this version?'].! !

!MCWorkingCopyBrowser methodsFor: 'orphanage' stamp: 'kph 5/10/2007 17:24'!
clearOrphans

	self orphanage clearOrphans.
	self workingCopyListChanged.! !

!MCWorkingCopyBrowser methodsFor: 'actions' stamp: 'ar 2/14/2004 02:46'!
clearRequiredPackages
	workingCopy ifNotNilDo:
		[:wc |
		wc clearRequiredPackages.
		self workingCopyListChanged]! !

!MCWorkingCopyBrowser methodsFor: 'morphic ui' stamp: 'SR 9/9/2006 19:41'!
defaultExtent
	^ 600@400! !

!MCWorkingCopyBrowser methodsFor: 'morphic ui' stamp: 'ab 7/19/2003 23:38'!
defaultLabel
	^ 'Monticello Browser'! !

!MCWorkingCopyBrowser methodsFor: 'morphic ui' stamp: 'stephaneducasse 2/4/2006 20:47'!
defaults
	^ defaults ifNil: [defaults := Dictionary new]! !

!MCWorkingCopyBrowser methodsFor: 'actions' stamp: 'ar 2/14/2004 02:46'!
deleteWorkingCopy
	workingCopy unregister.
	self workingCopySelection: 0.
	self workingCopyListChanged.! !

!MCWorkingCopyBrowser methodsFor: 'morphic ui' stamp: 'stephaneducasse 2/4/2006 20:47'!
editLoadScripts

	| menu |
	self hasWorkingCopy ifFalse: [^self].
	menu := MenuMorph new defaultTarget: self.
	menu add: 'edit preamble' selector: #editScript: argument: #preamble.
	menu add: 'edit postscript' selector: #editScript: argument: #postscript.
	menu add: 'edit preambleOfRemoval' selector: #editScript: argument: #preambleOfRemoval.
	menu add: 'edit postscriptOfRemoval' selector: #editScript: argument: #postscriptOfRemoval.
	menu popUpInWorld.! !

!MCWorkingCopyBrowser methodsFor: 'actions' stamp: 'kph 5/16/2007 12:53'!
editRepository
	| newRepo |
	
	newRepo := self repository openAndEditTemplateCopy.
	newRepo ifNotNil: [ 
		newRepo class = self repository class
			ifTrue: [self repository copyFrom: newRepo]
			ifFalse: [self inform: 'Must not change repository type!!']].
	self changed: #repositoryList.! !

!MCWorkingCopyBrowser methodsFor: 'morphic ui' stamp: 'kph 8/3/2008 03:23'!
editScript: scriptSymbol

| script |
script := workingCopy packageInfo propertyOrDefaultAt: scriptSymbol.
script openLabel: scriptSymbol asString, ' of the Package ', workingCopy package name.! !

!MCWorkingCopyBrowser methodsFor: 'actions' stamp: 'kph 5/14/2007 13:14'!
exploreWorkingCopy
	workingCopy ifNotNil: [workingCopy explore]! !

!MCWorkingCopyBrowser methodsFor: 'orphanage' stamp: 'kph 10/2/2007 18:40'!
fixAllObsoleteReferences
"
self fixAllObsoleteReferences
"
(SystemNavigation default obsoleteBehaviors select: [ :c | c isMeta not ])
	do: [ :obsClass | 
		
		([obsClass nonObsoleteClass] ifError: [ nil]) ifNotNilDo: 
			[ :actualClass |
				
				actualClass updateInstancesFrom: obsClass. 
				obsClass becomeForward: actualClass.	
			
				([obsClass metaclass] ifError:[nil]) ifNotNilDo:
					[:metaClass |
						metaClass becomeForward: actualClass class ]]].
			
		SmalltalkImage current fixObsoleteReferences! !

!MCWorkingCopyBrowser methodsFor: 'actions' stamp: 'bf 6/3/2005 15:08'!
flushAllCaches
	| beforeBytes afterBytes beforeVersions afterVersions |
	Cursor wait showWhile: [
		beforeBytes := Smalltalk garbageCollect.
		beforeVersions := MCVersion allSubInstances size.
		MCFileBasedRepository flushAllCaches.
		afterBytes := Smalltalk garbageCollect.
		afterVersions := MCVersion allSubInstances size.
	].
	^self inform: (beforeVersions - afterVersions) asString, ' versions flushed', String cr,
 		(afterBytes - beforeBytes) asStringWithCommas, ' bytes reclaimed'! !

!MCWorkingCopyBrowser methodsFor: 'actions' stamp: 'ar 11/4/2005 16:57'!
flushPasswords
	MCRepository allSubInstancesDo:[:repo| repo flushPasswords].! !

!MCWorkingCopyBrowser methodsFor: 'morphic ui' stamp: 'ab 7/22/2003 02:21'!
hasRepository
	^ self repository notNil! !

!MCWorkingCopyBrowser methodsFor: 'morphic ui' stamp: 'ab 7/22/2003 00:46'!
hasWorkingCopy
	^ workingCopy notNil! !

!MCWorkingCopyBrowser methodsFor: 'morphic ui' stamp: 'avi 11/16/2003 20:21'!
initialize
	MCWorkingCopy addDependent: self.
	self workingCopies do: [:ea | ea addDependent: self].! !

!MCWorkingCopyBrowser methodsFor: 'actions' stamp: 'bf 6/21/2005 15:58'!
loadRepositories
	FileStream fileIn: 'MCRepositories.st'.
	self changed: #repositoryList.
	self changedButtons.
! !

!MCWorkingCopyBrowser methodsFor: 'actions' stamp: 'stephaneducasse 2/4/2006 20:47'!
newRepository
	| types index |
	types := MCRepository allConcreteSubclasses asArray.
	index := (PopUpMenu labelArray: (types collect: [:ea | ea description]))
				startUpWithCaption: 'Repository type:'.
	^ index = 0 ifFalse: [(types at: index) morphicConfigure]! !

!MCWorkingCopyBrowser methodsFor: 'morphic ui' stamp: 'kph 5/17/2007 17:15'!
openDual

   |  dual r2 |

	r2 := self pickRepositorySatisfying: [:ea | true].
	
	dual :=	MCRepositoryDualVersionsInspector repository: self repository repository2: r2.
	 
	dual show.! !

!MCWorkingCopyBrowser methodsFor: 'actions' stamp: 'kph 4/25/2008 00:48'!
openRepository
	 
	[ self repository ifNotNilDo: [:repos | repos morphicOpen: workingCopy ] ] fork! !

!MCWorkingCopyBrowser methodsFor: 'orphanage' stamp: 'kph 5/5/2007 04:00'!
orphanage

	^ MCOrphanage current! !

!MCWorkingCopyBrowser methodsFor: 'morphic ui' stamp: 'stephaneducasse 2/4/2006 20:47'!
pickAncestorVersionInfo
	| ancestors index |
	ancestors := workingCopy ancestry breadthFirstAncestors.
	index := (PopUpMenu labelArray: (ancestors collect: [:ea | ea name]))
				startUpWithCaption: 'Ancestor:'.
	^ index = 0 ifFalse: [ ancestors at: index]! !

!MCWorkingCopyBrowser methodsFor: 'morphic ui' stamp: 'mtf 6/9/2008 20:58'!
pickRepositorySatisfying: aBlock
	| repos index |
	repos := MCRepositoryGroup default repositories select: aBlock.
	index := (PopUpMenu labelArray: (repos collect: [:ea | ea descriptionDisplay ]))
				startUpWithCaption: 'Repository:'.
	^ index = 0 ifFalse: [repos at: index]! !

!MCWorkingCopyBrowser methodsFor: 'morphic ui' stamp: 'nk 3/9/2004 14:39'!
pickWorkingCopy
	^self pickWorkingCopySatisfying: [ :c | true ]! !

!MCWorkingCopyBrowser methodsFor: 'morphic ui' stamp: 'stephaneducasse 2/4/2006 20:47'!
pickWorkingCopySatisfying: aBlock
	| copies index |
	copies := self workingCopies select: aBlock.
	copies isEmpty ifTrue: [ ^nil ].
	index := (PopUpMenu labelArray: (copies collect: [:ea | ea packageName]))
				startUpWithCaption: 'Package:'.
	^ index = 0 ifFalse: [ copies at: index]! !

!MCWorkingCopyBrowser methodsFor: 'orphanage' stamp: 'kph 4/24/2008 01:07'!
recompileAllPackages

	self workingCopies do: [ :each | self log monticello recompilingPackage: each packageName. each recompilePackage. ]! !

!MCWorkingCopyBrowser methodsFor: 'actions' stamp: 'kph 5/13/2007 01:43'!
recompilePackage
 
	workingCopy recompilePackage! !

!MCWorkingCopyBrowser methodsFor: 'actions' stamp: 'bf 6/21/2005 15:58'!
removeRepository
	self repository ifNotNilDo:
		[:repos |
		self repositoryGroup removeRepository: repos.
		self repositorySelection: (1 min: self repositories size)].
	self changed: #repositoryList.
	self changedButtons.
! !

!MCWorkingCopyBrowser methodsFor: 'actions' stamp: 'kph 6/28/2007 22:59'!
removeRepositoryFromPackagesMatching

	| result |
	result := FillInTheBlank request: 'packages matching: ' initialAnswer: 'My*'.
	result size > 0 ifTrue: [
		self removeRepositoryFromPackagesMatching: result.
	]! !

!MCWorkingCopyBrowser methodsFor: 'actions' stamp: 'kph 5/17/2007 00:37'!
removeRepositoryFromPackagesMatching: aString

	self repository ifNotNilDo:
		[:repos |
			(self workingCopies select: [:each | aString match: each package name ]) 
				do: [ :wc | 
					Transcript cr; show: 'remove ', repos description, ' from ', wc package name.
					wc repositoryGroup removeRepository: repos]]! !

!MCWorkingCopyBrowser methodsFor: 'morphic ui' stamp: 'ab 8/7/2003 21:32'!
repositories
	^ self repositoryGroup repositories! !

!MCWorkingCopyBrowser methodsFor: 'actions' stamp: 'stephaneducasse 2/4/2006 20:47'!
repository
	workingCopy ifNotNil: [repository := self defaults at: workingCopy ifAbsent: []].
	^ repository! !

!MCWorkingCopyBrowser methodsFor: 'actions' stamp: 'stephaneducasse 2/4/2006 20:47'!
repository: aRepository
	repository := aRepository.
	workingCopy ifNotNil: [self defaults at: workingCopy put: aRepository]! !

!MCWorkingCopyBrowser methodsFor: 'morphic ui' stamp: 'ab 8/7/2003 21:32'!
repositoryGroup
	^ workingCopy
		ifNil: [MCRepositoryGroup default]
		ifNotNil: [workingCopy repositoryGroup]! !

!MCWorkingCopyBrowser methodsFor: 'morphic ui' stamp: 'kph 5/16/2007 22:48'!
repositoryList
	^ self repositories collect: [:ea | ea descriptionDisplay]! !

!MCWorkingCopyBrowser methodsFor: 'morphic ui' stamp: 'kph 5/17/2007 17:14'!
repositoryListMenu: aMenu
	self repository ifNil: [^ aMenu].
	self fillMenu: aMenu fromSpecs:
		#(('open repository' #openRepository)
		    ('edit repository info' #editRepository)
		   ('add to package...' #addRepositoryToPackage)
		   ('add to packages matching...' #addRepositoryToPackagesMatching)
		   ('remove from packages matching...' #removeRepositoryFromPackagesMatching)
		   ('show feed' #showFeed)
		   ('remove repository' #removeRepository)	
		   ('load repositories' #loadRepositories)	
		   ('save repositories' #saveRepositories)
		   ('flush cached versions' #flushAllCaches)	
		   ('flush passwords' #flushPasswords)	
		   ('open dual changes browser' #openDual)	
		).
		aMenu
		add: (self repository alwaysStoreDiffs
					ifTrue: ['store full versions']
					ifFalse: ['store diffs'])
		target: self
		selector: #toggleDiffs.
	^ aMenu
				! !

!MCWorkingCopyBrowser methodsFor: 'morphic ui' stamp: 'ab 7/22/2003 02:22'!
repositorySelection
	^ self repositories indexOf: self repository! !

!MCWorkingCopyBrowser methodsFor: 'morphic ui' stamp: 'bf 6/21/2005 15:56'!
repositorySelection: aNumber
	aNumber = 0
		ifTrue: [self repository: nil]
		ifFalse: [self repository: (self repositories at: aNumber)].
	self changed: #repositorySelection.
	self changedButtons.
! !

!MCWorkingCopyBrowser methodsFor: 'actions' stamp: 'bf 5/11/2005 15:20'!
revertPackage
	self pickAncestorVersionInfo ifNotNilDo: [:info |
		(self repositoryGroup versionWithInfo: info
			ifNone: [^self inform: 'No repository found for ', info name]
		) load]! !

!MCWorkingCopyBrowser methodsFor: 'actions' stamp: 'avi 9/10/2004 17:46'!
saveRepositories
	| f |
	f := FileStream forceNewFileNamed: 'MCRepositories.st'.
	MCRepositoryGroup default repositoriesDo: [:r |
		f nextPutAll: 'MCRepositoryGroup default addRepository: (', r asCreationTemplate, ')!!'; cr.]! !

!MCWorkingCopyBrowser methodsFor: 'actions' stamp: 'mtf 9/26/2008 11:34'!
saveVersion

	| repo |
	
	self canSave ifFalse: [^self].
	self checkForNewerVersions ifFalse: [^self].
	repo := self repository.
	workingCopy newVersion ifNotNilDo: [:v |
		'Saving package' displayProgressAt: Sensor cursorPoint
		from: 0 to:100
		during:[:bar|
			bar value: 10.
			repo storeVersion: v.
			bar value: 80.
			
			MCCacheRepository default cacheAllFileNamesDuring: 
			[ repo cacheAllFileNamesDuring: [
				v allAvailableDependenciesDo:
					[:dep |
					(repo includesVersionNamed: dep info name)
						ifFalse: [repo storeVersion: dep]].
			]].
			bar value: 100.
		]
	]
! !

!MCWorkingCopyBrowser methodsFor: 'actions' stamp: 'bf 1/27/2006 18:31'!
showFeed
	| parser xml text |
	parser := Smalltalk at: #XMLDOMParser
		ifAbsent: [^self inform: 'Yaxo not installed'].
	Cursor wait showWhile: [
		xml := self repository readStreamForFileNamed: 'feed.rss'
			do: [:stream | parser parseDocumentFrom: stream ]].
	xml ifNil: [^self inform: 'feed.rss not found'].
	text := Text streamContents: [:stream |
		xml tagsNamed: #item do: [:item |
			#(title bold author italic pubDate normal description normal) pairsDo: [:tag :attr |
				stream withAttribute: (TextEmphasis perform: attr) do: [
					item tagsNamed: tag do: [:element | 
						stream nextPutAll: (element contentString
							copyReplaceAll: '<br/>' with: String cr) ; space]].
				attr = #normal ifTrue: [stream cr]]]].
	UIManager default edit: text label: 'Feed: ', self repository description! !

!MCWorkingCopyBrowser methodsFor: 'morphic ui' stamp: 'avi 8/31/2004 01:14'!
toggleDiffs
	self repository alwaysStoreDiffs
		ifTrue: [self repository doNotAlwaysStoreDiffs]
		ifFalse: [self repository doAlwaysStoreDiffs]! !

!MCWorkingCopyBrowser methodsFor: 'actions' stamp: 'avi 10/22/2005 19:34'!
trimAncestry
	self pickAncestorVersionInfo ifNotNilDo:
		[:ancestor |
		workingCopy ancestry trimAfterVersionInfo: ancestor]! !

!MCWorkingCopyBrowser methodsFor: 'actions' stamp: 'ar 2/14/2004 02:46'!
unloadPackage
	workingCopy unload.
	self workingCopySelection: 0.
	self workingCopyListChanged.! !

!MCWorkingCopyBrowser methodsFor: 'morphic ui' stamp: 'avi 9/10/2004 17:54'!
unsortedWorkingCopies
	^ MCWorkingCopy allManagers ! !

!MCWorkingCopyBrowser methodsFor: 'morphic ui' stamp: 'avi 9/10/2004 17:54'!
update: aSymbol
	self unsortedWorkingCopies do: [:ea | ea addDependent: self].
	self workingCopyListChanged.! !

!MCWorkingCopyBrowser methodsFor: 'actions' stamp: 'mtf 6/9/2008 20:58'!
viewChanges
	| patch |
	'Finding changes' displayProgressAt: Sensor cursorPoint from: 0 to: 10 during:[:bar|
		self canSave ifTrue:[
		bar value: 1.
		patch := workingCopy changesRelativeToRepository: self repository].
		patch isNil ifTrue: [^ self].
		bar value:3.
		patch isEmpty
			ifTrue: [ workingCopy modified: false.
				bar value: 10.
				self inform: 'No changes' ]
			ifFalse:
				[ workingCopy modified: true.
				bar value: 5.
				(MCPatchBrowser forPatch: patch)
					label: 'Patch Browser: ', workingCopy description;
					show]]! !

!MCWorkingCopyBrowser methodsFor: 'actions' stamp: 'avi 2/13/2004 01:13'!
viewHistory
	workingCopy ifNotNil:
		[(MCWorkingHistoryBrowser new
				ancestry: workingCopy ancestry;
				package: workingCopy package)
			label:  'Version History: ',  workingCopy packageName;
			show]! !

!MCWorkingCopyBrowser methodsFor: 'actions' stamp: 'kph 4/24/2008 04:33'!
viewMemoryUse

	| total sizeCalc classes  |
	
	total := 0.        
	sizeCalc := [ :roots | (ImageSegment new 
							copyFromRoots: 
								(roots reject: [:aRoot | 
										aRoot isNil or: 
										[ aRoot indexIfCompact > 0]]) 
								sizeHint: 100000) segment byteSize ].          
     
     classes :=         
        ((workingCopy packageInfo classes collect: [ :cls |
                | size  |
 
			size := sizeCalc value: ((cls classPool values asOrderedCollection) add: cls ; add: cls class; yourself).
                    
               	
        		total := total + size.

			cls name -> size.
             
        ]) asSortedCollection: [ :a :b | b value <= a value ]) asOrderedCollection.

        classes addFirst: 'Total' -> total. 
        
        classes explore.! !

!MCWorkingCopyBrowser methodsFor: 'actions' stamp: 'kph 4/24/2008 04:23'!
viewMemoryUseAll

	| total sizeCalc packages globals  |
	
	total := 0.        
	sizeCalc := [ :roots | (ImageSegment new 
							copyFromRoots: 
								(roots reject: [:aRoot | 
										aRoot isNil or: 
										[ aRoot indexIfCompact > 0]]) 
								sizeHint: 100000) segment byteSize ].          
     
     packages :=         
        ((MCWorkingCopy allManagers collect: [ :each |
                | size  |
 
			size := sizeCalc value:			
                 
                        (each packageInfo classes in: [ :cls |
                        (cls , (cls collect: [ :c | c class ]), (cls gather: [ :c | c classPool ]))
                         ]).
               	
        		total := total + size.

		     each package name -> size.
             
        ]) asSortedCollection: [ :a :b | b value <= a value ]) asOrderedCollection.


	   globals := Smalltalk globals collect: [ :assoc |
		
		| size | 
		
		size := (sizeCalc value: (Array with: assoc value)).

		total := total + size.

		assoc key -> size.
		
		].
	
	   packages addFirst: (#Globals -> globals).

        packages addFirst: 'Total' -> total. 
        
        packages explore.! !

!MCWorkingCopyBrowser methodsFor: 'morphic ui' stamp: 'SR 9/9/2006 19:50'!
widgetSpecs
	^ #(
		((buttonRow) (0 0 1 0) (0 0 0 30))
		((treeOrListMorph: workingCopy) (0 0 0.5 1) (0 30 0 0))
		((listMorph: repository) (0.5 0 1 1) (0 30 0 0))
		)! !

!MCWorkingCopyBrowser methodsFor: 'morphic ui' stamp: 'mtf 7/21/2008 18:41'!
workingCopies
"List the working copies with the changed packages at the top"

	| c |
	c := MCWorkingCopy allManagers asSortedCollection:
		[ :a :b | a package name <= b package name ].
	^ (c select: [:mc| mc needsSaving]) asOrderedCollection, (c reject: [:mc| mc needsSaving]) asOrderedCollection.
! !

!MCWorkingCopyBrowser methodsFor: 'morphic ui' stamp: 'stephaneducasse 2/4/2006 20:47'!
workingCopy: wc
	workingCopy := wc.
	self changed: #workingCopyList; changed: #workingCopySelection; changed: #repositoryList.
	self changedButtons.
! !

!MCWorkingCopyBrowser methodsFor: 'morphic ui' stamp: 'avi 1/19/2004 16:41'!
workingCopyList
	^ self workingCopies collect:
		[:ea |
		(workingCopy notNil and: [workingCopy requiredPackages includes: ea package])
			ifTrue: [Text string: ea description emphasis: (Array with: TextEmphasis bold)]
			ifFalse: [ea description]]! !

!MCWorkingCopyBrowser methodsFor: 'morphic ui' stamp: 'ar 2/23/2006 18:30'!
workingCopyListChanged
	self changed: #workingCopyList.
	self changed: #workingCopyTree.
	self changed: #workingCopyTreeRoots.
	self changedButtons.
! !

!MCWorkingCopyBrowser methodsFor: 'morphic ui' stamp: 'mtf 10/7/2008 17:20'!
workingCopyListMenu: aMenu
	workingCopy ifNil: [^ aMenu].
	self fillMenu: aMenu fromSpecs:
		#(('add required package' #addRequiredPackage)
			('add required all dirty packages' #addRequiredDirtyPackage)
			('clear required packages' #clearRequiredPackages)
			('browse package' #browseWorkingCopy)
			('view changes' #viewChanges)
			('view history' #viewHistory)
			('recompile package' #recompilePackage)
			('recompile all packages' #recompileAllPackages)		
			('memory use for all packages' #viewMemoryUseAll)		
			('memory use' #viewMemoryUse)	
			('revert package...' #revertPackage)
			('unload package' #unloadPackage)
			('delete working copy' #deleteWorkingCopy)).
	(Smalltalk includesKey: #SARMCPackageDumper) ifTrue: [
		aMenu add: 'make SAR' target: self selector: #fileOutAsSAR
	].
	^aMenu! !

!MCWorkingCopyBrowser methodsFor: 'morphic ui' stamp: 'ab 7/19/2003 23:11'!
workingCopyListMorph
	^ PluggableMultiColumnListMorph
		on: self
		list: #workingCopyList
		selected: #workingCopySelection
		changeSelected: #workingCopySelection:
		menu: #workingCopyListMenu:! !

!MCWorkingCopyBrowser methodsFor: 'morphic ui' stamp: 'ab 7/19/2003 22:05'!
workingCopySelection
	^ self workingCopies indexOf: workingCopy! !

!MCWorkingCopyBrowser methodsFor: 'morphic ui' stamp: 'ar 2/14/2004 02:48'!
workingCopySelection: aNumber
	self workingCopy: 
		(aNumber = 0 
			ifTrue:[nil]
			ifFalse:[self workingCopies at: aNumber]).	! !

!MCWorkingCopyBrowser methodsFor: 'morphic ui' stamp: 'ar 2/14/2004 02:38'!
workingCopySelectionWrapper
	^workingCopyWrapper! !

!MCWorkingCopyBrowser methodsFor: 'morphic ui' stamp: 'ar 2/14/2004 02:57'!
workingCopySelectionWrapper: aWrapper
	workingCopyWrapper := aWrapper.
	self changed: #workingCopySelectionWrapper.
	self workingCopy: (aWrapper ifNotNil:[aWrapper item])! !

!MCWorkingCopyBrowser methodsFor: 'morphic ui' stamp: 'ar 2/14/2004 02:30'!
workingCopyTree
	^ self workingCopies collect:[:each| MCDependentsWrapper with: each model: self].! !

!MCWorkingCopyBrowser methodsFor: 'morphic ui' stamp: 'ar 2/23/2006 18:10'!
workingCopyTreeChildrenOf: aWorkingCopy
	| workingCopies |
	workingCopies := self unsortedWorkingCopies.
	^aWorkingCopy requiredPackages collect:[:each | 
			workingCopies detect: [:wc | wc package = each] ifNone: [nil]]
			thenSelect: [:x | x notNil].! !

!MCWorkingCopyBrowser methodsFor: 'morphic ui' stamp: 'ar 2/23/2006 18:15'!
workingCopyTreeLabelOf: aWorkingCopy
	^aWorkingCopy description! !

!MCWorkingCopyBrowser methodsFor: 'morphic ui' stamp: 'kph 5/13/2007 01:27'!
workingCopyTreeMenu: aMenu

	workingCopy ifNotNilDo: [ :wc | wc myMenu: aMenu inBrowser: self  ].
	
	^ aMenu! !

!MCWorkingCopyBrowser methodsFor: 'morphic ui' stamp: 'ar 2/14/2004 02:24'!
workingCopyTreeMorph
	^ SimpleHierarchicalListMorph
		on: self
		list: #workingCopyTree
		selected: #workingCopyWrapper
		changeSelected: #workingCopyWrapper:
		menu: #workingCopyListMenu:! !

!MCWorkingCopyBrowser methodsFor: 'morphic ui' stamp: 'ar 2/23/2006 18:18'!
workingCopyTreeRoots
	^self workingCopies! !

Object subclass: #MCToolWindowBuilder
	instanceVariableNames: 'builder window currentFrame tool'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-UI'!

!MCToolWindowBuilder class methodsFor: 'as yet unclassified' stamp: 'avi 2/11/2005 12:02'!
builder: aBuilder tool: aTool
	^ self basicNew initializeWithBuilder: aBuilder tool: aTool! !

!MCToolWindowBuilder methodsFor: 'as yet unclassified' stamp: 'avi 2/11/2005 12:03'!
build
	^ builder build: window! !

!MCToolWindowBuilder methodsFor: 'as yet unclassified' stamp: 'avi 2/11/2005 12:04'!
buttonRow
	^ self buttonRow: tool buttonSpecs! !

!MCToolWindowBuilder methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
buttonRow: specArray
	| panel button |
	panel := builder pluggablePanelSpec new.
	panel children: OrderedCollection new.
	specArray do:
		[:spec |
		
		button := builder pluggableButtonSpec new.
		button model: tool.
		button label: spec first asString.
		button action: spec second.
		button help: spec third.
		button enabled: (spec at: 4 ifAbsent: [#buttonEnabled]).
		button state: (spec at: 5 ifAbsent: [#buttonSelected]).
		panel children add: button].
	panel layout: #horizontal.
	panel frame: currentFrame.
	window children add: panel! !

!MCToolWindowBuilder methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
frame: aLayoutFrame
	currentFrame := aLayoutFrame! !

!MCToolWindowBuilder methodsFor: 'as yet unclassified' stamp: 'kph 5/18/2007 12:46'!
initializeWithBuilder: aBuilder tool: aTool
	builder := aBuilder.
	tool := aTool.! !

!MCToolWindowBuilder methodsFor: 'as yet unclassified' stamp: 'avi 2/11/2005 12:04'!
listMorph: listSymbol
	^ self
		listMorph: (listSymbol, 'List') asSymbol
		selection: (listSymbol, 'Selection') asSymbol
		menu: (listSymbol, 'ListMenu:') asSymbol! !

!MCToolWindowBuilder methodsFor: 'as yet unclassified' stamp: 'avi 2/11/2005 12:04'!
listMorph: listSymbol keystroke: keystrokeSymbol
	^ (self
		listMorph: (listSymbol, 'List') asSymbol
		selection: (listSymbol, 'Selection') asSymbol
		menu: (listSymbol, 'ListMenu:') asSymbol)
		keystrokeActionSelector: keystrokeSymbol;
		yourself! !

!MCToolWindowBuilder methodsFor: 'as yet unclassified' stamp: 'avi 2/11/2005 12:15'!
listMorph: listSymbol selection: selectionSymbol
	self listMorph: listSymbol selection: selectionSymbol menu: nil! !

!MCToolWindowBuilder methodsFor: 'as yet unclassified' stamp: 'avi 2/11/2005 12:16'!
listMorph: listSymbol selection: selectionSymbol menu: menuSymbol
	self listMorph: listSymbol selection: selectionSymbol menu: menuSymbol keystroke: nil! !

!MCToolWindowBuilder methodsFor: 'as yet unclassified' stamp: 'avi 2/13/2005 17:51'!
listMorph: listSymbol selection: selectionSymbol menu: menuSymbol keystroke: keystrokeSymbol
	| list |
	list := builder pluggableListSpec new.
	list 
		model: tool;
		list: listSymbol; 
		getIndex: selectionSymbol; 
		setIndex: (selectionSymbol, ':') asSymbol;
		frame: currentFrame.
	menuSymbol ifNotNil: [list menu: menuSymbol].
	keystrokeSymbol ifNotNil: [list keyPress: keystrokeSymbol].
	window children add: list
! !

!MCToolWindowBuilder methodsFor: 'as yet unclassified' stamp: 'avi 2/13/2005 17:52'!
multiListMorph: listSymbol selection: selectionSymbol listSelection: listSelectionSymbol menu: menuSymbol
	| list |
	list := builder pluggableMultiSelectionListSpec new.
	list 
		model: tool;
		list: listSymbol; 
		getIndex: selectionSymbol; 
		setIndex: (selectionSymbol, ':') asSymbol;
		getSelectionList: listSelectionSymbol;
		setSelectionList: (listSelectionSymbol, 'put:') asSymbol;
		frame: currentFrame.
	menuSymbol ifNotNil: [list menu: menuSymbol].
	window children add: list
! !

!MCToolWindowBuilder methodsFor: 'as yet unclassified' stamp: 'kph 5/21/2007 13:01'!
panel: getChildrenSelector 
	| panel |
	panel := builder pluggablePanelSpec new.
	panel children: getChildrenSelector.
	panel model: tool.
	panel frame: currentFrame.
	window children add: panel! !

!MCToolWindowBuilder methodsFor: 'as yet unclassified' stamp: 'kph 5/18/2007 12:58'!
startWithPanel
	window := builder pluggablePanelSpec new.
	window children: OrderedCollection new.
	window model: tool.
! !

!MCToolWindowBuilder methodsFor: 'as yet unclassified' stamp: 'kph 5/18/2007 12:47'!
startWithWindow 
	window := builder pluggableWindowSpec new.
	window children: OrderedCollection new.
	window label: tool label asString.
	window model: tool.
	window extent: tool defaultExtent.! !

!MCToolWindowBuilder methodsFor: 'as yet unclassified' stamp: 'avi 2/13/2005 17:52'!
textMorph: aSymbol
	| text |
	text := builder pluggableTextSpec new.
	text 
		model: tool;
		getText: aSymbol; 
		setText: (aSymbol, ':') asSymbol;
		frame: currentFrame.
	window children add: text! !

!MCToolWindowBuilder methodsFor: 'as yet unclassified' stamp: 'ar 2/23/2006 18:24'!
treeMorph: listSymbol
	^ self
		treeMorph: listSymbol
		selection: listSymbol
		menu: (listSymbol, 'TreeMenu:') asSymbol! !

!MCToolWindowBuilder methodsFor: 'as yet unclassified' stamp: 'ar 2/23/2006 17:46'!
treeMorph: listSymbol selection: selectionSymbol menu: menuSymbol
	^self treeMorph: listSymbol selection: selectionSymbol menu: menuSymbol keystroke: nil! !

!MCToolWindowBuilder methodsFor: 'as yet unclassified' stamp: 'ar 2/23/2006 18:25'!
treeMorph: treeSymbol selection: selectionSymbol menu: menuSymbol keystroke: keystrokeSymbol
	| tree |
	tree := builder pluggableTreeSpec new.
	tree 
		model: tool;
		roots: (treeSymbol, 'TreeRoots') asSymbol;
		setSelected: (selectionSymbol, ':') asSymbol;
		getChildren: (treeSymbol,'TreeChildrenOf:') asSymbol;
		label: (treeSymbol,'TreeLabelOf:') asSymbol;
		frame: currentFrame.
	menuSymbol ifNotNil: [tree menu: menuSymbol].
	keystrokeSymbol ifNotNil: [tree keyPress: keystrokeSymbol].
	window children add: tree
! !

!MCToolWindowBuilder methodsFor: 'as yet unclassified' stamp: 'ar 2/23/2006 17:46'!
treeOrListMorph: listSymbol
	^self treeMorph: listSymbol! !

Object subclass: #MCVariableDefinition
	instanceVariableNames: 'name'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Modeling'!

MCVariableDefinition subclass: #MCClassInstanceVariableDefinition
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Modeling'!

!MCClassInstanceVariableDefinition class methodsFor: 'as yet unclassified' stamp: 'cwp 7/7/2003 22:59'!
type
	^ #classInstance! !

!MCClassInstanceVariableDefinition methodsFor: 'as yet unclassified' stamp: 'cwp 7/7/2003 23:31'!
isClassInstanceVariable
	^ true! !

MCVariableDefinition subclass: #MCClassVariableDefinition
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Modeling'!

!MCClassVariableDefinition class methodsFor: 'as yet unclassified' stamp: 'cwp 7/7/2003 22:58'!
type
	^ #class! !

!MCClassVariableDefinition methodsFor: 'as yet unclassified' stamp: 'cwp 7/7/2003 23:32'!
isClassVariable
	^ true! !

!MCClassVariableDefinition methodsFor: 'as yet unclassified' stamp: 'bf 8/29/2006 11:41'!
isOrderDependend
	^false! !

MCVariableDefinition subclass: #MCInstanceVariableDefinition
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Modeling'!

!MCInstanceVariableDefinition class methodsFor: 'as yet unclassified' stamp: 'cwp 7/7/2003 22:59'!
type
	^ #instance! !

!MCInstanceVariableDefinition methodsFor: 'as yet unclassified' stamp: 'cwp 7/7/2003 23:32'!
isInstanceVariable
	^ true! !

MCVariableDefinition subclass: #MCPoolImportDefinition
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Modeling'!

!MCPoolImportDefinition class methodsFor: 'as yet unclassified' stamp: 'cwp 7/7/2003 22:59'!
type
	^ #pool! !

!MCPoolImportDefinition methodsFor: 'testing' stamp: 'bf 8/29/2006 11:41'!
isOrderDependend
	^false! !

!MCPoolImportDefinition methodsFor: 'testing' stamp: 'cwp 7/7/2003 23:51'!
isPoolImport
	^ true! !

!MCVariableDefinition class methodsFor: 'as yet unclassified' stamp: 'cwp 7/7/2003 23:18'!
name: aString
	^ self new name: aString
	! !

!MCVariableDefinition methodsFor: 'comparing' stamp: 'cwp 7/7/2003 23:02'!
= other
	^ (self species = other species)
		and: [self name = other name]! !

!MCVariableDefinition methodsFor: 'comparing' stamp: 'cwp 7/7/2003 23:02'!
hash
	^ name hash! !

!MCVariableDefinition methodsFor: 'testing' stamp: 'cwp 7/7/2003 23:31'!
isClassInstanceVariable
	^ false! !

!MCVariableDefinition methodsFor: 'testing' stamp: 'cwp 7/7/2003 23:08'!
isClassInstanceVariableDefinition
	^ false! !

!MCVariableDefinition methodsFor: 'testing' stamp: 'cwp 7/7/2003 23:30'!
isClassVariable
	^ false! !

!MCVariableDefinition methodsFor: 'testing' stamp: 'cwp 7/7/2003 23:31'!
isInstanceVariable
	^ false! !

!MCVariableDefinition methodsFor: 'testing' stamp: 'cwp 7/7/2003 23:10'!
isInstanceVariableDefinition
	^ false! !

!MCVariableDefinition methodsFor: 'testing' stamp: 'bf 8/29/2006 11:41'!
isOrderDependend
	^true! !

!MCVariableDefinition methodsFor: 'testing' stamp: 'cwp 7/7/2003 23:51'!
isPoolImport
	^ false! !

!MCVariableDefinition methodsFor: 'accessing' stamp: 'cwp 11/25/2002 05:57'!
name
	^ name! !

!MCVariableDefinition methodsFor: 'accessing' stamp: 'stephaneducasse 2/4/2006 20:47'!
name: aString
	name := aString! !

!MCVariableDefinition methodsFor: 'as yet unclassified' stamp: 'nk 7/24/2003 14:56'!
printOn: aStream
	super printOn: aStream.
	aStream nextPut: $(; nextPutAll: self name; nextPut: $)! !

Object subclass: #MCVersion
	instanceVariableNames: 'package info snapshot dependencies'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Versioning'!

MCVersion subclass: #MCDiffyVersion
	instanceVariableNames: 'base patch'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Versioning'!

!MCDiffyVersion class methodsFor: 'name utilities' stamp: 'bf 5/30/2005 18:45'!
baseNameFrom: diffName
	| baseId verName |
	baseId := (diffName copyAfter: $() copyUpTo: $).
	baseId ifEmpty: [^baseId].
	(baseId beginsWith: '@')
		ifTrue: [^baseId copyAfter: $@].
	verName := self verNameFrom: diffName.
	^(baseId includes: $.)
		ifTrue: [(verName copyUpToLast: $-), '-', baseId]
		ifFalse: [(verName copyUpToLast: $.), '.', baseId]
! !

!MCDiffyVersion class methodsFor: 'name utilities' stamp: 'bf 5/30/2005 18:58'!
canonicalNameFor: aFileName
	^(self nameForVer: (self verNameFrom: aFileName)
		base: (self baseNameFrom: aFileName))
			, '.', MCMcdReader extension
! !

!MCDiffyVersion class methodsFor: 'name utilities' stamp: 'bf 5/30/2005 17:39'!
nameForVer: versionName base: baseName
	| baseId |
	baseId := (versionName copyUpToLast: $.) = (baseName copyUpToLast: $.)
		ifTrue: [baseName copyAfterLast: $.]
		ifFalse: [(versionName copyUpToLast: $-) = (baseName copyUpToLast: $-)
			ifTrue: [baseName copyAfterLast: $-]
			ifFalse: ['@', baseName]].
	^ versionName, '(', baseId, ')'! !

!MCDiffyVersion class methodsFor: 'instance creation' stamp: 'avi 2/13/2004 23:07'!
package: aPackage info: aVersionInfo dependencies: aCollection baseInfo: baseVersionInfo patch:
aPatch
	^ self basicNew initializeWithPackage: aPackage info: aVersionInfo dependencies: aCollection baseInfo: baseVersionInfo patch:
aPatch! !

!MCDiffyVersion class methodsFor: 'instance creation' stamp: 'avi 2/13/2004 23:06'!
package: aPackage info: aVersionInfo snapshot: aSnapshot dependencies: aCollection baseVersion: aVersion
	^ self 
		package: aPackage
		info: aVersionInfo
		dependencies: aCollection
		baseInfo: aVersion info
		patch: (aSnapshot patchRelativeToBase: aVersion snapshot)! !

!MCDiffyVersion class methodsFor: 'name utilities' stamp: 'bf 5/30/2005 18:19'!
verNameFrom: diffName
	^diffName copyUpTo: $(! !

!MCDiffyVersion methodsFor: 'as yet unclassified' stamp: 'avi 2/13/2004 23:17'!
baseInfo
	^ base! !

!MCDiffyVersion methodsFor: 'as yet unclassified' stamp: 'avi 2/13/2004 23:39'!
baseSnapshot
	^ (self workingCopy repositoryGroup versionWithInfo: base) snapshot! !

!MCDiffyVersion methodsFor: 'as yet unclassified' stamp: 'bf 5/23/2005 15:42'!
canOptimizeLoading
	"Answer wether I can provide a patch for the working copy without the usual diff pass"
	^ package hasWorkingCopy
		and: [package workingCopy modified not
			and: [package workingCopy ancestors includes: self baseInfo]]! !

!MCDiffyVersion methodsFor: 'as yet unclassified' stamp: 'bf 5/30/2005 17:39'!
fileName
	^ (self class nameForVer: info name base: base name), '.', self writerClass extension! !

!MCDiffyVersion methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
initializeWithPackage: aPackage info: aVersionInfo dependencies: aCollection baseInfo: baseVersionInfo patch: aPatch
	patch := aPatch.
	base := baseVersionInfo.
	super initializeWithPackage: aPackage info: aVersionInfo snapshot: nil dependencies: aCollection.
! !

!MCDiffyVersion methodsFor: 'as yet unclassified' stamp: 'avi 2/13/2004 23:24'!
isDiffy
	^ true! !

!MCDiffyVersion methodsFor: 'as yet unclassified' stamp: 'avi 2/13/2004 23:17'!
patch
	^ patch! !

!MCDiffyVersion methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
snapshot
	^ snapshot ifNil: [snapshot := MCPatcher apply: patch to: self baseSnapshot]! !

!MCDiffyVersion methodsFor: 'as yet unclassified' stamp: 'avi 2/19/2004 22:03'!
summary
	^ '(Diff against ', self baseInfo name, ')', String cr, super summary! !

!MCDiffyVersion methodsFor: 'as yet unclassified' stamp: 'avi 2/13/2004 23:17'!
writerClass
	^ MCMcdWriter ! !

!MCVersion class methodsFor: 'instance creation' stamp: 'ab 7/7/2003 16:13'!
package: aPackage
	^ self package: aPackage info: MCVersionInfo new! !

!MCVersion class methodsFor: 'instance creation' stamp: 'ab 7/7/2003 16:13'!
package: aPackage info: aVersionInfo
	^ self package: aPackage info: aVersionInfo snapshot: aPackage snapshot! !

!MCVersion class methodsFor: 'instance creation' stamp: 'cwp 11/7/2004 13:02'!
package: aPackage info: aVersionInfo snapshot: aSnapshot
	^ self package: aPackage info: aVersionInfo snapshot: aSnapshot dependencies: #()! !

!MCVersion class methodsFor: 'instance creation' stamp: 'avi 1/19/2004 13:11'!
package: aPackage info: aVersionInfo snapshot: aSnapshot dependencies: aCollection
	^ self new initializeWithPackage: aPackage info: aVersionInfo snapshot: aSnapshot dependencies: aCollection! !

!MCVersion methodsFor: 'actions' stamp: 'kph 5/16/2007 11:44'!
addToCache
	| repository |
	MCCacheRepository default storeVersion: self.
	 ! !

!MCVersion methodsFor: 'actions' stamp: 'avi 2/12/2004 19:37'!
adopt
	self workingCopy adopt: self! !

!MCVersion methodsFor: 'enumerating' stamp: 'stephaneducasse 2/4/2006 20:47'!
allAvailableDependenciesDo: aBlock
	| version |
	self dependencies do:
		[:ea |
		[version := ea resolve.
		version allAvailableDependenciesDo: aBlock.
		aBlock value: version]
			on: Error do: []]! !

!MCVersion methodsFor: 'enumerating' stamp: 'cwp 11/7/2004 11:58'!
allDependenciesDo: aBlock
	self allDependenciesDo: aBlock ifUnresolved: [:ignored | true]! !

!MCVersion methodsFor: 'enumerating' stamp: 'stephaneducasse 2/4/2006 20:47'!
allDependenciesDo: aBlock ifUnresolved: failBlock
	| dict |
	dict := Dictionary new.
	self allDependenciesNotIn: dict do: aBlock ifUnresolved: failBlock! !

!MCVersion methodsFor: 'enumerating' stamp: 'cwp 11/7/2004 14:24'!
allDependenciesNotIn: aDictionary do: aBlock ifUnresolved: failBlock
	| version |
	self dependencies do: 
		[:ea | 
		version := aDictionary at: ea ifAbsent: [ea resolve].
		version 
			ifNil: [failBlock value: ea]
			ifNotNil: [(aDictionary includes: version) ifFalse:
						[aDictionary at: ea put: version.
						version 
							allDependenciesNotIn: aDictionary 
							do: aBlock
							ifUnresolved: failBlock.
						aBlock value: version]]]! !

!MCVersion methodsFor: 'converting' stamp: 'avi 2/19/2004 21:00'!
asDiffAgainst: aVersion
	aVersion info = self info ifTrue: [self error: 'Cannot diff against self!!'].
	^ MCDiffyVersion
		package: self package
		info: self info
		snapshot: self snapshot
		dependencies: self dependencies
		baseVersion: aVersion! !

!MCVersion methodsFor: 'actions' stamp: 'bf 3/22/2005 22:12'!
browse
	(MCSnapshotBrowser forSnapshot: self snapshot)
		showLabelled: 'Snapshot of ', self fileName! !

!MCVersion methodsFor: '' stamp: 'kph 8/1/2007 12:25'!
browseFrom: repository
	 
	self browse! !

!MCVersion methodsFor: 'testing' stamp: 'bf 5/23/2005 15:43'!
canOptimizeLoading
	"Answer wether I can provide a patch for the working copy without the usual diff pass"
	^false! !

!MCVersion methodsFor: 'accessing' stamp: 'mtf 10/7/2008 13:29'!
changes
	^ self snapshot patchRelativeToBase: package patchSnapshot! !

!MCVersion methodsFor: 'accessing' stamp: 'avi 1/22/2004 00:24'!
dependencies
	^ dependencies ifNil: [#()]! !

!MCVersion methodsFor: 'printing' stamp: 'kph 5/6/2007 03:05'!
description
	^ self info name 
	 ! !

!MCVersion methodsFor: 'accessing' stamp: 'avi 1/22/2004 12:44'!
fileName
	^ info name, '.', self writerClass extension! !

!MCVersion methodsFor: 'actions' stamp: 'avi 1/22/2004 12:44'!
fileOutOn: aStream
	self writerClass fileOut: self on: aStream! !

!MCVersion methodsFor: 'accessing' stamp: 'ab 7/7/2003 14:28'!
info
	^ info! !

!MCVersion methodsFor: 'initialize-release' stamp: 'stephaneducasse 2/4/2006 20:47'!
initializeWithPackage: aPackage info: aVersionInfo snapshot: aSnapshot dependencies: aCollection
	package := aPackage.
	info := aVersionInfo.
	snapshot := aSnapshot.
	dependencies := aCollection.
	self addToCache.! !

!MCVersion methodsFor: 'actions' stamp: 'kph 5/9/2007 18:41'!
install

	^ self load! !

!MCVersion methodsFor: 'testing' stamp: 'bf 3/22/2005 23:00'!
isCacheable
	^true! !

!MCVersion methodsFor: 'testing' stamp: 'avi 2/13/2004 23:24'!
isDiffy
	^ false! !

!MCVersion methodsFor: 'actions' stamp: 'avi 1/24/2004 20:13'!
load
	MCVersionLoader loadVersion: self! !

!MCVersion methodsFor: '' stamp: 'kph 8/1/2007 12:06'!
loadFrom: repository

	self isCacheable
			ifTrue: [ self workingCopy repositoryGroup addRepository: repository].
	 
	self load.! !

!MCVersion methodsFor: 'actions' stamp: 'abc 2/13/2004 15:58'!
merge
	MCVersionMerger mergeVersion: self! !

!MCVersion methodsFor: '' stamp: 'kph 8/1/2007 12:53'!
mergeFrom: repository

	self isCacheable
			ifTrue: [ self workingCopy repositoryGroup addRepository: repository].
	 
	self merge.! !

!MCVersion methodsFor: 'actions' stamp: 'kph 5/13/2007 16:48'!
open
	^ (MCVersionInspector new version: self) show! !

!MCVersion methodsFor: 'accessing' stamp: 'ab 7/7/2003 14:19'!
package
	^ package! !

!MCVersion methodsFor: 'printing' stamp: 'kph 5/6/2007 03:06'!
printOn: aStream
	super printOn: aStream.
	aStream nextPut: $(.
	aStream nextPutAll: self description.
	aStream nextPut: $).! !

!MCVersion methodsFor: 'initialize-release' stamp: 'stephaneducasse 2/4/2006 20:47'!
setPackage: aPackage info: aVersionInfo snapshot: aSnapshot dependencies: aCollection
	package := aPackage.
	info := aVersionInfo.
	snapshot := aSnapshot.
	dependencies := aCollection! !

!MCVersion methodsFor: 'accessing' stamp: 'ab 7/7/2003 14:10'!
snapshot
	^ snapshot! !

!MCVersion methodsFor: 'accessing' stamp: 'avi 1/20/2004 16:07'!
summary
	^ String streamContents:
		[:s |
		s nextPutAll: info summaryHeader.
		(dependencies isNil or: [dependencies isEmpty]) ifFalse:
			[s cr; nextPutAll: 'Dependencies: '.
			dependencies
				do: [:ea | s nextPutAll: ea versionInfo name]
				separatedBy: [s nextPutAll: ', ']].
		s cr; cr; nextPutAll: info message]! !

!MCVersion methodsFor: 'enumerating' stamp: 'cwp 11/7/2004 14:24'!
withAllDependenciesDo: aBlock
	self allDependenciesDo: aBlock ifUnresolved: [:ignored].
	aBlock value: self! !

!MCVersion methodsFor: 'enumerating' stamp: 'stephaneducasse 2/4/2006 20:47'!
withAllDependenciesDo: aBlock ifUnresolved: failBlock
	| dict |
	dict := Dictionary new.
	self allDependenciesNotIn: dict do: aBlock ifUnresolved: failBlock.
	aBlock value: self! !

!MCVersion methodsFor: 'accessing' stamp: 'avi 2/12/2004 19:38'!
workingCopy
	^ package workingCopy! !

!MCVersion methodsFor: 'accessing' stamp: 'avi 1/22/2004 12:44'!
writerClass
	^ MCMczWriter ! !

MCVersion subclass: #MCVersionVirtual
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Versioning'!

MCVersionVirtual subclass: #MCCSVersion
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Versioning'!

MCVersionVirtual subclass: #MCSystemChangeSetVersion
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Versioning'!

Object subclass: #MCVersionDependency
	instanceVariableNames: 'package versionInfo'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Versioning'!

!MCVersionDependency class methodsFor: 'as yet unclassified' stamp: 'avi 1/19/2004 13:13'!
package: aPackage info: aVersionInfo
	^ self basicNew initializeWithPackage: aPackage info: aVersionInfo! !

!MCVersionDependency methodsFor: 'comparing' stamp: 'avi 1/19/2004 16:12'!
= other
	^ other species = self species
		and: [other versionInfo = versionInfo
				and: [other package = package]]! !

!MCVersionDependency methodsFor: 'comparing' stamp: 'avi 1/19/2004 16:06'!
hash
	^ versionInfo hash! !

!MCVersionDependency methodsFor: 'initialize-release' stamp: 'stephaneducasse 2/4/2006 20:47'!
initializeWithPackage: aPackage info: aVersionInfo
	package := aPackage.
	versionInfo := aVersionInfo! !

!MCVersionDependency methodsFor: 'testing' stamp: 'bf 4/19/2005 16:29'!
isCurrent
	^ package hasWorkingCopy
		and: [self isFulfilled
			and: [package workingCopy modified not]]! !

!MCVersionDependency methodsFor: 'testing' stamp: 'bf 4/19/2005 16:29'!
isFulfilled
	^package hasWorkingCopy
		and: [self isFulfilledBy: package workingCopy ancestry]! !

!MCVersionDependency methodsFor: 'testing' stamp: 'avi 3/4/2004 00:34'!
isFulfilledBy: anAncestry
	^ anAncestry ancestors includes: versionInfo! !

!MCVersionDependency methodsFor: 'testing' stamp: 'bf 4/19/2005 16:29'!
isFulfilledByAncestors
	^ package hasWorkingCopy
		and: [self isFulfilledByAncestorsOf: package workingCopy ancestry]! !

!MCVersionDependency methodsFor: 'testing' stamp: 'nk 7/13/2004 08:45'!
isFulfilledByAncestorsOf: anAncestry
	^ anAncestry hasAncestor: versionInfo! !

!MCVersionDependency methodsFor: 'testing' stamp: 'bf 4/19/2005 16:29'!
isOlder
	"Answer true if I represent an older version of a package that is loaded."
	^ package hasWorkingCopy
		and: [self isFulfilled not
			and: [ self isFulfilledByAncestors
				and: [package workingCopy modified not]]]! !

!MCVersionDependency methodsFor: 'accessing' stamp: 'avi 1/19/2004 15:40'!
package
	^ package! !

!MCVersionDependency methodsFor: 'accessing' stamp: 'avi 2/12/2004 19:38'!
repositoryGroup
	^ self package workingCopy repositoryGroup! !

!MCVersionDependency methodsFor: 'resolving' stamp: 'nk 6/13/2004 19:21'!
resolve
	^ self repositoryGroup
		versionWithInfo: versionInfo
		ifNone: [ MCRepositoryGroup default versionWithInfo: versionInfo ifNone: []]! !

!MCVersionDependency methodsFor: 'accessing' stamp: 'avi 1/19/2004 15:40'!
versionInfo
	^ versionInfo! !

Object subclass: #MCVersionLoader
	instanceVariableNames: 'versions'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Loading'!

!MCVersionLoader class methodsFor: 'as yet unclassified' stamp: 'avi 1/24/2004 20:06'!
loadVersion: aVersion
	self new
		addVersion: aVersion;
		load! !

!MCVersionLoader methodsFor: 'loading' stamp: 'cwp 11/7/2004 17:06'!
addDependency: aDependency
	| dep |
	aDependency isCurrent ifTrue: [^ self].
	(self depAgeIsOk: aDependency) ifFalse: [^ self].
	dep := aDependency resolve.
	dep
		ifNil: [self confirmMissingDependency: aDependency]
		ifNotNil: [(versions includes: dep) ifFalse: [self addVersion: dep]]! !

!MCVersionLoader methodsFor: 'loading' stamp: 'cwp 11/7/2004 17:04'!
addVersion: aVersion
	aVersion dependencies do: [ :ea | self addDependency: ea].
	versions add: aVersion.
! !

!MCVersionLoader methodsFor: 'checking' stamp: 'stephaneducasse 2/4/2006 20:47'!
checkForModifications
	| modifications |
	modifications := versions select: [:ea | ea package workingCopy modified].
	modifications isEmpty ifFalse: [self warnAboutLosingChangesTo: modifications].! !

!MCVersionLoader methodsFor: 'checking' stamp: 'cwp 11/7/2004 17:00'!
checkIfDepIsOlder: aDependency
	^ aDependency isOlder not 
		or: [self confirm: 'load older dependency ', aDependency versionInfo name , '?']! !

!MCVersionLoader methodsFor: 'checking' stamp: 'cwp 11/7/2004 17:06'!
confirmMissingDependency: aDependency
	| name |
	name := aDependency versionInfo name.
	(self confirm: 'Can''t find dependency ', name, '. ignore?')
		ifFalse: [self error: 'Can''t find dependency ', name]! !

!MCVersionLoader methodsFor: 'checking' stamp: 'cwp 11/7/2004 17:02'!
depAgeIsOk: aDependency
	^ aDependency isOlder not 
		or: [self confirm: 'load older dependency ', aDependency versionInfo name , '?']! !

!MCVersionLoader methodsFor: 'initialize-release' stamp: 'stephaneducasse 2/4/2006 20:47'!
initialize
	versions := OrderedCollection new! !

!MCVersionLoader methodsFor: 'loading' stamp: 'bf 3/16/2006 19:03'!
load
	self loadWithNameLike: versions first info name.
! !

!MCVersionLoader methodsFor: 'loading' stamp: 'kph 6/12/2007 09:37'!
loadWithNameLike: aString
	| loader |
	self checkForModifications.
	loader := MCPackageLoader new.
	versions size > 1 ifTrue: [ loader beMultiplePackage ].
	 
	versions do: [:ea |
		ea canOptimizeLoading
			ifTrue: [ea patch applyTo: loader]
			ifFalse: [loader updatePackage: ea package withSnapshot: ea snapshot
			]
	].
	loader 	installOrphanage;
			loadWithNameLike: aString.
	versions do: [:ea | ea workingCopy loaded: ea]! !

!MCVersionLoader methodsFor: 'checking' stamp: 'avi 1/24/2004 20:17'!
warnAboutLosingChangesTo: versionCollection
	self notify: (String streamContents: [:s |
		s nextPutAll: 'You are about to load new versions of the following packages that have unsaved changes in the image.  If you continue, you will lose these changes.'; cr.
		versionCollection do:
			[:ea |
			s cr; space; space; nextPutAll: ea package name]])! !

Object subclass: #MCVersionMerger
	instanceVariableNames: 'records merger'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Versioning'!

!MCVersionMerger class methodsFor: 'as yet unclassified' stamp: 'bf 12/5/2004 12:35'!
mergeVersion: aVersion
	self new
		addVersion: aVersion;
		mergeWithNameLike: aVersion info name! !

!MCVersionMerger methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
addVersion: aVersion
	| dep |
	records add: (MCMergeRecord version: aVersion).
	aVersion dependencies do:
		[:ea |
		dep := ea resolve.
		(records anySatisfy: [:r | r version = dep]) ifFalse: [self addVersion: dep]]! !

!MCVersionMerger methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
initialize
	records := OrderedCollection new.
	merger := MCThreeWayMerger new.! !

!MCVersionMerger methodsFor: 'as yet unclassified' stamp: 'abc 2/13/2004 17:15'!
merge
	records do: [:ea | merger addBaseSnapshot: ea packageSnapshot].
	records do: [:ea | merger applyPatch: ea mergePatch].
	self resolveConflicts ifTrue:
		[merger load.
		records do: [:ea | ea updateWorkingCopy]].! !

!MCVersionMerger methodsFor: 'as yet unclassified' stamp: 'bf 12/5/2004 12:32'!
mergeWithNameLike: baseName
	records do: [:ea | merger addBaseSnapshot: ea packageSnapshot].
	records do: [:ea | merger applyPatch: ea mergePatch].
	self resolveConflicts ifTrue:
		[merger loadWithNameLike: baseName.
		records do: [:ea | ea updateWorkingCopy]].! !

!MCVersionMerger methodsFor: 'as yet unclassified' stamp: 'bf 4/26/2005 14:29'!
resolveConflicts
	(records allSatisfy: [:ea | ea isAncestorMerge]) ifTrue: [MCNoChangesException signal. ^ false].
	^ ((MCMergeResolutionRequest new merger: merger)
		signal: 'Merging ', records first version info name) = true! !

Object subclass: #MCVersionNotification
	instanceVariableNames: 'version ancestor repository changes'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Repositories'!

!MCVersionNotification class methodsFor: 'as yet unclassified' stamp: 'avi 8/26/2004 14:27'!
version: aVersion repository: aRepository
	^ self basicNew initializeWithVersion: aVersion repository: aRepository! !

!MCVersionNotification methodsFor: 'as yet unclassified' stamp: 'avi 8/26/2004 15:13'!
fromAddress
	^ 'monticello@beta4.com'! !

!MCVersionNotification methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
initializeWithVersion: aVersion repository: aRepository
	version := aVersion.
	repository := aRepository.
	ancestor := repository closestAncestorVersionFor: version info ifNone: []. 
	changes := ancestor
				ifNil: [#()]
				ifNotNil: [(version snapshot patchRelativeToBase: ancestor snapshot) 							operations asSortedCollection]! !

!MCVersionNotification methodsFor: 'as yet unclassified' stamp: 'avi 8/26/2004 15:12'!
messageText
	^ String streamContents:
		[:s |
		s nextPutAll: 'Committed to repository: ', repository description; cr; cr.
		s nextPutAll: version summary.
		changes isEmpty ifFalse:
			[s cr; cr.
			s nextPutAll: '-----------------------------------------------------'; cr.
			s nextPutAll: 'Changes since ', ancestor info name, ':'; cr.
			changes do:
			[:ea |
			s cr; nextPutAll: ea summary; cr.
			s nextPutAll: ea sourceString]]]! !

!MCVersionNotification methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
messageTo: aString
	| message |
	message := MailMessage empty.
	message setField: 'from' toString: self fromAddress.
	message setField: 'to' toString: aString.
	message setField: 'subject' toString: '[MC] ', version info name.
	message body: (MIMEDocument contentType: 'text/plain' content: self messageText).
	^ message! !

!MCVersionNotification methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
notify: aString
	| message |
	message := self messageTo: aString.
	SMTPClient
		deliverMailFrom: message from
		to: (Array with: message to)
		text: message text
		usingServer: MailSender smtpServer! !

Object subclass: #MCVersionSorter
	instanceVariableNames: 'layers depthIndex depths stepparents roots'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Versioning'!

!MCVersionSorter commentStamp: 'kph 5/16/2007 19:47' prior: 0!
For sorting versions.

MCFileRepositories use their own scheme for sorting versions.!

MCVersionSorter subclass: #MCFilteredVersionSorter
	instanceVariableNames: 'target'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Versioning'!

!MCFilteredVersionSorter methodsFor: 'as yet unclassified' stamp: 'bf 5/28/2005 01:14'!
addVersionInfo: aVersionInfo
	(aVersionInfo hasAncestor: target)
		ifTrue: [super addVersionInfo: aVersionInfo]
! !

!MCFilteredVersionSorter methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
processVersionInfo: aVersionInfo
	| success |
	aVersionInfo = target ifTrue: [^ true].
	self pushLayer.
	success := (self knownAncestorsOf: aVersionInfo) anySatisfy:
				[:ea | self processVersionInfo: ea].
	self popLayer.
	success ifTrue: [self addToCurrentLayer: aVersionInfo].
	^ success	! !

!MCFilteredVersionSorter methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
target: aVersionInfo
	target := aVersionInfo! !

!MCVersionSorter methodsFor: 'as yet unclassified' stamp: 'avi 9/11/2004 14:19'!
addAllAncestorsOf: aVersionInfo to: aSet
	(aSet includes: aVersionInfo) ifTrue: [^ self].
	aSet add: aVersionInfo.
	(self knownAncestorsOf: aVersionInfo) do:
		[:ea |
		self addAllAncestorsOf: ea to: aSet]! !

!MCVersionSorter methodsFor: 'as yet unclassified' stamp: 'avi 8/31/2003 21:30'!
addAllVersionInfos: aCollection
	aCollection do: [:ea | self addVersionInfo: ea]! !

!MCVersionSorter methodsFor: 'as yet unclassified' stamp: 'kph 5/16/2007 20:06'!
addAllVersions: aCollection
	aCollection do: [:ea | self addVersionInfo: ea info]! !

!MCVersionSorter methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
addToCurrentLayer: aVersionInfo
	| layer |
	layer := layers at: depthIndex.
	(layer includes: aVersionInfo) ifFalse:
		[depths at: aVersionInfo ifPresent:
			[:i |
			i < depthIndex
				ifTrue: [(layers at: i) remove: aVersionInfo]
				ifFalse: [^ false]].
		layer add: aVersionInfo.
		depths at: aVersionInfo put: depthIndex.
		^ true].
	^ false ! !

!MCVersionSorter methodsFor: 'as yet unclassified' stamp: 'avi 9/11/2004 10:49'!
addVersionInfo: aVersionInfo
	roots add: aVersionInfo.
	self registerStepChildrenOf: aVersionInfo seen: Set new! !

!MCVersionSorter methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
allAncestorsOf: aVersionInfo
	| all |
	all := Set new.
	self addAllAncestorsOf: aVersionInfo to: all.
	^ all! !

!MCVersionSorter methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
initialize
	stepparents := Dictionary new.
	roots := OrderedCollection new.! !

!MCVersionSorter methodsFor: 'as yet unclassified' stamp: 'avi 9/11/2004 14:37'!
knownAncestorsOf: aVersionInfo
	^ aVersionInfo ancestors, (self stepParentsOf: aVersionInfo) asArray! !

!MCVersionSorter methodsFor: 'as yet unclassified' stamp: 'ab 8/17/2003 15:53'!
layers
	^ layers! !

!MCVersionSorter methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
popLayer
	depthIndex := depthIndex - 1! !

!MCVersionSorter methodsFor: 'as yet unclassified' stamp: 'avi 9/11/2004 10:39'!
processVersionInfo: aVersionInfo
	(self addToCurrentLayer: aVersionInfo) ifTrue:
		[self pushLayer.
		(self knownAncestorsOf: aVersionInfo) do: [:ea | self processVersionInfo: ea].
		self popLayer]
! !

!MCVersionSorter methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
pushLayer
	depthIndex := depthIndex + 1.
	depthIndex > layers size ifTrue: [layers add: OrderedCollection new].
	! !

!MCVersionSorter methodsFor: 'as yet unclassified' stamp: 'avi 9/11/2004 14:34'!
registerStepChildrenOf: aVersionInfo seen: aSet
	(aSet includes: aVersionInfo) ifTrue: [^ self].
	aSet add: aVersionInfo.
	aVersionInfo stepChildren do: [:ea | (self stepParentsOf: ea) add: aVersionInfo].
	aVersionInfo ancestors do: [:ea | self registerStepChildrenOf: ea seen: aSet].! !

!MCVersionSorter methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
sortedVersionInfos
	layers := OrderedCollection with: OrderedCollection new.
	depthIndex := 1.
	depths := Dictionary new.
	roots do: [:ea | self processVersionInfo: ea].
	^ layers gather: [:ea | ea]! !

!MCVersionSorter methodsFor: 'as yet unclassified' stamp: 'avi 9/11/2004 10:40'!
stepParentsOf: aVersionInfo
	^ (stepparents at: aVersionInfo ifAbsentPut: [Set new])! !

Object subclass: #MCWriter
	instanceVariableNames: 'stream'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Storing'!

MCWriter subclass: #MCMczWriter
	instanceVariableNames: 'zip infoWriter'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Storing'!

MCMczWriter subclass: #MCMcdWriter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Storing'!

!MCMcdWriter class methodsFor: 'as yet unclassified' stamp: 'avi 2/13/2004 23:09'!
readerClass
	^ MCMcdReader! !

!MCMcdWriter methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
writeBaseInfo: aVersionInfo
	| string |
	string := self serializeVersionInfo: aVersionInfo.
	self addString: string at: 'base'.
! !

!MCMcdWriter methodsFor: 'as yet unclassified' stamp: 'avi 2/17/2004 01:48'!
writeDefinitions: aVersion
	self writeBaseInfo: aVersion baseInfo.
	self writePatch: aVersion patch.! !

!MCMcdWriter methodsFor: 'as yet unclassified' stamp: 'avi 2/17/2004 02:07'!
writeNewDefinitions: aCollection
	self addString: (self serializeDefinitions: aCollection) at: 'new/source.', self snapshotWriterClass extension.! !

!MCMcdWriter methodsFor: 'as yet unclassified' stamp: 'avi 2/17/2004 02:07'!
writeOldDefinitions: aCollection
	self addString: (self serializeDefinitions: aCollection) at: 'old/source.', self snapshotWriterClass extension.! !

!MCMcdWriter methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
writePatch: aPatch
	| old new |
	old := OrderedCollection new.
	new := OrderedCollection new.
	aPatch operations do:
		[:ea |
		ea isRemoval ifTrue: [old add: ea definition].
		ea isAddition ifTrue: [new add: ea definition].
		ea isModification ifTrue: [old add: ea baseDefinition. new add: ea definition]].
	self writeOldDefinitions: old.
	self writeNewDefinitions: new.
	self addString: (self serializeInBinary: aPatch) at: 'patch.bin'.! !

!MCMczWriter class methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
fileOut: aVersion on: aStream
	| inst |
	inst := self on: aStream.
	inst writeVersion: aVersion.
	inst flush.
	
! !

!MCMczWriter class methodsFor: 'as yet unclassified' stamp: 'cwp 8/1/2003 12:35'!
readerClass
	^ MCMczReader! !

!MCMczWriter methodsFor: 'writing' stamp: 'stephaneducasse 2/4/2006 20:47'!
addString: string at: path
	| member |
	member := zip addString: string as: path.
	member desiredCompressionMethod: ZipArchive compressionDeflated 
	! !

!MCMczWriter methodsFor: 'writing' stamp: 'avi 2/17/2004 02:17'!
flush
	zip writeTo: stream.
	stream close! !

!MCMczWriter methodsFor: 'accessing' stamp: 'avi 2/17/2004 01:54'!
format
	^ '1'! !

!MCMczWriter methodsFor: 'initializing' stamp: 'stephaneducasse 2/4/2006 20:47'!
initialize
	zip := ZipArchive new.
! !

!MCMczWriter methodsFor: 'serializing' stamp: 'stephaneducasse 2/4/2006 20:47'!
serializeDefinitions: aCollection
	| writer s |
	s := RWBinaryOrTextStream on: String new.
	writer := self snapshotWriterClass on: s.
	writer writeDefinitions: aCollection.
	^ s contents! !

!MCMczWriter methodsFor: 'serializing' stamp: 'stephaneducasse 2/4/2006 20:47'!
serializeInBinary: aSnapshot
	| writer s |
	s := RWBinaryOrTextStream on: String new.
	writer := DataStream on: s.
	writer nextPut: aSnapshot.
	^ s contents! !

!MCMczWriter methodsFor: 'serializing' stamp: 'cwp 8/13/2003 01:06'!
serializePackage: aPackage
	^ '(name ''', aPackage name, ''')'! !

!MCMczWriter methodsFor: 'serializing' stamp: 'stephaneducasse 2/4/2006 20:47'!
serializeVersionInfo: aVersionInfo
	infoWriter ifNil: [infoWriter := MCVersionInfoWriter new].
	^ String streamContents:
		[:s |
		infoWriter stream: s.
		infoWriter writeVersionInfo: aVersionInfo]! !

!MCMczWriter methodsFor: 'accessing' stamp: 'avi 2/17/2004 02:07'!
snapshotWriterClass
	^ MCStWriter! !

!MCMczWriter methodsFor: 'visiting' stamp: 'avi 2/17/2004 01:48'!
writeDefinitions: aVersion
	self writeSnapshot: aVersion snapshot! !

!MCMczWriter methodsFor: 'visiting' stamp: 'avi 2/17/2004 01:56'!
writeFormat
"	self addString: self format at: 'format'."! !

!MCMczWriter methodsFor: 'visiting' stamp: 'avi 2/17/2004 01:48'!
writePackage: aPackage
	self addString: (self serializePackage: aPackage) at: 'package'! !

!MCMczWriter methodsFor: 'visiting' stamp: 'avi 9/28/2004 14:26'!
writeSnapshot: aSnapshot
	self addString: (self serializeDefinitions: aSnapshot definitions) at: 'snapshot/source.', self snapshotWriterClass extension.
	self addString: (self serializeInBinary: aSnapshot) at: 'snapshot.bin'! !

!MCMczWriter methodsFor: 'visiting' stamp: 'avi 9/13/2004 16:49'!
writeVersion: aVersion
	self writeFormat.
	self writePackage: aVersion package.
	self writeVersionInfo: aVersion info.
	self writeDefinitions: aVersion.
	aVersion dependencies do: [:ea | self writeVersionDependency: ea]! !

!MCMczWriter methodsFor: 'visiting' stamp: 'stephaneducasse 2/4/2006 20:47'!
writeVersionDependency: aVersionDependency
	| string |
	string := (self serializeVersionInfo: aVersionDependency versionInfo).
	self addString: string at: 'dependencies/', aVersionDependency package name! !

!MCMczWriter methodsFor: 'visiting' stamp: 'stephaneducasse 2/4/2006 20:47'!
writeVersionInfo: aVersionInfo
	| string |
	string := self serializeVersionInfo: aVersionInfo.
	self addString: string at: 'version'.
! !

!MCMczWriter methodsFor: 'accessing' stamp: 'cwp 8/1/2003 00:06'!
zip
	^ zip! !

MCWriter subclass: #MCStWriter
	instanceVariableNames: 'initStream'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Chunk Format'!

!MCStWriter class methodsFor: 'as yet unclassified' stamp: 'avi 1/20/2004 00:16'!
readerClass
	^ MCStReader! !

!MCStWriter methodsFor: 'writing' stamp: 'cwp 8/2/2003 02:34'!
chunkContents: aBlock
	stream cr; nextChunkPut: (String streamContents: aBlock); cr! !

!MCStWriter methodsFor: 'visiting' stamp: 'cwp 8/2/2003 11:02'!
visitClassDefinition: definition
	self writeClassDefinition: definition.
	definition hasClassInstanceVariables ifTrue: [self writeMetaclassDefinition: definition].
	definition hasComment ifTrue: [self writeClassComment: definition].! !

!MCStWriter methodsFor: 'visiting' stamp: 'al 10/9/2005 19:52'!
visitClassTraitDefinition: definition
	self chunkContents: [:s | s
		nextPutAll: definition baseTrait;
		nextPutAll: ' classTrait';
		cr; tab;
		nextPutAll: 'uses: ';
		nextPutAll: (definition classTraitComposition ifNil: ['{}'])]
! !

!MCStWriter methodsFor: 'visiting' stamp: 'al 10/9/2005 19:40'!
visitMetaclassDefinition: definition
	self writeMetaclassDefinition: definition! !

!MCStWriter methodsFor: 'visiting' stamp: 'avi 2/17/2004 02:23'!
visitMethodDefinition: definition
	self writeMethodPreamble: definition.
	self writeMethodSource: definition.
	self writeMethodPostscript.
	self writeMethodInitializer: definition.! !

!MCStWriter methodsFor: 'visiting' stamp: 'cwp 8/2/2003 11:02'!
visitOrganizationDefinition: defintion
	defintion categories do: [:cat | self writeCategory: cat].
! !

!MCStWriter methodsFor: 'visiting' stamp: 'al 10/9/2005 19:40'!
visitTraitDefinition: definition
	self writeClassDefinition: definition.
	definition hasComment ifTrue: [self writeClassComment: definition].! !

!MCStWriter methodsFor: 'writing' stamp: 'cwp 8/2/2003 01:46'!
writeCategory: categoryName
	stream
		nextChunkPut: 'SystemOrganization addCategory: ', categoryName printString;
		cr! !

!MCStWriter methodsFor: 'writing' stamp: 'ab 8/17/2003 17:09'!
writeClassComment: definition
	stream
		cr;
		nextPut: $!!;
		nextPutAll: definition className;
		nextPutAll: ' commentStamp: ';
		store: definition commentStamp;
		nextPutAll: ' prior: 0!!';
		cr;
		nextChunkPut: definition comment;
		cr.! !

!MCStWriter methodsFor: 'writing' stamp: 'cwp 8/2/2003 02:16'!
writeClassDefinition: definition
	self chunkContents: [:s | definition printDefinitionOn: stream]! !

!MCStWriter methodsFor: 'writing' stamp: 'dvf 9/8/2004 10:28'!
writeDefinitions: aCollection
	"initStream is an ugly hack until we have proper init defs"
	initStream := String new writeStream.

	(MCDependencySorter sortItems: aCollection)
		do: [:ea | ea accept: self]
		displayingProgress: 'Writing definitions...'.
	
	stream nextPutAll: initStream contents.! !

!MCStWriter methodsFor: 'writing' stamp: 'al 12/2/2005 15:17'!
writeMetaclassDefinition: definition
	self chunkContents: [:str |
		str	nextPutAll: definition className;
			nextPutAll: ' class';
			cr; tab.
			definition hasClassTraitComposition ifTrue: [
				str	nextPutAll: 'uses: ';
					nextPutAll: definition classTraitCompositionString;
					cr; tab].
			str	nextPutAll: 'instanceVariableNames: ''';
				nextPutAll: definition classInstanceVariablesString;
				nextPut: $']! !

!MCStWriter methodsFor: 'writing' stamp: 'avi 2/17/2004 02:24'!
writeMethodInitializer: aMethodDefinition
	aMethodDefinition isInitializer ifTrue:
		[initStream nextChunkPut: aMethodDefinition className, ' initialize'; cr]! !

!MCStWriter methodsFor: 'writing' stamp: 'cwp 8/2/2003 12:43'!
writeMethodPostscript
	stream
		space;
		nextPut: $!!;
		cr! !

!MCStWriter methodsFor: 'writing' stamp: 'avi 9/23/2003 17:42'!
writeMethodPreamble: definition
	stream
		cr;
		nextPut: $!!;
		nextPutAll: definition fullClassName;
		nextPutAll: ' methodsFor: ';
		nextPutAll: definition category asString printString;
		nextPutAll: ' stamp: ';
		nextPutAll: definition timeStamp asString printString;
		nextPutAll: '!!';
		cr! !

!MCStWriter methodsFor: 'writing' stamp: 'cwp 8/4/2003 01:35'!
writeMethodSource: definition
	stream nextChunkPut: definition source! !

!MCStWriter methodsFor: 'writing' stamp: 'avi 2/17/2004 02:25'!
writeSnapshot: aSnapshot
	self writeDefinitions: aSnapshot definitions! !

MCWriter subclass: #MCVersionInfoWriter
	instanceVariableNames: 'written'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Storing'!

!MCVersionInfoWriter methodsFor: 'as yet unclassified' stamp: 'avi 1/22/2004 21:10'!
isWritten: aVersionInfo
	^ self written includes: aVersionInfo! !

!MCVersionInfoWriter methodsFor: 'as yet unclassified' stamp: 'avi 9/13/2004 16:53'!
writeVersionInfo: aVersionInfo
	(self isWritten: aVersionInfo)
		ifTrue: [^ stream nextPutAll: '(id ', aVersionInfo id asString printString, ')'].
	stream nextPut: $(.
	#(name message id date time author) 
		do: [:sel | 
			stream nextPutAll: sel.
			stream nextPut: $ .
			((aVersionInfo perform: sel) ifNil: ['']) asString printOn: stream.
			stream nextPut: $ ].
	stream nextPutAll: 'ancestors ('.
	aVersionInfo ancestors do: [:ea | self writeVersionInfo: ea].
	stream nextPutAll: ') stepChildren ('.
	aVersionInfo stepChildren do: [:ea | self writeVersionInfo: ea].
	stream nextPutAll: '))'.
	self wrote: aVersionInfo! !

!MCVersionInfoWriter methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
written
	^ written ifNil: [written := Set new]! !

!MCVersionInfoWriter methodsFor: 'as yet unclassified' stamp: 'avi 1/22/2004 21:10'!
wrote: aVersionInfo
	self written add: aVersionInfo! !

!MCWriter class methodsFor: 'accessing' stamp: 'cwp 8/1/2003 15:00'!
extension
	^ self readerClass extension! !

!MCWriter class methodsFor: 'writing' stamp: 'cwp 8/1/2003 01:16'!
on: aStream
	^ self new stream: aStream! !

!MCWriter class methodsFor: 'accessing' stamp: 'cwp 7/28/2003 23:46'!
readerClass
	^ self subclassResponsibility ! !

!MCWriter methodsFor: 'as yet unclassified' stamp: 'cwp 8/1/2003 01:14'!
stream
	^ stream! !

!MCWriter methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
stream: aStream
	stream := aStream! !

!Object methodsFor: '*monticello' stamp: 'dvf 8/10/2004 23:25'!
isConflict
	^false! !

ListItemWrapper subclass: #MCDependentsWrapper
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-UI'!

!MCDependentsWrapper methodsFor: 'as yet unclassified' stamp: 'ar 2/14/2004 02:31'!
asString
	^item description! !

!MCDependentsWrapper methodsFor: 'as yet unclassified' stamp: 'avi 9/10/2004 17:54'!
contents
	| list workingCopies |
	workingCopies := model unsortedWorkingCopies.
	list := item requiredPackages collect: 
					[:each | 
					workingCopies detect: [:wc | wc package = each] ifNone: [nil]]
				thenSelect: [:x | x notNil].
	^list collect: [:each | self class with: each model: model]! !

!MCDependentsWrapper methodsFor: 'as yet unclassified' stamp: 'ar 2/14/2004 02:31'!
hasContents
	^item requiredPackages isEmpty not! !

!MCDependentsWrapper methodsFor: 'as yet unclassified' stamp: 'ar 2/14/2004 02:41'!
item
	^item! !

!PseudoMetaclass methodsFor: '*monticello-override' stamp: 'nk 2/18/2004 18:30'!
isMeta
	^true! !

!PackageInfo class methodsFor: '*monticello-orphanage' stamp: 'kph 5/14/2007 13:28'!
orphanage

	^ MCOrphanage current! !

!PackageInfo methodsFor: '*monticello-orphanage' stamp: 'kph 5/14/2007 16:58'!
allClassesAsDefinitions
	^ self loadedClassesAsDefinitions , self orphanedClassesAsDefinitons! !

!PackageInfo methodsFor: '*monticello' stamp: 'kph 8/3/2008 03:57'!
manager

	^ self propertyAt: #mc! !

!PackageInfo methodsFor: '*monticello' stamp: 'kph 8/3/2008 03:58'!
manager: mcPackageManager

	^ self propertyAt: #mc put: mcPackageManager! !

!PackageInfo methodsFor: '*monticello-orphanage' stamp: 'kph 5/14/2007 13:36'!
orphanedClassesAsDefinitons

	  "convenience method implemented here to enable users of packageInfo to be aware that there are additional classes in the orphanage which belong to this package."
	
	^ (self class orphanage orphansFor: self) select: [ :m | m isClassDefinition ]  ! !

!PackageInfo methodsFor: '*monticello-orphanage' stamp: 'kph 5/14/2007 13:31'!
orphanedMethodsAsDefinitons

	  "convenience method implemented here to enable users of packageInfo to be aware that there are additional methods in the orphanage which belong to this package."
	
	^ (self class orphanage orphansFor: self) select: [ :m | m isMethodDefinition ]  ! !

!CrLfFileStream methodsFor: '*monticello' stamp: 'stephaneducasse 2/4/2006 20:47'!
lineEndingConvention: aSymbol	
	lineEndConvention := aSymbol! !

!FilePackage methodsFor: '*monticello' stamp: 'al 12/2/2005 13:58'!
classDefinition: string with: chgRec
	| tokens theClass |
	
	self flag: #traits.
		
	tokens := Scanner new scanTokens: string.

	"tokens size = 11 ifFalse:[^doIts add: chgRec]."

	theClass := self getClass: (tokens at: 3).
	theClass definition: string.
	classOrder add: theClass.! !

!FilePackage methodsFor: '*monticello' stamp: 'avi 1/19/2004 23:47'!
doIts
	^ doIts! !

!ClassTrait methodsFor: '*monticello' stamp: 'al 3/26/2006 21:31'!
asMCDefinition
	^MCClassTraitDefinition
		baseTraitName: self baseTrait name
		classTraitComposition: self traitCompositionString
			! !

Error subclass: #MCNoVersionInfoAvailable
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Versioning'!

Notification subclass: #MCAbortNotification
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Loading'!

Notification subclass: #MCChangeSelectionRequest
	instanceVariableNames: 'patch label'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Versioning'!

!MCChangeSelectionRequest methodsFor: 'as yet unclassified' stamp: 'avi 9/14/2004 15:01'!
defaultAction
	^ (MCChangeSelector new patch: patch; label: label) showModally! !

!MCChangeSelectionRequest methodsFor: 'as yet unclassified' stamp: 'avi 9/14/2004 15:02'!
label
	^ label! !

!MCChangeSelectionRequest methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
label: aString
	label := aString! !

!MCChangeSelectionRequest methodsFor: 'as yet unclassified' stamp: 'avi 9/11/2004 15:12'!
patch
	^ patch! !

!MCChangeSelectionRequest methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
patch: aPatch
	patch := aPatch! !

Notification subclass: #MCMergeResolutionRequest
	instanceVariableNames: 'merger'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Versioning'!

!MCMergeResolutionRequest methodsFor: 'as yet unclassified' stamp: 'bf 4/26/2005 14:25'!
defaultAction
	^ (MCMergeBrowser new
		merger: merger;
		label: messageText) showModally! !

!MCMergeResolutionRequest methodsFor: 'as yet unclassified' stamp: 'ab 7/18/2003 18:19'!
merger
	^ merger! !

!MCMergeResolutionRequest methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
merger: aMerger
	merger := aMerger! !

Notification subclass: #MCVersionNameAndMessageRequest
	instanceVariableNames: 'suggestion'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Versioning'!

!MCVersionNameAndMessageRequest methodsFor: 'as yet unclassified' stamp: 'ab 8/24/2003 20:39'!
defaultAction
	^ MCSaveVersionDialog new
		versionName: suggestion;
		showModally! !

!MCVersionNameAndMessageRequest methodsFor: 'as yet unclassified' stamp: 'ab 7/10/2003 01:07'!
suggestedName
	^ suggestion! !

!MCVersionNameAndMessageRequest methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
suggestedName: aString
	suggestion := aString! !

!ChangeList class methodsFor: '*monticello' stamp: 'stephaneducasse 2/4/2006 20:47'!
recent: charCount on: origChangesFile 
	"Opens a changeList on the end of the specified changes log file"
	| changeList end changesFile |
	changesFile := origChangesFile readOnlyCopy.
	end := changesFile size.
	Cursor read
		showWhile: [changeList := self new
						scanFile: changesFile
						from: (0 max: end - charCount)
						to: end].
	changesFile close.
	^changeList! !

!ChangeList class methodsFor: '*monticello' stamp: 'stephaneducasse 2/4/2006 20:47'!
recentLogOn: origChangesFile startingFrom: initialPos 
	"Prompt with a menu of how far back to go when browsing a changes file."

	| end banners positions pos chunk i changesFile |
	changesFile := origChangesFile readOnlyCopy.
	banners := OrderedCollection new.
	positions := OrderedCollection new.
	end := changesFile size.
	pos := initialPos.
	[pos = 0
		or: [banners size > 20]]
		whileFalse: [changesFile position: pos.
			chunk := changesFile nextChunk.
			i := chunk indexOfSubCollection: 'priorSource: ' startingAt: 1.
			i > 0
				ifTrue: [positions addLast: pos.
					banners
						addLast: (chunk copyFrom: 5 to: i - 2).
					pos := Number
								readFrom: (chunk copyFrom: i + 13 to: chunk size)]
				ifFalse: [pos := 0]].
	changesFile close.
	banners size == 0 ifTrue: [^self recent: end on: origChangesFile].

	pos := (SelectionMenu labelList: banners selections: positions)
				startUpWithCaption: 'Browse as far back as...'.
	pos == nil
		ifTrue: [^ self].
	^self recent: end - pos on: origChangesFile! !

!ChangeList methodsFor: '*monticello' stamp: 'stephaneducasse 2/4/2006 20:47'!
changeTo: changeSubset
	| newList newChangeList |

	newChangeList := OrderedCollection new.
	newList := OrderedCollection new.

	1 to: changeList size do:
		[:i | (changeSubset includes: (changeList at: i)) ifTrue:
			[newChangeList add: (changeList at: i).
			newList add: (list at: i)]].
	newChangeList size < changeList size
		ifTrue:
			[changeList := newChangeList.
			list := newList.
			listIndex := 0.
			listSelections := Array new: list size withAll: false].
	self changed: #list

	! !

Warning subclass: #MCOrphanedNotification
	instanceVariableNames: 'orphan'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Orphanage'!

!MCOrphanedNotification class methodsFor: 'as yet unclassified' stamp: 'kph 5/15/2007 13:05'!
orphan: aDefinition signal: reason
	
	^(self new)
		orphan: aDefinition;
		signal: reason
		yourself! !

!MCOrphanedNotification methodsFor: 'accessing' stamp: 'kph 5/13/2007 04:53'!
orphan
	"Answer the value of orphan"

	^ orphan! !

!MCOrphanedNotification methodsFor: 'accessing' stamp: 'kph 5/13/2007 04:53'!
orphan: anObject
	"Set the value of orphan"

	orphan := anObject! !

!ChangeRecord methodsFor: '*monticello' stamp: 'avi 9/14/2004 14:27'!
asMethodDefinition
	^ MCMethodDefinition 
		className: class
		classIsMeta: meta
		selector: self methodSelector
		category: category
		timeStamp: stamp
		source: self string! !

!Encoder methodsFor: '*monticello-override' stamp: 'mtf 6/9/2008 20:43'!
undeclared: name

	| sym | 
	requestor interactive ifTrue: [
		requestor requestor == #error: ifTrue: [requestor error: 'Undeclared'].
		^ self notify: 'Undeclared'].
	"Transcript show: ' (' , name , ' is Undeclared) '."
	sym := name asSymbol.
	Undeclared at: sym put: nil.
	^self global: (Undeclared associationAt: sym) name: sym! !

!SequenceableCollection methodsFor: '*monticello' stamp: 'mtf 6/27/2008 23:03'!
mcLessThan: otherCollection ifEqual: aBlock
	| a b |
	1 to: (self size min: otherCollection size) do: [:index |
		a := self at: index. b := otherCollection at: index.
		a ~= b ifTrue: [ ^ a <= b ]].
	^ self size ~= otherCollection size
		ifTrue: [self size <= otherCollection size]
		ifFalse: [aBlock value]! !

Exception subclass: #MCNoChangesException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Versioning'!

!MCNoChangesException methodsFor: 'as yet unclassified' stamp: 'jf 8/21/2003 19:49'!
defaultAction
	self inform: 'No changes'! !

ClassBuilder subclass: #MCClassBuilder
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Loading'!

!MCClassBuilder methodsFor: 'as yet unclassified' stamp: 'kph 6/7/2008 03:28'!
doneCompiling: aClass
	"The receiver has finished modifying the class hierarchy.
	Do any necessary cleanup."
	aClass doneCompiling.
	! !

!MCClassBuilder methodsFor: 'as yet unclassified' stamp: 'kph 6/7/2008 01:31'!
name: className inEnvironment: env subclassOf: newSuper type: type instanceVariableNames: instVarString classVariableNames: classVarString poolDictionaries: poolString category: category unsafe: unsafe
	"Define a new class in the given environment.
	If unsafe is true do not run any validation checks.
	This facility is provided to implement important system changes."
	| oldClass newClass organization instVars classVars force needNew oldCategory copyOfOldClass newCategory |
 
	environ := env.
	instVars := Scanner new scanFieldNames: instVarString.
	classVars := (Scanner new scanFieldNames: classVarString) collect: [:x | x asSymbol].

	"Validate the proposed name"
	unsafe ifFalse:[(self validateClassName: className) ifFalse:[^nil]].
	oldClass := env at: className ifAbsent:[nil].
	oldClass isBehavior 
		ifFalse: [oldClass := nil]  "Already checked in #validateClassName:"
		ifTrue: [
			copyOfOldClass := oldClass copy.
			copyOfOldClass superclass addSubclass: copyOfOldClass].
	
	
	[unsafe ifFalse:[
		"Run validation checks so we know that we have a good chance for recompilation"
		(self validateSuperclass: newSuper forSubclass: oldClass) ifFalse:[^nil].
		(self validateInstvars: instVars from: oldClass forSuper: newSuper) ifFalse:[^nil].
		(self validateClassvars: classVars from: oldClass forSuper: newSuper) ifFalse:[^nil].
		(self validateSubclassFormat: type from: oldClass forSuper: newSuper extra: instVars size) ifFalse:[^nil]].

	"See if we need a new subclass"
	needNew := self needsSubclassOf: newSuper type: type instanceVariables: instVars from: oldClass.
	needNew == nil ifTrue:[^nil]. "some error"

	(needNew and:[unsafe not]) ifTrue:[
		"Make sure we don't redefine any dangerous classes"
		(self tooDangerousClasses includes: oldClass name) ifTrue:[
			self error: oldClass name, ' cannot be changed'.
		].
		"Check if the receiver should not be redefined"
		(oldClass ~~ nil and:[oldClass shouldNotBeRedefined]) ifTrue:[
			self notify: oldClass name asText allBold, 
						' should not be redefined. \Proceed to store over it.' withCRs]].

	needNew ifTrue:[
		"Create the new class"
		newClass := self 
			newSubclassOf: newSuper 
			type: type 
			instanceVariables: instVars
			from: oldClass.
		newClass == nil ifTrue:[^nil]. "Some error"
		newClass setName: className.
	] ifFalse:[
		"Reuse the old class"
		newClass := oldClass.
	].

	"Install the class variables and pool dictionaries... "
	force := (newClass declare: classVarString) | (newClass sharing: poolString).

	"... classify ..."
	newCategory := category asSymbol.
	organization := environ ifNotNil:[environ organization].
	oldClass isNil ifFalse: [oldCategory := (organization categoryOfElement: oldClass name) asSymbol].
	organization classify: newClass name under: newCategory.
	newClass environment: environ.

	"... recompile ..."
	newClass := self recompile: force from: oldClass to: newClass mutate: false.

	"... export if not yet done ..."
	(environ at: newClass name ifAbsent:[nil]) == newClass ifFalse:[
		[environ at: newClass name put: newClass]
			on: AttemptToWriteReadOnlyGlobal do:[:ex| ex resume: true].
		Smalltalk flushClassNameCache.
	].

	"... notify interested clients ..."
	oldClass isNil ifTrue: [
		SystemChangeNotifier uniqueInstance classAdded: newClass inCategory: newCategory.
		^ newClass].
	newCategory ~= oldCategory 
		ifTrue: [SystemChangeNotifier uniqueInstance class: newClass recategorizedFrom: oldCategory to: category]
		ifFalse: [SystemChangeNotifier uniqueInstance classDefinitionChangedFrom: copyOfOldClass to: newClass.].
] ensure: 
		[copyOfOldClass ifNotNil: [copyOfOldClass superclass removeSubclass: copyOfOldClass].
		Behavior flushObsoleteSubclasses.
		].
	^newClass! !
MCDefinition initialize!
MCMethodDefinition initialize!
MCPackageLoader initialize!
MCWorkingCopy initialize!
MCVersionReader initialize!
MCRepository initialize!
MCCacheRepository initialize!
MCWorkingCopyBrowser initialize!
